! MODULE Constants_Module
! $Id: Constants_Module.f90,v 1.0 2015/11/10 
!
! NAME:
!   Constants
!
! FUNCTION:
!   Contain the constants used by the R-CLIPER programs
!
! DESCRIPTION:
!   This module holds the constants needed to run the R-CLIPER code.
!
! REFERENCE:
!
! CALLING SEQUENCE:
!
! INPUTS:
!
! OUTPUTS:
!
! DEPENDENCIES:
!
! RESTRICTIONS:
!
! HISTORY:
!
!-------------------------------------------------------------------------------
MODULE Constants_Module

   IMPLICIT NONE

   !------ Maximum forecast lead time in hours ---------------------------------
   INTEGER, PARAMETER :: MAX_LEAD_TIME = 24

   !----- Time interval (h) between forecast points in track forecast ----------
   REAL, PARAMETER :: TRACK_FORECAST_INTERVAL = 12.0

   !------ Number of lead times to read from input track file ------------------
   INTEGER, PARAMETER :: N_TRACK_TIMES = 1 + MAX_LEAD_TIME / &
    NINT(TRACK_FORECAST_INTERVAL)

   !------ Output interval in hours --------------------------------------------
   INTEGER, PARAMETER :: OUTPUT_INTERVAL_HOURS = 6

   !------ Number of output files (at 6-h intervals) ---------------------------
   INTEGER, PARAMETER :: N_OUT_TIMES = 1 + MAX_LEAD_TIME / OUTPUT_INTERVAL_HOURS

   !------ Number of column and rows in R-CLIPER grid array --------------------
   INTEGER, PARAMETER :: NX_RCLIPER = 1000
   INTEGER, PARAMETER :: NY_RCLIPER = 1000

   !------ Grid spacing (degrees) of R-CLIPER grid array -----------------------
   REAL, PARAMETER :: GRID_SPACE = 0.04

   !------ Half-width of the R-CLIPER domain -----------------------------------
   REAL, PARAMETER :: HALFWIDTH_LON = 0.5 * FLOAT(NX_RCLIPER - 1) * GRID_SPACE
   REAL, PARAMETER :: HALFWIDTH_LAT = 0.5 * FLOAT(NY_RCLIPER - 1) * GRID_SPACE

   !------ Time step (hours) in calculations -----------------------------------
   REAL, PARAMETER :: TIME_STEP = 0.50

   !------ Maximum possible number of time steps -------------------------------
   INTEGER, PARAMETER :: MAX_TIME_STEPS = NINT(FLOAT(MAX_LEAD_TIME) / &
    TIME_STEP) + 1

   !------ Number of lead times in SHIPS diagnostic file -----------------------
   INTEGER, PARAMETER :: N_SHIPS_TIMES = 21

   !------ RCliper Home 
   CHARACTER(LEN=*), PARAMETER :: Root_Rcliper = '/home/Petrap/RCliper/'

   !------ SHIPS file name and unit number -------------------------------------
   INTEGER, PARAMETER :: UNIT_SHIPS = 10
   CHARACTER(LEN=*), PARAMETER :: Filename_SHIPS = Root_Rcliper // 'lsdiag.dat'

   !------ Storm track file name and unit number -------------------------------
   INTEGER, PARAMETER :: UNIT_TRACK = 10
   CHARACTER(LEN=*), PARAMETER :: FILENAME_TRACK =  Root_Rcliper // 'comab.dat'

   !------ Coastline file name and unit number ---------------------------------
   INTEGER, PARAMETER :: COASTLINE_UNIT = 55
   CHARACTER(LEN=*), PARAMETER :: FILENAME_COASTLINE =  Root_Rcliper // 'RCLIPERcoastlines.dat'

   !------ Log file name and unit number ---------------------------------------
   INTEGER, PARAMETER :: LOG_UNIT = 81
   CHARACTER(LEN=*), PARAMETER :: FILENAME_LOG =  Root_Rcliper // 'rcliperl.log'

   !------ Base unit number for output files and coastline file ----------------
   INTEGER, PARAMETER :: OUT_UNIT_BASE = 60

   !------ Missing value -------------------------------------------------------
   REAL, PARAMETER :: VAL_MSG = 9999.

   !------ Assumed radius of maximum winds -------------------------------------
   REAL, PARAMETER :: RADIUS_OF_MAX_WINDS = 35.

   !------ Global DEM for orographic correction --------------------------------
   INTEGER, PARAMETER :: NX_TOPO = 9000
   INTEGER, PARAMETER :: NY_TOPO = 4500
   REAL, PARAMETER :: DEM_LAT_MIN =  -90.0
   REAL, PARAMETER :: DEM_LAT_MAX =   90.0
   REAL, PARAMETER :: DEM_LON_MIN = -180.0
   REAL, PARAMETER :: DEM_LON_MAX =  180.0
   INTEGER, PARAMETER :: UNIT_TOPO = 10 

!------ Constants and conversion factors ---------------------------------------
   !------ Pi ------------------------------------------------------------------
   REAL, PARAMETER :: PI = 3.1415926536

   !------ Multiplier to convert meters per second to knots --------------------
   REAL, PARAMETER :: MPS_TO_KTS = 1.94384449

   !------ Multiplier to convert meters per second to km per hour --------------
   REAL, PARAMETER :: MPS_TO_KPH = 3.6

   !------ Multiplier to convert degrees latitude to nautical miles ------------
   REAL, PARAMETER :: DEG_TO_NAUTMI = 60.

   !------ Multipler to convert degrees latitude to kilometers -----------------
   REAL, PARAMETER :: DEG_TO_KM = 111.1

   !------ Multiplier to convert degrees to radians ----------------------------
   REAL, PARAMETER :: DEG_TO_RAD = PI / 180.

   !------ Source of climatology used for R-CLIPER: 1-gauge; 0=TRMM ------------
   INTEGER, PARAMETER :: USE_GAUGE_CLIMO = 0

   !------ Shear flag (1=account for shear; 0=do not account for it) -----------
   INTEGER, PARAMETER :: USE_SHEAR = 1

   !------ Multiplier for R-CLIPER climatological rain rate equation -----------
   REAL, PARAMETER :: A0  = 6.4 / 24.0

   !------ Intercept for R-CLIPER climatological rain rate equation ------------
   REAL, PARAMETER :: B0  = 1.0 / 24.0

   !------ Time adjustment for R-CLIPER rain rate equation ---------------------
   REAL, PARAMETER :: TIME_ADJ  = 0.029

   !------ Minimum radius (km) for R-CLIPER rain rate equation -----------------
   REAL, PARAMETER :: RADIUS_MIN = 20.0

   !------ Maximum radius (km) for R-CLIPER rain rate equation -----------------
   REAL, PARAMETER :: RADIUS_MAX = 160.0

   !----- Lower limit of maximum sustained winds (kt) --------------------------
   REAL, PARAMETER :: WIND_SPEED_MIN = 20.0

   !----- Acceptable range of maximum sustained winds (kt) for R-CLIPER climo --
   REAL, PARAMETER :: WIND_SPEED_MINBOUND = 35.0
   REAL, PARAMETER :: WIND_SPEED_MAXBOUND = 136.0

   !----- Shear amplitude thresholds -------------------------------------------
   REAL, PARAMETER :: SHEAR_LOW_MODERATE = 9.7
   REAL, PARAMETER :: SHEAR_MODERATE_HIGH = 19.4

   !----- Debug write flag and unit number for writing -------------------------
   INTEGER, PARAMETER :: FLAG_DEBUG = 0
   LOGICAL, PARAMETER :: FLAG_PRINT = .FALSE.
   INTEGER, PARAMETER :: UNIT_DEBUG = 0
   INTEGER, PARAMETER :: LOG_TIME_STEPS = 0

   !----- Parameters for the quadratic interpolation subroutine ----------------
   INTEGER, PARAMETER :: N_ITERATIONS = 100 
   REAL, PARAMETER :: CONVERGE_CHECK_INIT = 1.0e+10
   REAL, PARAMETER :: CONVERGE_COST_THRESHOLD = 1.0e-10
   REAL, PARAMETER :: LINEARITY_THRESHOLD = 1.0e-10

   !---- Maximum number of coastlines in 'aland.dat' and maximum number of -----
   !---- points per coastline --------------------------------------------------
   INTEGER, PARAMETER :: MAX_NUM_COASTLINES = 1000
   INTEGER, PARAMETER :: MAX_COASTLINE_POINTS = 2000

   !---- Base AREA file number to write output ---------------------------------
   INTEGER, PARAMETER :: OUT_AREA_BASE = 4000

   !---- Scale factor for converting output from inches to 0-255 ---------------
   REAL, PARAMETER :: SCALE_FACTOR_DAILY = 40. / 255.
   REAL, PARAMETER :: SCALE_FACTOR_HOURLY = 20. / 255.

END MODULE Constants_Module
