!-------------------------------------------------------------------------------
! NAME:
!    Error_Messaging_Module
!
! FUNCTION:
!    Provide a common error messaging format.
!
! DESCRIPTION:
!    This module defines three parameters and contains a subroutine that will
!    write the information contained in the input fields to the standard output
!    along with a time/date stamp.
!
! REFERENCE:
!
! CALLING SEQUENCE:
!    CALL Error_Messaging ( Routine_Name, Message, Level )
!
! INPUTS:
!    CHARACTER(*) Routine_Name     Name of routine issuing the error message
!    CHARACTER(*) Message          The message to be printed
!    INTEGER      Level            Severity level:
!                                     0 = NOTICE
!                                     1 = WARNING
!                                     2 = FATAL ERROR
!
! OUTPUTS:
!    Prints the error message to standard output.
!
! DEPENDENCIES:
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!    2007-06-05 Peter Keehn  PSGS/QSS  Initial development
!
!-------------------------------------------------------------------------------

MODULE Error_Messaging_Module

   USE Type_Kinds

   IMPLICIT NONE

!== PARAMETERS =================================================================
   INTEGER(LONG), PARAMETER :: NOTICE   = 0
   INTEGER(LONG), PARAMETER :: WARNING  = 1
   INTEGER(LONG), PARAMETER :: FATAL    = 2
           
   CONTAINS

      SUBROUTINE Error_Messaging (Routine_Name, Message, Level)
 
      IMPLICIT NONE
    
!== VARIABLES ==================================================================
      CHARACTER (LEN=*), INTENT (IN)  :: Routine_Name
      CHARACTER (LEN=*), INTENT (IN)  :: Message
      INTEGER (LONG), INTENT (IN) :: Level

      CHARACTER (LEN=8)  :: Date
      CHARACTER (LEN=10) :: Time
      CHARACTER (LEN=19) :: DateTime

!== MAIN SUBROUTINE ============================================================
 
      !--Get the current time --------------------------------------------------
      CALL DATE_AND_TIME(Date, Time)

      !-- Convert the Date and Time strings to one readable string -------------
      DateTime = Date(1:4)//"-"//Date(5:6)//"-"//Date(7:8)//" "// &
                 Time(1:2)//":"//Time(3:4)//":"//Time(5:6)

      ! -- Write the message to standard output --------------------------------
      SELECT CASE (Level)

         !-- Notice message ----------------------------------------------------
         CASE (NOTICE)
            WRITE (*, '(a, a, a, a, a)') &
              DateTime, " NOTICE: (",TRIM(Routine_Name),") ", TRIM(Message)

         !-- Warning message ---------------------------------------------------
         CASE (WARNING)
            WRITE (*, '(a, a, a, a, a)') &
              DateTime, " WARNING: (",TRIM(Routine_Name),") ", TRIM(Message)

         !-- (Fatal) error message ---------------------------------------------
         CASE (FATAL)
            WRITE (*, '(a, a, a, a, a)') &
              DateTime, " FATAL ERROR: (",TRIM(Routine_Name),") ", TRIM(Message)
 
         !-- All others (unknown) ----------------------------------------------
         CASE DEFAULT
            WRITE (*, '(a, a, a, a, a)') &
              DateTime, " UNKNOWN: (",TRIM(Routine_Name),") ", TRIM(Message)

      END SELECT

      RETURN

   END SUBROUTINE Error_Messaging

END MODULE Error_Messaging_Module
