!-------------------------------------------------------------------------------
! NAME: Rainfall_Module
!
! TYPE/LANGUAGE: F90 module
!
! FUNCTION:
!   Contains subroutines for computing the R-CLIPER rainfall, accounting for
!   the effects of shear, and adjusting for orographic effects.
!
! DESCRIPTION:
!   This module contains the subroutines for computing the R-CLIPER rainfall,
!   accounting for the effects of shear, and adjusting for orographic effects.
!
! FILES NEEDED: N/A
!
! MODULES/SUBROUTINES NEEDED:
!   - Type_Kinds.f90
!   - Constants_Module.f90
!
! SUBROUTINES CONTAINED:
!   - CalculateRainfall
!   - AccumulateRainfall
!
! CALLING SEQUENCE: N/A
!
! INPUTS: N/A
!
! OUTPUTS: N/A
!
! SYSTEM CALLS: N/A
!
! REFERENCE: None
!
! DEPENDENCIES:
!   -  Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!               M. DeMaria / F. Marks  Original code
!   2016-09-13  B. Kuligowski   Convert to F90; clean up to meet SPSRB standards
!-------------------------------------------------------------------------------
MODULE Rainfall_Module

USE TYPE_KINDS
USE Constants_Module

IMPLICIT NONE

CONTAINS

!===============================================================================
! NAME: CalculateRainfall
!
! TYPE/LANGUAGE: F90 subroutine
!
! FUNCTION:
!   Computes the R-CLIPER rain rates based on storm climatology and shear, with
!   an adjustment for orographic effects.
!
! DESCRIPTION:
!   This routine calculates the rain rate (RainRate, inches per hour) as a
!   function of the time inland (time, hr), the maximum wind speed
!   (WindSpeedMax, kt), the radius from the storm center (rad, km), and the
!   magnitude and direction of vertical shear (if option is selected). It also
!   makes an adjustment for orographic effects.
!
! FILES NEEDED:
!   - None.
!
! MODULES/SUBROUTINES NEEDED:
!   - Type_Kinds.f90
!   - Constants_Module.f90
!
! SUBROUTINES CONTAINED: N/A
!
! CALLING SEQUENCE:
!   CALL CalculateRainfall(TimeSinceLandfall, WindSpeedMax, DistanceToCenter, 
!      PolarAngle, ShearAmplitude, RainRate)
!
! INPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   TimeSinceLandfall    I     Time since landfall (gauge climo ONLY)
!   WindSpeedMax         I     Current maximum sustained wind speed (kt)
!   DistanceToCenter     I     Distance from pixel to storm center
!   PolarAngle           I     Polar bearing (degrees) from storm center
!   ShearAmplitude       I     Shear magnitude (kt)
!
! OUTPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   RainRate             O     Rainfall rate (in/hr) at pixel
!
! SYSTEM CALLS:
!   - None
!
! REFERENCE: None
!   - None
!
! DEPENDENCIES:
!   -  Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:

! SUBROUTINE: CalculateRainfall
!
! DESCRIPTION:
!===============================================================================
   SUBROUTINE CalculateRainfall(TimeSinceLandfall, WindSpeedMax, &
    DistanceToCenter, PolarAngle, ShearAmplitude, RainRate)

   USE TYPE_KINDS
   USE Constants_Module

   IMPLICIT NONE

!== VARIABLES ==================================================================
!-- INPUTS: --------------------------------------------------------------------
   REAL, INTENT(IN) :: TimeSinceLandfall
   REAL, INTENT(IN) :: WindSpeedMax
   REAL, INTENT(IN) :: DistanceToCenter
   REAL, INTENT(IN) :: PolarAngle
   REAL, INTENT(IN) :: ShearAmplitude

!-- OUTPUTS: -------------------------------------------------------------------
   REAL, INTENT(OUT) :: RainRate

!-- PROCESSING VARIABLES -------------------------------------------------------
   !-- Adjusted time since landfall (gauge climo only) -------------------------
   REAL :: TimeAdjusted

   !-- Fourier coefficients for rain rate --------------------------------------
   REAL :: a_1, a_2, b_1, b_2

   !-- Range-limited maximum wind speed ----------------------------------------
   REAL :: WindSpeedMaxLimited

   REAL :: UnshearedRainRate
   REAL :: StormCategory
   REAL :: RainRateIntercept
   REAL :: RainRateSlope
   REAL :: ThresholdRadius

!== MAIN SUBROUTINE ============================================================
   !-- Use rainfall rate from gauge climatology (function of t, r) -------------
   IF (USE_GAUGE_CLIMO == 1) THEN
      TimeAdjusted = EXP(-TIME_ADJ * MAX(TimeSinceLandfall, 0.))
!
      IF (DistanceToCenter <= RADIUS_MIN) THEN
         RainRate = (A0 * TimeAdjusted + B0)
      ELSE
         RainRate = (A0 * TimeAdjusted + B0) * &
          EXP(-(DistanceToCenter - RADIUS_MIN) / RADIUS_MAX)
      ENDIF
!
      RETURN
   ENDIF
!
!  Rainfall rate from TRMM climatology (function of WindSpeedMax,r)
!
   !-- Begin by limiting the wind speed to the range in the R-CLIPER climo -----
   WindSpeedMaxLimited = MIN(MAX(WindSpeedMax, WIND_SPEED_MINBOUND), &
    WIND_SPEED_MAXBOUND)
!
   StormCategory = 1.0 + (WindSpeedMaxLimited - WIND_SPEED_MINBOUND) / 33.0
!
   !-- Compute the unsheared R-CLIPER rainfall rate ----------------------------
   RainRateIntercept =  (3.96 * StormCategory - 1.10) / 24.0
   RainRateSlope =  (4.80 * StormCategory - 1.60) / 24.0
   ThresholdRadius =  -13.0 * StormCategory + 64.5
!
   !-- Rain rate is a linear function of radius out to the threshold radius ----
   IF (DistanceToCenter <= ThresholdRadius) THEN
      UnshearedRainRate = RainRateIntercept + &
       (RainRateSlope - RainRateIntercept) * &
       (DistanceToCenter / ThresholdRadius)

   !-- Rain rate is an exponential function of radius outside threshold radius -
   ELSE
      UnshearedRainRate = RainRateSlope * &
       EXP(-(DistanceToCenter - ThresholdRadius) / &
       (-16.0 * StormCategory + 150.0))
   ENDIF
!   WRITE (6,*) "DistC ThrRad Unsheared", DistanceToCenter,ThresholdRadius, UnshearedRainRate

   !-- If no shear, use the unsheared rain rate and return from subroutine -----
   IF (USE_SHEAR == 0) THEN
      RainRate = UnshearedRainRate
      RETURN
   ENDIF

   !-- Compute rain rates from 5 to 600 km from the storm center ---------------
   IF (DistanceToCenter >= 5.0001 .AND. DistanceToCenter < 600.) THEN

      !-- Calculations if the maximum wind speed < 65 kt (tropical storm) ------
      IF (WindSpeedMaxLimited < 65.) THEN

         !-- Low shear ---------------------------------------------------------
         IF (ShearAmplitude < SHEAR_LOW_MODERATE) THEN
            a_1 = -17.5 * EXP(-(DistanceToCenter - 100.) ** 2. / &
             ((DistanceToCenter+50.)*0.5) ** 2.)+7.5
            b_1 = 1.1 * (DistanceToCenter - 5.) ** 0.6 - &
             EXP((DistanceToCenter - 125.) / 100.)
            a_2 = 4. - 10. * EXP(-(DistanceToCenter - 245.) ** 2. / 250. ** 2.)
            b_2 = 1. - 10. * EXP(-(DistanceToCenter - 335.) ** 2. / 60. ** 2.)

         !-- Moderate shear ----------------------------------------------------
         ELSE IF (ShearAmplitude >= SHEAR_LOW_MODERATE .AND. &
          ShearAmplitude <  SHEAR_MODERATE_HIGH) THEN
            a_1 = -20. * EXP(-(DistanceToCenter - 65.) ** 2. / &
             ((DistanceToCenter + 50.) * 0.65) ** 2.) + 2.5
            b_1 = 1.1 * (DistanceToCenter - 5.) ** 0.75 - &
             ((DistanceToCenter - 25.) / 47.5) ** 2.
            a_2 = -2.5 - 12.5 * &
             EXP(0. - (DistanceToCenter - 245.) ** 2. / 100. ** 2.)
            b_2 = -1.5 * (DistanceToCenter - 5.) ** 0.25

         !-- High shear --------------------------------------------------------
         ELSE
            a_1 = -35. * EXP(-(DistanceToCenter - 85.) ** 2. / &
             ((DistanceToCenter + 150.) * 0.35) ** 2.) -5.
            b_1 = 1.1 * (DistanceToCenter - 5.) ** 0.75 - &
             ((DistanceToCenter - 25.) / 50.) ** 2. + 12.5
            a_2 = -1.5 - 20. * &
             EXP(-(DistanceToCenter - 205.) ** 2. / 100. ** 2.)
            b_2 = -2.5 * (DistanceToCenter - 5.) ** 0.32
         ENDIF

      !-- Calculations if the maximum wind speed is 65-95 kt (Category 1-2) ----
      ELSE IF (WindSpeedMaxLimited >= 65. .AND. WindSpeedMaxLimited < 95.) THEN

         !-- Low shear ---------------------------------------------------------
         IF (ShearAmplitude < SHEAR_LOW_MODERATE) THEN
            a_1 = -15. * EXP(-(DistanceToCenter - 100.) ** 2. / &
             ((DistanceToCenter + 300.) * 0.25) ** 2.) + &
             (-8. + DistanceToCenter / 20.) - &
             EXP((DistanceToCenter - 175.) / 85.)
            b_1 = 1.1 * (DistanceToCenter - 5.) ** 0.35
            a_2 = 10. -8. * EXP((DistanceToCenter) ** 2. / 600. ** 2.)
            b_2 = 6.5 -8. * EXP(-(DistanceToCenter -245.) ** 2. / 250. ** 2.) &
             - EXP((DistanceToCenter - 250.) / 100.) 

         !-- Moderate shear ----------------------------------------------------
         ELSE IF (ShearAmplitude >= SHEAR_LOW_MODERATE .AND.&
          ShearAmplitude < SHEAR_MODERATE_HIGH) THEN       
            a_1 = -15. * EXP(-(DistanceToCenter - 100.) ** 2. / &
             ((DistanceToCenter + 300.) * 0.2) ** 2.)- 9. - &
             EXP((DistanceToCenter - 250.) / 100.)
            b_1 = 1.1 * (DistanceToCenter - 5.) ** 0.75 - &
             2.5 * (DistanceToCenter / 50.) ** 1.5 + 2.5
            a_2 = -2.5 - 10. * EXP(-(DistanceToCenter - 225.) ** 2. / &
             (100. - (DistanceToCenter - 100.) / 10.) ** 2.)
            b_2 = -1.25 * (DistanceToCenter - 5.) ** 0.2 - &
             EXP((DistanceToCenter - 400.) / 100.)

         !-- High shear --------------------------------------------------------
         ELSE
            a_1 = -25. * EXP(-(DistanceToCenter - 100.) ** 2. / &
             ((DistanceToCenter + 100.) * 0.75) **2.) - 9. + &
             EXP((DistanceToCenter - 200.) / 75.)
            b_1 = 1.1 * (DistanceToCenter - 5.) ** 0.7 - &
             ((DistanceToCenter - 25.) / 75.) ** 2. + 5.
            a_2 = -1.5 - 16. * &
             EXP(-(DistanceToCenter - 375.) ** 2. / 200. ** 2.)
            b_2 = -2.5 * (DistanceToCenter - 5.) ** 0.23
         ENDIF

      !-- Calculations if the maximum wind speed > 95 kt (Category 3+) ---------
      ELSE

         !-- Low shear ---------------------------------------------------------
         IF (ShearAmplitude < SHEAR_LOW_MODERATE) THEN
            a_1 = -15. * EXP(-(DistanceToCenter - 100.) ** 2. / &
             ((DistanceToCenter + 300.) * 0.5) ** 2.) + &
             (5. + DistanceToCenter / 20.) -2. * &
             EXP((DistanceToCenter - 75.) / 150.)
            b_1 = 1.1 * (DistanceToCenter - 5.) ** 0.35
            a_2 = 1.5 - 11.5 * &
             EXP(-(DistanceToCenter - 375.) ** 2. / 125. ** 2.)
            b_2 = 6.5 -  8.0 * &
             EXP(-(DistanceToCenter - 245.) ** 2. / 250. ** 2.) + &
             22.5 * EXP(-(DistanceToCenter - 410.) ** 2. / 50. ** 2.)

         !-- Moderate shear ----------------------------------------------------
         ELSE IF (ShearAmplitude >= SHEAR_LOW_MODERATE .AND. &
          ShearAmplitude <  SHEAR_MODERATE_HIGH) THEN      
            a_1 = -15. * EXP(-(DistanceToCenter - 150.) ** 2. / &
             ((DistanceToCenter + 500.) * 0.15) ** 2.) - 11.
            b_1 = 1.1 * ((DistanceToCenter - 5.) / 10.) ** 1.3 - &
             7. * ((DistanceToCenter - 50.) / 100.) ** 2.
            a_2 = 1.  - 10. * EXP(-(DistanceToCenter - 350.) ** 2. / 125. ** 2.)
            b_2 = 4.5 - 20. * EXP(-(DistanceToCenter - 300.) ** 2. / 100. ** 2.)

         !-- High shear --------------------------------------------------------
         ELSE
            a_1 = -15. * EXP(-(DistanceToCenter - 100.) ** 2. / &
             ((DistanceToCenter + 300.) * 0.35) ** 2.) - 15.
            b_1 = 1.1 * (DistanceToCenter - 5.) ** 0.72 - &
             ((DistanceToCenter - 25.) / 50.) ** 2. + 5.
            a_2 = -1.5 - 20. * EXP(-(DistanceToCenter - 245.) ** 2. / &
             50. ** 2.) + 15. * EXP(-(DistanceToCenter - 340.) ** 2. / &
             50. ** 2.)
            b_2 = -5.  - 10. * EXP(-(DistanceToCenter - 245.) ** 2. / &
             100. ** 2.)
         ENDIF
      ENDIF

   !-- Rain rates are zero within 5 km or more than 600 km from center ---------
   ELSE
       a_1 = 0.
       b_1 = 0.
       a_2 = 0.
       b_2 = 0.
   ENDIF
!   WRITE (6,*) "Unsheared PolAng",UnshearedRainRate, PolarAngle

   RainRate = UnshearedRainRate + &
    a_1 * (UnshearedRainRate / 100.) * COS(PolarAngle * DEG_TO_RAD) + &
    b_1 * (UnshearedRainRate / 100.) * SIN(PolarAngle * DEG_TO_RAD) + &
    a_2 * (UnshearedRainRate / 100.) * COS(2. * PolarAngle * DEG_TO_RAD)+ &
    b_2 * (UnshearedRainRate / 100.) * SIN(2. * PolarAngle * DEG_TO_RAD)
 
   RETURN
   END SUBROUTINE CalculateRainfall


!===============================================================================
! NAME: AccumulateRainfall
!
! TYPE/LANGUAGE: F90 subroutine
!
! FUNCTION:
!   Calculates the R-CLIPER rainfall accumulation.
!
! DESCRIPTION:
!   This subroutine calculates the storm-total rainfall (inches) using a
!   climatological tropical cyclone rainfall rate as a function of radius from
!   the storm center and time since landfall, modified by the shear forecast
!   from the SHIPS diagnostic file and for orographic effects using a computed
!   wind field combined with digital topography.
!
! FILES NEEDED:
!   - None
!
! MODULES/SUBROUTINES NEEDED:
!   - Type_Kinds.f90
!   - Constants_Module.f90
!   - land_module.f90
!   - Rainrate_shear_topo.f90
!
! SUBROUTINES CONTAINED: N/A
!
! CALLING SEQUENCE:
!   CALL AccumulateRainfall(CenterLat, CenterLon, WindSpeedMax, StormTime,&
!    GridLat, GridLon, TotalRain, ShearAmplitude, ShearHeading)
!
! INPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   CenterLat            I     Storm center latitudes from forecast track
!   CenterLon            I     Storm center longitudes from forecast track
!   WindSpeedMax         I/O   Maximum wind speeds from forecast track
!   StormTime            I     Time of forecast point
!   ShearAmplitude       I     Magnitude of shear from SHIPS diagnostic file
!   ShearHeading       I     Direction of shear from SHIPS diagnostic file
!
! OUTPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   GridLon             O     Longitudes of each column of R-CLIPER subgrid
!   GridLat             O     Latitudes of each row of R-CLIPER subgrid
!   TotalRain           O     Accumulated rainfall
!
! SYSTEM CALLS:
!   - None
!
! REFERENCE:
!   - None
!
! DEPENDENCIES:
!   - Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!               M. DeMaria / F. Marks  Original code
!   2016-09-13  B. Kuligowski   Convert to F90; clean up to meet SPSRB standards
!
!-------------------------------------------------------------------------------
   SUBROUTINE AccumulateRainfall(CenterLat, CenterLon, WindSpeedMax, StormTime,&
    GridLat, GridLon, TotalRain, ShearAmplitude, ShearHeading)

   USE Type_Kinds
   USE Constants_Module
   USE land_module
   USE rcliper_subroutines

   IMPLICIT NONE

!== VARIABLES ==================================================================
!-- INPUTS: --------------------------------------------------------------------
   !-- Storm track information from track file ---------------------------------
   REAL, DIMENSION(1:N_TRACK_TIMES), INTENT(IN) :: CenterLat
   REAL, DIMENSION(1:N_TRACK_TIMES), INTENT(IN) :: CenterLon
   REAL, DIMENSION(1:N_TRACK_TIMES), INTENT(INOUT) :: WindSpeedMax
   REAL, DIMENSION(1:N_TRACK_TIMES), INTENT(IN) :: StormTime

   !-- Shear information from SHIPS diagnostic file ----------------------------
   REAL, DIMENSION(1:N_TRACK_TIMES), INTENT(IN) :: ShearAmplitude
   REAL, DIMENSION(1:N_TRACK_TIMES), INTENT(IN) :: ShearHeading

!-- OUTPUTS: -------------------------------------------------------------------
   !-- R-CLIPER subgrid coordinates --------------------------------------------
   REAL, DIMENSION(1:NX_RCLIPER), INTENT(OUT) :: GridLon
   REAL, DIMENSION(1:NY_RCLIPER), INTENT(OUT) :: GridLat

   !-- Accumulated rainfall for each 6-h time period and the total -------------
   REAL, DIMENSION(1:NX_RCLIPER,1:NY_RCLIPER,1:N_OUT_TIMES), INTENT(OUT) :: &
    TotalRain

!-- PROCESSING VARIABLES -------------------------------------------------------
   !-- Counters ----------------------------------------------------------------
   INTEGER :: i, k, iTrackTime, iTimeStep

   !-- Storm lat, lon, max wind speed, and time interpolated to time step ------
   REAL, DIMENSION(1:MAX_TIME_STEPS) :: CenterLatInterp
   REAL, DIMENSION(1:MAX_TIME_STEPS) :: CenterLonInterp
   REAL, DIMENSION(1:MAX_TIME_STEPS) :: WindSpeedMaxInterp
   REAL, DIMENSION(1:MAX_TIME_STEPS) :: StormTimeInterp

   !-- Storm speed and heading at each time step -------------------------------
   REAL, DIMENSION(1:MAX_TIME_STEPS) :: StormSpeed
   REAL, DIMENSION(1:MAX_TIME_STEPS) :: StormHeading
   REAL, DIMENSION(1:MAX_TIME_STEPS) :: TimeSinceLandfall
   REAL, DIMENSION(1:MAX_TIME_STEPS) :: StormDistanceToLand

   !-- Shear amplitude and heading interpolated to time step -------------------
   REAL, DIMENSION(1:MAX_TIME_STEPS) :: ShearAmpInterp
   REAL, DIMENSION(1:MAX_TIME_STEPS) :: ShearHeadInterp

   REAL :: dx, dy, dxy
   REAL :: GridCenterLat, GridCenterLon
   REAL :: HeadingAngle

   !-- Flags for interpolation error, initialization, and interp type ----------
   INTEGER :: InterpErrorFlag
   INTEGER :: InitializationFlag
   INTEGER :: InterpTypeFlag
   INTEGER :: index
   INTEGER :: itimes
   !-- Unused speed variable sent to 'CalcSpeedHeadingFromLatLons' subroutine --
   REAL :: UnusedSpeed
   INTEGER :: iFirstTimeStepInland
   REAL :: DistanceToCenter
   REAL :: Time_Landfall

   !-- Climatological rain rate at grid point ----------------------------------
   REAL :: RainRate
   REAL :: CurrentShearHeading, CurrentShearAmplitude
   REAL :: speed
   REAL :: ShearHeadingChange, ShearHeadingChangeRate
   REAL :: TimeStep1, TimeStep2
   REAL :: PolarHeading
   REAL :: time1
   REAL :: CurrentMaxWindSpeed

   !-- Variables for computing change in elevation during time step ------------
   INTEGER :: ixRCLIPER, iyRCLIPER

   !-- Real and integer output interval for determining which sub-daily output -
   !-- files to write to -------------------------------------------------------
   INTEGER :: OutputInterval
   REAL :: RealInterval
      
!== MAIN SUBROUTINE ============================================================

   !-- Compute the time (hours) at each time step (initial time = 0) -----------
   DO iTimeStep = 1, MAX_TIME_STEPS
      StormTimeInterp(iTimeStep) = TIME_STEP * FLOAT(iTimeStep - 1)
   END DO

   !-- Stop execution if maximum wind speed for initial track point is too low -
   IF (WindSpeedMax(1) <= WIND_SPEED_MIN) WRITE (LOG_UNIT,'(/,a)') &
       ' WARNING--There must be at least one intensity point'

   !-- If forecast maximum wind speed is below threshold value, replace with ---
   !-- value from previous forecast time ---------------------------------------
   DO iTimeStep = 2, N_TRACK_TIMES
      IF (WindSpeedMax(iTimeStep) < WIND_SPEED_MIN) WindSpeedMax(iTimeStep) = &
       WindSpeedMax(iTimeStep - 1)
   END DO

   !-- Write  time step length to log file -------------------------------------
   WRITE (LOG_UNIT,'(/,a,f4.0,a,i3,a,f4.1,a)') &
    ' Track forecast out to ', StormTimeInterp(MAX_TIME_STEPS), &
    ' hrs Divide track into ', MAX_TIME_STEPS, ' points', TIME_STEP, &
    ' hrs apart'

   !-- Set interpolation type flag to quadratic --------------------------------
   InterpTypeFlag = 1

   !-- Interpolate track forecast center latitudes to R-CLIPER time steps ------
   DO iTimeStep = 1, MAX_TIME_STEPS
      IF (iTimeStep == 1) THEN
         InitializationFlag = 1
      ELSE
         InitializationFlag = 0
      ENDIF
      CALL QuadraticInterpolation(StormTime, CenterLon, N_TRACK_TIMES, & 
       InitializationFlag, InterpTypeFlag, StormTimeInterp(iTimeStep), &
       CenterLonInterp(iTimeStep), InterpErrorFlag)
      CenterLonInterp(iTimeStep) = -CenterLonInterp(iTimeStep)
   END DO
!   WRITE (6,*) "CENTLONINTERPdb ", CenterLonInterp(1),CenterLonInterp(N_TRACK_TIMES - 1)

   !-- Interpolate track forecast center longitudes to R-CLIPER time steps -----
   DO iTimeStep = 1, MAX_TIME_STEPS
      IF (iTimeStep == 1) THEN
         InitializationFlag = 1
      ELSE
         InitializationFlag = 0
      ENDIF
      CALL QuadraticInterpolation(StormTime, CenterLat, N_TRACK_TIMES, &
       InitializationFlag, InterpTypeFlag, StormTimeInterp(iTimeStep), &
       CenterLatInterp(iTimeStep), InterpErrorFlag)
   END DO

   !-- Compute number of R-CLIPER time steps between each SHIPS forecast time --
   itimes = (MAX_TIME_STEPS - 1) / (N_TRACK_TIMES - 1)

   !-- Check this number and make sure it is consistent ------------------------
   IF ((itimes * (N_TRACK_TIMES - 2) + itimes + 1) /= MAX_TIME_STEPS) THEN
      CALL Error_Messaging('AccumulateRainfall', &
       'Computed number of time steps does not match MAX_TIME_STEPS', FATAL)
      STOP
   ENDIF

   !-- Interpolate the SHIPS forecast shear heading to the R-CLIPER time steps -
   ShearHeadInterp(1) = ShearHeading(1)
   DO iTrackTime = 1, N_TRACK_TIMES - 1
      DO iTimeStep = 1, itimes
         index = itimes * (iTrackTime - 1) + iTimeStep

         !-- Value bewteen forecast periods changes by more than +180 degrees --
         IF (ShearHeading(iTrackTime+1) - ShearHeading(iTrackTime) > 180.) &
          THEN
            ShearHeadingChange = (ShearHeading(iTrackTime+1) - &
             ShearHeading(iTrackTime)) - 360.
            ShearHeadingChangeRate = (ShearHeadingChange / FLOAT(itimes)) * &
             FLOAT(iTimeStep)
            ShearHeadInterp(index+1) = ShearHeading(iTrackTime) + &
             ShearHeadingChangeRate
            IF (ShearHeadInterp(index+1) < 0.) ShearHeadInterp(index+1) = &
             ShearHeadInterp(index+1) + 360.

         !-- Value bewteen forecast periods changes by more than -180 degrees --
         ELSE IF (ShearHeading(iTrackTime+1) - ShearHeading(iTrackTime) < &
          -180.) THEN
            ShearHeadingChange = (ShearHeading(iTrackTime+1) - &
             ShearHeading(iTrackTime)) + 360.
            IF (ShearHeadingChange > 360.) ShearHeadingChange = &
             ShearHeadingChange - 360.
            ShearHeadingChangeRate = (ShearHeadingChange / FLOAT(itimes)) * &
             FLOAT(iTimeStep)
            ShearHeadInterp(index+1) = ShearHeading(iTrackTime) + &
             ShearHeadingChangeRate
            IF (ShearHeadInterp(index+1) > 360.) ShearHeadInterp(index+1) = &
             ShearHeadInterp(index+1) - 360.

         !-- Value bewteen forecast periods changes by less than -180 to +180 --
         !-- degrees -----------------------------------------------------------
         ELSE
            ShearHeadInterp(index+1) = ShearHeading(iTrackTime) + &
             ((ShearHeading(iTrackTime+1) - ShearHeading(iTrackTime)) / &
             FLOAT(itimes)) * FLOAT(iTimeStep)
         ENDIF

      !-- Move on to the next R-CLIPER time step -------------------------------
      END DO

   !-- Move on to the next SHIPS time step -------------------------------------
   END DO
!   WRITE (6,*) "ShearHeadInterp ", ShearHeadInterp(1),ShearHeadInterp(2),ShearHeadInterp(N_TRACK_TIMES - 1)
!   WRITE (6,*) "ShearAmpInterp ", ShearAmpInterp(1),ShearAmpInterp(2),ShearAmpInterp(N_TRACK_TIMES - 1)

   !-- Interpolate SHIPS forecast shear amplitude to R-CLIPER time steps -------
   DO iTimeStep = 1, MAX_TIME_STEPS
      IF (iTimeStep == 1) THEN
         InitializationFlag = 1
      ELSE
         InitializationFlag = 0
      ENDIF
      CALL QuadraticInterpolation(StormTime, ShearAmplitude, N_TRACK_TIMES, &
       InitializationFlag, InterpTypeFlag, StormTimeInterp(iTimeStep), &
       ShearAmpInterp(iTimeStep), InterpErrorFlag)
   END DO

   !-- Linearly interpolate forecast max wind speed to R-CLIPER time steps and -
   !-- the storm speed and heading for each time step from the interpolated ----
   !-- center lat / lon coordinates --------------------------------------------
   InterpTypeFlag = 0

!   WRITE (6,*) "In Accum"
!   WRITE (6,*) "WINDSP1 ", WindSpeedMax
!   WRITE (6,*) "CENTLAT ", CenterLat
!   WRITE (6,*) "CENTLON ", CenterLon

   DO iTimeStep = 1, MAX_TIME_STEPS
      IF (iTimeStep == 1) THEN
         InitializationFlag = 1
      ELSE
         InitializationFlag = 0
      ENDIF
      CALL QuadraticInterpolation(StormTime, WindSpeedMax, N_TRACK_TIMES, &
       InitializationFlag, InterpTypeFlag, StormTimeInterp(iTimeStep), &
       WindSpeedMaxInterp(iTimeStep), InterpErrorFlag)

      !-- Compute current storm speed and heading from the interpolated track --
      !-- For the first time step, use the first two time steps ----------------
      IF (iTimeStep == 1) THEN
         CALL CalcSpeedHeadingFromLatLons &
          (CenterLonInterp(1), CenterLatInterp(1), StormTimeInterp(1), &
           CenterLonInterp(2), CenterLatInterp(2), StormTimeInterp(2), &
           StormSpeed(iTimeStep), StormHeading(iTimeStep))

      !-- For the tine steps in between, use the time steps immediately before
      !-- and after the current time step (centered difference) ----------------
      ELSE IF (iTimeStep == MAX_TIME_STEPS) THEN
         CALL CalcSpeedHeadingFromLatLons &
          (CenterLonInterp(N_TRACK_TIMES-1), CenterLatInterp(N_TRACK_TIMES-1), &
           StormTimeInterp(N_TRACK_TIMES-1), &
           CenterLonInterp(N_TRACK_TIMES), CenterLatInterp(N_TRACK_TIMES), &
           StormTimeInterp(N_TRACK_TIMES), &
           StormSpeed(iTimeStep), StormHeading(iTimeStep))

      !-- For the last time step, use the last two time steps ------------------
      ELSE
         CALL CalcSpeedHeadingFromLatLons &
          (CenterLonInterp(iTimeStep-1), CenterLatInterp(iTimeStep-1), &
           StormTimeInterp(iTimeStep-1), &
           CenterLonInterp(iTimeStep+1), CenterLatInterp(iTimeStep+1), &
           StormTimeInterp(iTimeStep+1), &
           StormSpeed(iTimeStep), StormHeading(iTimeStep))
      ENDIF
   END DO

   !-- Calculate the distance to land at the interpolated positions ------------
   DO iTimeStep = 1, MAX_TIME_STEPS
      CALL ComputeStormDistanceToLand(CenterLonInterp(iTimeStep), &
       CenterLatInterp(iTimeStep), StormDistanceToLand(iTimeStep))
   END DO

   !-- Calculate the time since landfall at any point along interpolated track -
   !-- If storm starts off over water, set the initial time since landfall to 0
   IF (StormDistanceToLand(1) >= 0.0) THEN
      TimeSinceLandfall(1) = 0.0

   !-- If the first position is over land, estimate the time since landfall ----
   !-- based upon the current speed and the distance to land. ------------------
   ELSE
      CALL DistanceBetweenPoints(CenterLonInterp(1), CenterLatInterp(1), &
       CenterLonInterp(2), CenterLatInterp(2), dx, dy, dxy)
      speed = dxy / (StormTimeInterp(2) - StormTimeInterp(1))
      time1 = -StormDistanceToLand(1) / speed
      TimeSinceLandfall(1) = TIME_STEP * FLOAT(ifix(time1/TIME_STEP))
   ENDIF

   !-- Caulculate the time since landfall for the remaining points -------------
   DO iTimeStep = 2, MAX_TIME_STEPS
      IF (StormDistanceToLand(iTimeStep) < 0.0) THEN
         TimeSinceLandfall(iTimeStep) = TimeSinceLandfall(iTimeStep-1) + &
          TIME_STEP
      ELSE 
         TimeSinceLandfall(iTimeStep) = 0.0
      ENDIF
   END DO

   !-- Find the time when the storm first moves inland -------------------------
   iFirstTimeStepInland = -99
   DO iTimeStep = 1, MAX_TIME_STEPS
      IF (StormDistanceToLand(iTimeStep) <= 0.0) THEN
         iFirstTimeStepInland = i
         EXIT
      ENDIF
    END DO

   !-- Write the time steps to the log file if flag is on ----------------------
   IF (LOG_TIME_STEPS == 1) THEN
      DO iTimeStep = 1, MAX_TIME_STEPS
         WRITE (LOG_UNIT,'(a,6(f7.2,1x),2(1x,f7.0))') &
          ' t,tld,lon,lat,vmx,c,head,StormDistanceToLand: ', &
          StormTimeInterp(iTimeStep), TimeSinceLandfall(iTimeStep), &
          CenterLonInterp(iTimeStep), CenterLatInterp(iTimeStep), &
          WindSpeedMaxInterp(iTimeStep), StormSpeed(iTimeStep),&
          StormHeading(iTimeStep),StormDistanceToLand(iTimeStep)
      END DO
   ENDIF

   !-- If the storm moves inland, determine the time step when it happens ------
   IF (iFirstTimeStepInland > 0) THEN
      WRITE (LOG_UNIT,'(/,a,f5.1,a,f7.1,1x,f6.1,a,f7.1)') &
       ' Storm inland at t=', StormTimeInterp(iFirstTimeStepInland), &
       ' lon/lat: ', CenterLonInterp(iFirstTimeStepInland), &
       CenterLatInterp(iFirstTimeStepInland), '  StormDistanceToLand=', &
       StormDistanceToLand(iFirstTimeStepInland)
      Time_Landfall = StormTimeInterp(iFirstTimeStepInland)

   !-- If the storm does not move inland, center the grid on the center of the -
   !-- forecast track instead of on the time it moved inland -------------------
   ELSE
      iFirstTimeStepInland = MAX_TIME_STEPS / 2
      WRITE (LOG_UNIT,'(/,a,f5.1,a,f7.1,1x,f6.1,a,f7.1)') &
       ' Storm not inland, center grid at t=', &
       StormTimeInterp(iFirstTimeStepInland), ' lon/lat: ', &
       CenterLonInterp(iFirstTimeStepInland), &
       CenterLatInterp(iFirstTimeStepInland), &
       ' StormDistanceToLand=', StormDistanceToLand(iFirstTimeStepInland)
     Time_Landfall = 999.9
   ENDIF

   !-- Determine center of local R-CLIPER grid (coordinates of storm center at -
   !-- landfall or forecast position halfway through forecast period otherwise)
   GridCenterLon =  FLOAT(ifix(CenterLonInterp(iFirstTimeStepInland) - 0.49))
   GridCenterLat =  FLOAT(ifix(CenterLatInterp(iFirstTimeStepInland) + 0.49))
   WRITE (LOG_UNIT,'(/,a,f7.1,1x,f6.1)') &
    ' Rainfall grid reference lon/lat: ', GridCenterLon, &
    GridCenterLat

   !-- Calculate the longitudes and latitudes of each grid point ---------------
   DO ixRCLIPER = 1, NX_RCLIPER
      GridLon(ixRCLIPER) = (GridCenterLon - HALFWIDTH_LON) + &
       GRID_SPACE * FLOAT(ixRCLIPER-1)
   END DO
   DO iyRCLIPER = 1, NY_RCLIPER
      GridLat(iyRCLIPER) = (GridCenterLat - HALFWIDTH_LAT) + GRID_SPACE * &
       FLOAT(iyRCLIPER-1)
   END DO

   !-- Initialize all rainfall totals ------------------------------------------
   DO ixRCLIPER = 1, NY_RCLIPER
      DO iyRCLIPER = 1, NX_RCLIPER
         DO k = 1, N_OUT_TIMES
            TotalRain(ixRCLIPER,iyRCLIPER,k) = 0.0
         END DO
      END DO
   END DO

   !-- Loop through R-CLIPER time steps ----------------------------------------
   DO iTimeStep = 1, MAX_TIME_STEPS
      CurrentMaxWindSpeed = WindSpeedMaxInterp(iTimeStep)
      CurrentShearHeading = ShearHeadInterp(iTimeStep)
      CurrentShearAmplitude = ShearAmpInterp(iTimeStep)

      !--Define time step for computing storm heading -----------------
      IF (iTimeStep == 1) THEN
         TimeStep1 = StormTimeInterp(1)
         TimeStep2 = StormTimeInterp(2)
      ELSE IF (iTimeStep == MAX_TIME_STEPS) THEN
         TimeStep1 = StormTimeInterp(N_TRACK_TIMES-1)
         TimeStep2 = StormTimeInterp(N_TRACK_TIMES)
      ELSE
         TimeStep1 = StormTimeInterp(iTimeStep-1)
         TimeStep2 = StormTimeInterp(iTimeStep+1)
      ENDIF

      !-- Loop through grid points on R-CLIPER nested grid ---------------------
      DO iyRCLIPER = 1, NY_RCLIPER
         DO ixRCLIPER = 1, NX_RCLIPER
!            WRITE (6,*) "Lon Lat Timestepped",CenterLonInterp(iTimeStep), &
!                  CenterLatInterp(iTimeStep)
!            WRITE (6,*) "GLon GLat ix iy",GridLon(ixRCLIPER), &
!                  GridLat(iyRCLIPER),ixRCLIPER,iyRCLIPER

            CALL DistanceBetweenPoints(CenterLonInterp(iTimeStep), &
             CenterLatInterp(iTimeStep), GridLon(ixRCLIPER), &
             GridLat(iyRCLIPER), dx, dy, DistanceToCenter)
            CALL CalcSpeedHeadingFromLatLons(CenterLonInterp(iTimeStep), &
             CenterLatInterp(iTimeStep), TimeStep1, GridLon(ixRCLIPER), &
              GridLat(iyRCLIPER), TimeStep2, UnusedSpeed, HeadingAngle)

            !-- Convert heading from meteorological to math angle --------------
            HeadingAngle = 90.0 - HeadingAngle
            IF (HeadingAngle < 0.0) HeadingAngle = HeadingAngle + 360.0
            PolarHeading = HeadingAngle - CurrentShearHeading + 90.0
            IF (PolarHeading < 0.)PolarHeading = PolarHeading + 360.0
            CALL CalculateRainfall(TimeSinceLandfall(iTimeStep), &
             CurrentMaxWindSpeed, DistanceToCenter, PolarHeading, &
             CurrentShearAmplitude, RainRate)
!            WRITE (6,*) "RainRate after CalcRain", RainRate

            !-- Increment 00-24 h rainfall using trapezoidal method (i.e., half
            !-- weight at end points) ------------------------------------------
            IF (iTimeStep > 1 .AND. iTimeStep < MAX_TIME_STEPS) THEN
               TotalRain(ixRCLIPER,iyRCLIPER,N_OUT_TIMES) = &
                TotalRain(ixRCLIPER,iyRCLIPER,N_OUT_TIMES) + &
                TIME_STEP * RainRate
            ELSE
               TotalRain(ixRCLIPER,iyRCLIPER,N_OUT_TIMES) = &
                TotalRain(ixRCLIPER,iyRCLIPER,N_OUT_TIMES) + &
                TIME_STEP * RainRate / 2.
            ENDIF

            !-- Accumulate rainfall in time range(s) corresponding to time step
            !-- using the trapezoidal method -----------------------------------
            RealInterval = 1. + FLOAT(iTimeStep - 1) * TIME_STEP / &
             FLOAT(OUTPUT_INTERVAL_HOURS)
            OutputInterval = 1 + INT(FLOAT(iTimeStep - 1) * TIME_STEP) / &
             OUTPUT_INTERVAL_HOURS

            !-- Instances where time step falls on the boundary bewteen two ----
            !-- output intervals and is thus split between the two -------------
            IF (ABS(RealInterval - FLOAT(OutputInterval)) < &
             TIME_STEP / FLOAT(2 * OUTPUT_INTERVAL_HOURS)) THEN

               !-- Half of time step goes to earlier interval except for very --
               !-- first time step (trapezoidal sum) ---------------------------
               IF (iTimeStep > 1) &
                TotalRain(ixRCLIPER,iyRCLIPER,OutputInterval-1) = &
                TotalRain(ixRCLIPER,iyRCLIPER,OutputInterval-1) + &
                TIME_STEP * RainRate / 2.

               !-- Half of last step goes to later interval except for very ----
               !-- last time step (trapezoidal sum) ----------------------------
               IF (iTimeStep < MAX_TIME_STEPS) &
                TotalRain(ixRCLIPER,iyRCLIPER,OutputInterval) = &
                TotalRain(ixRCLIPER,iyRCLIPER,OutputInterval) + &
                TIME_STEP * RainRate / 2.

            !-- Instances where the time step falls entirely within the output
            !-- interval (put whole amount in time interval) -------------------
            ELSE
               TotalRain(ixRCLIPER,iyRCLIPER,OutputInterval) = &
                TotalRain(ixRCLIPER,iyRCLIPER,OutputInterval) + &
                TIME_STEP * RainRate
            ENDIF

         !-- Move on to next R-CLIPER grid box ---------------------------------
         END DO
      END DO

   !-- Move on to next R-CLIPER time step --------------------------------------
   END DO

   RETURN
   END SUBROUTINE AccumulateRainfall

END MODULE Rainfall_Module
