!===============================================================================
! NAME:        land_module
!
! TYPE/LANGUAGE:        F90 module
!
! FUNCTION:
!  This module contains the subroutines for computing the distance between a
!  cylone and the nearest land mass (or if it is ever over land)
!
! DESCRIPTION:  N/A
!
! FILES NEEDED:  N/A
!
! MODULES/SUBROUTINES NEEDED:
!   - type_kinds.f90
!   - Constants_Module.f90
!
! SUBROUTINES CONTAINED:
!   -  ComputeStormDistanceToLand
!   -  CycloneDistanceToCoastSegment
!   -  ComputeDistance_PolarAngle
!   -  LongitudeLimit (function)
!   -  ReadCoastlineFile
!
! CALLING SEQUENCE:  N/A
!
! INPUTS:  N/A
!
! OOUTPUTS:  N/A
!
! SYSTEM CALLS:  N/A
!
! DEPENDENCIES:
!   -  Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!  2016-07-16  B. Kuligowski     Adapted from NCEP Central Operations F77 code
!
!===============================================================================
MODULE land_module

USE TYPE_KINDS
USE Constants_Module
USE Error_Messaging_Module

IMPLICIT NONE

!--- Publicly available subroutines and functions
PUBLIC :: ComputeStormDistanceToLand
PUBLIC :: CycloneDistanceToCoastSegment
PUBLIC :: ComputeDistance_PolarAngle
PUBLIC :: LongitudeLimit
PUBLIC :: ReadCoastlineFile

CONTAINS

!===============================================================================
! NAME:  ComputeStormDistanceToLand
!
! TYPE/LANGUAGE: F90 subroutine
!
! FUNCTION: 
!   Returns the distance from the cylone center to the nearest land mass
!   (negative value if over land)
!
! DESCRIPTION:
!   Computes the distance (km) from, and normal bearing to, the coastline 
!   nearest to a cyclone centered at latitude 'CycloneLat' and longitude
!   'CycloneLon'.  Coastlines are read from the coastline file.  The points of
!   each segment are stored in counterclockwise order around the coast.  The
!   last index in the 'xCoastlineLonLat' or 'xCoastlineLonLat' arrays is 1 for
!   latitude and 2 for longitude.
!
!   The distance from the cyclone to the nearest point on each coastal segment
!   is calculated, and the minimum absolute value is accepted as the coastal
!   distance for that particular land mass.  Negative values are inland.  The
!   minimum absolute value of all coastal distances for the cycloneis returned
!   as the coastal distance for this particular cyclone.
!                                                                
! File Needed:
!   - Lat / lon points of land mass boundaries "aland.dat"
!
! MODULES/SUBROUTINES NEEDED:
!   - Type_Kinds.f90
!   - Constants_Module.f90
!
! SUBROUTINES CONTAINED:
!   - None.
!
! CALLING SEQUENCE:
!   CALL ComputeStormDistanceToLand (CycloneLon, CycloneLat, DistanceToCoast)
!
! INPUTS:
!   Name             Type          Description
!   ----------------------------------------------------------------------------
!   CycloneLon         I    Longitude of cyclone center (degrees E) 
!   CycloneLat         I    Latitude of cyclone center (degrees N)
!
! OUTPUTS:
!   Name             Type          Description
!   ----------------------------------------------------------------------------
!   DistanceToCoast     O    Distance (km) to nearest coastline
!
! SYSTEM CALLS:
!   -  None
!
! DEPENDENCIES:
!   -  Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!  2016-07-16  B. Kuligowski     Adapted from NCEP Central Operations F77 code
!
!===============================================================================
   SUBROUTINE ComputeStormDistanceToLand (CycloneLon, CycloneLat, &
    DistanceToCoast)

   USE TYPE_KINDS
   USE Constants_Module
   USE Error_Messaging_Module

   IMPLICIT NONE

!-- INPUTS: --------------------------------------------------------------------
   !-- Storm center longitude and latitude -------------------------------------
   REAL, INTENT(IN) :: CycloneLon, CycloneLat

   !--- Variables which are read from the coastline file and saved for ---------
   !--- subsequent calls to the subroutine -------------------------------------
   LOGICAL, SAVE :: ReadAlreadyDone
   REAL, SAVE, DIMENSION(1:MAX_NUM_COASTLINES,1:MAX_COASTLINE_POINTS,1:2) :: &
    xCoastlineLonLat
   INTEGER, SAVE :: nCoastlines
   INTEGER, SAVE :: nCoastlinePoints
   INTEGER, SAVE, DIMENSION(1:MAX_NUM_COASTLINES) :: nCoastlinePointsFile

!-- OUTPUTS: -------------------------------------------------------------------
   REAL, INTENT(OUT) :: DistanceToCoast

!-- PROCESSING VARIABLES: ------------------------------------------------------
   !-- Distance and bearing from the storm to each coastline point -------------
   REAL, DIMENSION(1:MAX_COASTLINE_POINTS) :: Dist_StormTolinePoint
   REAL, DIMENSION(1:MAX_COASTLINE_POINTS) :: Bearing_StormTolinePoint

   !-- Distance and bearing from the storm to each land mass -------------------
   REAL, DIMENSION(1:MAX_NUM_COASTLINES) :: MinDist_StormToLand
   REAL, DIMENSION(1:MAX_NUM_COASTLINES) :: Bearing_StormToLand

   !-- Counters ----------------------------------------------------------------
   INTEGER :: iPoint
   INTEGER :: iCoastPointClosestToStorm
   INTEGER :: iClosestIsland
   INTEGER :: iCoastline

   !-- Variables related to the distance and angle between the storm and coast -
   REAL :: FinalAngle_CoastToStorm
   REAL :: PolarAngle_CoastToStorm, PolarAngle_Coastline
   REAL :: PolarAngle_CoastlineInit
   REAL :: StormBearingToLand
   REAL :: MinDistance_StormToCoast
   REAL :: DeltaX_Coastline, DeltaY_Coastline
   REAL :: DeltaX_CoastlineInit, DeltaY_CoastlineInit
   REAL :: DeltaX2storm, DeltaY2storm
   REAL :: StormDistanceToLand
   REAL :: FinalDist_CoastToStorm

   DATA ReadAlreadyDone /.FALSE./

!== MAIN SUBROUTINE ============================================================

   !-- Skip reading of coastline and island points if already done previously --
   IF (.NOT. ReadAlreadyDone) CALL ReadCoastlineFile (nCoastlines, &
    nCoastlinePointsFile, xCoastlineLonLat, ReadAlreadyDone)

   !-- Compute distance and bearing between cyclone and each coastline ---------
   DO iCoastline = 1, nCoastlines
      nCoastlinePoints = nCoastlinePointsFile(iCoastline)

      !-- Compute values for initial coastline segment -------------------------
      CALL ComputeDistance_PolarAngle(xCoastlineLonLat(iCoastline,1,1), &
       xCoastlineLonLat(iCoastline,nCoastlinePoints,1), &
       xCoastlineLonLat(iCoastline,1,2), &
       xCoastlineLonLat(iCoastline,nCoastlinePoints,2), &
       DeltaX_CoastlineInit, DeltaY_CoastlineInit, PolarAngle_CoastlineInit)
      PolarAngle_CoastlineInit = LongitudeLimit(PolarAngle_CoastlineInit + 180.)
!
      !--Loop through 'nCoastlinePointsFile(iCoastline)' coastal segments of
      !-- nth coastline --------------------------------------------------------
      DO iPoint = 1,nCoastlinePoints

         !-- Compute distance and polar bearing for coast segment --------------
         CALL ComputeDistance_PolarAngle &
          (xCoastlineLonLat(iCoastline,iPoint+1,1),&
           xCoastlineLonLat(iCoastline,iPoint  ,1), &
           xCoastlineLonLat(iCoastline,iPoint+1,2), &
           xCoastlineLonLat(iCoastline,iPoint  ,2), &
           DeltaX_Coastline, DeltaY_Coastline, PolarAngle_Coastline)

         !-- Compute distance and polar bearing from storm center to coast point
         CALL ComputeDistance_PolarAngle  &
                   (CycloneLon, xCoastlineLonLat(iCoastline,iPoint,1), &
                    CycloneLat, xCoastlineLonLat(iCoastline,iPoint,2), &
                           DeltaX2storm, DeltaY2storm, PolarAngle_CoastToStorm)

         !-- Compute distance and meteo bearing from storm center to coast point
         CALL CycloneDistanceToCoastSegment(DeltaX_Coastline, DeltaY_Coastline,&
                DeltaX2storm, DeltaY2storm, PolarAngle_CoastlineInit, &
                PolarAngle_Coastline, PolarAngle_CoastToStorm, &
                FinalAngle_CoastToStorm, FinalDist_CoastToStorm )
         Dist_StormTolinePoint(iPoint) = FinalDist_CoastToStorm
         Bearing_StormTolinePoint(iPoint) = FinalAngle_CoastToStorm

         PolarAngle_CoastlineInit = LongitudeLimit(PolarAngle_Coastline + 180.)

      !-- Move on to next coast segment ----------------------------------------
      END DO

      !-- Determine which coastal segment is closest to the storm for this land
      !-- mass -----------------------------------------------------------------
      MinDistance_StormToCoast = 1000.
      iCoastPointClosestToStorm = 1
      DO iPoint = 1,nCoastlinePoints
         IF (ABS(Dist_StormTolinePoint(iPoint)) < MinDistance_StormToCoast) &
          iCoastPointClosestToStorm = iPoint
         MinDistance_StormToCoast = &
          ABS(Dist_StormTolinePoint(iCoastPointClosestToStorm))
       END DO

      !-- Set the final distance and bearing for the storm to this land mass ---
      MinDist_StormToLand(iCoastline) = &
       Dist_StormTolinePoint(iCoastPointClosestToStorm)
      Bearing_StormToLand(iCoastline) = &
       Bearing_StormTolinePoint(iCoastPointClosestToStorm)

   !-- Move on to next coastline (collection of segments) ----------------------
   END DO

   !-- Determine which land mass has a coastal point closest to the storm ------
   StormDistanceToLand = 9999.
   iClosestIsland=1
   DO iCoastline = 1, nCoastlines
      IF( ABS(MinDist_StormToLand(iCoastline)) < StormDistanceToLand ) THEN
         iClosestIsland = iCoastline
         StormDistanceToLand= ABS(MinDist_StormToLand(iCoastline))
      ENDIF
   END DO

   !-- Set final distance and bearing for the storm to closest land ------------
   DistanceToCoast = MinDist_StormToLand(iClosestIsland) * DEG_TO_KM
   StormBearingToLand = Bearing_StormToLand(iClosestIsland)
   RETURN

   END SUBROUTINE ComputeStormDistanceToLand

!===============================================================================
! NAME:  ComputeDistanceToCoastSegment
!
! TYPE/LANGUAGE:  F90 subroutine
!
! FUNCTION:
!   Computes the distance (km) from the cyclone center to each coastal segment
!
! DESCRIPTION:
!   Computes the distance (km) from the cyclone center to each coastal segment
!
! FILES NEEDED:
!   - None
!
! MODULES/SUBROUTINES NEEDED:
!   - Type_Kinds.f90
!   - Constants_Module.f90
!
! SUBROUTINES CONTAINED: N/A
!
! CALLING SEQUENCE:
!   CALL CycloneDistanceToCoastSegment(DeltaX_Coastline, DeltaY_Coastline,
!        DeltaX2storm, DeltaY2storm, PolarAngle_CoastlineInit,u
!        PolarAngle_Coastline, PolarAngle_CoastToStorm,
!        FinalAngle_CoastToStorm, FinalDist_CoastToStorm)
!
! INPUTS:
!   Name                    Type          Description
!   ----------------------------------------------------------------------------
!   DeltaX_Coastline          I    X component of length of coastal segment 
!   DeltaY_Coastline          I    Y component of length of coastal segment 
!   DeltaX2storm              I    X component of cyclone - coast distance
!   DeltaY2storm              I    Y component of cyclone - coast distance
!   PolarAngle_Coastline      I    Polar angle of current coastline segment
!   PolarAngle_CoastlineInit  I    Polar angle of first coastline segment
!
! OUTPUTS:
!   Name                    Type          Description
!   ----------------------------------------------------------------------------
!   FinalAngle_CoastToStorm    O    Bearing from coastline to cyclone
!   FinalDist_CoastToStorm     O    Distance (km) from coastline to cyclone
!
! SYSTEM CALLS:
!   - None
!
! REFERENCE: None
!   - None
!
! DEPENDENCIES:
!   -  Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!  2016-07-16  B. Kuligowski     Adapted from NCEP Central Operations F77 code
!
!===============================================================================
   SUBROUTINE CycloneDistanceToCoastSegment(DeltaX_Coastline, &
    DeltaY_Coastline, DeltaX2storm, DeltaY2storm, PolarAngle_CoastlineInit, &
    PolarAngle_Coastline, PolarAngle_CoastToStorm, FinalAngle_CoastToStorm, &
    FinalDist_CoastToStorm)

   USE TYPE_KINDS
   USE Constants_Module

   IMPLICIT NONE

!-- INPUTS: --------------------------------------------------------------------
   REAL, INTENT(IN) :: DeltaX_Coastline, DeltaY_Coastline
   REAL, INTENT(IN) :: DeltaX2storm, DeltaY2storm
   REAL, INTENT(IN) :: PolarAngle_Coastline
   REAL, INTENT(IN) :: PolarAngle_CoastlineInit

!-- OUTPUTS: -------------------------------------------------------------------
   REAL, INTENT(OUT) :: FinalAngle_CoastToStorm
   REAL, INTENT(OUT) :: FinalDist_CoastToStorm

!-- PROCESSING VARIABLES -------------------------------------------------------
   REAL :: CoastlineAngleChange
   REAL :: PolarAngle_CoastToStorm
   REAL :: CoastSegmentLength
   REAL :: Dist_CoastToStorm
   REAL :: DistAlongCoast_vs_CoastSegmentLength
   REAL :: BearingDiff
   REAL :: DistNormalToCoast
   REAL :: DistAlongCoast
   REAL :: DirectionFlip
   REAL :: Sign_BearingDiff
   REAL :: Angle_Storm_Coast
   REAL :: CompassBearing_StormTo
   REAL :: PolarAngle_Coastline_Flip
                                                                
   !-- Compute distance from cyclone to coastal segment ------------------------
   BearingDiff = LongitudeLimit(PolarAngle_CoastToStorm - PolarAngle_Coastline)
   CoastSegmentLength = SQRT(DeltaX_Coastline * DeltaX_Coastline + &
                   DeltaY_Coastline * DeltaY_Coastline)
   Dist_CoastToStorm = SQRT(DeltaX2storm * DeltaX2storm + &
                           DeltaY2storm * DeltaY2storm)
   DistNormalToCoast = -Dist_CoastToStorm * SIN(BearingDiff * DEG_TO_RAD)
   DistAlongCoast =  Dist_CoastToStorm * COS(BearingDiff * DEG_TO_RAD)
   DistAlongCoast_vs_CoastSegmentLength =  DistAlongCoast / CoastSegmentLength

   !-- Assign distance from cyclone to coast:
   !-- + If distance ratio is in range 0-1, use normal distance
   !-- + Otherwise, use distance from cyclone to intial point of segment with
   !--    sign depending on the angle of the coast at that point. --------------
 
   !-- Determine distance and whether cyclone is over land or over water -------
   Sign_BearingDiff = 1.
   IF (ABS(BearingDiff) >= 1.E-5) Sign_BearingDiff = BearingDiff / &
    ABS(BearingDiff)
   Angle_Storm_Coast = LongitudeLimit(PolarAngle_CoastToStorm - &
    PolarAngle_CoastlineInit)
   CoastlineAngleChange= LongitudeLimit(PolarAngle_CoastlineInit - &
    PolarAngle_Coastline)

   DirectionFlip = 0.

   !-- If the storm is onshore, flip the direction 180 degrees -----------------
   IF (DistNormalToCoast < 0.) DirectionFlip = 180.
   CompassBearing_StormTo = -90. - PolarAngle_CoastToStorm
   PolarAngle_Coastline_Flip = DirectionFlip - PolarAngle_Coastline

   !-- Distance is approximated by distance to cyclone, but will be dfinitely --
   !-- longer than, and therefore superseded by, the distance from the initial -
   !-- point of the next coastal segment ---------------------------------------
   IF (DistAlongCoast_vs_CoastSegmentLength > 1.) THEN

      FinalDist_CoastToStorm = -Dist_CoastToStorm * Sign_BearingDiff
      FinalAngle_CoastToStorm = CompassBearing_StormTo

   !-- Distance is that from initial point to cyclone.  Point is over water ----
   !-- if bearing of vector to cyclone is less than bearing of previous coastal
   !-- segment, or if to right of current coastal segment ----------------------
   ELSE IF (DistAlongCoast_vs_CoastSegmentLength < 0.) THEN

      FinalDist_CoastToStorm = -Dist_CoastToStorm * Sign_BearingDiff
      FinalAngle_CoastToStorm = CompassBearing_StormTo

      !-- The distance offshore / onshore is now set up relative to the current
      !-- coastal segment vector: negative is onshore (to the left); positive is
      !-- offshore (to the right). Make final corrections here. ----------------

      !-- Cyclone is onshore: skip the correction immediately following --------
      IF (FinalDist_CoastToStorm <= 0.) THEN

         !-- Cyclone is onshore ------------------------------------------------
         IF ( Angle_Storm_Coast > 0. .AND. CoastlineAngleChange > 0. ) &
          FinalDist_CoastToStorm = -FinalDist_CoastToStorm

      !-- Cyclone is offshore --------------------------------------------------
      ELSE
         IF ((Angle_Storm_Coast <= 0.) .AND. (CoastlineAngleChange <= 0.)) &
         FinalDist_CoastToStorm = -FinalDist_CoastToStorm
     ENDIF

   !-- Distance is along normal to coast ---------------------------------------
   ELSE

      FinalDist_CoastToStorm = DistNormalToCoast
      FinalAngle_CoastToStorm = PolarAngle_Coastline_Flip
   ENDIF

   !-- Transform the bearing from (-180,180) to (0,360) for display purposes ---
    IF (FinalAngle_CoastToStorm < 0. ) FinalAngle_CoastToStorm = &
    FinalAngle_CoastToStorm + 360.

   RETURN
   END SUBROUTINE CycloneDistanceToCoastSegment


!===============================================================================
! NAME:  ComputeDistance_PolarAngle
!
! TYPE/LANGUAGE:  F90 subroutine
!
! FUNCTION:
!   Computes the distance and angle (in polar coordinates, NOT compass 
!    coordinates) between two points.
!
! DESCRIPTION:
!   Computes the distance and angle (in polar coordinates, NOT compass
!    coordinates) between two points.
!
! FILES NEEDED:
!  -  None
!
! MODULES/SUBROUTINES NEEDED:
!   - Type_Kinds.f90
!   - Constants_Module.f90
!
! SUBROUTINES CONTAINED: N/A
!
! CALLING SEQUENCE:
!   CALL ComputeDistance_PolarAngle (point2lon, point1lon, point2lat, point1lat,
!        DeltaX, DeltaY, PolarAngle)
!
! INPUTS:
!   Name                    Type          Description
!   ----------------------------------------------------------------------------
!   point1lon                I    Longitude of first point (degrees E)
!   point1lat                I    Latitude of first point (degrees N)
!   point2lon                I    Longitude of second point (degrees E)
!   point2lat                I    Latitude of second point (degrees N)
!
! OUTPUTS:
!   Name                    Type          Description
!   ----------------------------------------------------------------------------
!   DeltaX                   O    Zonal distance (km) between the two points
!   DeltaY                   O    Medirional distance (km) between points
!   PolarAngle               O    Bearing (degrees) between the two points
!
! SYSTEM CALLS:
!   - None
!
! REFERENCE: None
!   - None
!
! DEPENDENCIES:
!   -  Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!  2016-07-16  B. Kuligowski     Adapted from NCEP Central Operations F77 code
!
!===============================================================================
   SUBROUTINE ComputeDistance_PolarAngle (point2lon, point1lon, point2lat, &
    point1lat, DeltaLon, DeltaLat, PolarAngle)

   USE TYPE_KINDS
   USE Constants_Module

   IMPLICIT NONE

!== VARIABLES ==================================================================
!-- INPUTS: Latitudes and longitudes of start and end points -------------------
   REAL, INTENT(IN) :: point1lat, point1lon
   REAL, INTENT(IN) :: point2lat, point2lon

!-- OUTPUTS: Longitude and latitude differences and polar angle ----------------
   REAL, INTENT(OUT) :: DeltaLon
   REAL, INTENT(OUT) :: DeltaLat
   REAL, INTENT(OUT) :: PolarAngle

!== MAIN SUBROUTINE ============================================================
   !-- Compute initiali latitude and longitude differences ---------------------
   DeltaLat =  point2lat - point1lat
   DeltaLon = point2lon - point1lon

   !-- Restrict longitude range to -180 to +180 degrees ------------------------
   DeltaLon = LongitudeLimit(DeltaLon)

   !-- Adjust longitude difference for width of longitude spacing --------------
   DeltaLon = DeltaLon * COS((point2lat + point1lat) * DEG_TO_RAD / 2.)

   !-- Compute polar angle (set to zero if distance is small) ------------------
   IF ( (ABS(DeltaLon) + ABS(DeltaLat)) <= 1.E-5) THEN
      PolarAngle  = 0.
   ELSE
      PolarAngle  = ATAN2(DeltaLat,DeltaLon) / DEG_TO_RAD
   ENDIF
   RETURN
   END SUBROUTINE ComputeDistance_PolarAngle
 

!===============================================================================
! NAME:  LongitudeLimit
!
! TYPE/LANGUAGE:  F90 function
!
! FUNCTION:
!   Lmits a (longitude) value to between -180 and +180.
!
! DESCRIPTION:
!   Limits a (longitude) value to between -180 and +180.
!
! FILES NEEDED:
!   - None
!
! MODULES/SUBROUTINES NEEDED:
!   -  None
!
! SUBROUTINES CONTAINED: N/A
!
! CALLING SEQUENCE:
!    LongitudeLimit (aLongitude)
!
! INPUTS:
!   Name                    Type          Description
!   ----------------------------------------------------------------------------
!   aLongitude               I    Longitude in degrees E (0-360)
!
! OUTPUTS: Function value
!
! SYSTEM CALLS:
!   - None
!
! REFERENCE: None
!   - None
!
! DEPENDENCIES:
!   -  Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!  2016-07-16  B. Kuligowski     Adapted from NCEP Central Operations F77 code
!
!===============================================================================
   REAL FUNCTION LongitudeLimit(aLongitude)

   USE TYPE_KINDS

   IMPLICIT NONE

!== VRAIABLES ==================================================================
!-- INPUT: longitude in degrees (0-360) ----------------------------------------
   REAL, INTENT(IN) :: aLongitude

!== MAIN FUNCTION ==============================================================
   LongitudeLimit = aLongitude
   IF (aLongitude <= -180.) LongitudeLimit = aLongitude + 360.
   IF (aLongitude > 180.) LongitudeLimit = aLongitude - 360.
   RETURN
   END FUNCTION LongitudeLimit


!===============================================================================
! NAME:  ReadCoastlineFile
!
! TYPE/LANGUAGE:  F90 subroutine
!
! FUNCTION:
!   Reads the coastline file used by R-CLIPER to determine the distance from
!    the storm to the nearest land.
!
! DESCRIPTION:
!   Reads the coastline file used by R-CLIPER to determine the distance from
!    the storm to the nearest land.
!
! FILES NEEDED:
!  -  None
!
! MODULES/SUBROUTINES NEEDED:
!   - Type_Kinds.f90
!   - Constants_Module.f90
!
! SUBROUTINES CONTAINED: N/A
!
! CALLING SEQUENCE:
!   CALL ReadCoastlineFile (nCoastlines, nCoastlinePointsFile, &
!    xCoastlineLonLat, ReadAlreadyDone)
!
! INPUTS:
!   None (inputs from reading file)
!
! OUTPUTS:
!   Name                    Type          Description
!   ----------------------------------------------------------------------------
!   nCoastline               O    Total number of coastlines in coastline file
!   nCoastlinePointsFile     O    Total number of points in each coastline
!   xCoastlineLonLat         O    Longitute / latitude of each coastline point
!   ReadAlreadyDone          O    Logical; TRUE once the file has been read
!
! SYSTEM CALLS:
!   - None
!
! REFERENCE: None
!   - None
!
! DEPENDENCIES:
!   -  Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!  2016-07-16  B. Kuligowski     Adapted from NCEP Central Operations F77 code
!
!===============================================================================
   SUBROUTINE ReadCoastlineFile (nCoastlines, nCoastlinePointsFile, &
    xCoastlineLonLat, ReadAlreadyDone)

   USE TYPE_KINDS
   USE Constants_Module
   USE Error_Messaging_Module

   IMPLICIT NONE

!-- OUTPUTS: -------------------------------------------------------------------
   INTEGER, INTENT(OUT) :: nCoastlines
   INTEGER, DIMENSION(1:MAX_NUM_COASTLINES), INTENT(OUT) :: nCoastlinePointsFile
   REAL, DIMENSION(1:MAX_NUM_COASTLINES,1:MAX_COASTLINE_POINTS,1:2), &
    INTENT(OUT) :: xCoastlineLonLat
   LOGICAL, INTENT(OUT) :: ReadAlreadyDone

!-- PROCESSING VARIABLES: ------------------------------------------------------

   !-- Other values read from file ---------------------------------------------
   INTEGER :: OpenStatus
   INTEGER :: iPolygon
   INTEGER :: nCoastlinePoints

   !-- Counters ----------------------------------------------------------------
   INTEGER :: iCoastline
   INTEGER :: iPoint

!== MAIN SUBROUTINE ============================================================

   ReadAlreadyDone = .FALSE.

   !-- Open coastline file; make sure continent does not have too many points
   WRITE (6,*) "Coastline file ",FILENAME_COASTLINE
   OPEN (COASTLINE_UNIT, FILE=FILENAME_COASTLINE, FORM='FORMATTED', &
    STATUS='OLD', IOSTAT=OpenStatus)
   IF (OpenStatus /= 0) THEN
      CALL Error_Messaging('ComputeStormDistanceToLand', &
       'Cannot find coastline file', FATAL)
      STOP
   ENDIF

   !-- Read number of coastlines from file and stop execution if too many ---
   READ (COASTLINE_UNIT,'(20x,i4)',ERR=995) nCoastlines
   IF (nCoastlines >  MAX_NUM_COASTLINES) THEN
      CALL Error_Messaging('ComputeStormDistanceToLand', &
       'Number of coastline polygons exceeds limit', FATAL)
      STOP
   ENDIF

   !-- Read coastline polygons ----------------------------------------------
   DO iCoastline = 1, nCoastlines

      !-- Read number of points in coastline polygon ------------------------
      READ(COASTLINE_UNIT,'(7x,i5,4x,i7)',ERR=995) &
       iPolygon, nCoastlinePointsFile(iCoastline)
      nCoastlinePoints = nCoastlinePointsFile(iCoastline)

      DO iPoint = 1, nCoastlinePoints
         READ(COASTLINE_UNIT,'(f9.5,f11.5)',ERR=995) &
          xCoastlineLonLat(iCoastline,iPoint,2), &
          xCoastlineLonLat(iCoastline,iPoint,1)
      END DO

      !-- Copy first coastline point into end of array to complete the loop -
      xCoastlineLonLat(iCoastline,nCoastlinePoints+1,1) = &
       xCoastlineLonLat(iCoastline,1,1)
      xCoastlineLonLat(iCoastline,nCoastlinePoints+1,2) = &
       xCoastlineLonLat(iCoastline,1,2)

      !-- If print flag is on, write out the coastal points for the island --
      IF (FLAG_PRINT) THEN
         WRITE (LOG_UNIT,'(a,i3,a,i5,a)') 'Polygon ', iPolygon, &
          ' is defined by the following ', &
          nCoastlinePointsFile(iCoastline), ' points:'
         nCoastlinePoints = nCoastlinePointsFile(iCoastline)
         WRITE (LOG_UNIT,'(6(1x,i4,a,f10.5,f9.5,a))') (iPoint, '(', &
          xCoastlineLonLat(iCoastline,iPoint,1), &
         xCoastlineLonLat(iCoastline,iPoint,2), ')', &
          iPoint = 1, nCoastlinePoints)
      ENDIF

   !-- Read next island in coastline file -----------------------------------
   END DO

   !-- All coastlines read; now compute distance from storm to nearest land -
   ReadAlreadyDone = .TRUE.
   CLOSE (COASTLINE_UNIT)

   RETURN

   !-- Error message for reading of coastal segment file 'aland.dat' -----------
995 CALL Error_Messaging('ComputeStormDistanceToLand', &
     'EOF of coastline file; probable cause is incorrect number of '// &
     'land masses at beginning of file', FATAL)
   STOP                                                       

   END SUBROUTINE ReadCoastlineFile

END MODULE land_module
