!-------------------------------------------------------------------------------
! NAME: rcliper_main
!
! TYPE/LANGUAGE: F90 program
!
! FUNCTION:
!   R-CLIPER predicts storm total rainfall for 6-h intervals out to 24 hours
!   based on the forecast storm track, climatological tropical cyclone rain
!   rate, shear, and interaction of the wind field with orography.
!
! DESCRIPTION:
!   This program is the driver for R-CLIPER.
!
! FILES NEEDED:
!   - None
!
! MODULES/SUBROUTINES NEEDED:
!   - Type_Kinds.f90
!   - Constants_Module.f90
!   - Error_Messaging_Module.f90
!        - Error_Messaging
!   - land_module.f90
!        - ComputeStormDistanceToLand
!        - CycloneDistanceToCoastSegment
!        - ComputeDistance_PolarAngle
!   - Rainfall_Module.f90
!        - CalculateRainfall
!   - rcliper_subroutines.f90
!        - getShear
!        - getStormTrack
!        - getTimeDifference
!        - QuadraticInterpolation
!        - CartesianToDistanceHeading
!        - AccumulateRainfall
!        - DistanceBetweenPoints
!        - CalcSpeedHeadingFromLatLons
!   - write_area_module.f90
!        - writeAREA
!        - Julian_Day
!        - GouldValue
!
! SUBROUTINES CONTAINED: None
!
! CALLING SEQUENCE:
!    ./rcliper_main
!
! INPUTS:
!   - storm track file "comab.dat"
!   - SHIPS diagnostic file "lsdiag.dat"
!
! OUTPUTS:
!   - diagnostic message file "rcliper.log"
!
! REFERENCE: None
!
! DEPENDENCIES:
!   - Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!               M. DeMaria / F. Marks  Original code
!   2016-09-13  B. Kuligowski   Convert to F90; clean up to meet SPSRB standards
!-------------------------------------------------------------------------------
   PROGRAM rcliper_main

   USE Type_Kinds
   USE Constants_Module
   USE rcliper_subroutines
   USE Rainfall_Module
   USE land_module
   USE write_area_module
   USE Error_Messaging_Module

   IMPLICIT NONE

!== VARIABLES ==================================================================
!-- INPUTS ---------------------------------------------------------------------
!-- Storm initialization year, month, day, and GMT hour from track file header -
   INTEGER :: iYearInit
   INTEGER :: MonthInit
   INTEGER :: iDayInit
   INTEGER :: iHourInit
   INTEGER :: iBullID
   INTEGER :: iDay
   INTEGER :: iHour
   INTEGER :: iYear
   INTEGER :: Month
   INTEGER :: LogFileNumber
   INTEGER :: OpenError
   INTEGER :: ix
   INTEGER :: iy
   CHARACTER(LEN=100) :: CommandLineArgument
!-- Values of storm center lat (N positive), lon (W positive), and max wind
!-- speed (kt) at each forecast track time -------------------------------------
   REAL, DIMENSION (1:N_TRACK_TIMES) :: CenterLat
   REAL, DIMENSION (1:N_TRACK_TIMES) :: CenterLon
   REAL, DIMENSION (1:N_TRACK_TIMES) :: WindSpeedMax

!-- Shear heading and amplitude from SHIPS diagnostic file ---------------------
   REAL, DIMENSION (1:N_TRACK_TIMES) :: ShearHeading
   REAL, DIMENSION (1:N_TRACK_TIMES) :: ShearAmplitude

!-- OUTPUTS --------------------------------------------------------------------
!-- Outputs: accumulated rainfall for 0-6, 6-12, 12-18, 18-24, and 00-24 h
   REAL, DIMENSION (1:NX_RCLIPER,1:NY_RCLIPER,1:N_OUT_TIMES) :: TotalRain

!-- PROCESSING VARIABLES: ------------------------------------------------------
!-- Storm name -----------------------------------------------------------------
   CHARACTER(LEN=20) :: StormName
   CHARACTER(LEN=10) :: ATCFName  
   CHARACTER(LEN=6) :: wmo  
   CHARACTER(LEN=40) :: CMD  

!-- Time at each point in the storm track --------------------------------------
   REAL, DIMENSION (1:N_TRACK_TIMES) :: StormTime

!-- Latitude of each row / longitude of each column in nested grid -------------
   REAL, DIMENSION (1:NX_RCLIPER) :: GridLon
   REAL, DIMENSION (1:NY_RCLIPER) :: GridLat

!-- Counters: grid column and row, storm track time, output file number --------
   INTEGER :: iTrackTime
   CHARACTER(LEN=4) :: FC_cent
   CHARACTER(LEN=6), DIMENSION (1:49) :: WMOs
!   DATA WMOs / "WTNT21","WTNT22","WTNT23","WTNT24","WTNT25","WTPZ21","WTPZ2!2","WTPZ23","WTPZ24","WTPZ25", &
!               "WTPA21","WTPA22","WTPA23","WTPA24","WTPA25","WTPN31","WTPN3!2","WTPN33","WTPN34","WTPN35", &
!               "WTPS31","WTPS32","WTPS33","WTPS34","WTPS35","WTXS31","WTXS3!2","WTXS33","WTXS34","WTXS35", &
!               "WTIO30","WVID10","FKAU01","FKAU02","FKAU03","FKAU04","FKAU0!5", &
!               "FKAU06","FKPS01","FKPS02","FKPQ30","FKPQ31","FKPQ32","FKPQ3!3","FKPQ34","FKIN20", &
!               "FKIO20","WTNZ41","WHNZ41" /

!== MAIN PROGRAM ===============================================================
   CALL getarg(1, CommandLineArgument)
   wmo = TRIM(CommandLineArgument)

   !-- Open the log file -------------------------------------------------------
   OPEN (UNIT=LOG_UNIT, FILE=FILENAME_LOG, FORM='formatted', STATUS='replace')

   !-- Compute the time at each forecast interval and initialize track values --
   DO iTrackTime = 1, N_TRACK_TIMES
      StormTime(iTrackTime) = TRACK_FORECAST_INTERVAL * FLOAT(iTrackTime - 1)
      WindSpeedMax(iTrackTime) = 0.
      ShearHeading(iTrackTime) = 0.
      ShearAmplitude(iTrackTime) = 0.
   END DO

   !-- Get the storm track and intensity from the forecast file ----------------
!   CALL GetStormTrack(iYearInit, MonthInit, iDayInit, iHourInit, StormName, &
!    CenterLat, CenterLon, WindSpeedMax)
!   DO iBullID = 1,49
!     wmo = WMOs(iBullID)
      WRITE (LOG_UNIT,*) "wmo ",wmo
!     CMD="/bin/bash bull_select.sh "//wmo
!     CALL SYSTEM(CMD)
      OPEN (41, FILE='/home/Petrap/RCliper/Bul_'//wmo//'.log', STATUS='old',  IOSTAT=OpenError)
      IF (OpenError /= 0) THEN
         WRITE (LogFileNumber,'(6a)') 'No bulletin file for ', &
              TRIM(wmo)
      ELSE
         READ (41,141) StormName,ATCFName,iDayInit,iHourInit,iYearInit,MonthInit,WindSpeedMax,CenterLat,CenterLon,FC_cent
!         READ (41,141) StormName,ATCFName,iDayInit,iHourInit,iYearInit,MonthInit,WindSpeedMax,CenterLat
         WRITE (6,*) "STORMNAME ",StormName
         WRITE (6,*) "ATCFNAME ",ATCFName
         WRITE (6,*) "DAY ",iDayInit
         WRITE (6,*) "HOUR ",iHourInit
         WRITE (6,*) "YEAR ",iYearInit
         WRITE (6,*) "MONTH ",MonthInit
         WRITE (6,*) "WINDSP1 ", WindSpeedMax
         WRITE (6,*) "CENTLAT ", CenterLat
         WRITE (6,*) "CENTLON ", CenterLon
         WRITE (6,*) "FC_cent ", FC_cent
141      FORMAT (10x,a10/10x,a10/5x,i2/6x,i2/6x,i4/6x,i2/13x,3g3.0/10x,3g7.1/10x,3g7.1/,a4)
!141      FORMAT (10x,a10/10x,a10/5x,i2/6x,i2/6x,i4/5x,i3/13x,3f4.0/10x,3g6.1,a)
   !-- Get the shear forecast from the shear file -----------------------------
         CALL getShear(ShearHeading, ShearAmplitude)

   !-- Raise any nonzero max sustained winds below threshold to threshold value
         DO iTrackTime = 1, N_TRACK_TIMES
            IF ((WindSpeedMax(iTrackTime) > 0.) .AND. &
                 (WindSpeedMax(iTrackTime) < WIND_SPEED_MIN)) &
                 WindSpeedMax(iTrackTime) = WIND_SPEED_MIN
         END DO

   !-- Write forecast date, initialization time, and storm name to log file ----
         WRITE (LOG_UNIT,'(a,i4.4,1x,3(i2.2),1x,a10)') 'RCLIPER forecast for ', &
              iYearInit, MonthInit, iDayInit, iHourInit,StormName
 
   !-- Write storm track and intensity forecast to log file --------------------
         WRITE (LOG_UNIT,'(a5,a6,a7,a5)') 'Lead',' Lat ','  Lon  ','Speed'
         DO iTrackTime = 1, N_TRACK_TIMES
            WRITE (LOG_UNIT,'(f5.0,f6.1,f7.1,f5.0)') StormTime(iTrackTime), &
                 CenterLat(iTrackTime), CenterLon(iTrackTime), WindSpeedMax(iTrackTime)
         END DO
!         WRITE (6,*) "ShearHeading met", ShearHeading
!         WRITE (6,*) "ShearAmplitude met", ShearAmplitude
   !-- Convert shear heading from meteorological to math angle -----------------
         DO iTrackTime = 1, N_TRACK_TIMES
            ShearHeading(iTrackTime) = 90.0 - ShearHeading(iTrackTime)
            IF (ShearHeading(iTrackTime) < 0.0) ShearHeading(iTrackTime) = &
                 ShearHeading(iTrackTime) + 360.0
         END DO
!         WRITE (6,*) "ShearHeading math", ShearHeading
         WRITE (6,*) "Longitude before Accum", CenterLon
   !-- Compute rainfall accumulation -------------------------------------------
         CALL AccumulateRainfall(CenterLat, CenterLon, WindSpeedMax, StormTime, &
              GridLat, GridLon, TotalRain, ShearAmplitude, ShearHeading)
!         WRITE (6,*) "Here after Accum Rain"
!         WRITE (6,*) "iYear", iYearInit

!====== TEST ======
!   DO iy = 1, NY_RCLIPER
!      DO ix = 1, NX_RCLIPER
!         WRITE (*,'(f6.2,f8.2,5f7.1)') GridLat(iy), GridLon(ix), &
!          (TotalRain(ix,iy,iTrackTime), iTrackTime = 1, N_OUT_TIMES)
!      END DO
!   END DO
!   STOP
!====== TEST ======

   !-- Write output AREA files -------------------------------------------------
         CALL writeAREA(StormName, iYearInit, MonthInit, iDayInit, iHourInit, wmo, &
              GridLon, GridLat, TotalRain, FC_cent)
       END IF
!   END DO

END PROGRAM rcliper_main
