!-------------------------------------------------------------------------------
! NAME: rcliper_subroutines
!
! TYPE/LANGUAGE: F90 module
!
! FUNCTION:
!   Contains subroutines for reading the storm track and SHIPS shear information
!   for computing R-CLIPER rainfall totals; also calls the subroutine for
!   computing the actual rainfall amounts.
!
! DESCRIPTION:
!   This module contains the subroutines for reading the storm track and SHIPS
!   shear information and then calls the subroutine for computing the rainfall
!   totals.
!
! FILES NEEDED: N/A
!
! MODULES/SUBROUTINES NEEDED:
!   - Type_Kinds.f90
!   - Constants_Module.f90
!   - Error_Messaging_Module.f90
!
! SUBROUTINES CONTAINED:
!   - getShear
!   - getStormTrack
!   - getTimeDifference
!   - QuadraticInterpolation
!   - CartesianToDistanceHeading
!   - DistanceBetweenPoints
!   - CalcSpeedHeadingFromLatLon
!
! CALLING SEQUENCE: N/A
!
! INPUTS: N/A
!
! OUTPUTS: N/A
!
! SYSTEM CALLS: N/A
!
! REFERENCE: None
!
! DEPENDENCIES:
!   -  Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!               M. DeMaria / F. Marks  Original code
!   2016-09-13  B. Kuligowski   Convert to F90; clean up to meet SPSRB standards
!-------------------------------------------------------------------------------
MODULE rcliper_subroutines

USE Type_Kinds
USE Constants_Module
USE Error_Messaging_Module

IMPLICIT NONE

CONTAINS

!===============================================================================
! NAME: getShear
!
! TYPE/LANGUAGE: F90 subroutine
!
! FUNCTION:
!   Reads the shear forecasts from the SHIPS diagnostic file for R-CLIPER.
!
! DESCRIPTION:
!   This subroutine reads the shear forecasts from the SHIPS diagnostic file 
!   for R-CLIPER. Note that the shear forecasts are every 6 h but R-CLIPER uses
!   a 12-h time step so every other shear forecast value is ignored.
!
! FILES NEEDED:
!   - SHIPS diagnostic file specified by FILENAME_SHIPS in Constants_Module.f90
!       - Developmental data at http://rammb.cira.colostate.edu/research/
!                                 tropical_cyclones/ships/developmental_data.asp
!       - Real-time data at ftp://ftp.nhc.noaa.gov/atcf/lsdiag/
!
! MODULES/SUBROUTINES NEEDED:
!   - Type_Kinds.f90
!   - Constants_Module.f90
!
! SUBROUTINES CONTAINED: N/A
!
! CALLING SEQUENCE:
!   CALL getShear(ShearHeading, ShearAmplitude)
!
! INPUTS: 
!   - None
!
! OUTPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   ShearHeading         O     Direction of 850-200 mb shear vector (W=90 deg)
!   ShearAmplitude       O     Magnitude of 850-200 mb shear (kts*10) with
!                              vortex removed and averaged from 0-500 km
!                              relative to 850-hPa vortex center
!
! SYSTEM CALLS:
!   - None
!
! REFERENCE: None
!   - None
!
! DEPENDENCIES:
!   - Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!               M. DeMaria / F. Marks  Original code
!   2016-09-13  B. Kuligowski   Convert to F90; clean up to meet SPSRB standards
!-------------------------------------------------------------------------------
   SUBROUTINE getShear(ShearHeading, ShearAmplitude)

   USE Type_Kinds
   USE Constants_Module
   USE Error_Messaging_Module

   IMPLICIT NONE

!== VARIABLES ==================================================================
!-- INPUTS: shear heading and amplitude from SHIPS diagnostic file -------------
   INTEGER iShearHeading(N_SHIPS_TIMES)
   INTEGER iShearAmplitude(N_SHIPS_TIMES)

!-- OUTPUTS: Shear heading (deg) and amplitude (m/s) ---------------------------
   REAL, DIMENSION(1:N_TRACK_TIMES), INTENT(OUT) :: ShearHeading
   REAL, DIMENSION(1:N_TRACK_TIMES), INTENT(OUT) :: ShearAmplitude

!-- PROCESSING VARIBLES: -------------------------------------------------------
   !-- File open status --------------------------------------------------------
   INTEGER :: OpenError

   !-- Line from SHIPS diagnostic file that was just read ----------------------
   CHARACTER(LEN=130) cLine

   !-- Counters: storm track lead time and SHIPS file lead time ---------------
   INTEGER :: iTrackTime
   INTEGER :: iSHIPStime

!== MAIN SUBROUTINE ============================================================
   !-- Open the shear file -----------------------------------------------------
   OPEN (UNIT_SHIPS, FILE=FILENAME_SHIPS, STATUS='old', IOSTAT=OpenError)

   !-- Read file only if available ---------------------------------------------
   IF (OpenError == 0) THEN

      !-- Initialize all shear heading and amplitude values --------------------
      DO iTrackTime = 1, N_TRACK_TIMES
          ShearHeading(iTrackTime) = VAL_MSG
          ShearAmplitude(iTrackTime) = VAL_MSG
      END DO
      DO iSHIPStime = 1, N_SHIPS_TIMES
          iShearHeading(iSHIPStime) = NINT(VAL_MSG)
          iShearAmplitude(iSHIPStime) = NINT(VAL_MSG)
      END DO

      !-- Skip the header line -------------------------------------------------
      READ (UNIT_SHIPS,*,ERR=99) cline
 
      !-- Read the next line in the file if current line is not last in the file
      DO WHILE (cline(117:120) /= 'LAST')
         READ (UNIT_SHIPS,'(a130)',ERR=99) cline
 
         !-- Read shear heading in tenths of a degree -------------------------
         IF (cline(117:120) == 'SDDC') THEN
            READ (cline,'(11x,21(i4,1x))',ERR=99) &
             (iShearHeading(iSHIPStime), iSHIPStime = 1, N_SHIPS_TIMES)

            !-- Select only values every 12 h with leads starting at 0 h -------
            DO iTrackTime = 1, N_TRACK_TIMES
               ShearHeading(iTrackTime) = &
                FLOAT(iShearHeading((iTrackTime * 2) - 1))
            END DO
         ENDIF

         !-- Read shear amplitude in knots*10 ----------------------------------
         IF (cline(117:120) == 'SHDC') THEN
            READ (cline,'(11x,21(i4,1x))',ERR=99) &
             (iShearAmplitude(iSHIPStime), iSHIPStime = 1, N_SHIPS_TIMES)

            !-- Select only values every 12 h with leads starting at 0 h and ---
            !-- divide by 10 to convert to knots -------------------------------
            DO iTrackTime  = 1, N_TRACK_TIMES
               ShearAmplitude(iTrackTime) = &
                FLOAT(iShearAmplitude((iTrackTime * 2) - 1)) / 10.0
            END DO
         ENDIF
      END DO

      !-- Done reading file; close and return ----------------------------------
      CLOSE (UNIT_SHIPS)

   !-- Unable to find SHIPS diagnostic file; write error message and exit ------
   ELSE
      CALL Error_Messaging ('getShear', 'No SHIPS diagnostic file', FATAL)
   ENDIF
   RETURN

   !-- If read error occurs in line, give warning and set OpenError flag -------
99 CALL Error_Messaging('getShear', 'Read error in shear file', FATAL)
   OpenError = 1
   RETURN

   END SUBROUTINE getShear

!===============================================================================
! NAME: getStormTrack
!
! TYPE/LANGUAGE: F90 subroutine
!
! FUNCTION:
!   Reads the storm track forecasts from the official forecast bulletins.
!
! DESCRIPTION:
!   This subroutine reads the storm track forecasts from the official bulletins.
!
! FILES NEEDED:
!   - Forecast bulletin specified by FILENAME_TRACK in Constants_Module.f90
!
! MODULES/SUBROUTINES NEEDED:
!   - Type_Kinds.f90
!   - Constants_Module.f90
!
! SUBROUTINES CONTAINED: N/A
!
! CALLING SEQUENCE:
!   CALL GetStormTrack(iYear, Month, iDay, iHour, StormName, CenterLat, &
!    CenterLon, WindSpeedMax)
!
! INPUTS:
!
! OUTPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   iYear                O     Storm forecast initialization year
!   Month                O     Storm forecast initialization month
!   iDay                 O     Storm forecast initialization day of month
!   iHour                O     Storm forecast initialization hour
!   StormName            O     Storm name
!   CenterLat            O     Latitudes of storm center track
!   CenterLon            O     Longitudes of storm center track
!   WindSpeedMax         O     Max wind speeds of storm center track
!
! SYSTEM CALLS: N/A
!
! REFERENCE: None
!
! DEPENDENCIES:
!   - Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!               M. DeMaria / F. Marks  Original code
!   2016-09-13  B. Kuligowski   Convert to F90; clean up to meet SPSRB standards
!-------------------------------------------------------------------------------
   SUBROUTINE GetStormTrack(iYear, Month, iDay, iHour, StormName, CenterLat, &
    CenterLon, WindSpeedMax)

   USE Type_Kinds
   USE Constants_Module
   USE Error_Messaging_Module

   IMPLICIT NONE

!== VARIABLES ==================================================================
!-- OUTPUTS: -------------------------------------------------------------------
   !-- Initialization year, month, day, hour (GMT) and storm name from bulletin
   INTEGER, INTENT(OUT) :: iYear
   INTEGER, INTENT(OUT) :: Month
   INTEGER, INTENT(OUT) :: iDay
   INTEGER, INTENT(OUT) :: iHour
   CHARACTER(LEN=*), INTENT(OUT) :: StormName

   !-- Storm center latitude, longitude, and max wind speed from track forecast
   REAL, DIMENSION(1:N_TRACK_TIMES), INTENT(OUT) :: CenterLat
   REAL, DIMENSION(1:N_TRACK_TIMES), INTENT(OUT) :: CenterLon
   INTEGER :: iWindSpeedMax
   REAL, DIMENSION(1:N_TRACK_TIMES), INTENT(OUT) :: WindSpeedMax

!-- PROCESSING VARIABLES -------------------------------------------------------
   !-- File open status --------------------------------------------------------
   INTEGER :: OpenError

   !-- Counters: Month and storm track forecast time ---------------------------
   INTEGER :: MonthCounter
   INTEGER :: iTrackTime

   !-- Character string read from storm track file -----------------------------
   CHARACTER(LEN=50) ReadString

   !-- String parsing variables: string location and start/end of subsection ---
   INTEGER :: iStringPoint
   INTEGER :: iStartPoint
   INTEGER :: iEndPoint

   !-- String to check if this is the line with the initialization time --------
   CHARACTER(LEN=18) CenterCheckString

   !-- Character labels corresponding to each month of the year ----------------
   CHARACTER(LEN=3) cMonthName(12)
   DATA cMonthName / 'JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', &
                     'JUL', 'AUG', 'SEP', 'OCT', 'NOV', 'DEC' /

!== MAIN SUBROUTINE ============================================================
   OPEN (UNIT_TRACK, FILE=FILENAME_TRACK, STATUS='old', IOSTAT=OpenError)

   !-- Read file only if available ---------------------------------------------
   IF (OpenError == 0) THEN

      !---- Skip first 2 lines of file (and third, if needed) ------------------
      READ (UNIT_TRACK,'(a)',ERR=99) ReadString
      READ (UNIT_TRACK,'(a)',ERR=99) ReadString
      READ (UNIT_TRACK,'(a)',ERR=99) ReadString
      IF (ReadString == '') READ (UNIT_TRACK,'(a)',ERR=99) ReadString

      !---- Read storm name ----------------------------------------------------
      !---- Start point of storm name in string depends on storm type ----------
      IF (ReadString(1:19) == 'TROPICAL DEPRESSION') THEN
          iStartPoint = 21
      ELSE IF (ReadString(1:17) == 'SUBTROPICAL STORM') THEN
          iStartPoint = 19
      ELSE IF (ReadString(1:14) == 'TROPICAL STORM') THEN
          iStartPoint = 16
      ELSE IF (ReadString(1:9) == 'HURRICANE') THEN
          iStartPoint = 11
      ELSE IF (ReadString(1:21) == 'POST-TROPICAL CYCLONE') THEN
          iStartPoint = 23
      ELSE
         CALL Error_Messaging('GetStormTrack', 'Bad Storm Type', FATAL)
         STOP
      ENDIF

      !---- Find end point by reading from start point until a blank occurs ----
      iEndPoint = 0
      DO iStringPoint = iStartPoint, iStartPoint + 10
         IF (ReadString(iStringPoint:iStringPoint) == ' ') iEndPoint = &
          iStringPoint - 1
      END DO
      IF (iEndPoint == 0) THEN
         CALL Error_Messaging('GetStormTrack', 'No Storm Name', FATAL)
         STOP
      ENDIF
      StormName = ReadString(iStartPoint:iEndPoint)

      !---- Read through bulletin until time string is found -------------------
      DO WHILE ((ReadString(5:6) /= 'Z ') .AND. (ReadString(6:9) /= 'UTC '))
         READ (UNIT_TRACK,'(a)',ERR=99) ReadString
      END DO
      READ (ReadString(1:2),'(i2.2)',ERR=99) iHour
      IF (MOD(iHour,6) == 3) iHour = iHour - 3
      IF (MOD(iHour,3) /= 0) THEN
         CALL Error_Messaging('GetStormTrack', 'No Storm Name', FATAL)
         STOP
      ENDIF

      !--- Read time string if time ends in 'Z' --------------------------------
      IF (ReadString(5:6) == 'Z ') THEN
         DO MonthCounter = 1, 12
            IF (ReadString(11:13) == cMonthName(MonthCounter)) Month = &
             MonthCounter
         END DO
         READ (ReadString(15:16),'(i2.2)',ERR=99) iDay
         READ (ReadString(18:21),'(i4.4)',ERR=99) iYear

      !--- Read time string if time ends in 'UTC' ------------------------------
      ELSE IF (ReadString(6:9) == 'UTC ') THEN
         DO MonthCounter = 1, 12
            IF (ReadString(14:16) == cMonthName(MonthCounter)) Month = &
             MonthCounter
         END DO
         READ (ReadString(18:19),'(i2.2)',ERR=99) iDay
         READ (ReadString(21:24),'(i4.4)',ERR=99) iYear
      ELSE
         CALL Error_Messaging('GetStormTrack', 'Bad time parsing', FATAL)
         STOP
      ENDIF

      !---- Look for current max sustained winds -------------------------------
      DO WHILE (ReadString(1:19) /= 'MAX SUSTAINED WINDS')
         READ (UNIT_TRACK,'(a)',ERR=99) ReadString
      END DO
      READ (ReadString(21:23),'(i3.3)',ERR=99) iWindSpeedMax
      WindSpeedMax(1) = FLOAT(iWindSpeedMax)

      !---- Look for current storm position ------------------------------------
      !---- Create string to check for position at starting synoptic hour ------
      CenterCheckString = 'AT XX/XX00Z CENTER'
      WRITE (CenterCheckString(4:5),'(i2.2)') iDay
      WRITE (CenterCheckString(7:8),'(i2.2)') iHour

      !---- Look for string and extract center when found ----------------------
      DO WHILE (ReadString(1:18) /= CenterCheckString)
         READ (UNIT_TRACK,'(a)',ERR=99) ReadString
      END DO
      READ (ReadString(37:40),'(f4.1)',ERR=99) CenterLat(1)
      READ (ReadString(44:47),'(f4.1)',ERR=99) CenterLon(1)
      IF (CenterLon(1) < 60.) READ (ReadString(43:47),'(f5.1)',ERR=99) &
       CenterLon(1)
      READ (UNIT_TRACK,'(a)',ERR=99) ReadString

      !--- Read forecast positions and max sustained winds ---------------------
      DO iTrackTime = 2, N_TRACK_TIMES

         !---- Read forecast position -----------------------------------------
         DO WHILE (ReadString(1:14) /= 'FORECAST VALID')
            READ (UNIT_TRACK,'(a)',ERR=89) ReadString
         END DO
         READ (ReadString(25:28),'(f4.1)',ERR=89) CenterLat(iTrackTime)
         READ (ReadString(32:35),'(f4.1)',ERR=89) CenterLon(iTrackTime)
         IF (CenterLon(iTrackTime) < 60.) &
          READ (ReadString(31:35),'(f5.1)',ERR=99) CenterLon(iTrackTime)

         !--- Read forecast max sustained winds --------------------------------
         READ (UNIT_TRACK,'(a)',ERR=89) ReadString
         READ (ReadString(10:12),'(i3.3)',ERR=89) iWindSpeedMax
         WindSpeedMax(iTrackTime) = FLOAT(iWindSpeedMax)

      END DO

      !-- Done reading file ----------------------------------------------------
      CLOSE (UNIT_TRACK)

   !-- Give error message and exit if no storm track file ----------------------
   ELSE
      CALL Error_Messaging ('GetStormTrack', 'No storm track file', FATAL)
      STOP
   ENDIF
   RETURN

   !-- If unable to read all necessary track forecast periods, warn and set ----
   !-- OpenError flag to stop execution ----------------------------------------
89 CALL Error_Messaging('getStormTrack', &
    'Track forecast ends before end of forecast period', FATAL)
   OpenError = 1
   RETURN

   !-- If read error occurs, warn and set OpenError flag to stop execution -----
99 CALL Error_Messaging('getStormTrack', 'Read error in storm track file', &
    FATAL)
   OpenError = 1
   RETURN
   END SUBROUTINE GetStormTrack


!===============================================================================
! NAME: getTimeDifference
!
! TYPE/LANGUAGE: F90 subroutine
!
! FUNCTION:
!   Calculates the number of hours bewteen two dates/times.
!
! DESCRIPTION:
!   Used to compute the time difference between the official track and the
!   forecast time but not it is not clear that this would ever be needed; it
!   is being kept until we are sure of this.
!
! FILES NEEDED:
!   - None
!
! MODULES/SUBROUTINES NEEDED:
!   - Type_Kinds.f90
!   - Constants_Module.f90
!
! SUBROUTINES CONTAINED: N/A
!
! CALLING SEQUENCE:
!    CALL GetTimeDifference(iYearEnd, MonthEnd, iDayEnd, iHourEnd, &
!     iYearStart, MonthStart, iDayStart, iHourStart, iTimeDiffHours)
!
! INPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   iYearEnd             I     Year of initial time
!   MonthEnd             I     Month of initial time
!   iDayEnd              I     Day of month of initial time
!   iHourEnd             I     GMT hour of initial time
!   iYearStart           I     Year of initial time
!   MonthStart           I     Month of initial time
!   iDayStart            I     Day of month of initial time
!   iHourStart           I     GMT hour of initial time
!
! OUTPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   iTimeDiffHours       O     Time difference (hours) between start and end
!
! SYSTEM CALLS:
!   - None
!
! REFERENCE:
!   - None
!
! DEPENDENCIES:
!   - Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!               M. DeMaria / F. Marks  Original code
!   2016-09-13  B. Kuligowski   Convert to F90; clean up to meet SPSRB standards
!-------------------------------------------------------------------------------
   SUBROUTINE GetTimeDifference(iYearEnd, MonthEnd, iDayEnd, iHourEnd, &
    iYearStart, MonthStart, iDayStart, iHourStart, iTimeDiffHours)

   USE Type_Kinds
   USE Constants_Module

   IMPLICIT none

!== VARIABLES ==================================================================
!-- INPUTS: Starting and ending year, month, day, hour of time period ----------
   INTEGER, INTENT(IN) :: iYearEnd, MonthEnd, iDayEnd, iHourEnd
   INTEGER, INTENT(IN) :: iYearStart, MonthStart, iDayStart, iHourStart

!-- OUTPUT: Time difference in hours -------------------------------------------
   INTEGER, INTENT(OUT) :: iTimeDiffHours

!-- PROCESSING VARIABLES: ------------------------------------------------------
   !-- Julian day at the beginning of each month of the year NOTE: leap years --
   !-- are accounted for later in the subroutine -------------------------------
   INTEGER, DIMENSION(1:12) :: nday
   DATA nday / 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 /

   !-- Hours since reference time for start and end of period ------------------
   INTEGER iHourReferenceStart
   INTEGER iHourReferenceEnd

   !-- Reference year and year counter -----------------------------------------
   INTEGER iYearReference
   INTEGER iYearCounter

!== MAIN SUBROUTINE ============================================================
   !-- Calculate reference year (year before start year) -----------------------
   iYearReference = iYearStart - 2
   IF (iYearEnd < iYearReference) iYearReference = iYearEnd - 2

   !-- Compute number of hours from 00Z Jan. 1 of reference year ---------------
   !-- Start time --------------------------------------------------------------
   iHourReferenceStart = 0
   DO iYearCounter = iYearReference, iYearStart - 1

      !-- Leap year ------------------------------------------------------------
      IF (MOD(iYearCounter,4) == 0) THEN
         iHourReferenceStart = iHourReferenceStart + 24 * 366

      !-- Non-leap year --------------------------------------------------------
      ELSE
         iHourReferenceStart = iHourReferenceStart + 24 * 365
      ENDIF
   END DO

   !-- End time ----------------------------------------------------------------
   iHourReferenceEnd = 0
   DO iYearCounter = iYearReference, iYearEnd - 1

      !-- Leap year ------------------------------------------------------------
      IF (MOD(iYearReference,4) == 0) THEN
         iHourReferenceEnd = iHourReferenceEnd + 24 * 366

      !-- Non-leap year --------------------------------------------------------
      ELSE
         iHourReferenceEnd = iHourReferenceEnd + 24 * 365
      ENDIF
   END DO

   !-- Add number of hours from 00Z Jan. 1 of current year ---------------------
   iHourReferenceStart = iHourReferenceStart + 24 * nday(MonthStart)

   !-- Add 24 h if a leap year and after February ------------------------------
   IF ((MOD(iYearStart,4) == 0) .AND. MonthStart > 2) iHourReferenceStart = &
    iHourReferenceStart + 24

   !-- Add number of hours from 00Z Jan. 1 of current year ---------------------
   iHourReferenceEnd = iHourReferenceEnd + 24 * nday(MonthEnd)

   !-- Add 24 h if a leap year and after February ------------------------------
   IF ((MOD(iYearEnd,4) ==  0) .AND. MonthEnd > 2) iHourReferenceEnd = &
    iHourReferenceEnd + 24

   !-- Add number of hours since 00Z today -------------------------------------
   iHourReferenceStart = iHourReferenceStart + 24 * iDayStart + iHourStart
   iHourReferenceEnd = iHourReferenceEnd + 24 * iDayEnd + iHourEnd

   iTimeDiffHours = iHourReferenceEnd - iHourReferenceStart

   RETURN
   END SUBROUTINE GetTimeDifference


!===============================================================================
! NAME: QuadraticInterpolation
!
! TYPE/LANGUAGE: F90 subroutine
!
! FUNCTION:
!   Interpolates the value of a function.
!
! DESCRIPTION:
!   This subroutine applies a quadratic interpolation procedure to a function,
!   with the function value assimed to be represented by quadratic polynomials
!   between the points.  These polynomials are chosen to equal the given
!   function values, so that the first derivatives of either side of the point
!   match, and the second derivative of the approximated function integrated
!   over the domain is minimized.
!
! FILES NEEDED:
!   -  None
!
! MODULES/SUBROUTINES NEEDED:
!   - Type_Kinds.f90
!   - Constants_Module.f90
!
! SUBROUTINES CONTAINED: N/A
!
! CALLING SEQUENCE:
!   CALL QuadraticInterpolation(xValue, funcValue, nPoints, &
!    InitializationFlag, InterpTypeFlag, xValueInterp, funcValueInterp, &
!    InterpErrorFlag)
!
! INPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   nPoints              I     Number of data points in the function
!   xValue               I     Independent (x) values of function points
!   funcValue            I     Dependent (y=f(x)) values of function points
!   InitializationFlag   I     Initialization flag (see below)
!   InterpTypeFlag       I     Interpolation type flag (see below)
!   xValueInterp         I/O   Interpolated value of x (put in bounds if needed)
!
! OUTPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   funcValueInter       O     Interpolated function value at x
!
! SYSTEM CALLS:
!   - None
!
! REFERENCE:
!   - None
!
! DEPENDENCIES:
!    - Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!               M. DeMaria / F. Marks  Original code
!   2016-09-13  B. Kuligowski   Convert to F90; clean up to meet SPSRB standards
!-------------------------------------------------------------------------------
   SUBROUTINE QuadraticInterpolation(xValue, funcValue, NumPoints, &
    InitializationFlag, InterpTypeFlag, xValueInterp, funcValueInterp, &
    InterpErrorFlag)

   USE Type_Kinds
   USE Constants_Module

   IMPLICIT none

!== VARIABLES ==================================================================
!-- INPUTS: --------------------------------------------------------------------
    !-- Number of data points in the function to be interpolated ---------------
   INTEGER, INTENT(IN) :: NumPoints

   !-- Function x and f(x) values (NOTE: the x values must be sequential!) -----
   REAL, DIMENSION(1:NumPoints), INTENT(IN) :: xValue
   REAL, DIMENSION(1:NumPoints), INTENT(IN) :: funcValue

   !-- Initialization flag (1=calculate coefficients; 0=use prev coefficients) -
   INTEGER, INTENT(IN) :: InitializationFlag

   !-- Interpolation type flag (0=linear, 1=quadratic) -------------------------
   INTEGER, INTENT(IN) :: InterpTypeFlag

!-- INPUT/OUTPUT: Interpolated x value (adjusted by subroutine if out of bounds)
   REAL, INTENT(INOUT) :: xValueInterp

!-- OUTPUTS: -------------------------------------------------------------------
   !-- Interpolated f(x) value -------------------------------------------------
   REAL, INTENT(OUT) :: funcValueInterp

   !-- Error return flag (0 = normal; 1 = N_TRACK_TIMES is too small; 2 = x
   !-- values not sequential; 3=coefficient did not converge; 4=mix-up finding 
   !-- coefficients; 5=xi outside xValue range so xValueInterp set to nearest
   !-- endpoint before interpolation).
   !-- NOTE that funcValue = -99.9 if 1 <= InterpErrorFlag < =4. ---------------
   INTEGER, INTENT(OUT) :: InterpErrorFlag

!-- PROCESSING VARIABLES -------------------------------------------------------
   !-- Counters ----------------------------------------------------------------
   INTEGER iPoint, InterpInterval, iteration

   !-- Interpolation coefficients ----------------------------------------------
   REAL, DIMENSION(1:N_TRACK_TIMES), SAVE :: ax
   REAL, DIMENSION(1:N_TRACK_TIMES), SAVE :: bx
   REAL, DIMENSION(1:N_TRACK_TIMES), SAVE :: cx

   !-- Intervals in x and f(x) for calculating derivatives ---------------------
   REAL, DIMENSION(1:N_TRACK_TIMES) :: df
   REAL, DIMENSION(1:N_TRACK_TIMES) :: dx
   REAL, DIMENSION(1:N_TRACK_TIMES) :: gm

   !-- Trial value of cx -------------------------------------------------------
   REAL, DIMENSION(1:N_TRACK_TIMES) :: cxTrial

   !-- Other variables ---------------------------------------------------------
   REAL :: SegmentIntercept, SegmentSlope
   REAL :: CostFunctionCurrent
   REAL :: ConvergeCheck
   REAL :: CostFunctionTrial
   REAL :: DomainSize
   REAL :: den
   REAL :: DeviationFromLinear
   REAL :: CostFunctionGradient
   REAL :: CostFunctionChange
   REAL :: StepLengthOptimal, StepLengthTrial

!== MAIN SUBROUTINE ============================================================
 
   !--  Check to make sure N_TRACK_TIMES is large enough and NumPoints > 1 ----
   IF ((NumPoints > N_TRACK_TIMES) .OR. (NumPoints < 2)) THEN
      InterpErrorFlag = 1
      funcValueInterp = -99.9
      RETURN
   ENDIF
 
   !-- Perform the initialization for later interpolation ----------------------
   IF (InitializationFlag == 1) THEN
 
      !-- Check to make sure 'xValue' vector values are sequential -------------
      DO iPoint = 1, NumPoints - 1
         IF (xValue(iPoint) >= xValue(iPoint+1)) THEN
            InterpErrorFlag=2
            funcValueInterp = -99.9
            RETURN
         ENDIF
      END DO

      !-- Check for special case where NumPoints=2, where only linear ----------
      !-- interpolation is possible. -------------------------------------------
      IF (NumPoints == 2) THEN
         cx(1) = 0.0
         bx(1) = (funcValue(2) - funcValue(1)) / (xValue(2) - xValue(1))
         ax(1) = funcValue(1) - bx(1) * xValue(1)

      !-- Otherwise, perform quadratic interpolation ---------------------------
      ELSE

         !-- Calculate xValue and f differences --------------------------------
         DO iPoint = 1, NumPoints - 1
            df(iPoint) = funcValue(iPoint+1) - funcValue(iPoint)
            dx(iPoint) = xValue(iPoint+1) - xValue(iPoint)
         END DO

         !-- Calculate domain size ---------------------------------------------
         DomainSize = xValue(NumPoints) - xValue(1)

         !-- Check for linearity of input points -------------------------------
         SegmentSlope = (funcValue(2) - funcValue(1)) / (xValue(2) - xValue(1))
         SegmentIntercept = funcValue(1) - SegmentSlope * xValue(1)
         DeviationFromLinear = 0.0
         DO iPoint = 3, NumPoints
            DeviationFromLinear = DeviationFromLinear + &
             ABS(SegmentIntercept + SegmentSlope * xValue(iPoint) - &
            funcValue(iPoint))
         END DO

         !-- Function is linear; set all coefficient values to zero ------------
         IF (DeviationFromLinear < LINEARITY_THRESHOLD .OR. &
          InterpTypeFlag == 0) THEN

            DO iPoint = 1, NumPoints - 1
               cx(iPoint) = 0.0
            END DO

         !-- Iterate to find the c-coefficients --------------------------------
         ELSE

            !-- Initialize iteration -------------------------------------------
            cx(1) = 0.0
            StepLengthTrial  = 0.01
            ConvergeCheck = CONVERGE_CHECK_INIT

            !-- Loop through iterations ----------------------------------------
            DO iteration = 1, N_ITERATIONS

               !-- Compute c-coefficient values --------------------------------
               DO iPoint = 2, NumPoints - 1
                  cx(iPoint) = -cx(iPoint-1) * dx(iPoint-1) / dx(iPoint) - &
                   df(iPoint-1) / (dx(iPoint) * dx(iPoint-1)) + &
                   df(iPoint) / (dx(iPoint) * dx(iPoint))
               END DO

               !-- Calculate current value of cost function --------------------
               CostFunctionCurrent = 0.0
               DO iPoint = 1, NumPoints - 1
                  CostFunctionCurrent = CostFunctionCurrent + cx(iPoint) * &
                   cx(iPoint) * dx(iPoint)
               END DO
               CostFunctionCurrent = 0.5 * CostFunctionCurrent / DomainSize

               !-- Write cost function value if debug is turned on -------------
               IF (FLAG_DEBUG /= 0) WRITE (UNIT_DEBUG,'(/,a,e13.6)') &
                ' CostFunctionCurrent=',CostFunctionCurrent

               !-- Check for convergence ---------------------------------------
               CostFunctionChange = ABS(CostFunctionCurrent - ConvergeCheck) / &
                ABS(ConvergeCheck)
               IF (CostFunctionChange < CONVERGE_COST_THRESHOLD) EXIT
               ConvergeCheck = CostFunctionCurrent

               !-- Calculate values of Lagrange multipliers --------------------
               gm(NumPoints-1) = cx(NumPoints-1) * dx(NumPoints-1) / DomainSize
               IF (NumPoints > 3) THEN
                  DO iPoint = NumPoints - 2, 2, -1
                     gm(iPoint) = cx(iPoint) * dx(iPoint) / DomainSize - &
                      gm(iPoint+1) * dx(iPoint) / dx(iPoint+1)
                  END DO
               ENDIF

               !-- Calculate gradient of cost function -------------------------
               CostFunctionGradient = dx(1) * (cx(1) / DomainSize - &
                gm(2) / dx(2))

               !-- Adjust cx(1) using trial step -------------------------------
               cxTrial(1) = cx(1) - StepLengthTrial * CostFunctionGradient

               !-- Calculate remaining c values at trial step ------------------
               DO iPoint = 2, NumPoints - 1
                  cxTrial(iPoint) = -cxTrial(iPoint-1) * dx(iPoint-1) / &
                   dx(iPoint) - df(iPoint-1) / (dx(iPoint) * dx(iPoint-1)) + &
                   df(iPoint) / (dx(iPoint) * dx(iPoint))
               END DO

               !-- Calculate cost function at trial step -----------------------
               CostFunctionTrial = 0.0
               DO iPoint = 1, NumPoints - 1
                  CostFunctionTrial = CostFunctionTrial + cxTrial(iPoint) * &
                   cxTrial(iPoint) * dx(iPoint)
               END DO
               CostFunctionTrial = 0.5 * CostFunctionTrial / DomainSize

               !-- Calculate optimal step length and re-adjust cx(1) -----------
               den = 2.0 * ((CostFunctionTrial - CostFunctionCurrent) + &
                StepLengthTrial * CostFunctionGradient * CostFunctionGradient)
               IF (NINT(den) /= 0) THEN
                  StepLengthOptimal = CostFunctionGradient * &
                   CostFunctionGradient * StepLengthTrial * StepLengthTrial / &
                   den
               ELSE
                  StepLengthOptimal = 0.0
               ENDIF

               !-- Adjust optimal step length if desired (currently not done) --
               StepLengthOptimal = 1.0 * StepLengthOptimal

               !-- Adjust initial coefficient value ----------------------------
               cx(1) = cx(1) - StepLengthOptimal * CostFunctionGradient

               !-- Write iteration results if debug flag is on -----------------
               IF (FLAG_DEBUG /= 0) THEN
                  WRITE (UNIT_DEBUG,'(a,i4,3(a,e11.4))') ' Iteration=', &
                   iteration, '  cf1=', CostFunctionTrial, ' StepLengthTrial=',&
                   StepLengthTrial, ' StepLengthOptimal=', StepLengthOptimal
                  DO iPoint = 1, NumPoints - 1
                     WRITE (UNIT_DEBUG,'(4x,a,i2,a,f8.4)') 'iPoint=', iPoint, &
                      ' c=',cx(iPoint)
                  END DO
               ENDIF

               !-- Calculate trial step for next time step ---------------------
               StepLengthTrial = 0.5 * StepLengthOptimal
            END DO

            !-- Iteration did not converge -------------------------------------
            IF (CostFunctionChange >= CONVERGE_COST_THRESHOLD) THEN
               InterpErrorFlag = 3
               funcValueInterp = -99.9
               RETURN
            ENDIF

         !-- Iteration converged; write out if debug flag is on ----------------
         ENDIF
         IF (FLAG_DEBUG /= 0) THEN
            WRITE (UNIT_DEBUG,'(/,a)') ' Iteration converged'
         ENDIF

         !-- Calculate bx and ax coefficients ----------------------------------
         DO iPoint = 1, NumPoints - 1
            bx(iPoint) = df(iPoint) / dx(iPoint) - &
             cx(iPoint) * (xValue(iPoint+1) + xValue(iPoint))
            ax(iPoint) = funcValue(iPoint) - bx(iPoint) * xValue(iPoint) - &
             cx(iPoint) * xValue(iPoint) * xValue(iPoint)
         END DO
      ENDIF
   ENDIF

   !-- Interpolate the function ------------------------------------------------
   !-- If input x value is below lower bound of range, set to min value and flag
   IF (xValueInterp < xValue(1)) THEN
      xValueInterp = xValue(1)
      InterpErrorFlag = 5
   ENDIF

   !-- If input x value is above lower bound of range, set to max value and flag
   IF (xValueInterp > xValue(NumPoints)) THEN
      xValueInterp = xValue(NumPoints)
      InterpErrorFlag = 5
   ENDIF

   !-- Find the interval in the x values between which to interpolate ----------
   InterpInterval = 1
   DO iPoint = 2, NumPoints
      IF (xValueInterp <= xValue(iPoint)) THEN
         InterpInterval = iPoint - 1
         funcValueInterp = ax(InterpInterval) + bx(InterpInterval) * &
          xValueInterp + cx(InterpInterval) * xValueInterp * xValueInterp
         RETURN
      ENDIF
   END DO

   funcValueInterp = -99.9
   InterpErrorFlag = 4
   RETURN

   END SUBROUTINE QuadraticInterpolation

!===============================================================================
! NAME: CartesianToDistanceHeading
!
! TYPE/LANGUAGE: F90 subroutine
!
! FUNCTION:
!   Converts from Cartesian (x,y) to radial (radius,angle) map coordinates.
!
! DESCRIPTION:
!   This subroutine converts from Cartesian (x,y) to radial (radius,angle) map
!   coordinates, where the angle is in degrees measured clockwise from the
!   positive y axis (i.e., the standard meteorological heading where due north
!   is 0 degrees).
!
! FILES NEEDED:
!   -  None
!
! MODULES/SUBROUTINES NEEDED:
!   - Type_Kinds.f90
!   - Constants_Module.f90
!
! SUBROUTINES CONTAINED: N/A
!
! CALLING SEQUENCE:
!   CALL CartesianToDistanceHeading(x, y, distance, heading_meteo)
!
! INPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   deltax               I     Zonal (x-direction) distance (km)
!   deltay               I     Meridional (y-direction) distance (km)
!
! OUTPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   distance             O     Radial distance (km)
!   heading_meteo        O     Meteorological heading
!
! SYSTEM CALLS:
!   - None
!
! REFERENCE:
!   - None
!
! DEPENDENCIES:
!   - Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!               M. DeMaria / F. Marks  Original code
!   2016-09-13  B. Kuligowski   Convert to F90; clean up to meet SPSRB standards
!
!-------------------------------------------------------------------------------
   SUBROUTINE CartesianToDistanceHeading(deltax, deltay, distance, &
    heading_meteo)

   USE Type_Kinds
   USE Constants_Module

   IMPLICIT NONE

!== VARIABLES ==================================================================
!-- INPUTS: x and y coordinates ------------------------------------------------
   REAL, INTENT(IN) :: deltax, deltay

!-- OUTPUTS: distance and meteorological heading -------------------------------
   REAL, INTENT(OUT) :: distance, heading_meteo

!== MAIN SUBROUTINE ============================================================
   !-- Compute radius (distance from origin) -----------------------------------
   distance = SQRT(deltax * deltax + deltay * deltay)

   !-- If distance = 0, set azimuth to zero and return -------------------------
   IF (distance <= 0.) THEN
      heading_meteo = 0.

   !-- Otherwise, compute azumith and convert to heading -----------------------
   ELSE

      !-- Compute azimuth (compass; clockwise with origin at due north) --------
      heading_meteo = ACOS(deltax / distance) / DEG_TO_RAD
      IF (deltay < 0.) heading_meteo = 360. - heading_meteo

      !-- Convert azimuth to counterclockwise with origin at due east ----------
      heading_meteo = 90.0 - heading_meteo
      IF (heading_meteo < 0.) heading_meteo = heading_meteo + 360.0

   ENDIF

   RETURN
   END SUBROUTINE CartesianToDistanceHeading

!===============================================================================
! NAME: DistanceBetweenPoints
!
! TYPE/LANGUAGE: F90 subroutine
!
! FUNCTION:
!   Calculates the distance bewteen two lat / lon points.
!
! DESCRIPTION:
!   This routine calculates the distance in km (rad) between the points
!   (point1_lon,point1_lat) and (point2_lon,point2_lat) using an approximate
!   formula. The lon and lat are in deg E and N. The east and
!   north components of the distance (dx,dy) are also calculated.
!
! FILES NEEDED:
!   - None
!
! MODULES/SUBROUTINES NEEDED:
!   - Type_Kinds.f90
!   - Constants_Module.f90
!
! SUBROUTINES CONTAINED: N/A
!
! CALLING SEQUENCE:
!   CALL DistanceBetweenPoints(point1_lon, point1_lat, point2_lon, &
!    point2_lat, dx_km, dy_km, distance_km)
!
! INPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   point1_lat           I     Latitude of first point
!   point1_lon           I     Longitude of first point
!   point2_lat           I     Latitude of second point
!   point2_lon           I     Longitude of second point
!
! OUTPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   dx_km                O     Zonal component of distance bewteen the points
!   dy_km                O     Meridional component of distance b/w the points
!   distance_km          O     Distance (km) between the two points
!
! SYSTEM CALLS:
!   - None
!
! REFERENCE:
!   - None
!
! DEPENDENCIES:
!   - Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!               M. DeMaria / F. Marks  Original code
!   2016-09-13  B. Kuligowski   Convert to F90; clean up to meet SPSRB standards
!
!-------------------------------------------------------------------------------
   SUBROUTINE DistanceBetweenPoints(point1_lon, point1_lat, point2_lon, &
    point2_lat, dx_km, dy_km, distance_km)

   USE Type_Kinds
   USE Constants_Module

   IMPLICIT none

!== VARIABLES ==================================================================
!-- INPUTS ---------------------------------------------------------------------
   REAL, INTENT(IN) :: point1_lat, point1_lon
   REAL, INTENT(IN) :: point2_lat, point2_lon

!-- OUTPUTS --------------------------------------------------------------------
   REAL, INTENT(OUT) :: dx_km, dy_km
   REAL, INTENT(OUT) :: distance_km

!== MAIN SUBROUTINE ============================================================
   !-- Compute zonal distance in km between the two points ---------------------
!   WRITE (6,*) "point2_lat point1_lat",point2_lat,point1_lat
   dx_km  = DEG_TO_KM * (point2_lon - point1_lon) * &
    COS(0.5 * DEG_TO_RAD *(point1_lat + point2_lat))

   !-- Compute meridional distance in km between the two points ----------------
   dy_km  = DEG_TO_KM * (point2_lat - point1_lat)

   !-- Compute total distance in km between the two points ---------------------
   distance_km = SQRT(dx_km * dx_km + dy_km * dy_km)
!   WRITE (6,*) "dx dy dist",dx_km,dy_km,distance_km
   RETURN
   END SUBROUTINE DistanceBetweenPoints

!===============================================================================
! NAME: CalcSpeedHeadingFromLatLons
!
! TYPE/LANGUAGE: F90 subroutine
!
! FUNCTION:
!   Computes storm speed (kt) and heading (deg clockwise from due north) given
!   a pair of lat / lon coordinates at specified times.
!
! DESCRIPTION:
!   This subroutine computes storm speed (kt) and heading (deg clockwise from
!   due north) given a pair of lat / lon coordinates at specified times.
!
! FILES NEEDED:
!   - None
!
! MODULES/SUBROUTINES NEEDED:
!   - Type_Kinds.f90
!   - Constants_Module.f90
!
! SUBROUTINES CONTAINED: N/A
!
! CALLING SEQUENCE:
!   CALL CalcSpeedHeadingFromLatLons(x1, y1, time1, x2, y2, time2, &
!    speed_kts, heading_meteo)
!
! INPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   lon1                 I     Longitude (deg E positive) at first time
!   lat1                 I     Latitude (deg N positive) at first time
!   time1                I     First time (h)
!   lon2                 I     Longitude (deg E positive) at second time
!   lat2                 I     Latitude (deg N positive) at second time
!   time1                I     Second time (h)
!
! OUTPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   speed_kts            O     Storm forward speed (kts)
!   heading_meteo        O     Storm bearing (meteorological degrees)
!
! SYSTEM CALLS:
!   - None
!
! REFERENCE:
!   - None
!
! DEPENDENCIES:
!   - Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!               M. DeMaria / F. Marks  Original code
!   2016-09-13  B. Kuligowski   Convert to F90; clean up to meet SPSRB standards
!
!-------------------------------------------------------------------------------
   SUBROUTINE CalcSpeedHeadingFromLatLons(lon1, lat1, time1, lon2, lat2, &
    time2, speed_kts, heading_meteo)

   USE Type_Kinds
   USE Constants_Module

   IMPLICIT none

!== VARIABLES ==================================================================
!-- INPUTS ---------------------------------------------------------------------
   REAL, INTENT(IN) :: lon1, lon2
   REAL, INTENT(IN) :: lat1, lat2
   REAL, INTENT(IN) :: time1, time2

!-- OUTPUTS --------------------------------------------------------------------
   REAL, INTENT(OUT) :: speed_kts
   REAL, INTENT(OUT):: heading_meteo

!-- PROCESSING VARIABLES -------------------------------------------------------
   !-- Zonal and meridional components of distance -----------------------------
   REAL :: deltax, deltay

   !-- Distance between points in kilometers and nautical miles ----------------
   REAL :: distance_km
   REAL :: distance_nmi

!== MAIN SUBROUTINE ============================================================
   !-- Zero speed and heading if time is in wrong order ------------------------
   IF (time1 >= time2) THEN
      speed_kts = 0.0
      heading_meteo = 0.0
      return
   ENDIF

   !-- Compute Cartesian distance between the two points -----------------------
   CALL DistanceBetweenPoints(lon1, lat1, lon2, lat2, deltax, deltay, &
    distance_km)

   !-- Convert Cartesian distance from km to nautical miles --------------------
   distance_nmi = distance_km * DEG_TO_NAUTMI / DEG_TO_KM

   !-- Convert Cartesian distance in nautical miles to speed in kts ------------
   speed_kts  = distance_nmi / (time2 - time1)

   !-- Convert vector components to distance and heading -----------------------
   CALL CartesianToDistanceHeading(deltax, deltay, distance_km, heading_meteo)

   RETURN
   END SUBROUTINE CalcSpeedHeadingFromLatLons

END MODULE rcliper_subroutines
