!===============================================================================
! NAME:  write_area_module
!
! TYPE/LANGUAGE:  F90 module
!
! FUNCTION:
!  Writes the R-CLIPER output to a McIDAS AREA file.
!
! DESCRIPTION:
!  This module contains the subroutine for writing the R-CLIPER output to an
!  AREA file (the corresponding routine in etrap_utils.f90 cannot be used 
!  because it recycles the header from an AREA file that had previously been
!  read whereas here the header must be created from scratch.)
!
! FILES NEEDED:  N/A
!
! MODULES/SUBROUTINES NEEDED:
!   - type_kinds.f90
!   - Constants_Module.f90
!
! SUBROUTINES CONTAINED:
!   -  WriteAREA
!   -  Julian_Day
!   -  GouldValue
!
! CALLING SEQUENCE:  N/A
!
! INPUTS:  N/A
!
! OOUTPUTS:  N/A
!
! SYSTEM CALLS:  N/A
!
! DEPENDENCIES:
!   - Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!  2013-09-23  S. Kidder      Initial development.
!  2016-09-20  B. Kuligowski  Interface with R-CLIPER and clean-up to meet
!                              SPSRB coding standards
!
!===============================================================================
MODULE write_area_module

USE Type_Kinds
USE Constants_Module
USE Error_Messaging_Module

IMPLICIT NONE

!--- Publicly available subroutines and functions
PUBLIC :: writeAREA
PUBLIC :: Julian_Day
PUBLIC :: GouldValue

CONTAINS


!-------------------------------------------------------------------------------
! NAME: writeAREA
!
! FUNCTION:
!   Convert an R-CLIPER 24-h forecast into a McIDAS AREA file
!
! DESCRIPTION:
!   The first step in using R-CLIPER in eTRaP is converting the
!   R-CLIPER files into McIDAS format. This program does the conversion 
!   into the native lat-lon projection (called RECT). A further step
!   using the McIDAS imgremap command is necessary to get the file into
!   the Mercator projection used by eTRaP.
!
! FILES NEEDED:
!   - None
!
! MODULES/SUBROUTINES NEEDED:
!   - Type_Kinds module
!   - GouldValue function
!   - Julian_Day function
!
! SUBROUTINES CONTAINED: None
!
! CALLING SEQUENCE:
!   CALL writeAREA(StormName, iYear, Month, iDay, iHour, GridLon, GridLat,
!    TotalRain)
!
! INPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   StormName            I     Name of storm
!   iYear                I     Initialization year
!   Month                I     Initialization month
!   iDay                 I     Initialization day of month
!   iHour                I     Initialization GMT hour
!   GridLon              I     Longitude of each R-CLIPER grid column
!   GridLat              I     Latitude of each R-CLIPER grid row
!   TotalRain            I     R-CLIPER 6- and 24-h rainfall accumulations (in.)
!
! OUTPUTS:
!   - None
!
!  SYSTEM CALLS:
!   - None
!
! DEPENDENCIES:
!   - Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!   2013-09-23  S. Kidder     Initial development.
!   2016-09-20  B. Kuligowski Interface with R-CLIPER and clean-up to meet
!                              SPSRB coding standards
!
!-------------------------------------------------------------------------------
   SUBROUTINE writeAREA(StormName, iYear, Month, iDay, iHour, wmo, GridLon, &
    GridLat, TotalRain, FC_cent)

   USE Type_Kinds
   USE Constants_Module
   USE Error_Messaging_Module

   IMPLICIT NONE

!== VARIABLES ==================================================================
!-- INPUTS: --------------------------------------------------------------------
   !-- Storm name and initialization time --------------------------------------
   CHARACTER(LEN=20) :: StormName
   INTEGER, INTENT(IN) :: iYear, Month, iDay, iHour
!   CHARACTER(LEN=2) :: Month, Day, Hour
!   CHARACTER(LEN=1) :: Month1, Day1, Hour1
   !-- Latitude of each row, longitude of each column in grid ------------------
   REAL, DIMENSION(1:NX_RCLIPER), INTENT(IN) :: GridLon
   REAL, DIMENSION(1:NY_RCLIPER), INTENT(IN) :: GridLat

   !-- R-CLIPER rainfall for the intervals and total ---------------------------
   REAL*4, DIMENSION(1:NX_RCLIPER,1:NY_RCLIPER,1:N_OUT_TIMES), INTENT(IN) :: &
    TotalRain
   CHARACTER(LEN=40) :: CMD  

!-- OUTPUTS to AREA file: ------------------------------------------------------
   INTEGER(BYTE), DIMENSION(1:NX_RCLIPER / 4 * 4,1:NY_RCLIPER)  :: OutByteData
   INTEGER(LONG), DIMENSION(1:64) :: AREA_Directory
   INTEGER(LONG), DIMENSION(1:128) :: AREA_Navigation
   INTEGER(LONG), DIMENSION(1:128) :: AREA_Calibration

!-- PROCESSING VARIABLES -------------------------------------------------------
   CHARACTER(LEN = 255) :: FileName_Out
   CHARACTER(LEN = 32)  :: memo
   CHARACTER(LEN = 4) :: FC_cent
   CHARACTER(LEN = 6) :: wmo
   !-- Output file counter -----------------------------------------------------
   INTEGER :: iOutFile

   !-- Grid element and line counters ------------------------------------------
   INTEGER :: ix
   INTEGER :: iy

   !-- I/O error status on file open -------------------------------------------
   INTEGER :: iErrorStatus

   !-- Inititalization date and time in format for AREA file -------------------
   INTEGER :: start_date
   INTEGER :: start_time

   !-- Scaling factor for converting R-CLIPER output from inches to 0-255 ------
   REAL :: ScaleFactor

!== MAIN SUBROUTINE ============================================================
   !-- Loop through the lead times ---------------------------------------------
   DO iOutFile = 1, N_OUT_TIMES

      !-- Determine factor for scaling R-CLIPER output from inches to 0-255 ----
      !-- Use the daily scale factor for the total and the hourly for the ------
      !-- smaller intervals ----------------------------------------------------
      IF (iOutFile == N_OUT_TIMES) THEN
         ScaleFactor = SCALE_FACTOR_DAILY
!         ScaleFactor = SCALE_FACTOR_DAILY * 25.4
      ELSE
         ScaleFactor = SCALE_FACTOR_HOURLY
!         ScaleFactor = SCALE_FACTOR_HOURLY * 25.4
      ENDIF

      !-- Scale the rainfall amounts for output and truncate the right side of -
      !-- the grid if the number of elements is not divisible by 4 -------------
!      WRITE (6,*) "iOutFile GRIDLAT    GRIDLON    TOTAL RAIN"
      DO iy = 1, NY_RCLIPER
         DO ix = 1, (NX_RCLIPER / 4) * 4

!            WRITE (6,*) iOutFile, GridLon(ix), GridLat(iy),"TR", TotalRain(ix,iy,iOutFile)
            !-- Invert the grid since McIDAS is N->S and not S->N --------------
            !-- NOTE that this conversion assumes no missing values, since there
            !-- are no circumstances under which R-CLIPER will produce missing -
            !-- values ---------------------------------------------------------
            OutByteData(ix,NY_RCLIPER+1-iy) = NINT(MAX(0., MIN(255.,  &
             TotalRain(ix,iy,iOutFile) / ScaleFactor)),BYTE)
!            WRITE (6,*) iOutFile, GridLon(ix), GridLat(NY_RCLIPER+1-iy),"OBD", OutByteData(ix,NY_RCLIPER+1-iy)

         END DO
      END DO

      !-- Compute the start date and time for the AREA file header -------------
!      WRITE (6,*) "Year in b ", iYear
      start_date = (iYear - 1900) * 1000 + Julian_day(iYear, Month, iDay)
      start_time = iHour * 10000

      !-- Create memo string for AREA file header ------------------------------
      memo = "R-CLIPER 24-hour Rain Amount"

      !-- Create the AREA file directory block ---------------------------------
      !-- Word 1=0, word 2=4, word 3=sensor source number (0 for derived data) -
      AREA_Directory( 1) = 0
      AREA_Directory( 2) = 4
      AREA_Directory( 3) = 0

      !-- Words 5-6: nominal start date (YYDDD) and time (HHMMSS) --------------
      AREA_Directory( 4) = start_date
      AREA_Directory( 5) = start_time

      !-- Words 6-8: starting line and element (8 is unused) -------------------
      AREA_Directory( 6) = 1
      AREA_Directory( 7) = 1
      AREA_Directory (8) = 0

      !-- Words 9-10: Number of lines and elements -----------------------------
      AREA_Directory( 9) = NY_RCLIPER
      AREA_Directory(10) = (NX_RCLIPER / 4) * 4

      !-- Words 11-13: number of bytes per slement, line and element resolution
      AREA_Directory(11) = 1
      AREA_Directory(12) = 1
      AREA_Directory(13) = 1

      !-- Number of bands/line; length length of data block prefix, project no.
      AREA_Directory(14) = 1
      AREA_Directory(15) = 0
      AREA_Directory(16) = 0

      !-- Words 17-18: ingest date / time; make them same as start date / time -
      AREA_Directory(17) = AREA_Directory(4)
      AREA_Directory(18) = AREA_Directory(5)

      !-- Words 19-24: filter band info and satellite-spefic info (unused here)
      AREA_Directory(19) = 1
      AREA_Directory(20:24) = 0

      !-- Words 25-33: ASCII MEMO information and AREA file number -------------
      AREA_Directory(25:32) = TRANSFER(memo,AREA_Directory(25:32))
      AREA_Directory(33) = 1

      !-- Words 34-35: offsets (bytes) to start of data and navigation blocks --
      AREA_Directory(34) = 1280
      AREA_Directory(35) = 256

      !-- Words 36-45: validity code and satellite-specific info (unused) ------
      AREA_Directory(36:45) = 0

      !-- Words 46-47: actual image start date / time; make them same as start -
      AREA_Directory(46) = AREA_Directory(4)
      AREA_Directory(47) = AREA_Directory(5)

      !-- Word 48: actual starting scan line -----------------------------------
      AREA_Directory(48) = 1

      !-- Words 49-51: length (bytes) of DATA block line prefix documentation, -
      !-- calibration, and level map regions -----------------------------------
      AREA_Directory(49) = 0
      AREA_Directory(50) = 0
      AREA_Directory(51) = 0

      !-- Words 52-53: image source type and calibration time ------------------
      AREA_Directory(52) = TRANSFER("PRD ", AREA_Directory(52))
      AREA_Directory(53) = TRANSFER("BRIT", AREA_Directory(53))

      !-- Words 54-60: unused --------------------------------------------------
      AREA_Directory(54:60) = 0

      !-- Words 61-63: size of AUX block, unused, offset to start of CAL block -
      AREA_Directory(61) = 0
      AREA_Directory(62) = 0
      AREA_Directory(63) = 768

      !-- Word 64: number of comment records -----------------------------------
      AREA_Directory(64) = 0

      !-- Create the AREA navigation block -------------------------------------
      !-- Word 1: rectilinear map projection -----------------------------------
      AREA_Navigation( 1) = TRANSFER("RECT", AREA_Navigation(1))

      !-- Words 2-3: Starting line number and corresponding latitude * 10000 ---
      AREA_Navigation( 2) = 1
      AREA_Navigation( 3) = NINT(GridLat(NY_RCLIPER) * 10000.)

      !-- Words 4-5: Starting element number and corresponding longitude * 10000
      AREA_Navigation( 4) = 1
      AREA_Navigation( 5) = NINT(-GridLon(1) * 10000.)

      !-- Words 6-7: Latitude and longitude grid spacing (deg.) * 10000 --------
      AREA_Navigation( 6) = NINT(GRID_SPACE * 10000.)
      AREA_Navigation( 7) = NINT(GRID_SPACE * 10000.)

      !-- Words 8-9: Earth radius (m) and eccentricity * 1000000
      AREA_Navigation( 8) = 6378388
      AREA_Navigation( 9) = 81992

      !-- Word 10: Coordinate type (planetodetic) ------------------------------
      AREA_Navigation(10) = 0

      !-- Word 11: Longitude convention (west positive) ------------------------
      AREA_Navigation(11) = 0

      !-- Words 12-128: Unused -------------------------------------------------
      AREA_Navigation(12:128) = 0

      !-- Create the AREA file calibration block -------------------------------
      AREA_Calibration = 0

      !-- Change ScaleFactor to from inches to millimeters for calibration block
      ScaleFactor = ScaleFactor * 25.4
      AREA_Calibration(1) = GouldValue(ScaleFactor)

      !-- Write the data into the McIDAS AREA file -----------------------------
      WRITE (6,*) "YR Mon Day HR Changed?",iYear, Month, iDay, iHour

      WRITE (6,*) "FC_cent x",FC_cent,"x"
      WRITE (FileName_Out,'(a,i4.4,3a,i4.4,6a,2i0.2,a,3i0.2,a,i0.2,a)') '/data/Petrap/',iYear, &
         '/', TRIM(StormName),'/', iYear, TRIM(StormName), &
        ".", wmo, ".", FC_cent,".", iDay, iHour, "00.RCLIPER.", Month, iDay, iHour, "00.", 6 * (iOutFile - 1),'.AREA'
      WRITE (LOG_UNIT,'(a)') 'Writing '//TRIM(Filename_Out)
      CMD="mkdir /data/Petrap/2017/"
      CALL SYSTEM(CMD)
      CMD="mkdir /data/Petrap/2017/"//TRIM(StormName)
      CALL SYSTEM(CMD)
!      WRITE (6,*) "FileName_Out ",FileName_Out
!      exit
!      CMD="touch "//Filename_Out
!      CALL SYSTEM(CMD)
      OPEN (UNIT=OUT_UNIT_BASE, FILE=TRIM(FileName_Out), FORM='unformatted', & 
        ACCESS='direct', RECL=1280+NY_RCLIPER*(NX_RCLIPER/4)*4, STATUS='REPLACE', &
        IOSTAT=iErrorStatus)
      WRITE (6,*) "ErrStat",iErrorStatus

      IF (iErrorStatus /= 0) THEN
         CALL Error_Messaging ('writeAREA', 'Unable to open output file', FATAL)
         STOP
      ENDIF
      WRITE (OUT_UNIT_BASE, REC=1) AREA_Directory, AREA_Navigation, &
       AREA_Calibration, OutByteData
      CLOSE (OUT_UNIT_BASE)

      !-- Informational message for successful write ---------------------------
      CALL Error_Messaging('writeAREA', TRIM(FileName_Out)//' written', NOTICE)   
   !-- Move on to the next accumulation time period ----------------------------
   END DO

   RETURN

END SUBROUTINE writeAREA


!===============================================================================
! NAME: Julian_Day
!
! TYPE/LANGUAGE: F90 function
!
! FUNCTION:
!   Converts the year, month, and day of month to Julian day.
!
! DESCRIPTION:
!   This function converts the year, month, and day of month to Julian day.
!
! FILES NEEDED:
!   - None
!
! MODULES / SUBROUTINES NEEDED
!   Type_Kinds module
!
! SUBROUTINES CONTAINED: N/A
!
! CALLING SEQUENCE:
!   I = Julian_Day(iYear, Month, iDay))
!
! INPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   iYear, Month, iDay   I     Year, month, and day of month
!
! OUTPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   Julian_Day           O     Julian day corresponding to input values
!
! SYSTEM CALLS:
!   - None
!
! REFERENCE:
!   http://www.ssec.wisc.edu/mcidas
!
! DEPENDENCIES:
!   -  Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!   2013-09-23  S. Kidder       Initial development
!   2016-09-20  B. Kuligowski   Clean up to meet SPSRB standards
!
!-------------------------------------------------------------------------------
FUNCTION Julian_Day(iYear, Month, iDay)

   USE Type_Kinds
   USE Error_Messaging_Module

   IMPLICIT NONE

!== VARIABLES ==================================================================
!-- INPUTS: Year, month, and day of month to convert to Julian day -------------
   INTEGER, INTENT(IN) :: iYear, Month, iDay

!-- OUTPUT: Julian day value ---------------------------------------------------
   INTEGER :: Julian_Day
      
!-- Processing variables -------------------------------------------------------
   INTEGER, DIMENSION(1:12) :: nDaysInMonth
   DATA nDaysInMonth / 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /

!== MAIN FUNCTION ==============================================================
   !-- Correct number of days in February for leap year if needed --------------
!   WRITE (6,*) "Year in Jul Day ", iYear
   IF (iYear == (iYear / 4) * 4) nDaysInMonth(2) = 29

   !-- Check for year out of range ---------------------------------------------
   IF (iYear < 1901 .OR. iYear > 2099) THEN
      CALL Error_Messaging('Julian_Day', 'Year out of bounds (1901 - 2099)', &
       FATAL)
      STOP
   ENDIF

   !-- Check for month out of range --------------------------------------------
   IF (Month < 1 .OR. Month > 12) THEN
      CALL Error_Messaging('Julian_Day', 'Month out of bounds (1 - 12)', &
       FATAL)
      STOP
   ENDIF

   !-- Check for day of month out of range -------------------------------------
   IF (iDay < 1 .OR. iDay > nDaysInMonth(Month)) THEN
      CALL Error_Messaging('Julian_Day', 'Day of month out of bounds', &
       FATAL)
      STOP
   ENDIF

   !-- Compute the Julian day from the month and day of month ------------------
   Julian_Day = iDay
   IF(Month > 1) Julian_Day = Julian_Day + SUM(nDaysInMonth(1:Month-1))
   RETURN

END FUNCTION Julian_Day


!===============================================================================
! NAME: GouldValue
!
! TYPE/LANGUAGE: F90 function
!
! FUNCTION:
!   Converts a floating-point value into Gould format.
!
! DESCRIPTION:
!   Gould format is used by McIDAS calibration.
!
! FILES NEEDED:
!   - None
!
! MODULES / SUBROUTINES NEEDED
!   - Type_Kinds module
!
! SUBROUTINES CONTAINED: N/
!
! CALLING SEQUENCE:
!   I = GouldValue(value)
!
! INPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   value                I     The value to be converted to Gould fomat
!
! OUTPUTS:
!   Name                 Type  Description
!   ----------------------------------------------------------------------------
!   GouldValue       O     The Gould equivalent of 'value'
!
! SYSTEM CALLS:
!   - None
!
! REFERENCE:
!   http://www.ssec.wisc.edu/mcidas
!
! DEPENDENCIES:
!   -  Compiled using the GNU fortran (gfortran) compiler
!
! SIDE EFFECTS:
!   - Vertigo, hallucinations, psychosis
!
! HISTORY:
!   See McIDAS history (http://www.ssec.wisc.edu/mcidas)
!   2016-09-13  B. Kuligowski   C0; clean up to meet SPSRB standards
!
!-------------------------------------------------------------------------------
FUNCTION GouldValue(value)

   USE Type_Kinds

   IMPLICIT NONE

!== VARIABLES ==================================================================
!-- INPUT: original value ------------------------------------------------------
   REAL(SINGLE), INTENT(IN) :: value

!-- OUTPUT: Gould value --------------------------------------------------------
   INTEGER :: GouldValue
      
!-- Processing variables -------------------------------------------------------
   REAL(DOUBLE) :: y

!== MAIN FUNCTION ==============================================================

   y = LOG(ABS(DBLE(NINT(value)))) / LOG(16D0)
   GouldValue = IOR(ISHFT(IAND( 70 + INT(y) - 5, 127), 24), &
    IAND(INT(16. ** (SNGL(y) - FLOAT(INT(y)) + 5.)), 16777215))
   RETURN
      
END FUNCTION GouldValue

END MODULE write_area_module
