#!/bin/bash

############################################################################
#
# run_etrap.sh
#
# This script is the main driver for Etrap processing (Fortran version).
# It first creates a list of TRaP files for a given storm name, and computes
# the time offsets for each file.  This information is written to a file called
# "etrap_master_file_list.out".  
#
# Next the script calls the etrap.f90 code to process the TRaP files.
#
# See the Command Usage section for command line options.
# Usage: run_etrap.sh <storm name> <basin> [<basin2> <start date> <start time>]
#
# Example file format of etrap_master_file_list.out:
# 128
# 1208995200 20080424 0000 2008115
# 1209081600
# 1208942340 139260 117660 96060 74460 "<filename>"
# 1208963940 117660 96060 74460 52860 "<filename>"
#
# Definition of file contents:
# <number of filenames in list>
# <start date and time in seconds> <start date> <start time> <start date Julian>
# <start date and time plus 24 hours, in seconds>
# <forecast time of file in seconds> <difference between file forecast time and
#                                     24 hour forecast time>
#                                    <difference between file forecast time and
#                                     18 hour forecast time>
#                                    <difference between file forecast time and
#                                     12 hour forecast time>
#                                    <difference between file forecast time and
#                                     06 hour forecast time>
#                                    <filename of Trap file>
# 
#  Modified: 20081117 (J.Yang)
#                 get the clat/lon from the new format TRaP text file
#
#  Modified: 02/18/2009 (J.Yang)
#                 Add the area file number for the first product as input argument
#
#  Modified: 03/11/2009 (J.Yang)
#                 Fix the time for 06 hour
#
#  Modified: 05/21/2009 - 05/27 (J.Yang)
#                 Orginized the log files and do error checking for monitoring
#
#  Modified: 12/28/2009 (J. Yang)
#                 Uses double quotes for the file_list, otherwise it may fail in 01/01
#

#  Last modified: 08/30/2011 (R. Glassberg)
#                 Made changes to enable running on a hourly schedule in order to use
#                 late arriving TRaP sets
#

############################################################################

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#
# Default Settings: (change these according to your setup)
#
# Base location of Trap data files.  It is assumed subdirectories containing
# year/storm_name are located below this direcroty.  
##trap_data_dir="/home/pkeehn/test_code/fortran/etrap/trap_data/"
#trap_data_dir="/data/TRAP/"
trap_data_dir="$trapDataDir/"

# Base location for output data files.  A year/storm_name subdirectory will be 
# created in this location.
##etrap_output_dir="/home/pkeehn/mcidas/data/etrap_out/"
#etrap_output_dir="/data/TRAP/ETRAP"
#etrap_log_dir="/data/TRAP/ETRAP/logs"
etrap_output_dir=$etrapDataDir
etrap_log_dir=$logDir

# etrap bin directory for executable
#etrap_bin_dir=$HOME/mcidas/etrap/bin
etrap_bin_dir=$etrapBinDir

# etrap scripts directory
#etrap_script_dir=$HOME/mcidas/etrap/scripts
etrap_script_dir=$etrapSrcDir

# Data size.
rows=800
columns=800

# Rain thresholds for 6 hour accumulations.
thresh_06=( 25  50  75 100 )

# Rain thresholds for 24 hour accumulations.
thresh_24=( 50 100 150 200 )

# Maximum number of combinations to use.
max_members=200

# Mcidas data directory to use.
##mc_data_dir="/home/pkeehn/mcidas/data/"
#mc_data_dir="/home/oper1/mcidas/data/"
mc_data_dir="/data/Petrap/data/mcidas_data/"

# Log file for saving the GIF file, must match with the one on etrap.f90
saveGifLog="$mc_data_dir/saveGif.log"

# The last Forecast position file for digital eTRaP generation
lastPosFile=$mc_data_dir/LATEST_POS.TXT

# The Product filename list files and created AREA list, which will be created in the etrap.exe 
#pdListPre="$mc_data_dir/prodList"
#pdAreaList="$mc_data_dir/prodAreaList.txt"
pdListPre1="prodList"
pdAreaList1="prodAreaList.txt"
pdListPre="$mc_data_dir$pdListPre1"
pdAreaList="$mc_data_dir$pdAreaList1"

# Temporary Mcidas area numbers to use during processing.
mc_work_area_1=1000
mc_work_area_2=1001

# Product Mcidas area numbers for the first eTRaP product (avg QPF), the other five will be in sequency
mc_prod_areas=( 201 211 221 231 241 )


 
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

#
# Command usage.
#
if [ $# -lt 2 ] 
then
   echo "Usage: run_etrap.sh <storm name> <basin1> [<basin2> <start date> <start time>]"
   echo "    storm name = Name of storm."
   echo "    basin1      = basin area the storm fall in."
   echo "    basin2 (opt)= the new basin when the storm cross from the previous 'basin1'"
   echo "                 default is same as the basin1 and means no crossing basins"
   echo "    start date (opt) = Starting date (YYYYMMDD)"
   echo "                       default is todays date"
   echo "    start time (opt) = Starting time (hh:mm)"
   echo "                       default is the previous 6 hour synoptic time"
   echo " "
   exit
fi

#
# Collect inputs and expand on them.
#
# Get storm name and covert to uppercase.
storm_name="$1"
storm_name=`echo $storm_name | sed -r "s/\w*/\U&/"`

current_time="$2"
basin1="$3"
basin2=$basin1

#Get basin2, if entered.
if [ $# -ge 4 ]
then
   basin2=$4
fi

# Get current time, if entered.
if [ $# -ge 5 ]
then
   start_date=$5
fi

# Get start time if entered.
if [ $# -ge 6 ]
then
   start_time=$6
fi

#
# Define the start day and time (rounded to previous 6 hour synoptic time) 
# in case it is not entered on the command line.
#
start_date=`date -u +%Y%m%d`
# to embed distinguishing text in diagnostic file names
curr_date_time=`date -u +%m%d%H%M`
# to help determine that the 25,35,45,55 runs during production
# hours do NOT need to be run 
curr_mins=`date -u +%M`
#start_date=`date -d '3 days ago 18 hours ago' +%Y%m%d`
#current_time=`date -d '3 days ago 18 hours ago' +%H`
# to account for current times between 0000 and 0300 when shifting synoptic times
if [ $current_time -lt 3 ]
then
    current_time=`expr $current_time + 24`
    start_date=`date --date="1 day ago" +%Y%m%d`
fi
# the following line not work for hour 08 or 09: "value too great for base"
# synoptic_time=$(( (current_time / 6) * 6 ))
# Make sure each hour time, (just not 0300, 0900, 1500, 2100) is set within the
# correct synoptic period
 
shft_current=`expr $current_time - 3`
synoptic_time=`expr $shft_current / 6  \* 6`
# need to know when to force an eTRaP run even with old unrefreshed TRaPs
prod_time_tst=`expr $shft_current - $synoptic_time`
#if [ $synoptic_time -eq 0 ]
if [ $synoptic_time -lt 10 ]
then
   #synoptic_time="00"
   synoptic_time='0'$synoptic_time
fi

start_time="$synoptic_time:00"
#start_time="$synoptic_time:00:00"
start_YYYY=`echo $start_date | cut -c1-4`
start_MMDD=`echo $start_date | cut -c5-8`

echo " storm name = $storm_name"
echo " basin1 = $basin1, basin2 = $basin2"
echo " start date = $start_date"
echo " start time = $start_time"
#
# Lead time is always 24 for this script.  The actual lead time used in the 
# etrap.f90 code is specified in the call to etrap.exe.
#
lead_time=24

#
# Convert the start date/time to seconds, and convert the forecast time to 
# seconds.
#
etrap_start_seconds=`date -u -d "$start_date $start_time" +%s`
etrap_forecast_seconds=$(( etrap_start_seconds + ($lead_time*3600) ))

#
# Convert the start date to J-Date for use in Mcidas area files.
#
start_J_date=`date -u -d "$start_date" +%Y%j`
#
# Define the data directory for this storm and verify its existence.
#
data_dir=${trap_data_dir}${start_YYYY}/$storm_name
if [ ! -e $data_dir ] 
then
   echo " ERROR! TRaP data directory non-existent: $data_dir"
   echo " ERROR! TRaP data directory non-existent: $data_dir, program is terminated!" >> $logfile_img
   echo " ERROR! TRaP data directory non-existent: $data_dir, program is terminated!" >> $logfile_dist
#   exit 1
fi

#
# Get the list of all TRap files for this storm in order of most recent 
# file creation.
#
echo " Finding TRaP files..."
#file_list=`ls -1t $data_dir/2*.????????.[0,1][0-9].Z`
#######
   start_YYYY_previous=$(( start_YYYY - 1 ))
   data_dir_previous=${trap_data_dir}${start_YYYY_previous}/$storm_name
   file_list=`ls -1t $data_dir/2*.????????.[0,1][0-9].Z $data_dir_previous/2*.????????.[0,1][0-9].Z`
#file_list=`ls -1t $data_dir/2*.????????.[0,1][0-9].Z | grep -v "\.HEHR\."| grep -v "\.SSMIS\."`
#file_list=`ls -1t $data_dir/2*.????????.[0,1][0-9].Z | grep -v "\.GHEHR\."`

#
# If it's January 1, then get the files that may exist in the previous years
# directory for this storm name.
#
if [ start_MMDD = "0101" ]
then
   start_YYYY_previous=$(( start_YYYY - 1 ))
   data_dir_previous=${trap_data_dir}${start_YYYY_previous}/$storm_name
   file_list=`ls -1t $data_dir/2*.????????.[0,1][0-9].Z $data_dir_previous/2*.????????.[0,1][0-9].Z`
#   file_list=$file_list `ls -1t $data_dir_previous/2*.????????.[0,1][0-9].Z`
   #file_list=$file_list `ls -1t $data_dir_previous/2*.????????.[0,1][0-9].Z | grep -v "\.HEHR\."`
#   file_list="$file_list `ls -1t $data_dir_previous/2*.????????.[0,1][0-9].Z | grep -v "\.GHEHR\."`"
#   file_list="$file_list `ls -1t $data_dir_previous/2*.????????.[0,1][0-9].Z | grep -v "\.HEHR\."| grep -v "\.SSMIS\."`"
fi

#
# Get the center lat/lon.
#
#if [ $# -eq 1 ]
if [ $# -le 3 ]
then
   # If no date/time is entered then get the latest track file.
   echo "No date/time is entered, get the latest track file."
#   center_latlon_file=`ls -1t $data_dir/2*.????????.txt.Z | grep -v "\.SSMIS\."| head -1`
   center_latlon_file=`ls -1t $data_dir/2*.????????.txt.Z | head -1`
   #center_latlon=`zcat $center_latlon_file | tail -1 | cut -c22-`
   center_latlon=`zcat $center_latlon_file | tail -2 | head -1| cut -c22-`
   center_lat=`echo $center_latlon | cut -f1 -d" "`
   center_lon=`echo $center_latlon | cut -f2 -d" "`
else
   # If a date/time was entered then find the track file that is closest in
   # time to the date/time entered.
   echo "A date/time was entered, find the track file that is closest in time to the date/time entered."
   date_time_seconds=`date -u -d "$start_date $start_time" +%s`
   smallest_time_diff=999999
   for file_name in `ls -1t $data_dir_previous/2*.????????.txt.Z $data_dir/2*.????????.txt.Z`
#   for file_name in `ls -1t $data_dir/2*.????????.txt.Z`
   do
      basename=`basename $file_name`
      # Get the time components.
      YYYY=`echo $basename | cut -c1-4`
      MM=`echo $basename | cut -d"." -f6 | cut -c1-2`
      DD=`echo $basename | cut -d"." -f6 | cut -c3-4`
      hh=`echo $basename | cut -d"." -f6 | cut -c5-6`
      mm=`echo $basename | cut -d"." -f6 | cut -c7-8`
      # Convert the date to seconds.
      file_base_seconds=`date -u -d "$YYYY-$MM-$DD ${hh}:${mm}" +%s`
      # Calculate the time difference.
      time_diff=$(( date_time_seconds - file_base_seconds ))
      # Convert negative differences to positive.
      if [ $time_diff -lt 0 ]
      then
         time_diff=$(( time_diff * -1 ))
      fi
      # Keep the file with the smallest time difference.
      if [ $time_diff -lt $smallest_time_diff ]
      then
         smallest_time_diff=$time_diff
         center_latlon_file=$file_name
      fi
   done
   # Retrieve the center lat/lon 
   #center_latlon=`zcat $center_latlon_file | tail -1 | cut -c22-`
   center_latlon=`zcat $center_latlon_file | tail -2 | head -1| cut -c22-`
   center_lat=`echo $center_latlon | cut -f1 -d" "`
   center_lon=`echo $center_latlon | cut -f2 -d" "`

fi

#
# Add the error checking  for empty lat/lon file and lat/lon value
#

if [ ! -s $center_latlon_file ]
then
   echo "ERROR: failed to locate the center_latlon_file!"
fi

if [[ $center_latlon = "" ]] || [[ $center_latlon = *[a-z,A-Z]* ]]
then
   echo "ERROR: center_latlon for the latest TRaP of $storm_name is empty or invalid, Exit the program!"
   echo "ERROR: center_latlon for the latest TRaP of $storm_name is empty or invalid, program is terminated!" >> $logfile_img
   echo "ERROR: center_latlon for the latest TRaP of $storm_name is empty or invalid, program is terminated!" >> $logfile_dist
   echo "center_latlon = $center_latlon"
   exit 1
fi


# Copy the last forecast position file to a fixed name file for digital eTRaP to use
cp -p $center_latlon_file $lastPosFile.Z
uncompress -f $lastPosFile.Z

echo " Center lat = $center_lat  Center lon = $center_lon"
echo "center_latlon_file is $center_latlon_file"

number_of_files=`echo $file_list | wc -w`

#
# Clean up any old master lists.
#
echo "Current directory is "
pwd
rm -f ${etrap_output_dir}/etrap_master_list$storm_name.out ${etrap_output_dir}/etrap_master_list$storm_name.tmp

#
# Initialize some counters and loop through the file list.
#
echo " Scanning TRaP files..."

count=1
count_00=0
count_06=0
count_12=0
count_18=0
good_file_counter=0
while [ $count -le $number_of_files ]
do 

   name=`echo $file_list | cut -f${count} -d" "`
   count=$(( count + 1 ))

   #
   # Check to make sure the file size is greater than 2000 bytes.
   #
   file_size=`stat -c%s $name`
   if [ $file_size -gt 2000 ]
   then

      #
      # Decode the etrap filename into its time components and calculate the
      # number of seconds since 1970-01-01 00:00:00 UTC.  Write the seconds and 
      # the file name to the master file list.
      #

      # Get the base filename without the directory string.
      basename=`basename $name`

      # Get the time components.
      YYYY=`echo $basename | cut -c1-4`
      MM=`echo $basename | cut -d"." -f6 | cut -c1-2`
      DD=`echo $basename | cut -d"." -f6 | cut -c3-4`
      hh=`echo $basename | cut -d"." -f6 | cut -c5-6`
      mm=`echo $basename | cut -d"." -f6 | cut -c7-8`
      src=`echo $basename | cut -d"." -f5`
      fcst=`echo $basename | cut -d"." -f7`

      # Convert the forecast hour to seconds.
      file_fcst_seconds=`date -u -d "1970-01-01 $fcst hours" +%s`

      # Convert the date to seconds.
      file_base_seconds=`date -u -d "$YYYY-$MM-$DD $hh:$mm" +%s`
   
      # Add the date and forecast time to get the actual date, plus another 
      # 6 hours to account for the ending time of the forecast period.
      file_total_seconds=$(( file_base_seconds + file_fcst_seconds + (6*3600) ))

      #
      # Now calculate the time difference (in seconds) betweeen the forecast
      # bin (24, 18, 12, 6) and the file time.
      #
      # now also need to prevent early arriving TRaPs from getting into a synoptic period
      # before their time

      forecast_bin_outer=$etrap_forecast_seconds
      time_test_val=$(( forecast_bin_outer - file_total_seconds ))
      time_test=$(( 54000 - (($fcst/6) * 21600)))
      #echo "Timetestval "$time_test_val
      #echo "Timetest "$time_test
      if [ $time_test_val -ge $time_test ]
      then
      #	  echo "Into outer while"
      #
      # Count how many good files we find.
      #
	  no_go=1

	  if [[ $fcst == "00" ]] 
	      then
	      count_00=$(( count_00 + 1 ))
	      if [[ $count_00 -ge 5 ]] && [[ $src == "GHEHR" ]]
		  then
		  no_go=0
	      fi
	  fi
	      
	  if [[ $fcst == "06" ]] 
	      then
	      count_06=$(( count_06 + 1 ))
	      if [[ $count_06 -ge 4 ]] && [[ $src == "GHEHR" ]]
		  then
		  no_go=0
	      fi
	  fi
	      
	  if [[ $fcst == "12" ]] 
	      then
	      count_12=$(( count_12 + 1 ))
	      if [[ $count_12 -ge 3 ]] && [[ $src == "GHEHR" ]]
		  then
		  no_go=0
	      fi
	  fi
	      
	  if [[ $fcst == "18" ]] 
	      then
	      count_18=$(( count_18 + 1 ))
	      if [[ $count_18 -ge 3 ]] && [[ $src == "GHEHR" ]]
		  then
		  no_go=0
	      fi
	  fi
#	  echo "counts src hh mm" $count_00 $count_06 $count_12 $count_18 $hh $mm $srcq
	      
	  if [[ $no_go -eq 1 ]]
	      then
#	      echo "processing"
	      good_file_counter=$(( good_file_counter + 1 ))

	      time_diff=""
	      forecast_bin=$etrap_forecast_seconds
	      while [ $forecast_bin -gt $etrap_start_seconds ]
	      do
		  time_diff="$time_diff $(( (forecast_bin - file_total_seconds) ))"
		  forecast_bin=$(( forecast_bin - (6*3600) ))
	      done

      # Write the data time (in seconds), the time differences, and the 
      # filename to a temporary file.
	      echo $file_total_seconds $time_diff \"$name\" >> ${etrap_output_dir}/etrap_master_list$storm_name.tmp
	  fi
      fi
   fi
done

#
# Write the information to the master file list.
#
start_time_without_colon=`echo $start_time | cut -c1,2,4,5`
#start_time_without_colon=`echo $start_time | cut -c1,2,4,5,7,8`
echo "start_time is $start_time, start_time_without_colon is $start_time_without_colon"

echo $good_file_counter > ${etrap_output_dir}/etrap_master_list$storm_name.out
echo $etrap_start_seconds $start_date $start_time_without_colon $start_J_date >> ${etrap_output_dir}/etrap_master_list$storm_name.out
echo $etrap_forecast_seconds >> ${etrap_output_dir}/etrap_master_list$storm_name.out
# Sort the file list before writing it to the master list.  This will help
# speed up the Fortran code.
sort -k6 ${etrap_output_dir}/etrap_master_list$storm_name.tmp >> ${etrap_output_dir}/etrap_master_list$storm_name.out

# Compare last 2 versions of master file list per storm to determine possible redundency

calc_diff_file=0

# contains archived, diagnostic info of trap names for each
# previous run
# ie. sv_mast_listIRENE09151505

sv_mast_list_tm=${etrap_output_dir}/sv_mast_list$storm_name$curr_date_time

# contains archived, diagnostic info of trap names and associated 
# time stamps for each previous run
# ie. sv_mast_list_allIRENE09151505

sv_mast_list_tm_all=${etrap_output_dir}/sv_mast_list_all$storm_name$curr_date_time

# contains archived, diagnostic info of trap names
# for each current run
# ie. main_mast_listIRENE09151605

main_mast_list_tm=main_mast_list$storm_name$curr_date_time

# contains info of trap names
# for each previous run to use in detection of new traps
# ie. sv_mast_listIRENE.out. For diagnostic purposes, copied to
# sv_mast_list_tm

sv_mast_list=${etrap_output_dir}/sv_mast_list$storm_name.out

# contains info of trap names
# for each current run to use in detection of new traps
# ie. mast_listIRENE.out. For diagnostic purposes, copied to
# main_mast_list_tm

cat ${etrap_output_dir}/etrap_master_list$storm_name.out| awk '{ print $6}' >${etrap_output_dir}/mast_list$storm_name.out
if [ -e $sv_mast_list ]
then
    calc_diff_file=`diff $sv_mast_list ${etrap_output_dir}/mast_list$storm_name.out|wc -l`
fi

# cp etrap_master_list$storm_name.out $sv_mast_list_tm_all 
#
# Clean up the temporary file.
#
rm -f ${etrap_output_dir}/etrap_master_list$storm_name.tmp

#
# Make sure the output directory exists.
#
if [ ! -e $etrap_output_dir/$start_YYYY/$storm_name ]
then
   mkdir -p "$etrap_output_dir/$start_YYYY/$storm_name"
fi
# If no difference in master file list and not the normal production time slot,
# do NOT execute an eTRaP for this storm

if [[ $calc_diff_file -eq 0 ]] && [[ $prod_time_tst -ne 0 ]]
then 

    mv ${etrap_output_dir}/mast_list$storm_name.out $sv_mast_list

    echo "No new TRaPs, exiting"
    exit
fi

# also don't allow an etrap force at 35,45,55 minutes after
# production times

#if [[ $calc_diff_file -eq 0 ]] &&  [[ $prod_time_tst -eq 0 ]] && [[ $((10#$curr_mins)) -ge 55 ]]
#then 
#
#    cp mast_list$storm_name.out $sv_mast_list
#
#    echo "No new TRaPs, exiting"
#    exit
#fi

# cp $sv_mast_list $sv_mast_list_tm
# cp mast_list$storm_name.out $main_mast_list_tm
mv ${etrap_output_dir}/mast_list$storm_name.out $sv_mast_list

#
# Clean up the previous eTRaP Product filename list files and created AREA list, which will be created in the etrap.exe
#
rm -f $pdListPre.??.txt
rm -f $pdAreaList


if [ -s $saveGifLog ]
then
   mv $saveGifLog $saveGifLog.old
fi

sleep 10
#
# Run the etrap code.
#
echo " Calling ETRaP code..."
echo " "

echo "etrap.exe $storm_name 6 6 $center_lat $center_lon $rows $columns ${thresh_06[0]} ${thresh_06[1]} ${thresh_06[2]} ${thresh_06[3]} $etrap_output_dir $mc_data_dir $mc_work_area_1 $mc_work_area_2 $max_members ${mc_prod_areas[0]} ${etrap_output_dir}/etrap_master_list$storm_name.out"

time $etrap_bin_dir/etrap.exe $storm_name 6 6 $center_lat $center_lon $rows $columns ${thresh_06[0]} ${thresh_06[1]} ${thresh_06[2]} ${thresh_06[3]} $etrap_output_dir $mc_data_dir $mc_work_area_1 $mc_work_area_2 $max_members ${mc_prod_areas[0]} ${etrap_output_dir}/etrap_master_list$storm_name.out

if [ -s $saveGifLog ]
then 
   echo "Appended the GIF log file for 0-6 hr"
   echo "--------------" >> $logfile_img
   cat $saveGifLog >> $logfile_img
   mv $saveGifLog $saveGifLog.old
fi

echo " "
echo "--------------------------------------- "
echo "etrap.exe $storm_name 12 6"

time $etrap_bin_dir/etrap.exe $storm_name 12 6 $center_lat $center_lon $rows $columns ${thresh_06[0]} ${thresh_06[1]} ${thresh_06[2]} ${thresh_06[3]} $etrap_output_dir $mc_data_dir $mc_work_area_1 $mc_work_area_2 $max_members ${mc_prod_areas[1]} ${etrap_output_dir}/etrap_master_list$storm_name.out

if [ -s $saveGifLog ]
then
   echo "Appended the GIF log file for 6-12 hr"
   echo "--------------" >> $logfile_img
   cat $saveGifLog >> $logfile_img
   mv $saveGifLog $saveGifLog.old
fi

echo " "
echo "--------------------------------------- "
echo "etrap.exe $storm_name 18 6"

time $etrap_bin_dir/etrap.exe $storm_name 18 6 $center_lat $center_lon $rows $columns ${thresh_06[0]} ${thresh_06[1]} ${thresh_06[2]} ${thresh_06[3]} $etrap_output_dir $mc_data_dir $mc_work_area_1 $mc_work_area_2 $max_members ${mc_prod_areas[2]} ${etrap_output_dir}/etrap_master_list$storm_name.out

if [ -s $saveGifLog ]
then
   echo "Appended the GIF log file for 12-18 hr"
   echo "--------------" >> $logfile_img
   cat $saveGifLog >> $logfile_img
   mv $saveGifLog $saveGifLog.old
fi

echo " "
echo "--------------------------------------- "
echo "etrap.exe $storm_name 24 6"

time $etrap_bin_dir/etrap.exe $storm_name 24 6 $center_lat $center_lon $rows $columns ${thresh_06[0]} ${thresh_06[1]} ${thresh_06[2]} ${thresh_06[3]} $etrap_output_dir $mc_data_dir $mc_work_area_1 $mc_work_area_2 $max_members ${mc_prod_areas[3]} ${etrap_output_dir}/etrap_master_list$storm_name.out

if [ -s $saveGifLog ]
then
   echo "Appended the GIF log file for 18-24 hr"
   echo "--------------" >> $logfile_img
   cat $saveGifLog >> $logfile_img
   mv $saveGifLog $saveGifLog.old
fi

echo " "
echo "--------------------------------------- "
echo "etrap.exe $storm_name 24 24"

time $etrap_bin_dir/etrap.exe $storm_name 24 24 $center_lat $center_lon $rows $columns ${thresh_24[0]} ${thresh_24[1]} ${thresh_24[2]} ${thresh_24[3]} $etrap_output_dir $mc_data_dir $mc_work_area_1 $mc_work_area_2 $max_members ${mc_prod_areas[4]} ${etrap_output_dir}/etrap_master_list$storm_name.out

if [ -s $saveGifLog ]
then
   echo "Appended the GIF log file for 0-24 hr"
   echo "--------------" >> $logfile_img
   cat $saveGifLog >> $logfile_img
   mv $saveGifLog $saveGifLog.old
fi


#
# Call the Perl program to create digital eTRaP and deliver the product out
#

# Check if there is any eTRaP product generated
if [ -s $pdAreaList ]
then
   echo "Call the Perl program to create digital eTRaP and deliver the product out:"
   echo "$etrap_script_dir/area2digitetrap.pl $basin1 $basin2 $pdListPre $pdAreaList $lastPosFile"

   echo " "                                                                                        >> $logfile_dist 2>&1
   echo "$etrap_script_dir/area2digitetrap.pl $basin1 $basin2 $pdListPre $pdAreaList $lastPosFile" >> $logfile_dist 2>&1
   $etrap_script_dir/area2digitetrap.pl $basin1 $basin2 $pdListPre $pdAreaList $lastPosFile        >> $logfile_dist 2>&1
   cat $logfile_dist >> $etrap_log_dir/digitalETRAP.log
else
   echo "No eTRaP product generated for storm $storm_name at $start_date $start_time UTC! "
   echo "No eTRaP product generated for storm $storm_name at $start_date $start_time UTC! " >> $logfile_img 
   echo "No eTRaP product generated for storm $storm_name at $start_date $start_time UTC! " >> $logfile_dist 
fi

exit
