!-------------------------------------------------------------------------------
! NAME:
!   Etrap
!
! FUNCTION:
!   Compute the ensemble Trap products.
!
! DESCRIPTION:
!   Etrap is designed to read the TRAP data (in Mcidas format) for a 
!   particuler storm-name and generate the ensemble TRAP (or ETRAP) 
!   products (in Mcidas format).
!
! REFERENCE:
!
! CALLING SEQUENCE:
!   See the usage portion of the code for the most up to date list of command
!   line arguments.
!   etrap <storm name> <lead time> <accumulation time> <center lat> 
!         <center lon> <rows> <columns> <thresh 1> <thresh 2> <thresh 3> 
!         <thresh 4> <output dir> <mc data dir> <mc work area 1> 
!         <mc work area 2> <max combinations> <etrap output area>
!
! INPUTS:
!   <storm name>        = Name of storm to process
!   <lead time>         = Number of hours ahead of base time (6, 12, 18, or 24)
!   <accumulation time> = Number of hours of accumulate (6, 12, 18, or 24)
!   <center lat>        = Center latitude
!   <center lon>        = Center longitude
!   <rows>              = Number of rows (or lines) of data to process
!   <columns>           = Number of columns (or elements) of data to process
!   <thresh 1>          = First rain threshold for probability product (mm)
!   <thresh 2>          = Second rain threshold for probability product (mm)
!   <thresh 3>          = Third rain threshold for probability product (mm)
!   <thresh 4>          = Fourth rain threshold for probability product (mm)
!   <output dir>        = Location to write output results
!   <mc data dir>       = Mcidas data directory to use
!   <mc work area 1>    = First Mcidas area number for temporary use
!   <mc work area 2>    = Second Mcidas area number for temporary use
!   <max combinations>  = Maximum number of combinations allowed
!   <etrap output area> = The Mcidas area number for 1st etrap product(avg qpf), 
!                         the others five will be in syquency 
!
!   etrap_master_list.out - List of TRAP files with time offsets, generated by
!                           the run_etrap.sh calling script.
!   TRAP files in mcidas format.
!
! OUTPUTS:
!   1. Average QPF
!   2. Probability Matched QPF
!   3. Threshold Probabilities (4 separate files)
!   4. Log file
!
! DEPENDENCIES:
!   run_etrap.sh script
!   type_kinds module
!   Get_lun function 
!   error_messaging_module
!   etrap_utils module
!   imgcopy.k (mcidas command)
!
! RESTRICTIONS:
!
! HISTORY:
!   2008-05-22  P. Keehn     Initial development.
!   2008-07-30  P. Keehn     Added mcidas data directory input parameter.
!   2009-02-17  J. Yang      Added input parameter for eTRaP Product Mcidas area number
!   2009, Feb ~ April        by J. Yang
!                            1. Correct the time perid range for the print out
!                            2. Save the AREA file name and eTraP product name in some files for next steps.
!                            3. Change the MEMO fields and Call the script to create the GIF files 
!                            4. Print out the TRaP member number information to the log file
!   2009-05-05  J. Yang      Print the list of the individual TRaP members to the ETRAP log file for 6 hours period
!
!   2009-09-14  J. Yang      Correct the use of valid TRaP list and count after the adjustment
!
!   2013-07-22  S. Kidder    Added calibration
!
!-------------------------------------------------------------------------------
PROGRAM Etrap

   USE type_kinds
   USE Error_Messaging_Module
   USE etrap_utils

   IMPLICIT NONE

   CHARACTER(LEN=5) :: RoutineName = "Etrap"
   CHARACTER(LEN=2) :: EtrapFcstString
   CHARACTER(LEN=4) :: YearString
   CHARACTER(LEN=8) :: DateTimeString
   CHARACTER(LEN=10) :: BaseDateString
   CHARACTER(LEN=10) :: BaseTimeString
   CHARACTER(LEN=20) :: StormName
   CHARACTER(LEN=32) :: McidasMemo
   CHARACTER(LEN=100) :: ErrorString
   CHARACTER(LEN=100) :: CommandLineArgument
   CHARACTER(LEN=200) :: StormDir
   CHARACTER(LEN=200) :: OutputDirectory
   CHARACTER(LEN=200) :: McidasDataDirectory
   CHARACTER(LEN=200) :: AvgQpfFilename
   CHARACTER(LEN=200) :: PmQpfFilename
   CHARACTER(LEN=200), DIMENSION(4) :: PQpfFilename  ! Must be same size as RainThresholdsString
   CHARACTER(LEN=200) :: LogFilename
   CHARACTER(LEN=10), DIMENSION(4) :: RainThresholdsString
   CHARACTER(LEN=200), DIMENSION(:), POINTER :: Filenames6hour
   CHARACTER(LEN=200), DIMENSION(:), POINTER :: Filenames12hour
   CHARACTER(LEN=200), DIMENSION(:), POINTER :: Filenames18hour
   CHARACTER(LEN=200), DIMENSION(:), POINTER :: Filenames24hour
   CHARACTER(LEN=200), DIMENSION(:), ALLOCATABLE :: ValidFilenames6hour
   CHARACTER(LEN=200), DIMENSION(:), ALLOCATABLE :: ValidFilenames12hour
   CHARACTER(LEN=200), DIMENSION(:), ALLOCATABLE :: ValidFilenames18hour
   CHARACTER(LEN=200), DIMENSION(:), ALLOCATABLE :: ValidFilenames24hour
   CHARACTER(LEN=200), DIMENSION(:), ALLOCATABLE :: FilenamesAll

   ! For Mcidas Product area files
   CHARACTER(LEN=4) :: areaPre = "AREA"
   CHARACTER(LEN=200), DIMENSION(6):: ProdAreaNames
   CHARACTER(LEN=4) ::  ProdAreaStr
   CHARACTER(LEN=200) :: Command
   CHARACTER(LEN=8) :: FcstLeadTimeString
   CHARACTER(LEN=2) ::LeadTimeStr

   ! For GIF files
   CHARACTER(LEN=5) :: gifPre = "ETRAP"
   CHARACTER(LEN=1) :: counterStr
   CHARACTER(LEN=15) :: saveGifCmd = "MKPROBS.sh"

   ! For Products filename list
   !CHARACTER(LEN=15) :: pdList = "prodList.txt"
   CHARACTER(LEN=15) :: pdList = "prodList"
   CHARACTER(LEN=200) :: pdListFilename

   ! For one set of the created eTRaP AREA files list 
   CHARACTER(LEN=20) :: pdAreaList = "prodAreaList.txt"
   CHARACTER(LEN=200) :: pdAreaListFilename

   INTEGER(LONG) :: ErrorStatus
   INTEGER(LONG) :: LUN
   INTEGER(LONG) :: NumArguments
   INTEGER(LONG) :: NumRows
   INTEGER(LONG) :: NumColumns
   INTEGER(LONG) :: NumRainThresholds
   INTEGER(LONG) :: NumFilenames6hour
   INTEGER(LONG) :: NumFilenames12hour
   INTEGER(LONG) :: NumFilenames18hour
   INTEGER(LONG) :: NumFilenames24hour
   INTEGER(LONG) :: NumCombinations
   INTEGER(LONG) :: NumTRaPSets
   INTEGER(LONG) :: BaseDate
   INTEGER(LONG) :: BaseTime
   INTEGER(LONG) :: BaseJDate
   INTEGER(LONG) :: LeadTime
   INTEGER(LONG) :: AccumulationPeriod
   INTEGER(LONG) :: Okay
   INTEGER(LONG) :: Row
   INTEGER(LONG) :: Column
   INTEGER(LONG) :: AllIndex
   INTEGER(LONG) :: Counter
   INTEGER(LONG) :: InnerCounter
   INTEGER(LONG) :: ValidCounter
   INTEGER(LONG) :: LoopCounter
   INTEGER(LONG) :: Count_6
   INTEGER(LONG) :: Count_12
   INTEGER(LONG) :: Count_18
   INTEGER(LONG) :: Count_24
   INTEGER(LONG) :: Threshold
   INTEGER(LONG) :: CombinationNumber
   INTEGER(LONG) :: MaxCombinations
   INTEGER(LONG) :: HistogramBin
   INTEGER(LONG) :: DummyInteger
   INTEGER(LONG) :: StringLength
   INTEGER(LONG) :: QPF_AverageNonzeroCount
   INTEGER(LONG) :: NonzeroColumn
   INTEGER(LONG) :: NonzeroRow
   INTEGER(LONG), DIMENSION(4) :: RainThresholds
   INTEGER(LONG), DIMENSION(2) :: WorkAreas
   INTEGER(LONG), DIMENSION(1) :: MaxLocation
   INTEGER(LONG), DIMENSION(:), POINTER :: RainRateDistribution
   INTEGER(LONG), DIMENSION(:), ALLOCATABLE :: CurrentHistogram
   INTEGER(LONG), DIMENSION(:), ALLOCATABLE :: Keep
   INTEGER(LONG), DIMENSION(:,:), ALLOCATABLE :: QPF_AverageNonzeroLocation
   INTEGER(LONG), DIMENSION(:,:), ALLOCATABLE :: Mask
   INTEGER(BYTE), DIMENSION(:,:), ALLOCATABLE :: QPF_AverageByte
   INTEGER(BYTE), DIMENSION(:,:), ALLOCATABLE :: PM_QPF_Byte
   INTEGER(BYTE), DIMENSION(:,:,:), ALLOCATABLE :: PQPF_Byte

   ! For Mcidas Product area files
   INTEGER(LONG) :: NumProdAreas
   INTEGER(LONG) :: NextCount
   INTEGER(LONG), DIMENSION(6) :: ProdAreas

   REAL(SINGLE) :: WeightFcst
   REAL(SINGLE) :: WeightSensor
   REAL(SINGLE) :: Weight6hour
   REAL(SINGLE) :: Weight12hour
   REAL(SINGLE) :: Weight18hour
   REAL(SINGLE) :: Weight24hour
   REAL(SINGLE) :: BiasSensor
   REAL(SINGLE) :: CenterLat
   REAL(SINGLE) :: CenterLon
   REAL(SINGLE) :: QpfScale
!   REAL(SINGLE) :: delta_time
!   REAL(SINGLE), DIMENSION(2) :: time_array
   REAL(SINGLE), DIMENSION(:), ALLOCATABLE :: QPF_AverageNonzeroValue
   REAL(SINGLE), DIMENSION(:), ALLOCATABLE :: QPF_NonzeroValue
   REAL(SINGLE), DIMENSION(:), ALLOCATABLE :: RainRateHistogram
   REAL(SINGLE), DIMENSION(:,:), ALLOCATABLE :: PM_QPF
   REAL(SINGLE), DIMENSION(:,:), ALLOCATABLE :: Weight
   REAL(SINGLE), DIMENSION(:,:), ALLOCATABLE :: RainRate
   REAL(SINGLE), DIMENSION(:,:), ALLOCATABLE :: WeightSum
   REAL(SINGLE), DIMENSION(:,:), ALLOCATABLE :: QPF_Sum
   REAL(SINGLE), DIMENSION(:,:), ALLOCATABLE :: QPF_Average
   REAL(SINGLE), DIMENSION(:,:,:), ALLOCATABLE :: RainRate6hour
   REAL(SINGLE), DIMENSION(:,:,:), ALLOCATABLE :: RainRate12hour
   REAL(SINGLE), DIMENSION(:,:,:), ALLOCATABLE :: RainRate18hour
   REAL(SINGLE), DIMENSION(:,:,:), ALLOCATABLE :: RainRate24hour
   REAL(SINGLE), DIMENSION(:,:,:), ALLOCATABLE :: PQPF_Sum
   REAL(SINGLE), DIMENSION(:,:,:), ALLOCATABLE :: PQPF


   !
   ! Function declarations.
   !
   INTEGER :: iargc
   !REAL :: dtime
   INTEGER(LONG) :: Get_lun

   !
   ! Debug stuff
   !
   !INTEGER :: test_row = 30
   !INTEGER :: test_column = 411
   !INTEGER :: test_pixel = 401 !119742 !315556 !75514 !23611
   

   !
   ! Show the program usage if the number of input arguments is incorrect.
   !
   NumArguments = iargc()
   IF (NumArguments /= 18) THEN
      PRINT *," Usage: etrap <storm name> <lead time> <accumulation time>"
      PRINT *,"              <center lat> <center lon>"
      PRINT *,"              <rows> <columns>"
      PRINT *,"              <thresh 1> <thresh 2> <thresh 3> <thresh 4>"
      PRINT *,"              <output dir>"
      PRINT *,"              <mcidas data dir>"
      PRINT *,"              <mc work area 1> <mc work area 2>"
      PRINT *,"              <max combinations>"
      PRINT *," "
      PRINT *,"    storm mame        = Name of storm to process (all caps.)"
      PRINT *,"    lead time         = Forecast time as 6, 12, 18, or 24"
      PRINT *,"    accumulation time = Accumulation period as 6, 12, 18, or 24"
      PRINT *,"    center lat        = Center latitude to process around"
      PRINT *,"    center lon        = Center longitude to process around"
      PRINT *,"    rows              = Number of rows (lines) to process"
      PRINT *,"    columns           = Number of columns (elements) to process"
      PRINT *,"    thresh 1          = First rain threshold cutoff"
      PRINT *,"    thresh 2          = Second rain threshold cutoff"
      PRINT *,"    thresh 3          = Third rain threshold cutoff"
      PRINT *,"    thresh 4          = Fourth rain threshold cutoff"
      PRINT *,"    output dir        = File location for output"
      PRINT *,"    mcidas data dir   = McIDAS data directory to use"
      PRINT *,"    mc work area 1    = First temporary McIDAS area number"
      PRINT *,"    mc work area 2    = Second temporary McIDAS area number"
      PRINT *,"    max combinations  = Maximum number ensemble members to use"
      PRINT *,"    etrap output area = The Mcidas area number for 1st etrap product(avg qpf)"
      PRINT *," "
      STOP
   END IF

   !
   ! Read in all the command line arguments.
   !
   CALL getarg(1, CommandLineArgument)
   StormName = TRIM(CommandLineArgument)

   CALL getarg(2, CommandLineArgument)
   READ(CommandLineArgument,"(i8)") LeadTime

   CALL getarg(3, CommandLineArgument)
   READ(CommandLineArgument,"(i8)") AccumulationPeriod

   CALL getarg(4, CommandLineArgument)
   READ(CommandLineArgument,"(f8.2)") CenterLat

   CALL getarg(5, CommandLineArgument)
   READ(CommandLineArgument,"(f8.2)") CenterLon

   CALL getarg(6, CommandLineArgument)
   READ(CommandLineArgument,"(i8)") NumRows

   CALL getarg(7, CommandLineArgument)
   READ(CommandLineArgument,"(i8)") NumColumns

   CALL getarg(8, CommandLineArgument)
   READ(CommandLineArgument,"(i8)") RainThresholds(1)

   CALL getarg(9, CommandLineArgument)
   READ(CommandLineArgument,"(i8)") RainThresholds(2)

   CALL getarg(10, CommandLineArgument)
   READ(CommandLineArgument,"(i8)") RainThresholds(3)

   CALL getarg(11, CommandLineArgument)
   READ(CommandLineArgument,"(i8)") RainThresholds(4)

   CALL getarg(12, OutputDirectory)

   CALL getarg(13, McidasDataDirectory)

   CALL getarg(14, CommandLineArgument)
   READ(CommandLineArgument,"(i8)") WorkAreas(1)

   CALL getarg(15, CommandLineArgument)
   READ(CommandLineArgument,"(i8)") WorkAreas(2)

   CALL getarg(16, CommandLineArgument)
   READ(CommandLineArgument,"(i8)") MaxCombinations

   CALL getarg(17, CommandLineArgument)
   READ(CommandLineArgument,"(i8)") ProdAreas(1)

   CALL getarg(18, MasterFileList)

   !
   ! Write the command line arguments to the screen.
   !
   PRINT *, " storm name          = ",TRIM(StormName)
   PRINT *, " lead time           = ",LeadTime
   PRINT *, " accumulation period = ",AccumulationPeriod
   PRINT *, " center lat/lon      = ",CenterLat, CenterLon
   PRINT *, " dimensions          = ",NumRows, NumColumns
   PRINT *, " rain thresholds     = ",RainThresholds
   PRINT *, " output directory    = ",TRIM(OutputDirectory)
   PRINT *, " mcidas data dir     = ",TRIM(McidasDataDirectory)
   PRINT *, " mcidas work areas   = ",WorkAreas
   PRINT *, " max conbinations    = ",MaxCombinations
   PRINT *, " 1st Product mcidas  = ",ProdAreas(1)
   PRINT *, " "



   !
   ! Validate the input values as needed.
   !
   IF (AccumulationPeriod < 6 .OR. AccumulationPeriod > 24 .OR. &
   (AccumulationPeriod/6*6) /= AccumulationPeriod) THEN
      ErrorString = "Invalid accumulation value;  Use 6, 12, 18, or 24"
      CALL Error_Messaging(RoutineName, ErrorString, FATAL)
      STOP
   ENDIF

   IF (LeadTime < 6 .OR. LeadTime > 24 .OR. &
   LeadTime/6*6 /= LeadTime) THEN
      ErrorString = "Invalid leadtime value;  Use 6, 12, 18, or 24"
      CALL Error_Messaging(RoutineName, ErrorString, FATAL)
      STOP
   ENDIF

   IF (AccumulationPeriod > LeadTime) THEN
      ErrorString = "Accumulation cannot be greater than leadtime"
      CALL Error_Messaging(RoutineName, ErrorString, FATAL)
      STOP
   ENDIF

   IF (ProdAreas(1) < 0 .OR. ProdAreas(1) > 10000) THEN
      ErrorString = "Invalid Mcidas Area number; Must be 0-9999"
      CALL Error_Messaging(RoutineName, ErrorString, FATAL)
      STOP
   ENDIF

   NumRainThresholds = SIZE(RainThresholds)
   OutputDirectory = TRIM(OutputDirectory)//"/"

   !
   ! Get the other five Product area file numbers
   !

   NumProdAreas = SIZE(ProdAreas)

   print *, " 1st Mcidas product areas   = ",ProdAreas(1)

   DO Counter = 1, (NumProdAreas - 1)
       NextCount = Counter + 1
       ProdAreas(NextCount)= ProdAreas(Counter) + 1
   END DO

   DO Counter = 1, NumProdAreas
       WRITE(ProdAreaNames(Counter),FMT="(a,a,i4.4)") TRIM(McidasDataDirectory),TRIM(areaPre),ProdAreas(Counter)
       !WRITE(ProdAreaNames(Counter),FMT="(a,i4.4)") "AREA",ProdAreas(Counter)
   END DO


   !
   ! Get the appropriate files for each forecast period.
   !
   !delta_time = dtime(time_array)

   NumFilenames6hour = 0
   NumFilenames12hour = 0
   NumFilenames18hour = 0
   NumFilenames24hour = 0

   PRINT *, "Selecting TRaP files..."
   !
   ! 00-06 hour
   !
   IF (AccumulationPeriod > 0) THEN
      CALL Select_Traps(LeadTime, Filenames6hour, NumFilenames6hour, MasterFileList)
   PRINT *, "Before adjustment, NumFilenames6hour=", NumFilenames6hour
   !PRINT *, "00-06 hour files:"
   !PRINT *, LeadTime - AccumulationPeriod,"-",LeadTime," hour files: total ",NumFilenames6hour, " TRaP members."
   PRINT *, LeadTime - 6,"-",LeadTime," hour files: total ",NumFilenames6hour, " TRaP members."
   DO Counter = 1, NumFilenames6hour
      PRINT *, TRIM(Filenames6hour(Counter))
   END DO
   ENDIF

   !
   ! 06-12 hr
   !
   IF (AccumulationPeriod > 6) THEN
      CALL Select_Traps(LeadTime-6, Filenames12hour, NumFilenames12hour, MasterFileList)
   !PRINT *, "06-12 hour files:"
   PRINT *, LeadTime - 6 - 6,"-",LeadTime - 6," hour files: total ",NumFilenames12hour, " TRaP members."
   DO Counter = 1, NumFilenames12hour
      PRINT *, TRIM(Filenames12hour(Counter))
   END DO
   ENDIF

   !
   ! 12-18 hr
   !
   IF (AccumulationPeriod > 12) THEN
      CALL Select_Traps(LeadTime-12, Filenames18hour, NumFilenames18hour, MasterFileList)
   !PRINT *, "12-18 hour files:"
   PRINT *, LeadTime - 12 - 6,"-",LeadTime - 12," hour files: total ",NumFilenames18hour, " TRaP members."
   DO Counter = 1, NumFilenames18hour
      PRINT *, TRIM(Filenames18hour(Counter))
   END DO
   ENDIF

   !
   ! 18-24 hr
   !
   IF (AccumulationPeriod > 18) THEN
      CALL Select_Traps(LeadTime-18, Filenames24hour, NumFilenames24hour, MasterFileList)
   !PRINT *, "18-24 hour files:"
   PRINT *, LeadTime - 18 - 6,"-",LeadTime - 18," hour files: total ",NumFilenames24hour, " TRaP members."
   DO Counter = 1, NumFilenames24hour
      PRINT *, TRIM(Filenames24hour(Counter))
   END DO
   ENDIF

   !
   ! If their were no 6 hour forecast files found then the processing can not
   ! continue.
   !
   IF (NumFilenames6hour == 0) THEN
      ErrorString = "No ensemble members in first segment - quit!"
      !CALL Error_Messaging(RoutineName, ErrorString, FATAL)
      CALL Error_Messaging(RoutineName, ErrorString, WARNING)
      STOP
   ENDIF

   !delta_time = dtime(time_array)
   !print *,"Trap selection: ",delta_time

   !
   ! Check that it is possible to make the accumulation from existing data
   !
   Okay = 1
   IF (AccumulationPeriod <= 6 .AND. NumFilenames6hour == 0) THEN
      Okay = 0
   ENDIF
   IF (AccumulationPeriod > 6 .AND. NumFilenames12hour == 0) THEN
      Okay = 0
   ENDIF
   IF (AccumulationPeriod > 12 .AND. NumFilenames18hour == 0) THEN
      Okay = 0
   ENDIF
   IF (AccumulationPeriod > 18 .AND. NumFilenames24hour == 0) THEN
      Okay = 0
   ENDIF
   IF (AccumulationPeriod == 24 .AND. NumFilenames24hour > 0) THEN
      Okay = 1
   ENDIF

   IF (Okay == 0) THEN
      ErrorString = "Not enough data to make accumulation"
      CALL Error_Messaging(RoutineName, ErrorString, FATAL)
      STOP
   ENDIF

!! the following is moved after the ValidFilename is got
!! J.Yang
!   !
!   ! Count the number of independent TRaPs used to generate ensemble
!   ! and keep a list of the unique file names (names without the forecast
!   ! time).
!   !
!   ALLOCATE(FilenamesAll(NumFilenames6hour + NumFilenames12hour + &
!                         NumFilenames18hour + NumFilenames24hour), &
!            STAT=ErrorStatus)
!   IF (ErrorStatus /= 0) THEN
!      CALL Error_Messaging(RoutineName,"All filenames allocate error",FATAL)
!      STOP
!   END IF
!
!   AllIndex = 0
!   DO Counter = 1,NumFilenames6hour
!      AllIndex = AllIndex + 1
!      StringLength = LEN_TRIM(Filenames6hour(Counter)) - 5
!      FilenamesAll(AllIndex) = Filenames6hour(Counter)(:StringLength)
!   END DO
!   DO Counter = 1,NumFilenames12hour
!      AllIndex = AllIndex + 1
!      StringLength = LEN_TRIM(Filenames12hour(Counter)) - 5
!      FilenamesAll(AllIndex) = Filenames12hour(Counter)(:StringLength)
!   END DO
!   DO Counter = 1,NumFilenames18hour
!      AllIndex = AllIndex + 1
!      StringLength = LEN_TRIM(Filenames18hour(Counter)) - 5
!      FilenamesAll(AllIndex) = Filenames18hour(Counter)(:StringLength)
!   END DO
!   DO Counter = 1,NumFilenames24hour
!      AllIndex = AllIndex + 1
!      StringLength = LEN_TRIM(Filenames24hour(Counter)) - 5
!      FilenamesAll(AllIndex) = Filenames24hour(Counter)(:StringLength)
!   END DO
!   DO Counter = 1, AllIndex
!      DO InnerCounter = (Counter + 1), AllIndex
!         IF (TRIM(FilenamesAll(Counter)) == &
!             TRIM(FilenamesAll(InnerCounter))) THEN
!            FilenamesAll(Counter) = ""
!         END IF
!      END DO
!   END DO

   !
   ! Read in the Trap data for each valid accumulation period.
   !
   PRINT *, "Accumulating TRaP data..."

   ALLOCATE(RainRate(NumColumns, NumRows), STAT=ErrorStatus)
   IF (ErrorStatus /= 0) THEN
      CALL Error_Messaging(RoutineName,"Trap data allocate error",FATAL)
      STOP
   END IF

   IF (AccumulationPeriod > 0 .AND. NumFilenames6hour > 0) THEN
      ALLOCATE(RainRate6hour(NumColumns, NumRows, NumFilenames6hour), &
               ValidFilenames6hour(NumFilenames6hour), STAT=ErrorStatus)
      IF (ErrorStatus /= 0) THEN
         CALL Error_Messaging(RoutineName,"6 hour accum. allocate error",FATAL)
         STOP
      END IF
      ValidCounter = 1
      DO Counter = 1,NumFilenames6hour
         CALL Get_Trap_Rain_Rate(Filenames6hour(Counter), &
                                 NumRows, NumColumns, CenterLat, CenterLon, &
                                 McidasDataDirectory, &
                                 WorkAreas(1), WorkAreas(2), RainRate)

         IF (MAXVAL(RainRate) > 20.0) THEN
            RainRate6hour(:,:,ValidCounter) = RainRate
            ValidFilenames6hour(ValidCounter) = Filenames6hour(Counter)
            ValidCounter = ValidCounter + 1
            !PRINT *, "Maxval of rain rate > 20 for file:", TRIM(Filenames6hour(Counter))
         ELSE
            PRINT *, "Maxval of rain rate < 20, not use:", TRIM(Filenames6hour(Counter))
         END IF
      END DO
      NumFilenames6hour = ValidCounter - 1
      PRINT *, "After adjustment, NumFilenames6hour=", NumFilenames6hour, "ValidCounter=",ValidCounter

      IF (NumFilenames6hour <= 1) THEN
         ErrorString = "One or less ensemble members in SECOND segment - quit!"
         !CALL Error_Messaging(RoutineName, ErrorString, FATAL)
         CALL Error_Messaging(RoutineName, ErrorString, WARNING)
         STOP
      ENDIF

   ENDIF

   IF (AccumulationPeriod > 6) THEN
      ALLOCATE(RainRate12hour(NumColumns, NumRows, NumFilenames12hour), &
               ValidFilenames12hour(NumFilenames12hour), STAT=ErrorStatus)
      IF (ErrorStatus /= 0) THEN
         CALL Error_Messaging(RoutineName,"12 hour accum. allocate error",FATAL)
         STOP
      END IF
      ValidCounter = 1
      DO Counter = 1, NumFilenames12hour
         CALL Get_Trap_Rain_Rate(Filenames12hour(Counter), &
                                 NumRows, NumColumns, CenterLat, CenterLon, &
                                 McidasDataDirectory, &
                                 WorkAreas(1), WorkAreas(2), RainRate)

         IF (MAXVAL(RainRate) > 20.0) THEN
            RainRate12hour(:,:,ValidCounter) = RainRate
            ValidFilenames12hour(ValidCounter) = Filenames12hour(Counter)
            ValidCounter = ValidCounter + 1
         END IF
      END DO
      NumFilenames12hour = ValidCounter - 1
   ENDIF

   IF (AccumulationPeriod > 12) THEN
      ALLOCATE(RainRate18hour(NumColumns, NumRows, NumFilenames18hour), &
               ValidFilenames18hour(NumFilenames18hour), STAT=ErrorStatus)
      IF (ErrorStatus /= 0) THEN
         CALL Error_Messaging(RoutineName,"18 hour accum. allocate error",FATAL)
         STOP
      END IF
      ValidCounter = 1
      DO Counter = 1, NumFilenames18hour
         CALL Get_Trap_Rain_Rate(Filenames18hour(Counter), &
                                 NumRows, NumColumns, CenterLat, CenterLon, &
                                 McidasDataDirectory, &
                                 WorkAreas(1), WorkAreas(2), RainRate)

         IF (MAXVAL(RainRate) > 20.0) THEN
            RainRate18hour(:,:,ValidCounter) = RainRate
            ValidFilenames18hour(ValidCounter) = Filenames18hour(Counter)
            ValidCounter = ValidCounter + 1
         END IF
      END DO
      NumFilenames18hour = ValidCounter - 1
   ENDIF

   IF (AccumulationPeriod > 18) THEN
      ALLOCATE(RainRate24hour(NumColumns, NumRows, NumFilenames24hour), &
               ValidFilenames24hour(NumFilenames24hour), STAT=ErrorStatus)
      IF (ErrorStatus /= 0) THEN
         CALL Error_Messaging(RoutineName,"24 hour accum. allocate error",FATAL)
         STOP
      END IF
      ValidCounter = 1
      DO Counter = 1, NumFilenames24hour
         CALL Get_Trap_Rain_Rate(Filenames24hour(Counter), &
                                 NumRows, NumColumns, CenterLat, CenterLon, &
                                 McidasDataDirectory, &
                                 WorkAreas(1), WorkAreas(2), RainRate)

         IF (MAXVAL(RainRate) > 20.0) THEN
            RainRate24hour(:,:,ValidCounter) = RainRate
            ValidFilenames24hour(ValidCounter) = Filenames24hour(Counter)
            ValidCounter = ValidCounter + 1
         END IF
      END DO
      NumFilenames24hour = ValidCounter - 1
   ENDIF

   !delta_time = dtime(time_array)
   !print *," accumulation: ", delta_time

   !! the following IF blocks were moved after the ValidFilenames are got (they were in front of the ValidFilenames), 
   !! Change the Filenames6hour to ValidFilenames6hour, etc.  BY J.Yang 09/15/09

   !
   ! Count the number of independent TRaPs used to generate ensemble
   ! and keep a list of the unique file names (names without the forecast
   ! time).
   !
   ALLOCATE(FilenamesAll(NumFilenames6hour + NumFilenames12hour + &
                         NumFilenames18hour + NumFilenames24hour), &
            STAT=ErrorStatus)
   IF (ErrorStatus /= 0) THEN
      CALL Error_Messaging(RoutineName,"All filenames allocate error",FATAL)
      STOP
   END IF

   AllIndex = 0
   DO Counter = 1,NumFilenames6hour
      AllIndex = AllIndex + 1
      !StringLength = LEN_TRIM(Filenames6hour(Counter)) - 5
      !FilenamesAll(AllIndex) = Filenames6hour(Counter)(:StringLength)
      StringLength = LEN_TRIM(ValidFilenames6hour(Counter)) - 5
      FilenamesAll(AllIndex) = ValidFilenames6hour(Counter)(:StringLength)
   END DO
   DO Counter = 1,NumFilenames12hour
      AllIndex = AllIndex + 1
      StringLength = LEN_TRIM(ValidFilenames12hour(Counter)) - 5
      FilenamesAll(AllIndex) = ValidFilenames12hour(Counter)(:StringLength)
   END DO
   DO Counter = 1,NumFilenames18hour
      AllIndex = AllIndex + 1
      StringLength = LEN_TRIM(ValidFilenames18hour(Counter)) - 5
      FilenamesAll(AllIndex) = ValidFilenames18hour(Counter)(:StringLength)
   END DO
   DO Counter = 1,NumFilenames24hour
      AllIndex = AllIndex + 1
      StringLength = LEN_TRIM(ValidFilenames24hour(Counter)) - 5
      FilenamesAll(AllIndex) = ValidFilenames24hour(Counter)(:StringLength)
   END DO
   DO Counter = 1, AllIndex
      DO InnerCounter = (Counter + 1), AllIndex
         IF (TRIM(FilenamesAll(Counter)) == &
             TRIM(FilenamesAll(InnerCounter))) THEN
            FilenamesAll(Counter) = ""
         END IF
      END DO
   END DO

   !
   ! GENERATE ENSEMBLE.
   !
   ! Allocate arrays.
   !
   ALLOCATE(WeightSum(NumColumns, NumRows), &
            QPF_Sum(NumColumns, NumRows), &
            PQPF_Sum(NumColumns, NumRows, NumRainThresholds), &
            Mask(NumColumns, NumRows), &
            Weight(NumColumns, NumRows), &
            RainRateHistogram(2000), &
            CurrentHistogram(2000), STAT=ErrorStatus)
   IF (ErrorStatus /= 0) THEN
      CALL Error_Messaging(RoutineName,"Ensemble section allocate error",FATAL)
      STOP
   END IF
   WeightSum = 0.0
   QPF_Sum = 0.0
   PQPF_Sum = 0.0
   RainRateHistogram = 0.0

   !
   ! Calculate the number of combinations.
   !
   IF ( AccumulationPeriod == 6) THEN 
      NumCombinations =  NumFilenames6hour 
   ELSE IF (AccumulationPeriod == 12) THEN
      NumCombinations = NumFilenames6hour * NumFilenames12hour
   ELSE IF (AccumulationPeriod == 18) THEN
      NumCombinations = NumFilenames6hour * NumFilenames12hour &
                        * NumFilenames18hour
   ELSE IF (AccumulationPeriod == 24) THEN
      NumCombinations = NumFilenames6hour * NumFilenames12hour &
                        * NumFilenames18hour * NumFilenames24hour
   ELSE
      CALL Error_Messaging(RoutineName, &
                           "Invalid accumulation value encountered",FATAL)
      STOP

   END IF

   !
   ! If there are more than MaxCombinations ensemble members then randomly 
   ! choose MaxCombinations unique members to keep.
   !
   ALLOCATE(Keep(NumCombinations), STAT=ErrorStatus)
   IF (ErrorStatus /= 0) THEN
      CALL Error_Messaging(RoutineName,"MaxCombinations allocate error",FATAL)
      STOP
   END IF

   IF (NumCombinations <= MaxCombinations) THEN
      Keep = 1
   ELSE
      Keep = select_random(NumCombinations, MaxCombinations)
   END IF

   !
   ! Loop over forecasts, making an ensemble member out of each possible 
   ! combination of forecasts valid at times included in the accumulation
   !
   PRINT *, "Creating ensemble..."

   LoopCounter = 0

   !
   ! Change values of loop upper bound in case of accumulations < 24.
   !
   IF (AccumulationPeriod < 24) NumFilenames24hour = 1
   IF (AccumulationPeriod < 18) NumFilenames18hour = 1
   IF (AccumulationPeriod < 12) NumFilenames12hour = 1

   DO Count_6 = 1, NumFilenames6hour
      DO Count_12 = 1, NumFilenames12hour
         DO Count_18 = 1, NumFilenames18hour
            DO Count_24 = 1, NumFilenames24hour

               !
               ! Cull ensemble if necessary.
               !
               CombinationNumber = (Count_24 -1) * NumFilenames6hour &
                                                 * NumFilenames12hour &
                                                 * NumFilenames18hour &
                                 + (Count_18 -1) * NumFilenames6hour &
                                                 * NumFilenames12hour &
                                 + (Count_12 -1) * NumFilenames6hour &
                                 + Count_6

               IF (Keep(CombinationNumber) /= 1) CYCLE

               !
               ! Store a binary mask of good values (missing data assumed to 
               ! be negative).  Accumulate weights and rain
               !
               !
               ! 6 Hour data.
               !
               Weight6hour = 0.0
               CALL Get_Trap_Weights(ValidFilenames6hour(Count_6), WeightFcst, &
               WeightSensor, BiasSensor)
               Weight6hour = WeightFcst * WeightSensor
               DO Row = 1, NumRows
                  DO Column = 1, NumColumns
                     IF (RainRate6hour(Column,Row,Count_6) >= 0.0) THEN
                        Mask(Column,Row) = 1
                        Weight(Column,Row) = Weight6hour
                     ELSE
                        Mask(Column,Row) = 0
                        Weight(Column,Row) = 0
                     END IF
                     RainRate(Column,Row) = RainRate6hour(Column,Row,Count_6) &
                     * BiasSensor

                     !if (((Row-1)*800 + Column) == test_pixel) then
                     !print *,"rain rate = ",RainRate(Column,Row)
                     !end if
                  END DO
               END DO

               !
               ! 12 Hour data.
               !
               Weight12hour = 0
               IF (AccumulationPeriod > 6) THEN
                  CALL Get_Trap_Weights(ValidFilenames12hour(Count_12), &
                                        WeightFcst, WeightSensor, BiasSensor)
                  Weight12hour = WeightFcst * WeightSensor
                  DO Row = 1, NumRows
                     DO Column = 1, NumColumns
                        IF (RainRate12hour(Column,Row,Count_12) >= 0.0 .AND. &
                                         Mask(Column,Row) == 1) THEN
                           Weight(Column,Row) = Weight(Column,Row) &
                                              + Weight12hour 
                        ELSE
                           Mask(Column,Row) = 0
                        END IF
                        RainRate(Column,Row) = RainRate(Column,Row) &
                                    + (RainRate12hour(Column,Row,Count_12) &
                                    * BiasSensor)
                     END DO
                  END DO
               END IF

               !
               ! 18 Hour data.
               !
               Weight18hour = 0
               IF (AccumulationPeriod > 12) THEN
                  CALL Get_Trap_Weights(ValidFilenames18hour(Count_18), &
                                        WeightFcst, WeightSensor, BiasSensor)
                  Weight18hour = WeightFcst * WeightSensor
                  DO Row = 1, NumRows
                     DO Column = 1, NumColumns
                        IF (RainRate18hour(Column,Row,Count_18) >= 0.0 .AND. &
                                         Mask(Column,Row) == 1) THEN
                           Weight(Column,Row) = Weight(Column,Row) &
                                              + Weight18hour 
                        ELSE
                           Mask(Column,Row) = 0
                        END IF
                        RainRate(Column,Row) = RainRate(Column,Row) &
                             + (RainRate18hour(Column,Row,Count_18) &
                             * BiasSensor)
                     END DO
                  END DO
               END IF

               !
               ! 24 Hour data.
               !
               Weight24hour = 0
               IF (AccumulationPeriod > 18) THEN
                  CALL Get_Trap_Weights(ValidFilenames24hour(Count_24), &
                                        WeightFcst, WeightSensor, BiasSensor)
                  Weight24hour = WeightFcst * WeightSensor
                  DO Row = 1, NumRows
                     DO Column = 1, NumColumns
                        IF (RainRate24hour(Column,Row,Count_24) >= 0.0 .AND. &
                                         Mask(Column,Row) == 1) THEN
                           Weight(Column,Row) = Weight(Column,Row) &
                                              + Weight24hour 
                        ELSE
                           Mask(Column,Row) = 0
                        END IF
                        RainRate(Column,Row) = RainRate(Column,Row) &
                             + (RainRate24hour(Column,Row,Count_24) &
                             * BiasSensor)
                     END DO
                  END DO
               END IF

               !
               ! Accumulate qpf values and histogram of rainrate.
               !
               CurrentHistogram = 0
               DO Row = 1, NumRows
                  DO Column = 1, NumColumns
                     IF (Mask(Column,Row) == 1) THEN
                        WeightSum(Column, Row) = WeightSum(Column, Row) &
                                               + Weight(Column,Row)
                        QPF_Sum(Column,Row) = QPF_Sum(Column,Row) &
                                            + Weight(Column,Row) &
                                            * RainRate(Column,Row)
                     END IF
 
                     HistogramBin = INT(RainRate(Column, Row))
                     IF (HistogramBin >= 1 .AND. HistogramBin <= 2000) THEN
                        CurrentHistogram(HistogramBin) = &
                                        CurrentHistogram(HistogramBin) + 1
                     END IF
                     !if (((Row-1)*800 + Column) == test_pixel) then
                     !   print *, "Mask = ",Mask(Column,Row)
                     !   print *, "Weight = ",Weight(Column,Row)
                     !   print *, "WeightSum = ",WeightSum(Column,Row)
                     !   print *, "RainRate = ",RainRate(Column,Row)
                     !   print *, "HistogramBin = ",HistogramBin
                     !!   print *, "QPF_Sum = ",QPF_Sum(Column,Row)
                     !endif
                  END DO
               END DO

               !
               ! Accumulate threshold data.
               !
               DO Threshold = 1, NumRainThresholds
                  DO Row = 1, NumRows
                     DO Column = 1, NumColumns
                        IF (RainRate(Column,Row) >= &
                            RainThresholds(Threshold)) THEN
                           PQPF_Sum(Column,Row,Threshold) = &
                                      PQPF_Sum(Column,Row,Threshold) &
                                      + Weight(Column,Row)
                        END IF
                        !if (((Row-1)*800 + Column) == test_pixel) then
                        !   print *, "thresh = ",Threshold
                        !   print *, "Weight = ",Weight(Column,Row)
                        !   print *, "WeightSum = ",WeightSum(Column,Row)
                        !   print *, "RainRate = ",RainRate(Column,Row)
                        !   print *, "PQPF_Sum = ",PQPF_Sum(Column,Row,Threshold)
                        !endif
                     END DO
                  END DO
               END DO

               !
               ! Calulate histogram here
               !
               RainRateHistogram = RainRateHistogram + &
                                   (MAXVAL(Weight)*CurrentHistogram)
 
               LoopCounter = LoopCounter + 1


            END DO
         END DO
      END DO
   END DO

   !
   ! Now get weighted ensemble mean QPF, PQPF, and rain distribution.
   !

   ALLOCATE(QPF_Average(NumColumns,NumRows), &
            QPF_AverageNonzeroValue(NumColumns*NumRows), &
            QPF_AverageNonzeroLocation(2, NumColumns*NumRows), &
            QPF_AverageByte(NumColumns,NumRows), STAT=ErrorStatus)
   IF (ErrorStatus /= 0) THEN
      CALL Error_Messaging(RoutineName,"QPF allocate error",FATAL)
      STOP
   END IF

   QPF_AverageNonzeroCount = 0
   DO Row = 1, NumRows
      DO Column = 1, NumColumns
         QPF_Average(Column,Row) = QPF_Sum(Column,Row) &
                                 / WeightSum(Column,Row) 
         IF (QPF_Average(Column,Row) >= 1.0 ) THEN
            QPF_AverageNonzeroCount = QPF_AverageNonzeroCount + 1
            QPF_AverageNonzeroValue(QPF_AverageNonzeroCount) = &
                                     QPF_Average(Column, Row)
            QPF_AverageNonzeroLocation(1,QPF_AverageNonzeroCount) = &
                                     Column
            QPF_AverageNonzeroLocation(2,QPF_AverageNonzeroCount) = &
                                     Row
         END IF

         !if (((Row-1)*800 + Column) == test_pixel) then
         !   print *, (Row-1)*800 + Column, "=", test_pixel
         !   print *, "QPF_Average = ",QPF_Average(Column,Row)
         !   print *, "scaled aver. = ",(QPF_Average(Column,Row)*255.0/(40.0*25.4))
         !   print *, "nint(scaled avg) = ",NINT((QPF_Average(Column,Row)*255.0/(40.0*25.4)))
         !endif
      END DO
   END DO

   !delta_time = dtime(time_array)
   !print *," qpf calculation: ", delta_time

   ALLOCATE(PQPF(NumColumns,NumRows,NumRainThresholds), &
            PQPF_Byte(NumColumns,NumRows,NumRainThresholds), STAT=ErrorStatus)
   IF (ErrorStatus /= 0) THEN
      CALL Error_Messaging(RoutineName,"P-QPF allocate error",FATAL)
      STOP
   END IF

   DO Threshold = 1, NumRainThresholds
      DO Row = 1, NumRows
         DO Column = 1, NumColumns
            PQPF(Column,Row,Threshold) = PQPF_Sum(Column,Row,Threshold) &
            / WeightSum(Column,Row) 
            !if (((Row-1)*800 + Column) == test_pixel) then
            !   print *, (Row-1)*800 + Column, "=", test_pixel
            !   print *, "pqpf_average = ",PQPF(Column,Row,Threshold)
            !   print *, "scaled aver. = ",(PQPF(Column,Row,Threshold)*100.0)
            !   print *, "nint(scaled avg) = ",NINT((PQPF(Column,Row,Threshold)*100.0))
            !endif
         END DO
      END DO
   END DO
   !delta_time = dtime(time_array)
   !print *," PQPF calculation: ", delta_time

   RainRateHistogram = RainRateHistogram / WeightSum(1,1)

   !
   ! Probability matching to transform ensemble mean rain rates.
   !

   CALL Rain_Distribution(RainRateHistogram, RainRateDistribution)

   ALLOCATE(PM_QPF(NumColumns,NumRows), &
            PM_QPF_Byte(NumColumns,NumRows), &
            QPF_NonzeroValue(QPF_AverageNonzeroCount), STAT=ErrorStatus)
   IF (ErrorStatus /= 0) THEN
      CALL Error_Messaging(RoutineName,"PM-QPF allocate error",FATAL)
      STOP
   END IF

   QPF_NonzeroValue = QPF_AverageNonzeroValue(1:QPF_AverageNonzeroCount)
   PM_QPF = 0.0
   Counter = 1
   PRINT *,"Probability matching..."
   DO WHILE (Counter <= SIZE(RainRateDistribution))

      MaxLocation = MAXLOC(QPF_NonzeroValue)
      NonzeroColumn = QPF_AverageNonzeroLocation(1, MaxLocation(1))
      NonzeroRow = QPF_AverageNonzeroLocation(2, MaxLocation(1))

      PM_QPF(NonzeroColumn, NonzeroRow) = RainRateDistribution(Counter)
      QPF_NonzeroValue(Maxlocation) = 0
      Counter = Counter + 1

   END DO

   !delta_time = dtime(time_array)
   !print *," PM_QPF calculation: ", delta_time
      


   !
   ! Prepare to write the results to output files.
   !
   !
   ! Define the scale factore based on the accumulation time.
   !
   IF (AccumulationPeriod == 24) THEN
      EtrapFcstString = "24"
      FcstLeadTimeString = "24HRS"
      QpfScale = 40.0
   ELSE
      WRITE(EtrapFcstString,"(i2.2)") (LeadTime - AccumulationPeriod) 
      WRITE(FcstLeadTimeString,FMT="(a,a,i2.2,a)") &
            TRIM(EtrapFcstString),"-",LeadTime,"HR"
      QpfScale = 20.0
   END IF

   !
   ! Open the master file list and read the first few lines to get the 
   ! base date, time, and J-date.
   !
   LUN = Get_lun()
   OPEN(UNIT=LUN, FILE=MasterFileList, FORM="FORMATTED", ACTION="READ", &
        STATUS="OLD", IOSTAT=ErrorStatus)
   IF (ErrorStatus /= 0) THEN
      ErrorString = "Unable to open file: "//TRIM(MasterFileList)
      CALL Error_Messaging(RoutineName, ErrorString, FATAL)
      STOP
   END IF

   READ(UNIT=LUN,FMT=*,IOSTAT=ErrorStatus) DummyInteger
   READ(UNIT=LUN,FMT=*,IOSTAT=ErrorStatus) DummyInteger, &
                                           BaseDate, BaseTime, BaseJDate
   CLOSE(UNIT=LUN)

   !
   ! Convert serveral numeric values into character strings.
   !
   WRITE(BaseDateString,"(i8)") BaseDate
   WRITE(BaseTimeString,"(i4.4)") BaseTime
   YearString = BaseDateString(1:4)
   DateTimeString = BaseDateString(5:8)//TRIM(BaseTimeString)
   WRITE(RainThresholdsString(1),"(i4)") RainThresholds(1)
   WRITE(RainThresholdsString(2),"(i4)") RainThresholds(2)
   WRITE(RainThresholdsString(3),"(i4)") RainThresholds(3)
   WRITE(RainThresholdsString(4),"(i4)") RainThresholds(4)
   StormDir = TRIM(OutputDirectory)//TRIM(YearString)//"/"//TRIM(StormName)

   !
   ! Write average QPF output. 
   !
   AvgQpfFilename = TRIM(StormDir)//"/"// &
                    TRIM(YearString)// &
                    TRIM(StormName)// &
                    ".avgqpf."// &
                    TRIM(DateTimeString)// &
                    "."// &
                    TRIM(EtrapFcstString)

   PRINT *,"avg qpf filename = ",TRIM(AvgQpfFilename)

   !
   ! Convert to byte and write to output file.
   !
   QPF_AverageByte = NINT((QPF_Average * 255.0 / (QpfScale * 25.4)), BYTE)

   !McidasMemo = "Ensemble TRaP simple mean QPF   "
   McidasMemo = TRIM(StormName)//":"//TRIM(FcstLeadTimeString)//" Simple mean QPF "

   CALL Write_ETrap_Data(AvgQpfFilename, QPF_AverageByte, BaseJdate, &
                         BaseTime, McidasMemo)

   !
   !  Copy the AvgQpfFilename to some AREA number for the GIF file next step
   !
   Counter = 1
   Command = "cp -p "//TRIM(AvgQpfFilename)//" "//ProdAreaNames(Counter)
   CALL SYSTEM(TRIM(Command))

! call the Perl program to save the GIF files 03/03

   !
   ! Write probability matched QPF output.
   !
   PmQpfFilename = TRIM(StormDir)//"/"// &
                   TRIM(YearString)// &
                   TRIM(StormName)// &
                   ".pmqpf."// &
                   TRIM(DateTimeString)// &
                   "."// &
                   TRIM(EtrapFcstString)

   PRINT *," pm qpf filename = ",TRIM(PmQpfFilename)

   !
   ! Convert to byte and write to output file.
   !
   PM_QPF_Byte = NINT((PM_QPF * 255.0 / (QpfScale * 25.4)), BYTE)
   !McidasMemo = "Ensemble TRaP prob-matched QPF  "
   McidasMemo = TRIM(StormName)//":"//TRIM(FcstLeadTimeString)//" Prob-matched QPF "

   CALL Write_ETrap_Data(PmQpfFilename, PM_QPF_Byte, BaseJdate, &
                         BaseTime, McidasMemo)

   !
   !  Copy the PmQpfFilename to some AREA number for the GIF file
   !
   Counter = Counter + 1
   Command = "cp -p "//TRIM(PmQpfFilename)//" "//ProdAreaNames(Counter)
   CALL SYSTEM(TRIM(Command))

   ! 
   ! Apply calibration to PoPs (S. Kidder,  2013-07-22)
   !
   DO Threshold = 1, NumRainThresholds
      CALL Calibrate_PoP(AccumulationPeriod, RainThresholds(Threshold), PQPF(:,:,Threshold))
   END DO

   !
   ! Write threshold probability QPF output.
   !
   ! Convert to byte.
   !
   PQPF_Byte = NINT((PQPF * 100.0), BYTE)
   DO Threshold = 1, NumRainThresholds
      PQpfFilename(Threshold) = TRIM(StormDir)//"/"// &
                     TRIM(YearString)// &
                     TRIM(StormName)// &
                     ".p"// &
                     TRIM(ADJUSTL(RainThresholdsString(Threshold)))// &
                     "."// &
                     TRIM(DateTimeString)// &
                     "."// &
                     TRIM(EtrapFcstString)

      PRINT *,"  p qpf filename = ",TRIM(PQpfFilename(Threshold))
      !McidasMemo = "Ensemble TRaP prob ge "// &
      !              TRIM(ADJUSTL(RainThresholdsString(Threshold)))//" mm"
      McidasMemo = TRIM(StormName)//":"//TRIM(FcstLeadTimeString)//" Prob >= "// &
                    TRIM(ADJUSTL(RainThresholdsString(Threshold)))//" mm "

      CALL Write_ETrap_Data(PQpfFilename(Threshold), PQPF_Byte(:,:,Threshold), &
                            BaseJdate, BaseTime, McidasMemo)
   
      !
      !  Copy the PQpfFilename to some AREA number for the GIF file
      !
      Counter = Counter + 1
      Command = "cp -p "//TRIM(PQpfFilename(Threshold))//" "//ProdAreaNames(Counter)
      !print *, " Copy command = ", Command
      CALL SYSTEM(TRIM(Command))

   END DO

   
   !
   !  Write the product file names
   !
   pdListFilename = TRIM(McidasDataDirectory)//TRIM(pdList)//"."//TRIM(EtrapFcstString)//".txt"

   LUN = Get_lun()
   OPEN(UNIT=LUN, FILE=pdListFilename, FORM="FORMATTED", STATUS="REPLACE", &
        IOSTAT=ErrorStatus)
   IF (ErrorStatus /= 0) THEN
      ErrorString = "Unable to open file: "//TRIM(pdListFilename)
      CALL Error_Messaging(RoutineName, ErrorString, FATAL)
      STOP
   END IF
   ! write the list
   WRITE(UNIT=LUN,FMT="(a)") TRIM(AvgQpfFilename)
   WRITE(UNIT=LUN,FMT="(a)") TRIM(pmQpfFilename)
   DO Threshold = 1, NumRainThresholds
      WRITE(UNIT=LUN,FMT="(a)") TRIM(PQpfFilename(Threshold))
   END DO

   CLOSE(UNIT=LUN)  

   !
   !  Write the created eTRaP AREA file list
   !
   pdAreaListFilename = TRIM(McidasDataDirectory)//TRIM(pdAreaList)
   LUN = Get_lun()
   OPEN(UNIT=LUN, FILE=pdAreaListFilename, FORM="FORMATTED", STATUS="UNKNOWN", &
        POSITION="APPEND", IOSTAT=ErrorStatus)
!        IOSTAT=ErrorStatus)
   IF (ErrorStatus /= 0) THEN
      ErrorString = "Unable to open file: "//TRIM(pdAreaListFilename)
      CALL Error_Messaging(RoutineName, ErrorString, FATAL)
      STOP
   END IF
   ! write the list
   !WRITE(UNIT=LUN,FMT="(a,a,i4.4,a,i)") TRIM(EtrapFcstString)," ",ProdAreas(1)," ",SIZE(ProdAreas)
   WRITE(UNIT=LUN,FMT="(a,a,i4.4,a,i2.2)") TRIM(EtrapFcstString)," ",ProdAreas(1)," ",SIZE(ProdAreas)
   !print *, "Prodarea line:", TRIM(EtrapFcstString)," ",ProdAreas(1)," ",SIZE(ProdAreas)
   CLOSE(UNIT=LUN)


   !
   !  Save the GIF files from the AREA files "ProdAreaNames"
   !
   !!WRITE(ProdAreaStr,FMT="(i4.4)") ProdAreas(1)
   !WRITE(ProdAreaStr,FMT="(i)") ProdAreas(1)
   !WRITE(counterStr, FMT="(i)") SIZE(ProdAreas)
   WRITE(ProdAreaStr,FMT="(i4.4)") ProdAreas(1)
   WRITE(counterStr, FMT="(i1.1)") SIZE(ProdAreas)
   WRITE(LeadTimeStr,FMT="(i2.2)") LeadTime
   PRINT *, "ProdAreaStr = ", ProdAreaStr
   PRINT *, "counterStr = ", counterStr
   !Command = TRIM(McidasDataDirectory)//TRIM(saveGifCmd)//" "//TRIM(McidasDataDirectory)//" "//TRIM(ProdAreaStr)//" "// &
   !           TRIM(counterStr)//" "//TRIM(gifPre)//" "//TRIM(LeadTimeStr)//" > "//TRIM(McidasDataDirectory)//"saveGif.log"
   Command = TRIM(McidasDataDirectory)//TRIM(saveGifCmd)//" "//TRIM(McidasDataDirectory)//" "//TRIM(ProdAreaStr)//" "// &
              TRIM(counterStr)//" "//TRIM(gifPre)//" "//TRIM(EtrapFcstString)//" > "//TRIM(McidasDataDirectory)//"saveGif.log"
   print *, " Create GIF command = ", Command
   CALL SYSTEM(TRIM(Command))

   ! Save the GIF with the long name
   Counter = 1
   !WRITE(counterStr, FMT="(i)") Counter
   WRITE(counterStr, FMT="(i1.1)") Counter
   Command = "mv "//TRIM(McidasDataDirectory)//TRIM(gifPre)//TRIM(counterStr)//".GIF "//TRIM(AvgQpfFilename)//".GIF"
   !print *, " Copy Avg QPF GIF command = ", Command
   CALL SYSTEM(TRIM(Command))
  
   Counter = Counter + 1
   !WRITE(counterStr, FMT="(i)") Counter
   WRITE(counterStr, FMT="(i1.1)") Counter
   Command = "mv "//TRIM(McidasDataDirectory)//TRIM(gifPre)//TRIM(counterStr)//".GIF "//TRIM(PmQpfFilename)//".GIF"
   !print *, " Copy PM QPF GIF command = ", Command
   CALL SYSTEM(TRIM(Command))

   DO Threshold = 1, NumRainThresholds
      Counter = 2 + Threshold
      WRITE(counterStr, FMT="(i1.1)") Counter
      Command = "mv "//TRIM(McidasDataDirectory)//TRIM(gifPre)//TRIM(counterStr)//".GIF "//TRIM(PQpfFilename(Threshold))//".GIF"
      !print *, " Copy Probability GIF command = ", Command
      CALL SYSTEM(TRIM(Command))
   END DO

   !
   ! Write log file information.
   !
   LogFilename = TRIM(StormDir)//"/"// &
                 TRIM(YearString)// &
                 TRIM(StormName)// &
                 ".ETRAP."// &
                 TRIM(DateTimeString)// &
                 "."// &
                 TRIM(EtrapFcstString)// &
                 ".log"

   PRINT *,"  log filename = ",TRIM(LogFilename)

   LUN = Get_lun()
   OPEN(UNIT=LUN, FILE=LogFilename, FORM="FORMATTED", STATUS="REPLACE", &
        IOSTAT=ErrorStatus)
   IF (ErrorStatus /= 0) THEN
      ErrorString = "Unable to open file: "//TRIM(LogFilename)
      CALL Error_Messaging(RoutineName, ErrorString, FATAL)
      STOP
   END IF

! work up to here, 09/14 J.Yang

   NumTRaPSets = 0
   DO Counter = 1, AllIndex
      IF (LEN_TRIM(FilenamesAll(Counter)) > 0) THEN
         WRITE(UNIT=LUN,FMT="(a)") TRIM(FilenamesAll(Counter))
         NumTRaPSets = NumTRaPSets + 1
      END IF
   END DO

   ! Write the individual TRaP member file name for 6 hours period
   IF (AccumulationPeriod == 6) THEN
      REWIND(UNIT=LUN,IOSTAT=ErrorStatus)
      IF (ErrorStatus /= 0) THEN
         ErrorString = "Unable to rewind file: "//TRIM(LogFilename)
         CALL Error_Messaging(RoutineName, ErrorString, FATAL)
         !STOP
      END IF

      WRITE(UNIT=LUN,FMT="(i4,a,i4,a,i4,a)") LeadTime - 6, " -", LeadTime, &
                         " hour files: total ",NumFilenames6hour, " TRaP members."
!      DO Counter = 1, NumFilenames6hour
!         WRITE(UNIT=LUN,FMT="(a)") TRIM(Filenames6hour(Counter))
!      END DO

!      WRITE(UNIT=LUN,FMT="(a)") "Valid TRaP members:"
      DO Counter = 1, NumFilenames6hour
         WRITE(UNIT=LUN,FMT="(a)") TRIM(ValidFilenames6hour(Counter))
      END DO

   END IF

   WRITE(UNIT=LUN,FMT="(a,i8,i8,f8.2,f8.2)") &
                       "Grid information: nc,nr,clat,clon: ", &
   NumColumns, NumRows, CenterLat, CenterLon

   !
   ! Write the TRaP members counts to the log file
   !

   WRITE(UNIT=LUN,FMT="(a,i4)") "Total independent TRaPs used:", NumTRaPSets
   PRINT *, "Total independent TRaPs used:", NumTRaPSets
   IF (AccumulationPeriod == 24) THEN
      WRITE(UNIT=LUN,FMT="(a,i4,a,i4,a,i4,a,i4,a,i4,a,i8)") "Ensemble members before cull to ", &
                          MaxCombinations, " : ", NumFilenames6hour, " x", & 
                          NumFilenames12hour, " x", NumFilenames18hour, " x", &
                          NumFilenames24hour, " =", NumCombinations 
      PRINT *,"Ensemble members before cull to ", MaxCombinations, " : ", NumFilenames6hour, " x", &
                          NumFilenames12hour, " x", NumFilenames18hour, " x", &
                          NumFilenames24hour, " =", NumCombinations
   END IF
   WRITE(UNIT=LUN,FMT="(a,i4)") "Total Ensemble TRaP members:", MIN(NumCombinations, MaxCombinations)

   CLOSE(UNIT=LUN)

   PRINT *, "Total TRaP members:", MIN(NumCombinations, MaxCombinations)
 
   !delta_time = dtime(time_array)
   !print *," output: ", delta_time

END PROGRAM etrap
