!-------------------------------------------------------------------------------
! NAME:
!   Etrap_Utils
!
! FUNCTION:
!   Module containing the utility routines used by etrap.f90.
!
! DESCRIPTION:
!   This module contains the following routines used by etrap.f90.
!   1. FUNCTION Select_Random
!   2. SUBROUTINE Select_Traps
!   3. SUBROUTINE Get_Trap_Rain_Rate
!   4. SUBROUTINE Get_Trap_Weights
!   5. SUBROUTINE Write_ETrap_Data
!   6. SUBROUTINE Rain_Distribution
!   7. SUBROUTINE Calibrate_PoP
!
! REFERENCE:
!
! CALLING SEQUENCE:
!   use etrap_utils
!
! INPUTS:
!
! OUTPUTS:
!
! DEPENDENCIES:
!   type_kinds module
!   error_messaging_module module
!   Get_lun() function
!
! RESTRICTIONS:
!
! HISTORY:
!   2008-05-05  P. Keehn     Initial development
!   2008-11-18  J. Yang      Modified to run on SSD
!   2009-05-14/21  J. Yang   Modified the weights for sensors and latencies
!   2011-08-30  R. Glassberg Modified gating of files in CASE block to support
!                            more frequent eTRaP execution to catch late TRaPs
!   2013-07-22  S. Kidder    Added Subroutine Calibrate_PoP 
!
!-------------------------------------------------------------------------------
MODULE Etrap_Utils

   USE type_kinds
   USE error_messaging_module

   IMPLICIT NONE

   !
   ! Global variables.
   !
   !CHARACTER(LEN=100) :: MasterFileList = "etrap_master_list.out"

   INTEGER(LONG), DIMENSION(4), PARAMETER :: ValidForecastTimes = &
                                              (/6, 12, 18, 24/)

   !REAL(SINGLE), DIMENSION(4), PARAMETER :: ValidForecastWeights = &
   !                                           (/1.0, 0.7, 0.4, 0.1/)
   REAL(SINGLE), DIMENSION(4), PARAMETER :: ValidForecastWeights = &
                                              (/1.0, 0.85, 0.70, 0.55/)

   CHARACTER(LEN=10), DIMENSION(17), PARAMETER :: ValidSensorNames = &
                                              (/"RRH_GPM   ", &
                                                "AMSUNM    ", &
                                                "AMSUNN    ", &
                                                "AMSUNP    ", &
                                                "AMSUM1    ", &
                                                "AMSUM2    ", &
                                                "AMSU      ", &
                                                "SSMI      ", &
                                                "SSMISA    ", &
                                                "SSMISB    ", &
                                                "SSMISC    ", &
                                                "SSMIS     ", &
                                                "AMSRE     ", &
                                                "GHEHR     ", &
                                                "RAIN      ", &
                                                "RRH_AMSR  ", &   
                                                "RCLIPER   "/)

!   CHARACTER(LEN=10), DIMENSION(16), PARAMETER :: ValidSensorNames = &
!                                              (/"AMSUNM    ", &
!                                                "AMSUNN    ", &
!                                                "AMSUNP    ", &
!                                                "AMSUM1    ", &
!                                                "AMSUM2    ", &
!                                                "AMSU      ", &
!                                                "SSMI      ", &
!                                                "SSMISA    ", &
!                                                "SSMISB    ", &
!                                                "SSMISC    ", &
!                                                "SSMIS     ", &
!                                                "AMSRE     ", &
!                                                "GHEHR     ", &
!                                                "RAIN      ", &
!                                                "RRH_AMSR  ", &   
!                                                "RCLIPER   "/)

   CHARACTER(LEN=100) :: MasterFileList

   !REAL(SINGLE), DIMENSION(6), PARAMETER :: ValidSensorWeights = &
   !                                         (/1.0, 1.3, 0.7, 1.0, 0.4, 1.0/)
   REAL(SINGLE), DIMENSION(17), PARAMETER :: ValidSensorWeights = &
                                            (/1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0/)

   REAL(SINGLE), DIMENSION(17), PARAMETER :: ValidSensorBiases = &
                                            (/1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.7, 1.0, 1.0, 1.8/)

!   REAL(SINGLE), DIMENSION(16), PARAMETER :: ValidSensorWeights = &
!                                            (/1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0/)

!   REAL(SINGLE), DIMENSION(16), PARAMETER :: ValidSensorBiases = &
!                                            (/1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.7, 1.0, 1.0, 1.8/)

   INTEGER(LONG), DIMENSION(320) :: McidasAreaHeader
   INTEGER(LONG), DIMENSION(64) :: McidasAreaHeader_gpm

   CONTAINS

   !----------------------------------------------------------------------------
   ! NAME:
   !   Select_Random
   !
   ! FUNCTION:
   !   Randomly select N pieces from M options.
   !
   ! DESCRIPTION:
   !   This routine randomly selects N elements of an array of size M.  Given
   !   an array of size M (NumSelections), randomly select N (MaxSelections)
   !   elements.  The output array of size NumSelections will contain 0's and 
   !   1's, where 1's are the radomly selected elements.
   !
   ! REFERENCE:
   !
   ! CALLING SEQUENCE:
   !   result = Select_Random(NumSelections, MaxSelections)
   !
   ! INPUTS:
   !   NumSelections    INTEGER 
   !   MaxSelections    INTEGER 
   !
   ! OUTPUTS:
   !   RandomSelections INTEGER,DIMENSION(NumSelections) 
   !
   ! DEPENDENCIES:
   !
   ! RESTRICTIONS:
   !
   ! HISTORY:
   !   2008-05-05  P. Keehn     Initial development
   !
   !----------------------------------------------------------------------------
   FUNCTION Select_Random(NumSelections, MaxSelections) RESULT(RandomSelections)

      INTEGER(LONG), INTENT(IN) :: NumSelections
      INTEGER(LONG), INTENT(IN) :: MaxSelections
      INTEGER(LONG), DIMENSION(NumSelections) :: RandomSelections

      CHARACTER(LEN=50) :: RoutineName = "Select_Random"
      INTEGER(LONG) :: NumRandomSelections
      INTEGER(LONG) :: RandomInteger
      INTEGER(LONG) :: RandomNumberAttempts
      INTEGER(LONG) :: MaxRandomNumberAttempts
      REAL(SINGLE) :: Harvest

      !
      ! Initialize some variables.
      !
      NumRandomSelections = 1
      RandomSelections = 0 
      MaxRandomNumberAttempts = MaxSelections*10
      RandomNumberAttempts = 0

      !
      ! Seed the random number generator.
      !
      CALL RANDOM_SEED()

      !
      ! Continue generating random numbers between 1 and NumSelections until
      ! a total of MaxSelections has been generated.
      !
      DO WHILE (NumRandomSelections <= MaxSelections)

         !
         ! Keep track of the number of random number attempts, and make sure
         ! we don't get stuck in some never ending loop.
         !
         RandomNumberAttempts = RandomNumberAttempts + 1
         IF (RandomNumberAttempts > MaxRandomNumberAttempts) THEN
            CALL Error_Messaging(RoutineName, &
                                 "Exceeded maximum iteration attempts",FATAL)
            STOP
         END IF

         !
         ! Generate a random number and convert it to an integer between 1 and 
         ! NumSelections.  Set that value to 1 in the RandomSelections array.  
         ! If it has already been selected, skip to the next random number.
         !
         CALL RANDOM_NUMBER(Harvest)
         RandomInteger = INT(Harvest*NumSelections) + 1
         IF (RandomSelections(RandomInteger) == 1) CYCLE
         RandomSelections(RandomInteger) = 1
         NumRandomSelections = NumRandomSelections + 1

      END DO

   END FUNCTION Select_Random


   !----------------------------------------------------------------------------
   ! NAME:
   !   Select_Traps
   !
   ! FUNCTION:
   !   Select the Trap files to use based on the lead time.
   !
   ! DESCRIPTION:
   !   This routine will look through the master file list and collect the 
   !   filenames that meet the given lead time criteria.
   !
   ! REFERENCE:
   !
   ! CALLING SEQUENCE:
   !   CALL Select_Traps (LeadTime, GoodFileNames, GoodFileCounter)
   !
   ! INPUTS:
   !   LeadTime        INTEGER
   !
   ! OUTPUTS:
   !   GoodFileNames   CHARACTER,DIMENSION(:),POINTER
   !   GoodFileCounter INTEGER                       
   !
   ! DEPENDENCIES:
   !
   ! RESTRICTIONS:
   !
   ! HISTORY:
   !   2008-05-05  P. Keehn     Initial development
   !
   !----------------------------------------------------------------------------
   SUBROUTINE Select_Traps(LeadTime, GoodFileNames, GoodFileCounter, MasterFileList)

      INTEGER(LONG), INTENT(IN) :: LeadTime
      CHARACTER(LEN=200), DIMENSION(:), POINTER :: GoodFileNames
      INTEGER(LONG), INTENT(OUT) :: GoodFileCounter

      CHARACTER(LEN=20) :: RoutineName = "Select_Traps"
      CHARACTER(LEN=100) :: ErrorMessage
      CHARACTER(LEN=200) :: TrapFilename
      CHARACTER(LEN=200), DIMENSION(:), POINTER :: Filenames
      CHARACTER(LEN=100) :: MasterFileList
      INTEGER(LONG) :: FileCounter
      INTEGER(LONG) :: ErrorStatus
      INTEGER(LONG) :: TotalFiles
      INTEGER(LONG) :: TotalSeconds
      INTEGER(LONG) :: TotalSecondsBase
      INTEGER(LONG) :: TotalSecondsFcst
      INTEGER(LONG) :: DeltaTime_24
      INTEGER(LONG) :: DeltaTime_18
      INTEGER(LONG) :: DeltaTime_12
      INTEGER(LONG) :: DeltaTime_06
      INTEGER(LONG) :: ADeltaTime_24
      INTEGER(LONG) :: ADeltaTime_18
      INTEGER(LONG) :: ADeltaTime_12
      INTEGER(LONG) :: ADeltaTime_06
      INTEGER(LONG) :: DeltaTime
      INTEGER(LONG) :: Counter
      INTEGER(LONG) :: Separator_1
      INTEGER(LONG) :: Separator_2
      INTEGER(LONG) :: Separator_3
      INTEGER(LONG) :: Separator_4
      INTEGER(LONG) :: Separator_5
      INTEGER(LONG) :: Separator_6
      INTEGER(LONG) :: Separator_7
      INTEGER(LONG) :: StringLength
      INTEGER(LONG) :: FileFcstTime
!     INTEGER(LONG) :: Close_Gate24
!     INTEGER(LONG) :: Close_Gate18
!     INTEGER(LONG) :: Close_Gate12
!     INTEGER(LONG) :: Close_Gate6
      INTEGER(LONG) :: Close_Gate
      INTEGER(LONG) :: LUN
      INTEGER(LONG), DIMENSION(:), POINTER :: FileTimes

      INTEGER(LONG) :: Get_lun

      !
      ! Open the master file list for reading.
      !
      !PRINT *,"In Select Traps"
!!$      Close_Gate24 = 0
!!$      Close_Gate18 = 0
!!$      Close_Gate12 = 0
!!$      Close_Gate6 = 0
      Close_Gate  = 0
      LUN = Get_lun()
      OPEN(UNIT=LUN, FILE=MasterFileList, FORM="FORMATTED", ACTION="READ", &
           STATUS="OLD", IOSTAT=ErrorStatus)
      IF (ErrorStatus /= 0) THEN
         ErrorMessage = "Unable to open file: "//TRIM(MasterFileList)
         CALL Error_Messaging(RoutineName, ErrorMessage, FATAL)
         STOP
      END IF

      !
      ! Read the first three lines, and allocate some space.
      !
      !PRINT *,"Before Reads"

      READ(UNIT=LUN,FMT=*) TotalFiles
      !PRINT *,"TotalFiles ", TotalFiles
      READ(UNIT=LUN,FMT=*) TotalSecondsBase
      !PRINT *,"TotalSecondsBase ", TotalSecondsBase
      READ(UNIT=LUN,FMT=*) TotalSecondsFcst
      !PRINT *,"TotalSecondsFcst ", TotalSecondsFcst
      !PRINT *,"After Reads"


      ALLOCATE(Filenames(TotalFiles), FileTimes(TotalFiles), STAT=ErrorStatus)
      IF (ErrorStatus /= 0) THEN
         ErrorMessage = "Unable to allocate memory"
         CALL Error_Messaging(RoutineName, ErrorMessage, FATAL)
         STOP
      END IF


      !
      ! Loop through the rest of the master file list.
      !
      FileCounter = 0
      DO Counter = 1, TotalFiles

         !
         ! Read the time offsets and the file name.
         !
         READ(UNIT=LUN,FMT=*) TotalSeconds, DeltaTime_24, &
                              DeltaTime_18, DeltaTime_12, &
                              DeltaTime_06, TrapFilename
         ADeltaTime_24 = ABS(DeltaTime_24)
         ADeltaTime_18 = ABS(DeltaTime_18)
         ADeltaTime_12 = ABS(DeltaTime_12)
         ADeltaTime_06 = ABS(DeltaTime_06)

         !
         ! Convert the forecast hour of the filename to an integer and add 6 to
         ! get the ending hour forecast time.
         !
         StringLength = LEN_TRIM(TrapFilename)
         READ(TrapFilename((StringLength-3):(StringLength-2)),FMT="(i2)") &
                                                          FileFcstTime
         FileFcstTime = FileFcstTime + 6

         !PRINT *,"LeadTime, FileFctTime ",LeadTime, FileFcstTime

         !
         ! Find the files that are within 3 hours of the given leadtime, and
         ! whose forecast time is greater then or equal to the leadtime.
         !
         !PRINT *,"LeadTime ", LeadTime
         Close_Gate = 0
         SELECT CASE (LeadTime)
            CASE(24)
               IF (ADeltaTime_24 <= 3*3600 .AND. &
               FileFcstTime >= LeadTime) THEN
                  IF (ADeltaTime_24 == 3*3600 .AND. Close_Gate == 1) THEN
                     PRINT *,"Preventing processing of adjacent boundary traps"
                  ELSE
                     FileCounter = FileCounter + 1
                     Filenames(FileCounter) = TrapFilename
                     FileTimes(FileCounter) = TotalSeconds
                     IF (ADeltaTime_24 == 3*3600) Close_Gate   = 1 
                  END IF
               END IF
            CASE(18)
               IF (ADeltaTime_18 <= 3*3600 .AND. &
               FileFcstTime >= LeadTime) THEN
                  IF (ADeltaTime_18 == 3*3600 .AND. Close_Gate == 1) THEN
                     PRINT *,"Preventing processing of adjacent boundary traps"
                  ELSE 
                     FileCounter = FileCounter + 1
                     Filenames(FileCounter) = TrapFilename
                     FileTimes(FileCounter) = TotalSeconds
                     IF (ADeltaTime_18 == 3*3600) Close_Gate   = 1 
                  END IF
               END IF
            CASE(12)
               IF (ADeltaTime_12 <= 3*3600 .AND. &
               FileFcstTime >= LeadTime) THEN
                  IF (ADeltaTime_12 == 3*3600 .AND. Close_Gate == 1) THEN
                     PRINT *,"Preventing processing of adjacent boundary traps"
                  ELSE
                     FileCounter = FileCounter + 1
                     Filenames(FileCounter) = TrapFilename
                     FileTimes(FileCounter) = TotalSeconds
                     IF (ADeltaTime_12 == 3*3600) Close_Gate   = 1 
                  END IF
               END IF
            CASE(6)
               IF (ADeltaTime_06 <= 3*3600 .AND. &
               FileFcstTime >= LeadTime) THEN
                  IF (ADeltaTime_06 == 3*3600 .AND. Close_Gate == 1) THEN
                     PRINT *,"Preventing processing of adjacent boundary traps"
                  ELSE
                     FileCounter = FileCounter + 1
                     Filenames(FileCounter) = TrapFilename
                     FileTimes(FileCounter) = TotalSeconds
                     IF (ADeltaTime_06 == 3*3600) Close_Gate  = 1 
                  END IF
               END IF
            CASE DEFAULT

         END SELECT

      END DO

      !
      ! Close the master file list.
      !
      CLOSE(UNIT=LUN)

      ALLOCATE (GoodFileNames(FileCounter), STAT=ErrorStatus)
      IF (ErrorStatus /= 0) THEN
         ErrorMessage = "Unable to allocate memory"
         CALL Error_Messaging(RoutineName, ErrorMessage, FATAL)
         STOP
      END IF

      !
      ! Make sure we are using the most recent trap file from the same 
      ! forecast centers.
      ! Sample file name: 2008DURGA.FKAU04.ADRM.230812.AMSU.04230323.00.Z
      ! Separator_1 to 7 are the locations of the periods (.) starting at the
      ! END of the filename (the right side).
      !
      GoodFileCounter = 0
      Separator_1 = INDEX(Filenames(1),".",.TRUE.) 
      Separator_2 = INDEX(Filenames(1)(:Separator_1-1),".",.TRUE.)
      Separator_3 = INDEX(Filenames(1)(:Separator_2-1),".",.TRUE.)
      Separator_4 = INDEX(Filenames(1)(:Separator_3-1),".",.TRUE.)
      Separator_5 = INDEX(Filenames(1)(:Separator_4-1),".",.TRUE.)
      Separator_6 = INDEX(Filenames(1)(:Separator_5-1),".",.TRUE.)
      Separator_7 = INDEX(Filenames(1)(:Separator_6-1),".",.TRUE.)

      !
      ! Since the filenames from the master trap list are already sorted 
      ! alpha-numerically (by run_etrap.sh) we can simply check each filename
      ! against the next file name to see if their center information matches.
      ! If the center and sensor are the same, and the time difference is 
      ! less then 20 minutes then don't include this file in the good list.
      ! 
      DO Counter = 1, FileCounter - 1
         DeltaTime = ABS(FileTimes(Counter) - FileTimes(Counter+1))
      !   PRINT *,"Filenames ",Filenames(Counter)
         IF ((FIlenames(Counter)(Separator_7:Separator_6) /= &
              Filenames(Counter+1)(Separator_7:Separator_6)) .OR. &
             (Filenames(Counter)(Separator_5:Separator_4) /= &
              Filenames(Counter+1)(Separator_5:Separator_4)) .OR. &
             (Filenames(Counter)(Separator_4:Separator_3) /= &
              Filenames(Counter+1)(Separator_4:Separator_3)) .OR. &
             (Filenames(Counter)(Separator_2:Separator_1) /= &
              Filenames(Counter+1)(Separator_2:Separator_1)) .OR. &
             (DeltaTime > 20*60)) THEN
            GoodFileCounter = GoodFileCounter + 1
            GoodFileNames(GoodFileCounter) = Filenames(Counter)
      !   PRINT *,"GoodFilenames ",GoodFilenames(GoodFileCounter)
      !   PRINT *,"GoodFilenames GFC",GoodFilenames(GoodFileCounter), & 
      !        GoodFileCounter
         END IF
      END DO
      !
      ! Now add the last file to the good list.
      !
      IF (GoodFileCounter > 0) THEN
         GoodFileCounter = GoodFileCounter + 1
         GoodFileNames(GoodFileCounter) = Filenames(FileCounter)
      !   PRINT *,"GoodFilenames GFC last",GoodFilenames(GoodFileCounter), & 
      !   GoodFileCounter
      END IF

      DEALLOCATE (Filenames)
      

   END SUBROUTINE Select_Traps


   !----------------------------------------------------------------------------
   ! NAME:
   !   Get_Trap_Rain_Rate
   !
   ! FUNCTION:
   !   Read the Trap data from a Mcidas file.
   !
   ! DESCRIPTION:
   !   This routine reads the Trap data from a Mcidas file.  It first copys the
   !   file to a Mcidas area, uncompresses the file, then calls the Mcidas
   !   imgcopy.k command to extract the desired portion of the data.
   !
   ! REFERENCE:
   !
   ! CALLING SEQUENCE:
   !   CALL Get_Trap_Rain_Rate(Filename, NumRows, NumColumns, CenterLat,
   !                           CenterLon, McidasDataDir, WorkArea1, 
   !                           WorkArea2, RainRate)
   !
   ! INPUTS:
   !   Filename        CHARACTER
   !   NumRows         INTEGER
   !   NumColumns      INTEGER
   !   CenterLat       REAL
   !   CenterLon       REAL
   !   McidasDataDir   CHARACTER
   !   WorkArea1       INTEGER
   !   WorkArea2       INTEGER
   !
   ! OUTPUTS:
   !   RainRate        REAL,DIMENSION(:,:)
   !
   ! DEPENDENCIES:
   !   imgcopy.k
   !
   ! RESTRICTIONS:
   !
   ! HISTORY:
   !   2008-05-05  P. Keehn     Initial development
   !   2008-07-30  P. Keehn     Added McidasDataDir to inputs.
   !
   !----------------------------------------------------------------------------
   SUBROUTINE Get_Trap_Rain_Rate(Filename, NumRows, NumColumns, CenterLat, &
                                 CenterLon, McidasDataDir, WorkArea1, &
                                 WorkArea2, RainRate)
   
      CHARACTER(LEN=200), INTENT(IN) :: Filename
      INTEGER(LONG), INTENT(IN) :: NumRows
      INTEGER(LONG), INTENT(IN) :: NumColumns
      REAL(SINGLE), INTENT(IN) :: CenterLat
      REAL(SINGLE), INTENT(IN) :: CenterLon
      CHARACTER(LEN=200), INTENT(IN) :: McidasDataDir
      INTEGER(LONG), INTENT(IN) :: WorkArea1
      INTEGER(LONG), INTENT(IN) :: WorkArea2
      REAL(SINGLE), DIMENSION(:,:), INTENT(OUT) :: RainRate

      CHARACTER(LEN=20) :: RoutineName = "Get_Trap_Rain_Rate"
      CHARACTER(LEN=200) :: WorkAreaName1
      CHARACTER(LEN=200) :: WorkAreaName2
      CHARACTER(LEN=200) :: Command
      CHARACTER(LEN=100) :: ErrorMessage
      INTEGER(LONG) :: Record
      INTEGER(LONG) :: Row
      INTEGER(LONG) :: Column
      INTEGER(LONG) :: LUN
      INTEGER(LONG) :: ErrorStatus
      INTEGER(BYTE) :: RainRateByte
      INTEGER(LONG) :: GPMsat
      INTEGER(LONG) :: RowLim
      INTEGER(LONG) :: ColLim

      INTEGER(LONG) :: Get_lun

      !
      ! Convert the mcidas work area numbers to strings that also contain the
      ! Mcidas data directory string.
      !
      !WRITE(WorkAreaName1,FMT="(a,i4.4)") "AREA",WorkArea1
      !WRITE(WorkAreaName2,FMT="(a,i4.4)") "AREA",WorkArea2
      WRITE(WorkAreaName1,FMT="(a,a,i4.4)") TRIM(McidasDataDir),"AREA",WorkArea1
      WRITE(WorkAreaName2,FMT="(a,a,i4.4)") TRIM(McidasDataDir),"AREA",WorkArea2

      !
      ! Delete the existing workarea1, workarea1.Z, and workarea2 files to
      ! make sure we don't read an old file in the event the 
      ! following system commands are unsuccessful.
      !
      Command = "rm -f "//TRIM(WorkAreaName1)//" "// &
                          TRIM(WorkAreaName1)//".Z "// &
                          TRIM(WorkAreaName2)
      CALL SYSTEM(TRIM(Command))
      GPMsat = INDEX(TRIM(Filename),'GPM')
      !
      ! Copy the Trap file to the mcidas work area (with a .Z extension).
      !
      Command = "cp "//TRIM(Filename)//" "//TRIM(WorkAreaName1)//".Z"
      CALL SYSTEM(TRIM(Command))

      !
      ! Uncompress the mcidas work area.
      ! NOTE: some Linux machines do not have the compress/uncompress command,
      ! so use gzip/gunzip instead.
      ! 
      !Command = "uncompress -f "//TRIM(WorkAreaName1)//".Z"
      Command = "gunzip -f "//TRIM(WorkAreaName1)//".Z"
      CALL SYSTEM(TRIM(Command))

      !
      ! Use the mcidas imgcopy command to copy the desired portion of the Trap
      ! data into the other mcidas working area.
      !
      WRITE (Command, FMT="(a,i4.4,a,i4.4,a,f8.2,a,f8.2,a,i6,a,i6,a)") &
         "imgcopy.k A/A.",WorkArea1," A/A.",WorkArea2," PRO=MERC LAT=", &
         CenterLat," ",CenterLon," SIZE=",NumRows," ",NumColumns," RES=4"
      CALL SYSTEM(TRIM(Command))

      !
      ! Open the mcidas file and get the header information.
      ! We are assuming the header size is no more than 1280 bytes.
      !
      LUN = Get_lun()
      OPEN(UNIT=LUN, FILE=WorkAreaName2, FORM="UNFORMATTED", ACCESS="DIRECT", &
           RECL=1280, STATUS="OLD", ACTION="READ", IOSTAT=ErrorStatus)
      IF (ErrorStatus /= 0) THEN
         ErrorMessage = "Unable to open file: "//TRIM(WorkAreaName2)
         CALL Error_Messaging(RoutineName, ErrorMessage, FATAL)
         STOP
      END IF
      IF (GPMsat == 0) THEN
         READ(UNIT=LUN,REC=1) McidasAreaHeader
      ELSE
         READ(UNIT=LUN,REC=1) McidasAreaHeader_gpm
      END IF
      CLOSE(UNIT=LUN)

      !
      ! Open the mcidas file again and read the data.
      ! Word 34 of the area header contains the byte offset to the data record.
      ! Words 9 and 10 are the row and column sizes.
      !
      LUN = Get_lun()
      OPEN(UNIT=LUN, FILE=WorkAreaName2, FORM="UNFORMATTED", ACCESS="DIRECT", &
           RECL=1, STATUS="OLD", ACTION="READ", IOSTAT=ErrorStatus)
      IF (ErrorStatus /= 0) THEN
         ErrorMessage = "Unable to open file: "//TRIM(WorkAreaName2)
         CALL Error_Messaging(RoutineName, ErrorMessage, FATAL)
         STOP
      END IF

      IF (GPMsat == 0) THEN
         Record = McidasAreaHeader(34)
      ELSE
         Record = McidasAreaHeader_gpm(34)
      END IF

!!$      IF (Record > 10000) THEN
!!$         ErrorMessage = "AREA file has incorrect byte order for this system"
!!$         CALL Error_Messaging(RoutineName, ErrorMessage, FATAL)
!!$         STOP
!!$      END IF
      
      IF (GPMsat == 0) THEN
         RowLim = McidasAreaHeader(9)
         ColLim = McidasAreaHeader(10)
      ELSE
         RowLim = McidasAreaHeader_gpm(9)
         ColLim = McidasAreaHeader_gpm(10)
      END IF
     DO Row = 1, RowLim
         DO Column = 1, ColLim
            Record = Record + 1
            READ(UNIT=LUN,REC=Record) RainRateByte

            !
            ! The 1-byte data in the mcidas file is in the range 0-255.
            ! Fortran interprets these as signed bytes with values 
            ! between -128 and +127.
            ! We need to add 256 to the data values that are negative and then
            ! convert to rain values.
            ! Rainfall is scaled (20 for 6 hour accumulations) and converted 
            ! from inches to mm.
            !
            IF (RainRateByte < 0) THEN
               RainRate(Column, Row) = & 
                       ((REAL(RainRateByte+256))/255.0) * 20.0 * 25.4
!               PRINT *,"RainRate 1 ",RainRate(Column,Row),Column,Row
            ELSE
               RainRate(Column, Row) = & 
                       (REAL(RainRateByte)/255.0) * 20.0 * 25.4
!               PRINT *,"RainRate 2 ",RainRate(Column,Row),Column,Row
            END IF

         END DO
      END DO

      !
      ! Close input file.
      !
      CLOSE(UNIT=LUN)

   END SUBROUTINE Get_Trap_Rain_Rate


   !----------------------------------------------------------------------------
   ! NAME:
   !   Get_Trap_Weights
   !
   ! FUNCTION:
   !   Get the forecast weight, sensor weight, and sensor bias.
   !
   ! DESCRIPTION:
   !   This routine gets the forecast weight, sensor weight, and sensor bias
   !   for the given filename.  The filename contains the sensor and forecast
   !   time.
   !
   ! REFERENCE:
   !
   ! CALLING SEQUENCE:
   !   CALL Get_Trap_Weights(Filename, WeightForecast, WeightSensor, BiasSensor)
   !
   ! INPUTS:
   !   Filename       CHARACTER
   !
   ! OUTPUTS:
   !   WeightForecast REAL
   !   WeightSensor   REAL
   !   BiasSensor     REAL
   !
   ! DEPENDENCIES:
   !
   ! RESTRICTIONS:
   !
   ! HISTORY:
   !   2008-05-05  P. Keehn     Initial development
   !
   !----------------------------------------------------------------------------
   SUBROUTINE Get_Trap_Weights(Filename, WeightForecast, WeightSensor, &
                               BiasSensor)

      CHARACTER(LEN=200), INTENT(IN) :: Filename
      REAL(SINGLE), INTENT(OUT) :: WeightForecast
      REAL(SINGLE), INTENT(OUT) :: WeightSensor
      REAL(SINGLE), INTENT(OUT) :: BiasSensor

      CHARACTER(LEN=20) :: RoutineName = "Get_Trap_Weights"
      CHARACTER(LEN=200) :: BaseFilename
      CHARACTER(LEN=8) :: SensorName
      INTEGER(LONG) :: ForecastTime
      INTEGER(LONG) :: Separator_1
      INTEGER(LONG) :: Separator_2
      INTEGER(LONG) :: Separator_3
      INTEGER(LONG) :: Separator_4
      
      !
      ! Get the filename without the directory info.
      ! Separator_1 to 4 are the locations of the periods (.) in the filename
      ! starting from the END of the string (right side).
      !
      BaseFilename = Filename(SCAN(Filename, '/', .TRUE.)+1:)
      Separator_1 = INDEX(BaseFilename,".",.TRUE.) 
      Separator_2 = INDEX(BaseFilename(:Separator_1-1),".",.TRUE.)
      Separator_3 = INDEX(BaseFilename(:Separator_2-1),".",.TRUE.)
      Separator_4 = INDEX(BaseFilename(:Separator_3-1),".",.TRUE.)
      IF (Separator_3 == 0 .OR. Separator_4 == 0) THEN
         CALL Error_Messaging(RoutineName, "invalid file name", FATAL)
         STOP
      END IF

      !
      ! Get the sensor name and the forecast time from the base filename.
      !
      SensorName = BaseFilename(Separator_4+1:Separator_3-1)
      READ(BaseFilename(Separator_2+1:Separator_1-1), FMT=*) ForecastTime
      ForecastTime = ForecastTime + 6

      !
      ! Get the appropriate forecast weight.
      !
      SELECT CASE (ForecastTime)
         CASE(ValidForecastTimes(1))
            WeightForecast = ValidForecastWeights(1)
         CASE(ValidForecastTimes(2))
            WeightForecast = ValidForecastWeights(2)
         CASE(ValidForecastTimes(3))
            WeightForecast = ValidForecastWeights(3)
         CASE(ValidForecastTimes(4))
            WeightForecast = ValidForecastWeights(4)
         CASE DEFAULT
            CALL Error_Messaging(RoutineName,"invalid forecast time", FATAL)
      END SELECT

      !
      ! Get the appropriate sensor weight and sensor bias.
      !

      IF (TRIM(SensorName) == TRIM(ValidSensorNames(1))) THEN
         WeightSensor = ValidSensorWeights(1)
         BiasSensor = ValidSensorBiases(1)
      ELSE IF (TRIM(SensorName) == TRIM(ValidSensorNames(2))) THEN
         WeightSensor = ValidSensorWeights(2)
         BiasSensor = ValidSensorBiases(2)
      ELSE IF (TRIM(SensorName) == TRIM(ValidSensorNames(3))) THEN
         WeightSensor = ValidSensorWeights(3)
         BiasSensor = ValidSensorBiases(3)
      ELSE IF (TRIM(SensorName) == TRIM(ValidSensorNames(4))) THEN
         WeightSensor = ValidSensorWeights(4)
         BiasSensor = ValidSensorBiases(4)
      ELSE IF (TRIM(SensorName) == TRIM(ValidSensorNames(5))) THEN
         WeightSensor = ValidSensorWeights(5)
         BiasSensor = ValidSensorBiases(5)
      ELSE IF (TRIM(SensorName) == TRIM(ValidSensorNames(6))) THEN
         WeightSensor = ValidSensorWeights(6)
         BiasSensor = ValidSensorBiases(6)
      ELSE IF (TRIM(SensorName) == TRIM(ValidSensorNames(7))) THEN
         WeightSensor = ValidSensorWeights(7)
         BiasSensor = ValidSensorBiases(7)
      ELSE IF (TRIM(SensorName) == TRIM(ValidSensorNames(8))) THEN
         WeightSensor = ValidSensorWeights(8)
         BiasSensor = ValidSensorBiases(8)
      ELSE IF (TRIM(SensorName) == TRIM(ValidSensorNames(9))) THEN
         WeightSensor = ValidSensorWeights(9)
         BiasSensor = ValidSensorBiases(9)
      ELSE IF (TRIM(SensorName) == TRIM(ValidSensorNames(10))) THEN
         WeightSensor = ValidSensorWeights(10)
         BiasSensor = ValidSensorBiases(10)
      ELSE IF (TRIM(SensorName) == TRIM(ValidSensorNames(11))) THEN
         WeightSensor = ValidSensorWeights(11)
         BiasSensor = ValidSensorBiases(11)
      ELSE IF (TRIM(SensorName) == TRIM(ValidSensorNames(12))) THEN
         WeightSensor = ValidSensorWeights(12)
         BiasSensor = ValidSensorBiases(12)
      ELSE IF (TRIM(SensorName) == TRIM(ValidSensorNames(13))) THEN
         WeightSensor = ValidSensorWeights(13)
         BiasSensor = ValidSensorBiases(13)
      ELSE IF (TRIM(SensorName) == TRIM(ValidSensorNames(14))) THEN
         WeightSensor = ValidSensorWeights(14)
         BiasSensor = ValidSensorBiases(14)
      ELSE IF (TRIM(SensorName) == TRIM(ValidSensorNames(15))) THEN
         WeightSensor = ValidSensorWeights(15)
         BiasSensor = ValidSensorBiases(15)
      ELSE IF (TRIM(SensorName) == TRIM(ValidSensorNames(16))) THEN
         WeightSensor = ValidSensorWeights(16)
         BiasSensor = ValidSensorBiases(16)
      ELSE IF (TRIM(SensorName) == TRIM(ValidSensorNames(17))) THEN
         WeightSensor = ValidSensorWeights(17)
         BiasSensor = ValidSensorBiases(17)
      ELSE
         CALL Error_Messaging(RoutineName,"invalid sensor name", FATAL)
      END IF

   END SUBROUTINE Get_Trap_Weights


   !----------------------------------------------------------------------------
   ! NAME:
   !   Write_ETrap_Data
   !
   ! FUNCTION:
   !   Write the Etrap results to file.
   !
   ! DESCRIPTION:
   !   This routine writes the Etrap results to the output file specified in
   !   Mcidas format.
   !
   ! REFERENCE:
   !
   ! CALLING SEQUENCE:
   !   CALL Write_ETrap_Data(Filename, DataValues, Jdate, Time, Memo) 
   !
   ! INPUTS:
   !   Filename   CHARACTER
   !   DataValues BYTE,DIMENSION(:,:)
   !   Jdate      INTEGER
   !   Time       INTEGER
   !   Memo       CHARACTER
   !
   ! OUTPUTS:
   !
   ! DEPENDENCIES:
   !
   ! RESTRICTIONS:
   !
   ! HISTORY:
   !   2008-05-05  P. Keehn     Initial development
   !
   !----------------------------------------------------------------------------
   SUBROUTINE Write_ETrap_Data(Filename, DataValues, Jdate, Time, Memo) 

      CHARACTER(LEN=100), INTENT(IN) :: Filename
      INTEGER(BYTE), DIMENSION(:,:), INTENT(IN) :: DataValues
      INTEGER(LONG), INTENT(IN) :: Jdate
      INTEGER(LONG), INTENT(IN) :: Time
      CHARACTER(LEN=32), INTENT(IN) :: Memo

      CHARACTER(LEN=20) :: RoutineName = "Write_ETrap_Data"
      CHARACTER(LEN=100) :: ErrorMessage
      CHARACTER(LEN=80) :: Comment
      CHARACTER (LEN=8)  :: CurrentDate
      CHARACTER (LEN=10) :: CurrentTime
      CHARACTER (LEN=19) :: CurrentDateTime
      INTEGER(LONG) :: ErrorStatus
      INTEGER(LONG) :: Row
      INTEGER(LONG) :: Column
      INTEGER(LONG) :: Record
      INTEGER(LONG) :: DataOffset
      INTEGER(LONG) :: WordOffset
      INTEGER(LONG) :: Counter
      INTEGER(LONG) :: LUN

      INTEGER(LONG) :: Get_lun

      !
      ! To write the output in mcidas format (area file) we will use the 
      ! header information optained during the reading of the input Trap data.
      ! The size of the header is stored in word 34 and is usually 1280 bytes.
      !
      DataOffset = McidasAreaHeader(34)
      WordOffset = DataOffset / 4

      !
      ! The date and time fields of the header need to be updated.  Mcidas 
      ! dates are in the form yyyddd where;
      !   yyy = MOD(YYYY, 1900)  YYYY = 4 digit year
      !   ddd = day of year
      ! Mcidas time is in the form hhmmss.
      ! The memo (32 character max) gets converted to integer and inserted.
      ! Word 64 is the number of comment cards at the end
      ! of the mcidas file and needs to be set to 1 to accomodate our 
      ! comment.
      !

      ! Modify the sensor source number to composite , by J. Yang
      !McidasAreaHeader(2) = 10
      !   print *, "before modified Header 3 = ",McidasAreaHeader(3)
      McidasAreaHeader(3) = 10
      !   print *, "After modified Header 3 = ",McidasAreaHeader(3)

      McidasAreaHeader(4) = MOD(Jdate,1900000)
      !McidasAreaHeader(5) = Time 
      ! Add the second, By J.Yang
      McidasAreaHeader(5) = Time * 100
      McidasAreaHeader(25:32) = TRANSFER(Memo, McidasAreaHeader(25:32))
      McidasAreaHeader(64) = 1


      !
      ! Write the header information to the output file.
      !
      LUN = Get_lun()
      OPEN(UNIT=LUN, FILE=Filename, FORM="UNFORMATTED", ACCESS="DIRECT", &
           RECL=DataOffset, STATUS="REPLACE", IOSTAT=ErrorStatus)
      IF (ErrorStatus /= 0) THEN
         ErrorMessage = "Unable to open file "//TRIM(Filename)
         CALL Error_Messaging(RoutineName, ErrorMessage, FATAL)
         STOP
      END IF
      WRITE(UNIT=LUN,REC=1) McidasAreaHeader(1:WordOffset)
      CLOSE(UNIT=LUN)

      !
      ! Write the data to the output file.
      !
      LUN = Get_lun()
      OPEN(UNIT=LUN, FILE=Filename, FORM="UNFORMATTED", ACCESS="DIRECT", & 
           RECL=1, STATUS="OLD", IOSTAT=ErrorStatus)
      IF (ErrorStatus /= 0) THEN
         ErrorMessage = "Unable to open file "//TRIM(Filename)
         CALL Error_Messaging(RoutineName, ErrorMessage, FATAL)
         STOP
      END IF

      Record = DataOffset
      DO Row = 1, McidasAreaHeader(9)
         DO Column = 1, McidasAreaHeader(10)
            Record = Record + 1
            WRITE(UNIT=LUN, REC=Record) DataValues(Column, Row)
         END DO
      END DO

      !
      ! Get the current time and convert to a readable string.
      !
      CALL DATE_AND_TIME(CurrentDate, CurrentTime)
      CurrentDateTime = CurrentDate(1:4)//"-"// &
                        CurrentDate(5:6)//"-"// &
                        CurrentDate(7:8)//" "// &
                        CurrentTime(1:2)//":"// &
                        CurrentTime(3:4)//":"// &
                        CurrentTime(5:6)


      !
      ! Write a mcidas comment.
      !
      Comment = "00000 "//CurrentTime(1:6)// &
                " file generated by etrap.f90 on "//CurrentDateTime
      DO Counter = 1, 80
         Record = Record + 1
         WRITE(UNIT=LUN, REC=Record) TRANSFER(Comment(Counter:Counter), &
                                              DataValues(1,1))
      END DO

      !
      ! Close the output file.
      !
      CLOSE(UNIT=LUN)

   END SUBROUTINE Write_ETrap_Data


   !----------------------------------------------------------------------------
   ! NAME:
   !   Rain_Distribution
   !
   ! FUNCTION:
   !   Calculate the distribution of a weighted histogram.
   !
   ! DESCRIPTION:
   !   This routine takes a weighted histogram of rain rates and creates an 
   !   array of rain rates (from largest to smallest) representative of all 
   !   the values recorded in the histogram.
   !
   ! REFERENCE:
   !
   ! CALLING SEQUENCE:
   !   CALL Rain_Distribution(Histogram, Distribution)
   !
   ! INPUTS:
   !   Histogram    REAL,DIMENSION(:)
   !
   ! OUTPUTS:
   !   Distribution INTEGER,DIMENSION(:)
   !
   ! DEPENDENCIES:
   !
   ! RESTRICTIONS:
   !
   ! HISTORY:
   !   2008-05-05  P. Keehn     Initial development
   !
   !----------------------------------------------------------------------------
   SUBROUTINE Rain_Distribution(Histogram, Distribution)

      REAL(SINGLE), DIMENSION(:), INTENT(IN) :: Histogram
      INTEGER(LONG), DIMENSION(:), POINTER :: Distribution

      CHARACTER(LEN=20) :: RoutineName = "Rain_Distribution"
      CHARACTER(LEN=100) :: ErrorMessage
      INTEGER(LONG) :: Bin
      INTEGER(LONG) :: Previous
      INTEGER(LONG) :: Summation
      INTEGER(LONG) :: LongCounter
      INTEGER(LONG) :: ErrorStatus
      REAL(SINGLE) :: Counter

      !
      ! Sum the histogram counts and use it as an estimate for our distribution
      ! array size.
      !
      Summation = INT(SUM(Histogram))

      ALLOCATE(Distribution(Summation+2), STAT=ErrorStatus)
      IF (ErrorStatus /= 0) THEN
         ErrorMessage = "Unable to allocate memory"
         CALL Error_Messaging(RoutineName, ErrorMessage, FATAL)
         STOP
      END IF

      Counter = 0.5
      Previous = 0
      DO Bin = SIZE(Histogram), 1, -1
         Counter = Counter + Histogram(bin)
         LongCounter = INT(Counter)
         IF (LongCounter > Previous) THEN
            Distribution(Previous+1:LongCounter) = Bin + 1
            Previous = LongCounter
         END IF
      END DO

   END SUBROUTINE Rain_Distribution

   !-------------------------------------------------------------------------------
   ! NAME:
   !   Calibrate_PoP
   !
   ! FUNCTION:
   !   Apply calibration to adjust eTRaP-generated PoPs toward observed values
   !
   ! DESCRIPTION:
   !   Calibration is accomplished by multiplying the input probability of
   !   precipitation by a factor between 0.0 and 1.0 depending on the 
   !   accumulation period and the rain intensity threshold (see code below).
   !   These calibration factors were developed by comparing eTRaP-calculated
   !   PoPs with Stage IV-observed PoPs.
   !
   ! REFERENCE:
   !
   ! CALLING SEQUENCE:
   !   CALL Calibrate_PoP(accum, thresh, pop)
   !
   ! INPUTS:
   !   <accum>     = Accumulation period (hours, 6 or 24)
   !   <thresh>    = Threshold rain intensity corresponding to the pop field
   !                 (25,  50,  75, or 100 mm for accum=6,
   !                  50, 100, 150, or 200 mm for accum=24)
   !   <pop>       = On input this is the "raw" (uncalibrated) probability of 
   !                 precipitation array calculated by eTRaP, values between 
   !                 0.0 and 1.0
   !
   ! OUTPUTS:
   !   <pop>       = On output this is the calibrated probability of 
   !                 precipitation array, values between 0.0 and 1.0
   !
   ! DEPENDENCIES:
   !   type_kinds module
   !   error_messaging_module
   !
   ! RESTRICTIONS:
   !
   ! HISTORY:
   !   2008        E. Ebert (probably)  Initial development.
   !   2013-07-22  S. Kidder            Adapted to apply the recently developed 
   !                                    calibration values
   !
   !-------------------------------------------------------------------------------
   SUBROUTINE Calibrate_PoP(accum, thresh, pop)

      INTEGER(LONG), INTENT(IN) :: accum
      INTEGER(LONG), INTENT(IN) :: thresh
      REAL(SINGLE), DIMENSION(:,:), INTENT(INOUT) :: pop

      CHARACTER(LEN=25) :: RoutineName = "Calibrate_PoP"
      CHARACTER(LEN=256) :: ErrorString
      INTEGER(LONG) :: ErrorStatus
      REAL(SINGLE)  :: cal_factor

      !
      !Check input for valid values
      !
      IF (accum /= 6 .and. accum /= 24) THEN
         ErrorString = "accum must be 6 or 24 -- Not calibrated!"
         CALL Error_Messaging(RoutineName, ErrorString, WARNING)
         RETURN
      END IF

      IF (accum == 6 .and. (thresh /= 25 .and. thresh /= 50 .and. thresh /= 75 &
                     .and. thresh /= 100)) then
         ErrorString = "for accum=6 thresh must be one of 25, 50, 75, 100 -- Not calibrated!"
         CALL Error_Messaging(RoutineName, ErrorString, WARNING)
         RETURN
      END IF
   
      IF (accum == 24 .and. (thresh /= 50 .and. thresh /= 100 .and. thresh /= 150 &
                      .and. thresh /= 200)) then
         ErrorString = "for accum=24 thresh must be one of 50, 100, 150, 200 -- Not calibrated!"
         CALL Error_Messaging(RoutineName, ErrorString, WARNING)
         RETURN
      END IF

      ! 
      ! Assign cal_factor
      !
      cal_factor = 1.00
      IF(accum == 6) THEN
         SELECT CASE(thresh)
            CASE(25)
               cal_factor = 0.65
            CASE(50)
               cal_factor = 0.48
            CASE(75)
               cal_factor = 0.30
            CASE(100)
               cal_factor = 0.13
         END SELECT
      ELSE ! (accum = 24)
         SELECT CASE(thresh)
            CASE(50)
               cal_factor = 0.84
            CASE(100)
               cal_factor = 0.72
            CASE(150)
               cal_factor = 0.61
            CASE(200)
               cal_factor = 0.50
         END SELECT
      END IF   
   
      !
      ! Apply calibration
      !
      pop = pop * cal_factor

   END SUBROUTINE Calibrate_PoP

END MODULE Etrap_Utils
