#!/usr/bin/perl -w   

## =====================================================================
## Name:            getBulData4.pl  
##                  
## Author:          Jianbin Yang/RSIS   March 2003
## Language:        Perl
## Purpose:         
## Usage:  	    getBulData.pl <WMO header> [<WXT file name>] [<bulletin data File>]
##                
## Description:     This program input specific bulletin WMO header
##		    and get the storm name, time and position information
##	 	    from the bulletin
## 
## Modified:   12/18/2003
##                  For new Darwin forecast bulletin format
##
## Modified:   01/29/2004
##                  Only use WT**2* ADVISORY bulletin for TPC/CPHC forecast bulletin
##                  Won't use the WT**4* (WTNT4*, WTPZ4* and WTPA4*) any more. 
##
## Modified:   02/09/2004
##                  Modify the Australia Perth bulletin around the name and initial time 
##                  for another format
##
## Modified:   02/11/2004
##                  Add the Brisbane bulletin WOAU01, 02
##
## Modified:   02/12/2004
##                  Modify the program around the initial position of the Darwin bulletin
##                  to make archiving more flexible to avoid the failure when format change.
## 
## Modified:   02/18/2004
##                  Add the name of the center in Australia in front of the storm name, if the system 
##                  Tropical LOW.  For example, for Perth bulletin, the storm name will be "PERTH_LOW"
##
## Modified:   03/03/2004
##                  Add code to "writeInfo" to delete the non-word charater (except '-') 
##                  from the storm name if there is any
##
## Modified: 03/05/2004
## 		    Add the bulletin header WTAU07 for Perth Center in Australia
##
## Modified: 03/11/2004
##        	    Modify the code to get the correct storm name from Reunion bulletinin
##                  in the following case. 
##                  "STORM 10 (HELMA EX-NICKY)", storm name "HELM" was got incorrectly.
##
## Modified: 03/16/2004
##        	    Add the code to read the new bulletin FKAU for Australia
##
##
## Modified: 01/13/2006
##        	    Change the subroutine "getBulInfo6" to read the lower case FIJI bulletin WTPS01 
##		    as well as upper case, since it only issue lower case this time for system TAM	
##
## Modified: 01/19/2006
##        	    Change the subroutine "getPerthBul" to read the lower case Perth bulletin WTAU
##		    as well as upper case, also change to read the new format of Perth bulletin	
##
## Modified: 01/23/2006
##        	    Change the subroutine "getDarwin", "getBrisbane", and "getBulInfo3"- Reunion 
##                  to read the lower case bulletins as well as upper case, 
##                  also change to "getBrisbane" to read the new format (with DEGREE in the forecast position)
##
##
## Modified: 02/23/2006
##        	    Found that not only WOAU is Brisbane forecast, but WTAU01. Change the subroutine getBulInfo4
##                  and "getBrisbaneBul". 
##
## Modified: 02/27/2006
##        	    Add more code to "getPerthBul" subroutine to handle the forecast positions when it is broken into a couple lines. 
##
## Modified: 02/06/2007
##                  Modify the "getDarwin" to read the forecast position for another format
##
## Modified: 04/17/2007
##		    Change the name of the extratropical system to add extra-, or sub-  (For Reunion bulletin)
##
## Modified: 06/06/2007
##		    Add FKIN, WTIN, FKPQ and WWNZ
##
## Modified: 07/03/2007
##		    Handle the subtropical or extra-tropical system for other bulletins
##		    Add FKPS01 for Fiji and FKIO20 for Reunion    07/05/2007
##
## Modified: 09/05/2007
##  		    Modify the the getBulInfo1 for the initial position
##
## Modified: 10/15/2007
##           	    Modify the the getBulInfo2 to eliminate the incorrect issued bulletin from JTWC(PHNC vs PGTW)
## 
## Modified: 11/13/07
##                  Fixed the getBulInfo7 for the FKIN20
##                 
## Modified: 12/27/07
##                  Fixed the getPerthBul for the tropical low from WTAU05
##
## Modified: 03/31/2008
##                  Handle the N/A in forecast position for ICAO bulletin
##
## Modified: 04/10/08
##                  Added AXAU Australian bulletin    By E.Tipton & J.Yang
##
## Modified: 06/20/2008
##                  Change the NIL to TD as Noname for the ICAO (FKPQ) bulletin
## 
## Last Modified: 09/08/2008
##                  Checked for the Zero value for the positions, and eliminate
##                  The incorrect forecast positions from bulletin
##                  This prevent the blank TRaP
##                  Fixed the getBulInfo3 & getBulInfo6 for the other position formats - 09/08
## 
## Modified: 09/19/2008
##                  Fixed the getBulInfo2 (WTPN) for the corrected bulletin
##
## Modified: 09/22/2008
##                  Modified the getBulInfo2 to get the storm number as storm name if it is not named
##                  Example: "18W (EIGHTEEN)" as "18W", not "EIGHTEEN"; another is for "NO NAME"
## 
## Modified: 09/30/08
##         Added FKPQ32-34
##
## Modified: 12/19/08
##         Added the Jakarta bulletin
##
## Modified: 02/06/09
##         Changed to use $HOME without specifying the oper account
##
## Modified: 05/24/09
##         Changed the code for the FKIN bulletin
##
## Modified: 11/23/09
##         Allow the last character missed from the number for NW Pac.
##
## Modified: 12/15/09
##         Fix the sub "checkPosition" for the case passing the dateline
##
## Modified: 5/12/11
##         Treat NPac storms labeled "NN" same as "NIL" (help 3869) cd
## 
## Last modified: 6/8/2011
##        KNHC header change added space   changed lines[3] to lines[4] for basin
## =====================================================================

use Class::Struct;

####---------------------- Define structures and data ------------------------####

# define constant for some error codes
*SUCCESS  		= \0;
*NOMATCH		= \1;
*FAILURE_BUL_FORMAT	= \2;
*FAILURE_GETPOSITION	= \3;
*FAILURE_BUL_JDATE	= \-2;

*FAILURE__WR_JTWCSTAT	= \4;	# Wrong JTWC station

# create a definition for a "Storm" struct
struct Storm =>
{
   name      => '$',        # storm name for pctrp.k to call
   nameNNB   => '$',        # storm name with NNB if is available, this used for name convention
   stNumber  => '$',	    # number of the storm for that basin for that year, for Australia bul. it is 99 
   wmoHd     => '$',        # the WMO header id 
   stationId => '$',        # the station id
   bulTime   => '$',        # the forecast bulletin time
   bulYJd    => '$',        # store the bulletin year and julian day (forecast day)
   bulMmdd   => '$',  	    # the month and day of the bulletin (forecast date)
   iniTime   => '$',        # the initial time of the storm
   forecastHour => '$',	    # TPC and CPHC bulletin will -3 from the hh in iniTime, other is same
   lat_0     => '$',        # the initial latitude of the center of the storm
   lon_0     => '$',        # the initial longitude of the center of the storm
   lat_6     => '$',        # the latitude of 6 hour forecast 
   lon_6     => '$',        # the longitude of 6 hour forecast
   lat_12    => '$',        # the latitude of 12 hour forecast
   lon_12    => '$',        # the longitude of 12 hour forecast
   lat_18    => '$',        # the latitude of 18 hour forecast
   lon_18    => '$',        # the longitude of 18 hour forecast
   lat_24    => '$',        # the latitude of 24 hour forecast
   lon_24    => '$',        # the longitude of 24 hour forecast
   isValid   => '$',        # the storm structure is valid or not
};


# Define the directories and files

$HOME = $ENV{"HOME"};

# $srcDir       = '/home/oper/mcidas/pctrap';
$dataDir        = "/data/Petrap/data/mcidas_data";
$util_dir       = "/home/Plocal/bin";
# $trapDataDir  = "/data/TRAP";
$positionFile   = "$dataDir/POSTRACK.TXT";
$infoFile       = "$dataDir/bulInfo.txt";            	# initialize the bulletin data file as default

# Define the wmolists for the WMO header of the bulletin
# @wmolist1 = qw(
#   WTNT41 WTNT42 WTNT43 WTNT44 WTNT45
#   WTPZ41 WTPZ42 WTPZ43 WTPZ44 WTPZ45
#   WTPA41 WTPA42 WTPA43 WTPA44 WTPA45 
# );
@wmolist1 = qw(
   WTNT21 WTNT22 WTNT23 WTNT24 WTNT25
   WTPZ21 WTPZ22 WTPZ23 WTPZ24 WTPZ25
   WTPA21 WTPA22 WTPA23 WTPA24 WTPA25 
);

## must specify the WMO station (WSTN=PGTW) for the following WMO header
@wmolist2 = qw(
   WTPN31 WTPN32 WTPN33 WTPN34 WTPN35
   WTPS31 WTPS32 WTPS33 WTPS34 WTPS35
   WTXS31 WTXS32 WTXS33 WTXS34 WTXS35
   WTIO31 WTIO32 WTIO33 WTIO34 WTIO35
);

# @wmolist3 = qw(
#    WTIO31 WTIO32 WTIO33 WTIO34 WTIO35
# );

## bulletins from Reunion(IO21 or IO31?), station (WSTN=FMEE)

# ?? Not sure the headers are all included
# remove the WTIO21, since most of the time it is in French, will confirm if it is always in French

@wmolist3 = qw(
   WTIO20 WTIO22 WTIO23 WTIO24 WTIO25 WTIO30       
);

## bulletins from Australia , station (WSTN=APRF)
# Will confirm the WTAU01-02, 04, and find more bulletin header

# Add the Jakarta hearder here
## WTAU replaced by AXAU ##
@wmolist4 = qw(
   WTAU01 WTAU02 WTAU03 WTAU04 WTAU05 WTAU06 WTAU07
   WOAU01 WOAU02              
   WVID10
);

## bulletins from RSMC Tokyo, station (WSTN=RJTD)

@wmolist5 = qw(
   WTJP21 WTJP22 WTJP23 WTJP24 WTJP25
);

## bulletins from Fiji, station (WSTN=NFFN)

@wmolist6 = qw(
   WTPS01 WTPS02 WTPS03 WTPS04 WTPS05
   WHPS01 WHPS02 WHPS03 WHPS04 WHPS05  
);


# the number range to be determined

@wmolist7 = qw(
   FKAU01 FKAU02 FKAU03 FKAU04 FKAU05 FKAU06 
   FKPS01 FKPS02 
   FKPQ30 FKPQ31 FKPQ32 FKPQ33 FKPQ34
   FKIN20
   FKIO20
);

@wmolist8 = qw(
   WTIN20
 );

@wmolist9 = qw(
   WTNZ41 WHNZ41
 );


@wmolist10 = qw(
   AXAU01 AXAU02 AXAU03 AXAU04 AXAU05 AXAU21
 );


# Define the number array to handle the JTWC bulletin for unnamed storm.  09/22/08
# take care those without dash inside the subroutine 
@numbers1 = qw(
   ONE TWO THREE FOUR FIVE
   SIX SEVEN EIGHT NINE TEN
   ELEVEN TWELVE THIRTEEN FOURTEEN FIFTEEN
   SIXTEEN SEVENTEEN EIGHTEEN NINETEEN TWENTY
   TWENTY-ONE TWENTY-TWO TWENTY-THREE TWENTY-FOUR TWENTY-FIVE
   TWENTY-SIX TWENTY-SEVEN TWENTY-EIGHT TWENTY-NINE THIRTY
   THIRTY-ONE THIRTY-TWO THIRTY-THREE THIRTY-FOUR THIRTY-FIVE
   THIRTY-SIX THIRTY-SEVEN THIRTY-EIGHT THIRTY-NINE FORTY
);

$unname = "NO NAME";
@numbers = ($unname, @numbers1);


## initialize some variable
$filename = "tempWxt.txt";
$arrayPrefix = "wmolist";            # the prefix of the above WMO header arrays
$functionPrefix = "getBulInfo";          # the prefix of functions to get positions
$bulArrNumber = 10;                   # there are 6 kinds of bulletins
				     #	for the corresponding WMO headers

$basin = "";			     # the basin will only got from the TPC & CPHC (wmolist1) bulletin
				     # other are empty

## initialize the array, hash table and structure
$storm1 = Storm->new();

# Define the array contained all the  bulletins 
@bulArray = (@wmolist1, @wmolist2, @wmolist3, @wmolist4, @wmolist5, @wmolist6, @wmolist7, @wmolist8, @wmolist9, @wmolist10);

## comment out on 06/24/03, will use 24 after testing
$hourRange = 24;
# $hourRange = 96;

####------------------------------- Start -------------------------------------####


# $path = $ENV{"PATH"};
# $ENV{"PATH"} = $path;

# $ENV{"MCPATH"} = "$HOME/mcidas/data:/home/mcidas/data";


###!!!! will turn off these print statement

print "\n--------------------- Start the get position Program ---------------------\n";

$startdate=`date`;
print "Start $0 at $startdate.\n"; 

if (@ARGV < 1)
{
   print "Usage: getBulData.pl <WMO header> [<WXT file name>] [<bulletin data File>]\n";
   die "No WMO header - exit program";
}
elsif (@ARGV == 1)
{
   $wmohd = $ARGV[0];
   $wxtFile = "";		# if it is empty, will get the bulletin file from wxtlist later
}

elsif (@ARGV == 2)
{
   $wmohd = $ARGV[0];
   $wxtFile = $ARGV[1];
}

elsif (@ARGV == 3)
{
   $wmohd = $ARGV[0];
   $wxtFile = $ARGV[1];
   $infoFile = $ARGV[2];
}

else
{
   print "Usage: getBulData.pl <WMO header> [<WXT file name>] [<bulletin data File>]\n";
   die "Too many arguments - exit program";
}


chdir("$dataDir") || die "Cannot change to directory $dataDir";
print  "Data directory:";
system("pwd");

# check if the input file is exist
if (($wxtFile ne "") && (!(-f $wxtFile)) )
{
   die "file $wxtFile is not exist: $!";
}


# open STDOUT, ">getBul.out" || die "Can't create getBul.out:$!";
# open STDERR, ">getBul.err" || die "Can't create getBul.err:$!";

$functionName = "";

for ($i = 1; $i < $bulArrNumber + 1; $i++)
{
   $arrayName = $arrayPrefix . $i;
   $isExist = isElement(\@{$arrayName}, $wmohd);
   if ($isExist >= 0)
   {
      $functionName = $functionPrefix . $i;
      last;
   }
}

print "out of the for loop is Exist = $isExist\n";
if ($isExist < 0)
{
   die "WMO header \"$wmohd\" is not in the lists - Exit program."
}

if ($functionName eq "")
{
   die "function error - Exit program.";
}

print "functionName is $functionName\n"; 

# call the specific function via the variable
# return 0 for success, -1 for failed to get possion 
$success = &$functionName();           

if ($success == $SUCCESS)
{
   # check for incorrect forecat position here
   # will call a subroutine, 07/29/08

   if (checkPositions() != $SUCCESS)
   {
      print "ERROR! check Position not pass, postion track is retrieved unsuccessfully for $wmohd!\n";
   }
   elsif (writeInfo() == $FAILURE_BUL_JDATE)
   {
      print "ERROR: Bulletin Julian date is retrieved unsuccessfully for $wmohd!\n";
   }

   print "Bulletin information are retrieved successfully. Write them into the POS file.\n";
   # writeInfo();
#   writeTrack();

   printf "IN MAIN, the storm name, initial time, lat_12 and lon_12 in the storm structure are %s %s%8.2f%8.2f\n",
              $storm1->nameNNB, $storm1->iniTime, $storm1->lat_12, $storm1->lon_12;

}
elsif ($success == $FAILURE_GETPOSITION)
{
   print "Warning: position track is retrieved unsuccessfully for $wmohd!\n";
   #print "ERROR: position track is retrieved unsuccessfully for $wmohd!\n";
   print "The WMO Header for this is $wmohd.\n";
}
elsif ($success == $FAILURE_BUL_FORMAT)
{
   #print "Warning: Bulletin format unexpected, retrieved unsuccessfully!\n";
   print "ERROR: Bulletin format unexpected, retrieved unsuccessfully for $wmohd!\n";
   print "The WMO Header for this is $wmohd.\n";
}
elsif ($success == $NOMATCH)
{
   print "Warning: No Matches Found for this WMO header!\n";
   print "The WMO Header for this is $wmohd.\n";
}
elsif ($success == $FAILURE_BUL_JDATE)
{
   #print "Warning: Bulletin Julian date is retrieved unsuccessfully!\n";
   print "ERROR: Bulletin Julian date is retrieved unsuccessfully for $wmohd!\n";
   print "The WMO Header for this is $wmohd.\n";
}
elsif ($success == $FAILURE__WR_JTWCSTAT)
{
   print "Warning: Incorrect JTWC bulletin, the storm is in EASTPAC, but it is issued under PGTW station!\n";
   print "The WMO Header for this is $wmohd.\n";
}

chdir("$dataDir") || die "Cannot change to dataDir $dataDir";

##printf "IN MAIN, the storm name, initial time, lat_12 and lon_12 in the storm structure are %s %s%8.2f%8.2f\n",
              $storm1->nameNNB, $storm1->iniTime, $storm1->lat_12, $storm1->lon_12;

####--------------------------------------------------------------------------####

print "\nEnd of the PERL program $0.\n";

##%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  Subroutines %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%##

############  Subroutine "isElement" ###############
########################################################
## if it is one of the element of the array the index is return,
## otherwise return -1

sub isElement
{
   # my($element, @array) = @_;
   my($array, $element) = @_;
   my($foundIndex) = -1;

   # must specify $i as local variable, otherwise will conflict with $i in 
   # the calling phase (main function)
   my($i);           

   # can also write as @{$array}
   ### print " the array is @array\n";

   for ($i= 0; $i < @$array; $i++)
   {
      ### print "inside the isElement, the element is @$array[$i], i is $i\n";
      if (@$array[$i] eq $element)
      {
         $foundIndex = $i;
         last;
      }
   }

   return $foundIndex;
}

############  Subroutine "adjustBulYJd" ###############
##################################################
# Noted at 08/15, in the subroutine will compare the forecast date with the issued date. (forecast-issed) + Jd in bulYjd,
# if the difference <=1. if difference >27,then it is over two months, then if forecast > issued, jd -1,
# otherwise jd +1, use the TK utility to +/- 1 from Jd

sub adjustBulYJd
{
   my($bulTime, $bulIssuedYJd) = @_;
   my($bulYjd);
   my($dmy) = `$util_dir/TKcydtodmyExe -cyd $bulIssuedYJd`;
#   my($dmy) ="11 2 2013";
   print "DMY $dmy\n";
   @splitMsg = split(/\s+/, $dmy);
   my($issuedDay) = $splitMsg[0];
   my($bulDay) = substr($bulTime, 0, 2);

   print "IssueDay bulDay $issuedDay $bulDay\n";   
   if ($issuedDay == $bulDay)
   {
      return $bulIssuedYJd;
   }

   # in the same month
   if ( abs($issuedDay - $bulDay) <= 1 )
   {
      $jdIncrease = $bulDay - $issuedDay;
   }
   # over two months
   elsif (abs($issuedDay - $bulDay) >= 27 )
   {
      if ($bulDay > $issuedDay)
      {
         $jdIncrease = -1;
      }
      else
      {
         $jdIncrease = 1;
      }
   }
   else
   {
      return $FAILURE_BUL_JDATE;
   }

   $bulYjd = `$util_dir/TKincdayExe -srcday $bulIssuedYJd -n_days $jdIncrease`;

   return $bulYjd;
}


############  Subroutine "getBulInfo1" ###############
##################################################

sub getBulInfo1
{
   my($i,$j);
   print "This is in function getBulInfo1.\n";
   
   if ($wxtFile ne "")
   {
      $filename = $wxtFile;
   }
   else
   {
      system("wxtlist.k WMO=$wmohd DTIME=$hourRange DEV=T $filename");
   }

   # If no bulletin match for this WMO header get back to the main program
   if (($tempMsg = `grep "No matches found" $filename`) =~ /^No matches found$/)
   {
      print "No matches found for bulletin $wmohd\n";      ## will delete the print
      return $NOMATCH;
   }

   ##------------ Start to get the storm information ------------##

   #-------- get the storm name --------#

   $tempMsg = `grep "ADVISORY NUMBER" $filename`;

   if ($tempMsg  eq "")
   {
      return $FAILURE_BUL_FORMAT;
   }

   #* @splitMsg = split(/DISCUSSION/, $tempMsg);
   @splitMsg = split(/FORECAST/, $tempMsg);
   @nameMsg = split(/\s+/, $splitMsg[0]);
# To avoid the SPECIAL in front of DISCUSSION, change -1 to 2
#   $stormName = $nameMsg[-1];
#   $nameNNB = $nameMsg[-1]; 

# Noted at 08/25 0027 Z, since the format change to "HURRICANE IGNACIO DISCUSSION"
# change back to use nameMsg[-1] from nameMsg[2], use if/esle to handle the "SPECIAL DISCUSSION"  
#   $stormName = $nameMsg[2];
#   $nameNNB = $nameMsg[2];

   $stormName = $nameMsg[-1];
   $nameNNB = $nameMsg[-1];

   if ($nameMsg[-1] eq "SPECIAL")
   {
      $stormName = $nameMsg[-2];
      $nameNNB = $nameMsg[-2];
   }

   # Try to catch the EX or SUBTROPICAL
   $prefix = ""; 
   if ($splitMsg[0] =~ /SUBTROPICAL/i)
   {
      $prefix = "SUB";
   }
   # the following has no example collected.  07/03/2007
   elsif ($splitMsg[0] =~ /EX[\-,\_,\s+]?TROPICAL/i)
   {
      $prefix = "EX";
   }
   elsif ($splitMsg[0] =~ /EXTRA[\-,\_]?TROPICAL/i)
   {
      $prefix = "EXTRA";
   }

   if ($prefix ne "")
   {
      $stormName = $prefix . '-' . $stormName;
      $nameNNB = $prefix . '-' . $nameNNB;
   }
 

   #-------- get the storm number and basin  --------#

   open(FILE, $filename) || die "Can not open $filename: $!";
   my(@lines) = <FILE>;
   @numberMsg = split(/\s+/, $lines[4]);

   # Noted at 08/13/2003, Get the basin here. When storm crossed 140W 
   # from the East Pac into the Central Pac, warning responsibility was passed 
   # from TPC to CPHC in Honolulu. But basin is still "EP"  

   $basin= substr($numberMsg[-1], 0, 2);

   $stNumber = substr($numberMsg[-1], 2, 2);
   close(FILE) || die "Can not close $filename: $!";

   #-------- get the time of the bulletine --------#
   $timeMsg = `grep $wmohd $filename`;
   @splitMsg = split(/\s+/, $timeMsg);
   $stationId = $splitMsg[1];
   $bulTime = $splitMsg[2];
   $bulYJd  = $splitMsg[-2];


   ##------------ Get the position track ------------##
   #--- the initial time of the storm will be got here ---#
   #--- 	   and the forecast hour	       ---#
   $posMsg = `grep  "\/" $filename`;
   print "the positon msg is ***\n$posMsg\n***\n";
   if ($posMsg eq "")
   {
      print "WARNING, there is no position listed in the bulletin.\n";
      return $FAILURE_GETPOSITION;
   }

   @splitMsg = split(/\n/, $posMsg);

   # $i for counting the number of forecast position lines read 
   $i = 0;

   foreach $posLine (@splitMsg)
   {
      # get the initial position

      # if ($posLine =~ /^INITIAL\s+\d{2}\/\d{4}Z/)
      if ($posLine =~ /CENTER WAS LOCATED NEAR\s+\d/)
      {
         print "this is the initial position.\n";
         @tempMsg = split(/\s+/, $posLine);

         #-------- get the initial time of the storm --------#
         # TPC and CPHC bulletin will minus 3 from the bulletin time 
         #(usually is 00, 06, 12 or 18), other is same

         #-------- get the storm initial position  --------#
  
       ## the corresponding WT**2* ADVISORY bulletin have the -3 hour position (initial)
## use the WTNT2* bulletin to get the initial position, since that is the right position for the forecast hour
## This modification is made in operational at 9/15/2003


         $iniTime = $tempMsg[1];                                      # modified at 10/20/2003
         $iniTime =~ s/\///g;          # delete the backslash between the day and hour
         $forecastHour = substr($iniTime, 2, 2);

         # get position
         $lat_0 = $tempMsg[-2];
         $latPos = chop($lat_0);
 
         # will add the following in the final latPos, 09/03/08
         if ($latPos !~ /^[NnSs]/)
         {
            print "ERROR! latPos has invalid value:$latPos\n";
            return $FAILURE_GETPOSITION;
         }
  
         $lat_0 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         $lon_0 = $tempMsg[-1];

         # for the pattern "AT 05/0000Z CENTER WAS LOCATED NEAR 14.1N  85.4W...INLAND", 09/05/2007
         if ($lon_0 =~ /\.\./)
         {
            @tempMsg2 = split(/\.\./, $lon_0);
            $lon_0 = $tempMsg2[0];
         }

         $lonPos = chop($lon_0);

         # will add the following in the final lonPos, 09/03/08
         if ($lonPos !~ /^[WwEe]/)
         {
            print "ERROR! lonPos has invalid value:$lonPos\n";
            return $FAILURE_GETPOSITION;
         }


         # the conversion follows Mcidas definition (East-lon is negative)
         $lon_0 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         print "lat is $lat_0, pos is $latPos, lon is $lon_0, pos is $lonPos\n";

      }  # end of getting the initial position

      # get the 12/24 hour forecast position,
      elsif ($posLine =~ /^\s*FORECAST VALID\s+\d{2}\/\d{4}Z\s*\d/)
      {
         $i++;
        #  print "posLine is $posLine\n";
         @tempMsg = split(/\s+/, $posLine);
         # skip the first element if it is empty
         $j = 0;
         if ($tempMsg[$j] eq "")
         {
            $j++;
         }
         $lat = $tempMsg[$j + 3];
         $latPos =  chop($lat);
         $lon = $tempMsg[$j + 4];

         # for the pattern "FORECAST VALID 25/1200Z 18.8N 110.7W...REMNANT LOW"
         if ($lon =~ /\.\./)
         {
            @tempMsg2 = split(/\.\./, $lon);
            $lon = $tempMsg2[0];
         }

         $lonPos = chop($lon);

         if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
         {
            print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
            return $FAILURE_GETPOSITION;
         }

         $lat *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         $lon *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         print "lat is $lat, pos is $latPos, lon is $lon, pos is $lonPos\n";

         if ($i == 1)
         {
            $lat_12 = $lat;
            $lon_12 = $lon;
         }
         elsif ($i == 2)
         {
            $lat_24 = $lat;
            $lon_24 = $lon;
            last;          # get up to 24 hour forecast
         }

      }  # end of "elseif" for checking the 12/24 forecast position

   }     # end of "foreach"

  if ($lat_24 !~ /\d$/)
  # if (($lat_12 !~ /\d$/) || ($lat_24 !~ /\d$/))
  {
     print "WARNING, 12/24 hour forecast is not the right one or not available!!\n";
     return $FAILURE_GETPOSITION;
  }

   get6_18_points();

   return $SUCCESS;

}  	## End of subroutine getBulInfo1

############  Subroutine "getBulInfo2" ###############
##################################################

sub getBulInfo2
{
   print "This is in function getBulInfo2.\n";

   #****** Please Note ********
   # specify the station as PGTW here, since the "WTPN PHNC" 
   # is duplicated of "WTPN PGTW", and I believe the other
   # "WTPS31", "WTXS31" is from PGTW station, will confirm this!!!
   # "WTIO31" must be from PGTW station.
   #***************************

   if ($wxtFile ne "")
   {
      $filename = $wxtFile;
   }
   else
   {
      if ($wmohd =~ /WTPN/)
      {
         system("wxtlist.k WMO=$wmohd WSTN=PGTW MATCH='NORTHWESTPAC' DTIME=$hourRange DEV=T $filename");
      }
      else
      {
         system("wxtlist.k WMO=$wmohd WSTN=PGTW DTIME=$hourRange DEV=T $filename");
      }
      #system("wxtlist.k WMO=$wmohd WSTN=PGTW DTIME=$hourRange DEV=T $filename");
   }


   # If no bulletin match for this WMO header get back to the main program
   if (($tempMsg = `grep "No matches found" $filename`) =~ /^No matches found$/)
   {
      print "No matches found for bulletin $wmohd\n";      ## will delete the print
      return $NOMATCH;
   }

   # WTPN31 PGTW 311500                                              2003304 1431
   # 01 ACTIVE TROPICAL CYCLONE IN NORTHWESTPAC
   #
   # WTPN31 PHNC 151000 COR
   # 01 ACTIVE TROPICAL CYCLONE IN EASTPAC

   $tempMsg = `grep "IN EASTPAC" $filename`;
   if ($tempMsg  ne "")
   {
      return $FAILURE__WR_JTWCSTAT;
   }
  

   ##------------ Start to get the storm information ------------##

   #-------- get the storm name and storm number --------#

   # To eliminate the additional line when it is a corrected bulletin
   # Example: "SUBJ/TROPICAL CYCLONE WARNING NR 03A CORRECTED//" - 09/19/08

   #$tempMsg = `grep "WARNING NR" $filename`;
   $tempMsg = `grep "WARNING NR" $filename|grep -v -i "SUBJ\/"`;
   if ($tempMsg  eq "")
   {
      return $FAILURE_BUL_FORMAT;
   }

   @splitMsg = split(/WARNING NR/, $tempMsg);
   @nameMsg = split(/\s+/, $splitMsg[0]);
   # -1 is another subscript for the last element
   $stormName = $nameMsg[-1];
   $nameNNB = $stormName;
   $stNumber = $stormName;

   #if ($stormName =~ /\(.+\)/)
   if (($tempMsg =~ /\(.+\)/) && ($stormName =~ /[\(\)]/) )
   {
      #   @f1 = split(/[\(\)]/, $&);
      #@splitName = split(/[\(\)]/, $stormName);
      @splitName = split(/[\(\)]/, $tempMsg);
      $stormName = $splitName[1];
 
      # check for space inside the ( ), especially for "NO NAME"
      if ($stormName !~ /\s+/) 
      {
         # -2 is another subscript for the second last element
         $stormNNB = $nameMsg[-2];
         $nameNNB = $stormName . $stormNNB;
         # the storm number should be the one in front of the storm name in this case
         $stNumber = $nameMsg[-2];
      }
      else
      {
         $stormNNB = $nameMsg[-3];
         $nameNNB = $stormName . $stormNNB;
         $stNumber = $nameMsg[-3];
      }

      # check if the storm name is actual a number
      $stNum = $stormNNB;
      chop($stNum);
      # take care those number without dash '-'
      $numberNDash = $numbers[$stNum];
      $numberNDash =~ s/\-//;
      $misNumber = $numbers[$stNum];
      chop($misNumber);
      $misNumberNDash = $numberNDash; 
      chop($misNumberNDash);
      if ( ($stormName eq $numbers[$stNum]) || ($stormName eq $numberNDash ) || ($stormName eq $numbers[0])
            || ($stormName eq $misNumber ) || ($stormName eq $misNumberNDash ) )
      {
         $stormName = $stormNNB;
      }
# print "stormName is $stormName, stormNNB is $stormNNB, nameNNB is $nameNNB, stNumber is $stNumber\n";
   }
   # Storm name has been got

   chop($stNumber);		# remove the last basin character 
   
   if ($stNumber !~ /^\d{2}$/)
   {
      print "ERROR! storm number is invalid:$stNumber, stormNNB is $stormNNB. The WMO header is $wmohd.\n";
      return $FAILURE_BUL_FORMAT;
   }

   #-------- get the time of the bulletine --------#

   $timeMsg = `grep $wmohd $filename`;
   @splitMsg = split(/\s+/, $timeMsg);

   $stationId = $splitMsg[1];
   $bulTime = $splitMsg[2];
   $bulYJd  = $splitMsg[-2];
   print "XX $bulYJd XX\n";
 #  $bulYJd = chomp($bulYJd);

   ##------------ Get the position track ------------##
   #--- the initial time of the storm and the forecast hour will be got here ---#

   # open and read the file to get the positions
   open(POS2, $filename) || die "Can not open $filename: $!"; 
   while (defined($line = <POS2>))
   {
      # chomp $line;
      # get the initial position
      if ($line =~ /^\s*WARNING POSITION:$/)
      {
         # read the next line
         $line = <POS2>;       
         if ($line =~ / --- NEAR/)
         {
           print "initial posLine is $line\n";
            @tempMsg = split(/\s+/, $line); 
      
            #-------- get the initial time of the storm --------#
 
            $iniTime = $tempMsg[-5];
            chop($iniTime);   # remove the last correction digit
            $forecastHour = substr ($iniTime, 2, 2);

            # use index -1 is better than 3, since whether there is space
            # in front of the line will result in different element "3" 
            $lat_0 = $tempMsg[-2];        
            $latCorNum = chop($lat_0);     		# remove the correction digit

            # If there is no correction digit at the end of lat or lon, unchop it.
            if ($latCorNum =~ /[NnSs]/)
            {
               print "No correction digit at the lat end, append the chopped char again.\n";
               $lat_0 .= $latCorNum;
            }
            $latPos = chop($lat_0);

            $lon_0 = $tempMsg[-1];
            $lonCorNum = chop($lon_0);              	# remove the correction digit
            if ($lonCorNum =~ /[EeWw]/)
            {
               print "No correction digit at the lon end, append the chopped char again.\n";
               $lon_0 .= $lonCorNum;
            }
            $lonPos = chop($lon_0);

            if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
            {
               print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
               return $FAILURE_GETPOSITION;
            }

            $lat_0 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
            $lon_0 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
            print "lat is $lat_0, pos is $latPos, lon is $lon_0, pos is $lonPos\n";
         }
         else
         {
            #print "WARNING, initial position is not in the right format or not available!!\n";
            print "ERROR! initial position is not in the right format or not available!!\n";
            return $FAILURE_GETPOSITION;;
         }
      }     # end of the "if" for initial position

      # get the 12 hour forecast
      elsif ($line =~ /^\s*12 HRS,\s+VALID AT:$/)
      {
         $line = <POS2>;
         if ($line =~ / --- /)
         {
           print "12 hour posLine is $line\n";
            @tempMsg = split(/\s+/, $line);
            $lat_12 = $tempMsg[-2];
            $latCorNum = chop($lat_12);               # remove the correction digit
             
            # add the following as we did for initial position on 07/13/2005, since the format change to no correction digit for 12/24 hour
 
            # If there is no correction digit at the end of lat or lon, unchop it.
            if ($latCorNum =~ /[NnSs]/)
            {
               print "No correction digit at the lat end, append the chopped char again.\n";
               $lat_12 .= $latCorNum;
            }
            $latPos = chop($lat_12);

  
            $lon_12 = $tempMsg[-1];
            $lonCorNum = chop($lon_12);               # remove the correction digit
            if ($lonCorNum =~ /[EeWw]/)
            {
               print "No correction digit at the lon end, append the chopped char again.\n";
               $lon_12 .= $lonCorNum;
            }
            $lonPos = chop($lon_12);

            if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
            {
               print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
               return $FAILURE_GETPOSITION;
            }

            $lat_12 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
            $lon_12 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
            print "lat is $lat_12, pos is $latPos, lon is $lon_12, pos is $lonPos\n";
         }
         else
         {
            print "WARNING, 12 hour forecast position is not in the right format or not available!!\n";
            return $FAILURE_GETPOSITION;;
         }
      }     # end of "elseif" for checking the 12 forecast position

      # get the 24 hour forecast position
      elsif ($line =~ /^\s*24 HRS,\s+VALID AT:$/)
      {
         $line = <POS2>;
         if ($line =~ / --- /)
         {
           print "24 hour posLine is $line\n";
            @tempMsg = split(/\s+/, $line);
            $lat_24 = $tempMsg[-2];
            $latCorNum = chop($lat_24);               # remove the correction digit

            # If there is no correction digit at the end of lat or lon, unchop it.
            if ($latCorNum =~ /[NnSs]/)
            {
               print "No correction digit at the lat end, append the chopped char again.\n";
               $lat_24 .= $latCorNum;
            }

            $latPos =  chop($lat_24);

            $lon_24 = $tempMsg[-1];
            $lonCorNum = chop($lon_24);               # remove the correction digit
            if ($lonCorNum =~ /[EeWw]/)
            {
               print "No correction digit at the lon end, append the chopped char again.\n";
               $lon_24 .= $lonCorNum;
            }
    
            $lonPos = chop($lon_24);

            if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
            {
               print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
               return $FAILURE_GETPOSITION;
            }

            $lat_24 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
            $lon_24 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
            print "lat is $lat_24, pos is $latPos, lon is $lon_24, pos is $lonPos\n";
         }
      }     # end of "elseif" for checking the 24 forecast position
   
   }        # End of the while loop to read the forecast file

   close(POS2) || die "Can not close $filename: $!";

   if ($lat_24 !~ /\d$/)
   {
      #print "WARNING, 24 hour forecast is not the right one or not available!!\n";
      print "Warning, 24 hour forecast is not the right one or not available!!\n";
      return $FAILURE_GETPOSITION;;
   }
   get6_18_points();  
   return $SUCCESS;

}  	## End of subroutine getBulInfo2


############  Subroutine "getBulInfo3" ###############
############   Reunion bulletin    ###############
## WTIO20 - 25
## If the bulletin is not matched, or no 24 hour position
## This subroutine will return -1 for error.
##################################################

sub getBulInfo3
{
   my($i) = 0;
   my($j);
   my($lat, $lon, $latPos, $lonPos);

   print "This is in function getBulInfo3.\n";

   if ($wxtFile ne "")
   {
      $filename = $wxtFile;
   }
   else
   {
      system("wxtlist.k WMO=$wmohd WSTN=FMEE DTIME=$hourRange DEV=T $filename");   
   }

   # If no bulletin match for this WMO header get back to the main program
   if (($tempMsg = `grep "No matches found" $filename`) =~ /^No matches found$/)
   {
      print "No matches found for bulletin $wmohd\n";      ## will delete the print
      return $NOMATCH;
   }

   ##------------ Start to get the storm information ------------##

   #-------- get the storm name and storm number --------#
   $tempMsg = `grep -i "PHENOMENON   " $filename`;
   if ($tempMsg  eq "")
   {
      $tempMsg = `grep -i "PHENOMENON:" $filename`;
      if ($tempMsg  eq "")
      {
         return $FAILURE_BUL_FORMAT;
      }
   }

   @nameMsg = split(/\s+/, $tempMsg);
  
   # the storm has a name
   if ($tempMsg =~ /^PHENOMENON\s*\:.*\(.*\)/i)
   {
      @nameMsg = split(/\(/,  $tempMsg);
      @nameMsg1 = split(/\s+/, $nameMsg[0]);
      $stNumber = $nameMsg1[-1];
      @nameMsg2 = split(/\s+/, $nameMsg[1]);
      $stormName = $nameMsg2[0];
      
      # to handle case like "(HELMA EX-NICKY)", use if statement.  03/11/2004
      if ($stormName =~ /\)$/)
      {
         chop($stormName);
      }

      print "rlg nameMsg @nameMsg\n";
      print "rlg nameMsg1 @nameMsg1\n";
      print "rlg nameMsg2 @nameMsg2\n";
print "###--- the line is $tempMsg, the name is $stormName, the storm number is $stNumber\n";
      
      # remove the ( in front of the storm name,
      # please note that there is a whitespace in front of ) 
 #     $stormName = substr($stormName, 1, (length($stormName) - 1) );
      $nameNNB = $stormName;
  #    $stNumber = $nameMsg[-3];
   }

   # disturbed weather
   elsif ($tempMsg =~ /^PHENOMENON\s*\: *ZONE/i)
   {
      @nameMsg1 = split(/\:/, $tempMsg);
      @nameMsg = split(/\s+/, $nameMsg1[1]);
      $stormName = $nameMsg[3] . $nameMsg[5];
      $nameNNB = $stormName;
      $stNumber = $nameMsg[5];
   }

   # there is not enought to give a name, it is disturbance or depression
   elsif ($tempMsg =~ /^PHENOMENON\s*\:.* \d+ HPA$/i)
   {
      if ((@nameMsg <= 7) && ($nameMsg[-3] =~ /\d+/))
      {
         $stormName = $nameMsg[-4]. $nameMsg[-3];
         $nameNNB = $stormName;
         $stNumber = $nameMsg[-3];
      }
      elsif ((@nameMsg > 7) && ($nameMsg[-3] =~ /\d+/))
      {
         $stormName = $nameMsg[-5] . $nameMsg[-4];
         $nameNNB = $stormName;
         $stNumber = $nameMsg[-4];
      }
   }

   else
   {
      return $FAILURE_BUL_FORMAT;
   }

   # To check if that is extratropical/subtropical system
   # pattern like this:   "PHENOMENON:        EXTRATROPICAL DEPRESSION 15 1000 HPA"
   # Need to filter this: "PHENOMENON:        EXTRATROPICAL DEPRESSION 8 (EX-BOLOETSE) 985 HPA"

   print "In getInfo3 the tempMsg is $tempMsg\n\n";


   if (($tempMsg =~ /EXTRATROPICAL/i) && ($stormName !~ /^EX[-,_]/i))
   {
       $stormName = "EXTRA-" . $stormName;
   }
   elsif (($tempMsg =~ /SUBTROPICAL/i) && ($stormName !~ /^EX[-,_]/i))
   {
       $stormName = "SUB-" . $stormName;
   }
   elsif (($tempMsg !~ /EXTRATROPICAL|SUBTROPICAL/i) && ($stormName =~ /^EX[-,_]/i) && ($tempMsg =~ /DEPRESSION/i))
   {
       print "The EX storm is a depression, before remove the EX-, stormName is $stormName\n";
       $stormName = substr($stormName, 3);
       print "The EX storm is a depression, remove the EX-, stormName is now $stormName\n";
   }


   

   #-------- get the time of the bulletine --------#
   #--- the initial time of the storm is same as bulletin time ---#

   $timeMsg = `grep $wmohd $filename`;
   @splitMsg = split(/\s+/, $timeMsg);
   $stationId = $splitMsg[1];
   $bulTime = $splitMsg[2];
   $bulYJd  = $splitMsg[-2];
 
   $iniTime = $bulTime . 'Z';   	# add 'Z' at the end
   $forecastHour = substr ($iniTime, 2, 2);

   ##------------ Get the position track ------------##

   $posMsg = `grep -i "POSITION" $filename`;
   print "the positon msg is ***\n$posMsg\n***\n"; 
   if ($posMsg eq "")
   {
      print "WARNING, there is no position listed in the bulletin.\n";
      return $FAILURE_GETPOSITION;
   } 
   @splitMsg = split(/\n/, $posMsg);
   print "split 1 is $splitMsg[0]\n";
   print "split 2 is $splitMsg[1]\n";
   # print "split 3 is $splitMsg[2]\n";

   foreach $posLine (@splitMsg)
   {
      # get the initial position, sometime there are space in front of the "POSITION"
      # and sometimes there is any space between 'POSITION' and ':'
      if ($posLine =~ /^\s*POSITION\s*:.*\//i)
      # if (($posLine =~ /^POSITION/i) && ($posLine =~ /[:\/]/))
      {
         print "this is the initial position.\n";
         @tempMsg = split(/\//, $posLine);
         if (@tempMsg != 2)
         {
            next;
         }
         @temp1 = split(/\s+/, $tempMsg[0]);
         $lat_0 = $temp1[-1];     # the last element in "temp1"
         $latPos = chop($lat_0);

         # This if statement is for the format of "17.3 S / 55.0 E",
         # the another format is "17.3S/55.0E" 
         if ($lat_0 eq "")
         {
            $lat_0 = $temp1[-2];
         }
         $lat_0 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);

         @temp2 = split(/\s+/, $tempMsg[1]);
         $j = 0;
         if ($temp2[$j] eq "")
         {
            $j++;
         }
         $lon_0 = $temp2[$j];
         
         if ($lon_0 =~ /[A-Z]$/i)
         {
            $lonPos = chop($lon_0);
         }
         else
         {
            $lonPos = $temp2[$j+1];
         }
         # This if statement is for the format of "17.3 S / 55.0 E",
         # the another format is "17.3S/55.0E"
         if ($lon_0 eq "")
         {
            $lon_0 = $temp2[$j+1];
            $lonPos = $temp2[$j+2];
         }
         if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
         {
            print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
            return $FAILURE_GETPOSITION;
         }

         # the conversion follows Mcidas definition (East-lon is negative)
         $lon_0 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         print "lat is $lat_0, pos is $latPos, lon is $lon_0, pos is $lonPos\n";          
      }

      # get the forecast position, expecting forecast for every 12 hour
      # forecast position Format 1, have to read into the forecast file
      elsif ($posLine =~ /^\s*FORECAST POSITION AND INTENSITY:/i)
      {
         open(POS3, $filename) || die "Can not open $filename: $!";
         while (defined($line = <POS3>))
         {
            if ($line =~ /^\s*FORECAST POSITION AND INTENSITY:/i)
            {
               $line = <POS3>;
               # get the 12 hr forecast
               if ($line =~ /^\s*12H, VALID/)
               {
                  $line = <POS3>;
                  if ($line =~ /^\s*\d.*\/\s*\d/)
                  {  
                     $i++;	# to check whether the 12, 24 hour forecast are all archived

                     @tempMsg = split(/\//, $line);
                     @temp1 = split(/\s+/, $tempMsg[0]);
                     $lat_12 = $temp1[-1];     # the last element in "temp1"
                     $latPos = chop($lat_12);
                     # This if statement is for the format of "17.3 S / 55.0 E", "17.3 SOUTH/  51.5 EAST"
                     # the another format is "17.3S/55.0E", or "17.3S / 55.0E"
                     # one more :17.3 SOUTH/  51.5 EAST
                     if (($lat_12 eq "") || ($lat_12 !~ /\d/))
                     {
                        $lat_12 = $temp1[-2];
                        $latPos = $temp1[-1];
                     }
                     $lat_12 *= (($latPos =~ /^[Nn]/)  ? 1.0 : -1.0);

                     @temp2 = split(/\s+/, $tempMsg[1]);
                     $j = 0;
                     if ($temp2[$j] eq "")
                     {
                       $j++;
                     }

                     $lon_12 = $temp2[$j];
                     
                     if ($lon_12 =~ /[A-Z]/i)
                     {
                        $lonPos = chop($lon_12);
                        if ($lonPos !~/[A-Z]/i)
                        {
                           $lonPos = chop($lon_12);
                        }
                     }
                     if (($lon_12 eq "") || ($lon_12 !~ /\d/))
                     {
                        $lon_12 = $temp2[$j+1];
                        $lonPos = $temp2[$j+2];
                     }
                     # remove the punctuation in the end for the one "$lon_12 = $temp2[$j]"
                     elsif ($lonPos !~ /[a-zA-Z]/)
                     {
                        $lonPos = chop($lon_12);
                     }
                     if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
                     {
                        print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
                        return $FAILURE_GETPOSITION;
                     }

                     # the conversion follows Mcidas definition (East-lon is negative)
                     $lon_12 *= (($lonPos =~ /^[Ww]/)  ? 1.0 : -1.0);

                  }     # End of "if" for the position line
               }        # End of "if" for the 12 hour forecast

               # get the 24 hour forecast
               $line = <POS3>;
               if ($line =~ /^\s*24H, VALID/)
               {
                  $line = <POS3>;
                  if ($line =~ /^\s*\d.*\/\s*\d/)
                  {
                     $i++;

                     @tempMsg = split(/\//, $line);
                     @temp1 = split(/\s+/, $tempMsg[0]);
                     $lat_24 = $temp1[-1];     # the last element in "temp1"
                     $latPos = chop($lat_24);

                     # This if statement is for the format of "17.3 S / 55.0 E",
                     # the another format is "17.3S/55.0E"
                     if (($lat_24 eq "") || ($lat_24 !~ /\d/))
                     {
                        $lat_24 = $temp1[-2];
                        $latPos = $temp1[-1];
                     }
                     # $lat_24 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
                     $lat_24 *= (($latPos =~ /^[Nn]/)  ? 1.0 : -1.0);

                     @temp2 = split(/\s+/, $tempMsg[1]);
                     $j = 0;
                     if ($temp2[$j] eq "")
                     {
                        $j++;
                     }
                     $lon_24 = $temp2[$j];
                     if ($lon_24 =~ /[A-Z]/i)
                     {
                        $lonPos = chop($lon_24);
                        if ($lonPos !~/[A-Z]/i)
                        {
                           $lonPos = chop($lon_24);
                        }
                     }

                     if (($lon_24 eq "") || ($lon_24 !~ /\d/))
                     {
                        $lon_24 = $temp2[$j+1];
                        $lonPos = $temp2[$j+2];
                     }
                     # remove the punctuation in the end for the one "$lon_24 = $temp2[$j]"
                     elsif ($lonPos !~ /[a-zA-Z]/)
                     {
                        $lonPos = chop($lon_24);
                     }

                     if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
                     {
                        print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
                        return $FAILURE_GETPOSITION;
                     }

                     # the conversion follows Mcidas definition (East-lon is negative)
                     $lon_24 *= (($lonPos =~ /^[Ww]/)  ? 1.0 : -1.0);

                  }     # End of "if" for the position line
               }        # End of "if" for the 12 hour forecast
               last;    # If the "FORECAST POSITION" paragraph is read, don't check any more lines

            }           # End of "if" for the "FORECAST POSITION" paragraph
         }              # End of the "while" to read the forecast file
         close(POS3) || die "Can not close $filename: $!";

      }                 # End of  the "elsif" for the forecast Format 1

      # forecast position Format 2
      elsif ($posLine =~ /^\s*FORECASTED POSITION\s*\d.*:.*\//i)
      {
        #  print "posLine is $posLine\n";
         # i is a counter to determine if it is 12 or 24 hour forecast
         $i++;       
         print "this is $i position.\n";
         @tempMsg = split(/:/, $posLine);
         if (@tempMsg != 2)
         {
            next;
         }
         @temp1 = split(/[\s\/]/, $tempMsg[1]);
         # skip the first element if it is empty
         $j = 0;
         if ($temp1[$j] eq "")
         {
            $j++;
         }
         $lat = $temp1[$j];
         $latPos =  $temp1[$j + 1];
         $lon = $temp1[$j + 2];
         $lonPos =  $temp1[$j + 3];

         if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
         {
            print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
            return $FAILURE_GETPOSITION;
         }

         if ($latPos =~ /^[Ss]/)
         {
            $lat *= -1;
         }
         if ($lonPos =~ /^[Ee]/)
         {
            $lon *= -1;
         }

         print "lat is $lat, pos is $latPos, lon is $lon, pos is $lonPos\n";
         if ($i == 1)
         {
            $lat_12 = $lat;
            $lon_12 = $lon;
         } 
         elsif ($i == 2)
         {
            $lat_24 = $lat;
            $lon_24 = $lon;
         }
      }  # end of "elseif" for checking the 12/24 forecast position

   }      # end of "foreach"

   if ($i < 2)
   {
      print "WARNING, there is no 24 hour forecast!!\n";
      return $FAILURE_GETPOSITION;
   }

   get6_18_points();

   return $SUCCESS;

}  	## End of subroutine getBulInfo3

############  Subroutine "getBulInfo4" ###############
############    Australia bulletin     ###############
############ RSMC Perth, RSMC Darwin   ###############
## WTAU01-07, WOAU01-02             -- what else?
##
## Please Note that: don't have the storm number in the bulletin!!! 04/14/03
##################################################

sub getBulInfo4
{
   my($i) = 0;
   my($status);

   print "This is in function getBulInfo4.\n";

   if ($wxtFile ne "")
   {
      $filename = $wxtFile;
   }
   else
   {
      system("wxtlist.k WMO=$wmohd DTIME=$hourRange DEV=T $filename");
   }


   # If no bulletin match for this WMO header get back to the main program
   if (($tempMsg = `grep "No matches found" $filename`) =~ /^No matches found$/)
   {
      print "No matches found for bulletin $wmohd\n";      ## will delete the print
      return $NOMATCH;
   }

   # currently WTAU05, 06 are for Perth, and WTAU03 for Darwin,
   # Not sure whether there is relationship, will confirm it if possible.
   # Noted by Jianbin Yang on 03/12/03

   if (($tempMsg = `grep -i "WEATHER PERTH" $filename`) ne "")
   {
      #  my($status) = getPerthBul();	# PLEASE NOTE!!!!! this is not working, don't know why
                                        # $ status get nothing!!!!
                                        # I guess if the "my" is declared in the "if", won't work
      $status = getPerthBul();    #!!! will return $status in the end of the subroutine
   }
   elsif (($tempMsg = `grep -i "DARWIN TROPICAL CYCLONE WARNING CENTRE" $filename`) ne "")
   {
      $status = getDarwinBul();
   }
   # found that WTAU01 is also from Brisbane, so add the second condition, 02/23/06
   #  elsif ($wmohd =~ /WOAU/) 
   elsif ( ($wmohd =~ /WOAU/) or (($tempMsg = `grep -i "WEATHER BRISBANE" $filename`) ne ""))
   {
      $status = getBrisbaneBul();
   }

   else
   {
      print "WMO Header is $wmohd, This is for Jakarta, try getPerthBul\n";
     # $status = getPerthBul();
      $status = getDarwinBul(); 
     # $status = getBrisbaneBul();
   }

   #-------- get the storm number --------#
   # these bulletins don't included storm number, use 99 
   # $stNumber = substr($wmohd, 4, 2);
   $stNumber = 99;

print "return status is $status\n";

   return $status;

}  	## End of subroutine getBulInfo4


###########  Subroutine "getPerthBul" ############
# Modified at 12/09/2003 for the position track
##################################################
sub getPerthBul
{
   my($lat, $lon, $latPos, $lonPos); 
   ##------------ Start to get the storm information ------------##

   #-------- get the storm name and the storm initial time --------#

   # open and read the file to build uo the complete lines for the 
   # storm name until reaching the "UTC"
   open(POS4, $filename) || die "Can not open $filename: $!";
   while (defined($line = <POS4>))
   {
      if ($line =~ /^SITUATION$/i)
      {   
         $nameLine = <POS4>;
      
         # Noted at 02/09/04, sometimes the UTC is not in the end
         # while ($nameLine !~ /\d+UTC$/)
         # changed the following on 12/27/07, not sure if that will cause some other cases failed
         #while (($nameLine !~ /\d+\s?UTC/i) && ($nameLine !~ /AREA AFFECTED/i))
         while ($nameLine !~ /AREA AFFECTED/i)
         {
            chomp $nameLine;
            if (defined($line = <POS4>))
            { 
               print "the name line in while is:\n$nameLine\n";
               $nameLine .= $line;
            }
            else
            {
               last;
            }
         }
         print "the name line outside while is:\n$nameLine\n";
         last;
      }
   }

   close(POS4) || die "Can not close $filename: $!";   

   # find out the split string for the storm name,
   # since sometimes when the line concatenate, two words may catenate

   # if ($nameLine =~ /WITH CENTRAL PRESSURE.* \d+UTC$/)
   #if ($nameLine =~ /WITH CENTRAL PRESSURE.* \d+\s?UTC/)
   if (($nameLine =~ /WITH CENTRAL PRESSURE.*\d+\s?UTC/i) || ($nameLine =~ /WITH CENTRAL PRESSURE.*\d+\s?Z/i))
   {
      $splitStr = "WITH CENTRAL PRESSURE";
   }
   # elsif ($nameLine =~ /WITHCENTRAL PRESSURE.* \d+UTC$/)
   # elsif ($nameLine =~ /WITHCENTRAL PRESSURE.* \d+\s?UTC/)
   elsif (($nameLine =~ /WITHCENTRAL PRESSURE.*\d+\s?UTC/i) || ($nameLine =~ /WITHCENTRAL PRESSURE.*\d+\s?Z/i))
   {
      $splitStr = "WITHCENTRAL PRESSURE";
   }
   # elsif ($nameLine =~ /WITH CENTRALPRESSURE.* \d+UTC$/)
   #elsif ($nameLine =~ /WITH CENTRALPRESSURE.* \d+\s?UTC/)
   elsif (($nameLine =~ /WITH CENTRALPRESSURE.*\d+\s?UTC/i) || ($nameLine =~ /WITH CENTRALPRESSURE.*\d+\s?Z/i))
   {
      $splitStr = "WITH CENTRALPRESSURE";
   }

   # add on 01/19/2006
   elsif ($nameLine =~ /was located within.*/i)
   {
       $splitStr = "was located within"; 

   }
   # add on 03/28/2006
   elsif ($nameLine =~ /located within.*/i)
   {
       $splitStr = "located within"; 

   }
   # add on 12/27/2007
   elsif ($nameLine =~ /was centred within.*/i)
   {
       $splitStr = "was centred within";

   }

   else
   {
      return $FAILURE_BUL_FORMAT;
   }

   #-------- get the storm name --------#
   # @splitMsg = split(/$splitStr/, $nameLine);
   @splitMsg = split(/$splitStr/i, $nameLine);
   @nameMsg = split(/\s+/, $splitMsg[0]);
   $stormName = $nameMsg[-1];
   $nameNNB = $nameMsg[-1];

   #if (($stormName eq "LOW") || ($stormName eq "low") )
   if ($stormName =~ /^LOW$/i )
   {
      $stormName = "PERTH_LOW";
      $nameNNB = $stormName;
   }

   # handle case "Tropical Cyclone Jacob [corrected] located within", 03/07/07
   if ($stormName =~ /\[/)
   {
      print "storm name contain [], and it was $stormName, correct it.\n";
      $stormName = $nameMsg[-2];
      $nameNNB = $stormName;
      print "Correct storm name is $stormName\n";
   }

   # Catch the Ex-tropical
   if (($splitMsg[0] =~ /EX[\-,\_,\s+]?TROPICAL/i) && ($stormName !~ /_LOW$/i ))
   {
      $stormName = "EX-" . $stormName;
      $nameNNB = $stormName;
   }

   #-------- get the storm initial time --------#
   ## @tempMsg = split(/\s+/, $splitMsg[1]);
   
   if ($splitMsg[1] =~ /UTC/)
   {
      @splitMsg2 = split(/UTC/, $splitMsg[1]);
      @tempMsg = split(/\s+/, $splitMsg2[0]);
      $iniTime = $tempMsg[-1];
   ### remove the UTC at the end
   ## $iniTime = substr($iniTime, 0, 4);
      $iniTime .= 'Z';
      print "It is UTC, iniTime is $iniTime\n";
   }
   elsif ($splitMsg[1] =~ /(\d+)\s?Z/)
   {
      $iniTime = $1;
      $iniTime .= 'Z';
      print "It is Z, iniTime is $iniTime\n";
   }

   # for the case $splitStr is "was located within"
   elsif ($splitMsg[0] =~ /(\d+)\s?UTC/)
   {
      $iniTime = $1;
      $iniTime .= 'Z';
   }

   # will get the day for it from the bulletin time

   #-------- get the time of the bulletine --------#
   $timeMsg = `grep $wmohd $filename`;
   @splitMsg = split(/\s+/, $timeMsg);
   $stationId = $splitMsg[1];
   $bulTime = $splitMsg[2];
   $bulYJd  = $splitMsg[-2];

   $iniTime = substr($bulTime, 0, 2) . $iniTime;
   $forecastHour = substr ($iniTime, 2, 2); 

print "#####the nameNNB is $nameNNB, iniTime is $iniTime, forecastHour is $forecastHour. filename is $filename\n";


   ##------------ Get the position track ------------##

   ## get the initial position

   # $intPosMsg = `grep '\[' $filename`;
   $intPosMsg = `grep -i "TUDE" $filename`;
   print "the initial positon msg is ***\n$intPosMsg\n***\n";
   if ($intPosMsg eq "")
   {
      print "WARNING, there is no initial position listed in the bulletin.\n";
      return $FAILURE_GETPOSITION;
   }
   @splitMsg = split(/\n/, $intPosMsg);
   print "split 1 is $splitMsg[0]\n";
   print "split 2 is $splitMsg[1]\n";

   # this foreach loop get the initial position
   foreach $posLine (@splitMsg)
   {
      # get the initial latitude position
      if ($posLine =~ /^\s*LATITUDE .* \[.*\]$/i)
      {
         print "this is the initial latitude position.\n";

         # Note that the position format is something like "[19.0S]"
         # So use the '[' and ']' to split the message
         @tempMsg = split(/[\[\]]/, $posLine);
     print "temp0 is $tempMsg[0]\n temp1 is $tempMsg[1]\n";
         $lat_0 = $tempMsg[1];
         $latPos = chop($lat_0);

         # Added on 01/10/05, for "LATITUDE FIFTEEN DECIMAL NINE SOUTH [15.9]", no S following the lat
         if ( $latPos =~ /\d/)
         {
            $lat_0 .= $latPos;
            @splitTemp = split(/\s+/, $tempMsg[0]);
            
            $latPos = substr($splitTemp[-1], 0, 1);
         } 
         $lat_0 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         print "initial latPos is $latPos.\n"; 
      }

      # get the longitude position
      elsif ($posLine =~ /^\s*LONGITUDE .* \[.*\]$/i)
      {
         print "this is the initial longitude position.\n";

         # Note that the position format is something like "[91.0E]"
         # So use the '[' and ']' to split the message
         @tempMsg = split(/[\[\]]/, $posLine);
     print "temp0 is $tempMsg[0]\n temp1 is $tempMsg[1]\n";
         $lon_0 = $tempMsg[1];
         $lonPos = chop($lon_0);

         if ( $lonPos =~ /\d/)
         {
            $lon_0 .= $lonPos;
            @splitTemp = split(/\s+/, $tempMsg[0]);

            $lonPos = substr($splitTemp[-1], 0, 1);
         }
         # the conversion follows Mcidas definition (East-lon is negative)
         $lon_0 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         print "lat is $lat_0, pos is $latPos, lon is $lon_0, pos is $lonPos\n";
      }  # end of "elseif" for longitude position
      
      elsif ($posLine =~ /^\s*LATITUDE\s+\d+/i)
      {
         print "this is the initial latitude position.\n";

         @tempMsg = split(/LATITUDE/i, $posLine); 
     print "temp0 is $tempMsg[0]\n temp1 is $tempMsg[1]\n";
         @tempMsg1 = split(/\s+/, $tempMsg[1]);
         $lat_0 = $tempMsg1[1];
         $latPos = $tempMsg1[2];

         $lat_0 *= (($latPos  =~ /^[Nn]/) ? 1.0 : -1.0);
         print "initial latPos is $latPos, lat_0 is $lat_0.\n";

      }

      elsif ($posLine =~ /^\s*LONGITUDE\s+\d+/i)
      {
         print "this is the initial longitude position.\n";

         @tempMsg = split(/LONGITUDE/i, $posLine);
     print "temp0 is $tempMsg[0]\n temp1 is $tempMsg[1]\n";
         @tempMsg1 = split(/\s+/, $tempMsg[1]);
         $lon_0 = $tempMsg1[1];
         $lon_0 = $tempMsg1[1];
         $lonPos = $tempMsg1[2];

         $lon_0 *= (($lonPos  =~ /^[Ww]/) ? 1.0 : -1.0);
         print "initial lonPos is $lonPos, lon_0 is $lon_0.\n";
      }

   }      # end of "foreach" for initial position

   ## get the 12/24 forecast position

   $posMsg = `grep -i "UTC " $filename`;
   print "The positon msg is ***\n$posMsg\n***\n";
   if ($posMsg eq "")
   {
      print "WARNING, there is no forecast position listed in the bulletin.\n";
      return $FAILURE_GETPOSITION;
   }
   @splitMsg1 = split(/\n/, $posMsg);

   $i = 0;
   foreach $posLine (@splitMsg1)
   {
      # get the 12/24 hour forecast position, expecting forecast for every 12 hour
      if ($posLine =~ /^\s*AT \d{4} ?UTC .* \d+\.\d+ .*H?P?A?/i)
      {
         print "posLine is $posLine\n";
         # i is a counter to determine if it is 12 or 24 hour forecast
         $i++;
         print "this is $i position.\n";

         # there are two formats for the position, For example,
         # "AT 0400 UTC 27 FEBRUARY 17.8S 119.5E 990HPA"  - 8 splits
         # "AT 1600UTC 10 MARCH  32.2 SOUTH  109.6 EAST  1002HPA" - 9 splits
         # "AT 1600UTC 09 DECEMBER 12.1S  099.5E  965HPA" - 7 splits     added on 12/09/2003
         # maybe this exist:
         # "AT 1600 UTC 09 DECEMBER 12.1 SOUTH  099.5 EAST  965HPA" - 10 splits
     
         # At 0600UTC 20 January: Within 40 nautical miles of 17.7 south  120.7 east

         @splitMsg = split(/UTC/i, $posLine);
         @tempMsg = split(/\s+/, $splitMsg[0]);
         $foreTime = $tempMsg[-1];


         $_ = $posLine;
         ($first, $second) = /(\d+\.\d+\s?\w+)\D*(\d+\.\d+\s?\w+)/;

         if (($first eq "") || ($second eq ""))
         {
            print "WARNING, forecast position can not be archived successfully.\n";
            return $FAILURE_GETPOSITION;
         }

         $lat = $first;
         # if there is space "16.5 S/SOUTH"
         if ($lat =~ /\d\s+\w/)
         {
            @tempMsg = split(/\s+/, $lat);
            $lat = $tempMsg[0];
            $latPos = $tempMsg[1];
         }
         else
         {
            $latPos = chop($lat);
         }

         $lon = $second;
         # there is space "146.5 E/EAST"
         if ($lon =~ /\d\s+\w/)
         {
            @tempMsg = split(/\s+/, $lon);
            $lon = $tempMsg[0];
            $lonPos = $tempMsg[1];
         }
         else
         {
            $lonPos = chop($lon);
         }

         if ($latPos =~ /^[Ss]/)
         {
            $lat *= -1;
         }
         # the conversion follows Mcidas definition (East-lon is negative)
         if ($lonPos =~ /^[Ee]/)
         {
            $lon *= -1;
         }

         $forecastHh = substr($foreTime, -4, 2);
         $iniHh =  substr($iniTime, 2, 2);
         $diffHh = abs ($forecastHh - $iniHh);

 print "i is $i, foreTime is $foreTime, forecastHh is $forecastHh, iniHh is $iniHh, diffHh is $diffHh\n";
         # there is 12 hour forecast
        # if (($i == 1) && ($diffHh == 12))
         if (($i == 1) && (($diffHh >= 11) && ($diffHh <= 13)) )
         {
            $lat_12 = $lat;
            $lon_12 = $lon;
         }
        # elsif (($i == 1) && ($diffHh == 0))
         elsif (($i == 1) && (($diffHh >= -1) && ($diffHh <= 1)) )
         {
            $lat_24 = $lat;
            $lon_24 = $lon;
            get12_point();
            last;
         }
        # elsif (($i == 2) && ($diffHh == 0))
         elsif (($i == 2) && (($diffHh >= -1) && ($diffHh <= 1)) )
         {
            $lat_24 = $lat;
            $lon_24 = $lon;
            last;
         }

     print "i is $i, lat is $lat, pos is $latPos, lon is $lon, pos is $lonPos.\n";

      }  # end of "if" for checking the 12/24 forecast position

   }     # end of "foreach" for forecast position

   if ($i < 2)
   {
      # There is another format that the position is in the other line of the pattern "UTC", noted on 02/27/2006
      # "At 0600UTC 28 February: Inland, within 70 nautical miles of
      #                    21.6 South 116.2 East "
      # so add the following

         open(POS4, $filename) || die "Can not open $filename: $!";
         while (defined($line = <POS4>))
         {
            if ($line =~ /^FORECAST/i)
            {
               $posLine2 = $line;

               #while ($nameLine !~ /\d+UTC/)
               while ($posLine2 !~ /WEATHER PERTH/i)
               {
                  chomp $posLine2;
                  if (defined($line = <POS4>))
                  {
                   #  print "the position line in while is:\n$posLine2\n";
                     $posLine2 .= $line;
                  }
                  else
                  {
                     last;
                  }
               }
               print "the pos line outside while is:\n$posLine2\n";
               last;
            }
         }

         close(POS4) || die "Can not close $filename: $!";


         print "The connected posLine is $posLine2\n";

         $_ = $posLine2;
         ($first, $second, $third, $forth) = /(\d+\.\d+\s?\w+)\D*(\d+\.\d+\s?\w+).*\D+(\d+\.\d+\s?\w+)\D*(\d+\.\d+\s?\w+)/;
print "%%%%first, second, third, forth is $first, $second, $third, $forth\n\n";


         if (($first eq "") || ($second eq "") || ($third eq "") || ($forth eq ""))
         {
            print "WARNING, forecast position can not be archived successfully.\n";
            return $FAILURE_GETPOSITION;
         }

         $lat_12 = $first;
         # if there is space "16.5 S/SOUTH"
         if ($lat_12 =~ /\d\s+\w/)
         {
            @tempMsg = split(/\s+/, $lat_12);
            $lat_12 = $tempMsg[0];
            $latPos = $tempMsg[1];
         }
         else
         {
            $latPos = chop($lat_12);
         }

         $lon_12 = $second;
         # there is space "146.5 E/EAST"
         if ($lon_12 =~ /\d\s+\w/)
         {
            @tempMsg = split(/\s+/, $lon_12);
            $lon_12 = $tempMsg[0];
            $lonPos = $tempMsg[1];
         }
         else
         {
            $lonPos = chop($lon_12);
         }



         if ($latPos =~ /^[Ss]/)
         {
            $lat_12 *= -1;
         }
         # the conversion follows Mcidas definition (East-lon is negative)
         if ($lonPos =~ /^[Ee]/)
         {
            $lon_12 *= -1;
         }


         $lat_24 = $third;
         # if there is space "16.5 S/SOUTH"
         if ($lat_24 =~ /\d\s+\w/)
         {
            @tempMsg = split(/\s+/, $lat_24);
            $lat_24 = $tempMsg[0];
            $latPos = $tempMsg[1];
         }
         else
         {
            $latPos = chop($lat_24);
         }

         $lon_24 = $forth;
         # there is space "146.5 E/EAST"
         if ($lon_24 =~ /\d\s+\w/)
         {
            @tempMsg = split(/\s+/, $lon_24);
            $lon_24 = $tempMsg[0];
            $lonPos = $tempMsg[1];
         }
         else
         {
            $lonPos = chop($lon_24);
         }

        if ($latPos =~ /^[Ss]/)
         {
            $lat_24 *= -1;
         }
         # the conversion follows Mcidas definition (East-lon is negative)
         if ($lonPos =~ /^[Ee]/)
         {
            $lon_24 *= -1;
         }
         
   }

   get6_18_points();
   return $SUCCESS;

}	# End of subroutin getPerthBul


############  Subroutine "getDarwinBul" ###############
##################################################

sub getDarwinBul
{
   my($lat, $lon, $latPos, $lonPos);
   my($j);

   ##------------ Start to get the storm information ------------##

   # Add the Jakarta bulletin here, the different is that the time is in hh:mm format 
   # other than hhmm format in Darwin bulletin, 12/19/2008.

   #-------- get the storm name --------#
   # open and read the bulletin file for the storm name 
   open(POS4, $filename) || die "Can not open $filename: $!";
   while (defined($line = <POS4>))
   {
      if ($line =~ /^SITUATION$/i)
      {
         $line = <POS4>;
         $nameLine = $line;
         $positionLine = $line;   # read the next line

         # if ($nameLine =~ /^AT \d+ UTC .* \d+ HPA/)
         if (($nameLine =~ /^AT \d+ UTC/i) || ($nameLine =~ /^AT \d+\:\d+ UTC/i))
         {
            @splitMsg = split(/TROPICAL /i, $nameLine);
            @nameMsg = split(/\s+/, $splitMsg[0]);
            $iniTime = $nameMsg[1];
            $iniTime =~ s/\://g;
            $iniTime .= 'Z'; 		# will get the day for it from the bulletin time
            $iniHhmm = $nameMsg[1];
            @nameMsg = split(/\s+/, $splitMsg[1]);
            $stormName = $nameMsg[1];
            #if ( ($nameMsg[0] eq "LOW") || ($nameMsg[0] eq "low") )
            if ($nameMsg[0] =~ /^LOW$/i )
            {
               $stormName = "DARWIN_LOW";
            }
            $nameNNB = $stormName;

            # check if the bulletin format is valid by the archived storm name and initial time
            if (($stormName eq "") || ($iniTime !~ /^\d+/))
            {
               return $FAILURE_BUL_FORMAT;
            }

            # Catch the Ex-tropical
            if (($nameLine =~ /EX[\-,\_,\s+]?TROPICAL/i) && ($stormName !~ /_LOW$/i ))
            {
               $stormName = "EX-" . $stormName;
               $nameNNB = $stormName;
            }

            ##-- get the initial storm position --##

            # will out of while if line is ended with a no-digit character and '.'
            # or the line is "AREA AFFECTED"
            while (($line !~ /\D\.$/) && ($line !~ /^AREA AFFECTED$/i))
            {
               chomp $positionLine;
               if (defined($line = <POS4>))
               {
                  $positionLine .= $line;
               }
               else
               {
                  last;
               }
            }	# End of the inside while loop to get initial position line
print "%%%%the initial position line outside while is:\n$positionLine%%%%\n";      
 

            $_ = $positionLine;
            ($lat_0, $lon_0) = /(\d+\.\d+)\D*(\d+\.\d+)/;
            ($latPos, $lonPos) = /\d+\.\d+\s?([A-Za-z]+).*\d+\.\d+\s?([A-Za-z]+)/;

            $lat_0 *= (($latPos  =~ /^[Nn]/) ? 1.0 : -1.0);
            $lon_0 *= (($lonPos =~ /^[Ww]/)  ? 1.0 : -1.0);
print "### the lat 0 is $lat_0 , latPos is $latPos, lon_0 is $lon_0, lonPos is $lonPos\n";          
         }	# End of the if to get storm name and position
                # for the 'if' $nameline match the 'UTC'

         last;  # exit the while loop if above match
      }		# End of the 'if' $line match 'SITUATION'
   }  		# End of the outside while loop
   
   if ($nameLine eq "")
   {
      return $FAILURE_BUL_FORMAT;
   }
 
   close(POS4) || die "Can not close $filename: $!";

   #-------- get the time of the bulletine --------#
   $timeMsg = `grep $wmohd $filename`;
   @splitMsg = split(/\s+/, $timeMsg);
   $stationId = $splitMsg[1];
   $bulTime = $splitMsg[2];
   $bulYJd  = $splitMsg[-2];
   $iniTime = substr($bulTime, 0, 2) . $iniTime;
   $forecastHour = substr ($iniTime, 2, 2);

print "%%%%%%%% the storm name is $stormName, iniTime is $iniTime, lat_0 is $lat_0, lon_0 is $lon_0, bulYJd is $bulYJd,
                bulTime is $bulTime.\n";

   ##------------ Get the position track ------------##

   ## get the 12/24 forecast position
   $posMsg = `grep -i "UTC " $filename`;
   print "the positon msg is ***\n$posMsg\n***\n";
   if ($posMsg eq "")
   {
      print "WARNING, there is no forecast position listed in the bulletin.\n";
      return $FAILURE_GETPOSITION;
   }
   @splitMsg = split(/\n/, $posMsg);

   $i = 0;
   # assign the initial hour and minute to previous forecast time
   #$preForecaseTime = $iniHhmm;
   $lat_18_real = "";
   $lon_18_real = "";

   foreach $posLine (@splitMsg)
   {
      # get the 12/24 hour forecast position, expecting forecast for every 12 hour
      # but sometimes the format change, maybe 18 hour forecast show up
      #if ($posLine =~ /^\d{4} UTC .*\: .*\d+\s*HPA/i)

      # add the second condition on 02/06/2007
      # Add the third one for Jakarta on 12/19/2008
      if (($posLine =~ /^\d{4} UTC .*\: .*\d+\s*HPA/i) || ($posLine =~ /^\d{4} UTC .*\: .*\d+/i) 
            || ($posLine =~ /^\d{2}\:\d{2} UTC .*\: .*\d+/i))
      {
         #print "posLine is $posLine\n";
         print "First format: posLine is $posLine\n";

         # i is a counter to determine if it is 12 or 24 hour forecast
         $i++;
         print "this is $i position.\n";
         @tempMsg = split(/\s+/, $posLine);
         $currForecaseTime = $tempMsg[0];
         $currForecaseTime =~ s/\://g;
         # curModifyTime is for subtraction
         $curModifyTime = $currForecaseTime;
         if ($curModifyTime eq "0000")
         {
            $curModifyTime = 2400;
         }


 #        $lat = $tempMsg[4];
 #        if ($tempMsg[5] =~ /^\d/)
 #        {
#print "POSITION is close to digit.\n";
 #           $latPos = chop($lat);
 #           $lon = $tempMsg[5];
 #           $lonPos = chop($lon);
 #           # in case there is '.' in the end of longitude, such as "137.6E."
 #           if ($lonPos !~ /^[A-Za-z]/)
 #           {
 #              $lonPos = chop($lon);
 #           }
 #        }
 #        else 
 #        {
 #           $latPos =  $tempMsg[5];
 #           $lon = $tempMsg[6];
 #           $lonPos =  $tempMsg[7];
 #        }

         $_ = $posLine;
         ($first, $second) = /(\d+\.\d+\s?\w+)\D*(\d+\.\d+\s?\w+)/;
   print "!!!NEW!!! first is $first, second is $second\n\n";

         if (($first eq "") || ($second eq ""))
         {
            print "WARNING, forecast position can not be archived successfully.\n";
            return $FAILURE_GETPOSITION;
         }

         $lat = $first;

         # if there is space "16.5 S/SOUTH"
         if ($lat =~ /\d\s+\w/)
         {
            @tempMsg = split(/\s+/, $lat);
            $lat = $tempMsg[0];
            $latPos = $tempMsg[1];
         }
         else
         {
            $latPos = chop($lat);
         }

         $lon = $second;
         # there is space "146.5 E/EAST"
         if ($lon =~ /\d\s+\w/)
         {
            @tempMsg = split(/\s+/, $lon);
            $lon = $tempMsg[0];
            $lonPos = $tempMsg[1];
         }
         else
         {
            $lonPos = chop($lon);
         }

print "****lat is $lat, pos is $latPos, lon is $lon, pos is $lonPos\n";

         if ($latPos =~ /^[Ss]/)
         {
            $lat *= -1;
         }
         if ($lonPos =~ /^[Ee]/)
         {
            $lon *= -1;
         }

         print "lat is $lat, pos is $latPos, lon is $lon, pos is $lonPos\n";
         if ($i == 1)
         {
            $lat_12 = $lat;
            $lon_12 = $lon;
         }
         elsif (($i == 2) && (abs($curModifyTime - $preForecastTime) == 1200))
         {
            $lat_24 = $lat;
            $lon_24 = $lon;
         }
         elsif (($i == 2) && (abs($curModifyTime - $preForecastTime) == 600))
         {
            $lat_18_real = $lat;
            $lon_18_real = $lon;
         }
         elsif (($i == 3) && (abs($curModifyTime - $preForecastTime) == 600))
         {
            $lat_24 = $lat;
            $lon_24 = $lon;
         }

         $preForecastTime = $currForecaseTime;

      }  # end of "if" for checking the 12/24 forecast position

      # else do something for the new format 12/18
      elsif ($posLine =~ /^\d{4} UTC .*\:$/i)
      {
print "## another format:$posLine\n";
         $posMsg2 = `grep -i "PA" $filename`;
         @splitMsg2 = split(/\n/, $posMsg2);
         
         $i = 0;
         foreach $posLine2 (@splitMsg2)
         {
            # get the 12/24 hour forecast position, expecting forecast for every 12 hour
            if ($posLine2 =~ /^\s*\d+.*\d+H?PA.*KNOTS/i)      
            {  
               print "posLine is $posLine2\n";
               # i is a counter to determine if it is 12 or 24 hour forecast
               $i++;
               @tempMsg = split(/\s+/, $posLine2);
               # skip the first element if it is empty
               $j = 0;
               if ($tempMsg[$j] eq "")
               {
                  $j++;
               }

               $lat = $tempMsg[$j++];
               $latPos = $tempMsg[$j++];
               $lon = $tempMsg[$j++];
               $lonPos = $tempMsg[$j++];
               $lat *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
               # Sometimes there is ',' at the end of the lon position
               $lon *= (($lonPos =~ /^[Ww]/)  ? 1.0 : -1.0);

print "### the lat is $lat , latPos is $latPos, lon is $lon, lonPos is $lonPos\n";

               if ($i == 1)
               {
                  $lat_12 = $lat;
                  $lon_12 = $lon;
               }
               elsif ($i == 2)
               {
                  $lat_24 = $lat;
                  $lon_24 = $lon;
               }

            } # end of the if "posLine2"  

         }    # end of the foreach "posLine2"
      last;

      }       # end of elsif "posLine"


   }          # end of "foreach" for forecast position

   if ($i < 2)
   {
      print "WARNING, there is no 24 hour forecast!!\n";
      return $FAILURE_GETPOSITION;
   }

   get6_18_points();

   if (($lat_18_real ne "") && ($lat_18_real ne ""))
   {
      $lat_18 = $lat_18_real;
      $lon_18 = $lon_18_real;
   }
   return $SUCCESS;

}       ## End of subroutine getDarwinBul





######################


###########  Subroutine "getBrisbaneBul" ############
# Add this subroutine on 02/11/2004 
##################################################
sub getBrisbaneBul
{
   my($lat, $lon, $latPos, $lonPos); 
   my($j);
   ##------------ Start to get the storm information ------------##

   #-------- get the storm name and the storm initial time --------#

   # open and read the file to build uo the complete lines for the
   # storm name until reaching the "UTC"
   open(POS4, $filename) || die "Can not open $filename: $!";
   while (defined($line = <POS4>))
   {
      if ($line =~ /^SITUATION$/i)
      {
         $nameLine = <POS4>;

         #while ($nameLine !~ /\d+UTC/)
         while ($nameLine !~ /AREA AFFECTED/i)
         {
            chomp $nameLine;
            if (defined($line = <POS4>))
            {
               print "the name line in while is:\n$nameLine\n";
               $nameLine .= $line;
            }
            else
            {
               last;
            }
         }
         print "the name line outside while is:\n$nameLine\n";
         last;
      }
   }

   close(POS4) || die "Can not close $filename: $!";

   #-------- get the storm name --------#

   # add if to handle "A 996 HPA LOW IS LOCATED" as well as "TROPICAL LOW WITH" , 03/18/04
   # add to handle "A BROAD LOW WITH A CENTRAL PRESSURE OF " , 01/24/2005

   if ($nameLine =~ /TROPICAL/i)
   {
      @splitMsg = split(/TROPICAL /i, $nameLine);
     # @nameMsg = split(/\s+/, $splitMsg[0]);
      @nameMsg = split(/\s+/, $splitMsg[1]);

      #if ( ($nameMsg[0] eq "LOW") || ($nameMsg[0] eq "low") )
      if ($nameMsg[0] =~ /^LOW/i)
      {
         #$stormName = $nameMsg[0];
         $stormName = "LOW";
      }
      else
      {
         $stormName = $nameMsg[1];
      }
print "---first if - TROPICAL, stormName is $stormName\n\n";
   }
   elsif ($nameLine =~ /LOCATED AT/i)
   {
      @splitMsg = split(/LOCATED AT/i, $nameLine);
      @nameMsg = split(/\s+/, $splitMsg[0]);
      # @nameMsg = split(/\s+/, $splitMsg[1]);
      if (($nameMsg[-1] eq "IS") || ($nameMsg[-1] eq "is") )
      {
         $stormName = $nameMsg[-2];
      }
      else
      {
         $stormName = $nameMsg[-1];
      }
print "---second if - LOCATED AT, stormName is $stormName\n\n";
   }
   elsif ($nameLine =~ /CENTRAL PRESSURE/i)
   {
      @splitMsg = split(/CENTRAL PRESSURE/i, $nameLine);
      @nameMsg = split(/\s+/, $splitMsg[0]);
      # @nameMsg = split(/\s+/, $splitMsg[1]);
      if (($nameMsg[-1] eq "A") || ($nameMsg[-1] eq "a") )
      {
         $stormName = $nameMsg[-3];
      }
      elsif (($nameMsg[-1] eq "WITH") ||($nameMsg[-1] eq "with" ))
      {
         $stormName = $nameMsg[-2];
      }
      else
      {
         $stormName = $nameMsg[-1];
      }
print "---third if - CENTRAL PRESSURE, stormName is $stormName\n\n";
   }


   #if (($stormName eq "LOW") || ($stormName eq "low"))
   if ($stormName =~ /^LOW$/i)
   {
      $stormName = "BRISBANE_LOW";
   }
   $nameNNB = $stormName;

   # Catch the Ex-tropical
   if (($nameLine =~ /EX[\-,\_,\s+]?TROPICAL/i) && ($stormName !~ /_LOW$/i ))
   {
       $stormName = "EX-" . $stormName;
       $nameNNB = $stormName;
   }

print "!!!!!the storm name is $stormName\n";

   #-------- get the time of the bulletine --------#
   $timeMsg = `grep $wmohd $filename`;
   @splitMsg = split(/\s+/, $timeMsg);
   $stationId = $splitMsg[1];
   $bulTime = $splitMsg[2];
   $bulYJd  = $splitMsg[-2];


   #-------- get the storm initial time --------#
   ## @tempMsg = split(/\s+/, $splitMsg[1]);

   if ($nameLine =~ /\d\s?UTC/i)
   {
      @splitMsg = split(/UTC/i, $nameLine);
      @tempMsg = split(/\s+/, $splitMsg[0]);
      $iniTime = $tempMsg[-1];
      # remove the "AT" in the beginning if there is any
      $iniTime = substr($iniTime, -6);
   }
   else
   {
      $iniTime = substr($bulTime, 0, 4) . "00";
   }

   $iniTime .= 'Z';

   # 03/18/04, some time there is 1800Z only in the initial time, no date in it
   if ($iniTime =~ /^\d{4}Z/)
   {
      $iniTime = substr($bulTime, 0, 2) . $iniTime;
   }

   $forecastHour = substr ($iniTime, 2, 2);


   # check if the bulletin format is valid by the archived storm name and initial time
   if (($stormName eq "") || ($iniTime !~ /^\d+/))
   {
      return $FAILURE_BUL_FORMAT;
   }



print "%%%%%%%%%the storm name is $stormName, the iniTime is $iniTime, bulTime is $bulTime, bulYJd is $bulYJd, forecastHour is $forecastHour.\n";


   ##------------ Get the position track ------------##

   ## get the initial position

   # for "10.0 DEGREES SOUNTH AND 153.0 DEGREES EAST" format
   if ($nameLine =~ /\d DEGREES/i)
   {
      $_ = $nameLine;
      # ($first, $second) = /(\d+.\d+\s?DEGREES\s?\w+).*\W(\d+.\d+\s?DEGREES\s?\w+)/;
      ($first, $second) = /(\d+.\d+\s?DEGREES\s?\w+)\D*(\d+.\d+\s?DEGREES\s?\w+)/i;

      # get initial latitude

      @splitMsg = split(/DEGREES/i, $first);
      @tempMsg = split(/\s+/, $splitMsg[0]);

      # skip the first element if it is empty
      $j = -1;
      if ($tempMsg[$j] eq "")
      {
         $j--;
      }

      $lat_0 = $tempMsg[$j];

      @tempMsg = split(/\s+/, $splitMsg[1]);


      # skip the first element if it is empty
      $j = 0;
      if ($tempMsg[$j] eq "")
      {
         $j++;
      }

      $latPos = $tempMsg[$j];

      # get initial longitude

      @splitMsg = split(/DEGREES/i, $second);
      @tempMsg = split(/\s+/, $splitMsg[0]);

      # skip the first element if it is empty
      $j = -1;
      if ($tempMsg[$j] eq "")
      {
         $j--;
      }

      $lon_0 = $tempMsg[$j];

      @tempMsg = split(/\s+/, $splitMsg[1]);

      # skip the first element if it is empty
      $j = 0;
      if ($tempMsg[$j] eq "")
      {
         $j++;
      }

      $lonPos = $tempMsg[$j];


   }

   # for "16.5S 146.5E" or "15.6S146.3EMOVING " format
   else
   {
      $_ = $nameLine;
      # ($first, $second) = /(\d+\.\d+\s?\w+)\D*(\d+\.\d+\s?\w+)/;
      #($first, $second) = /(\d+\.\d+\s?[a-zA-Z]+)\D*(\d+\.\d+\s?\w+)/;
      #($first, $second) = /NEAR\s*(\d+\.?\d?\s?[a-zA-Z]+)\D*(\d+\.?\d?\s?\w+)/;

      # changed on 01/10/2005, for "WITH A CENTRAL PRESSURE OF 970 HECTOPASCALSCENTRED AT 19.2S 159.3E AT "  
      ($first, $second) = /N?E?A?R?\s*(\d+\.?\d?\s?S[a-zA-Z]*)\D*(\d+\.?\d?\s?\w+)/i;
print "FIRST is $first, second is $second\n";

      $lat_0 = $first;

      # there is space "16.5 S/SOUTH"
      if ($lat_0 =~ /\d\s+\w/)
      {
         @tempMsg = split(/\s+/, $lat_0);
         $lat_0 = $tempMsg[0];
         $latPos = $tempMsg[1];
      }
      # there is no space between lat and pos "NEAR 19.0SOUTH  148.5 EAST"
      else
      {
         $lat_0 =~ /[a-zA-Z]+$/;
         $latPos = $&;          # get the part of string that matched
         $lat_0 =~ /^\d+\.\d/;
         $lat_0 = $&;

         # not simply use the following, because sometime when connected the text
         #NEAR 19.0SOUTH 148.5 EAST
         # $latPos = chop($lat_0);


      }

      $lon_0 = $second;
      # there is space "146.5 E/EAST"
      if ($lon_0 =~ /\d\s+\w/)
      {
         @tempMsg = split(/\s+/, $lon_0);
         $lon_0 = $tempMsg[0];
         $lonPos = $tempMsg[1];
      }
      else
      {
         $lon_0 =~ /[a-zA-Z]+$/;
         $lonPos = $&;          # get the part of string that matched
         $lon_0 =~ /^\d+\.\d/;
         $lon_0 = $&;

         # not simply use the following, because sometime when connected the text
         # it might be like that:"15.6S146.3EMOVING"
         # $lonPos = chop($lon_0);

      }

   }

   $lat_0 *= (($latPos =~ /^[Nn]/) ? 1.0 : -1.0);
   $lon_0 *= (($lonPos =~ /^[Ww]/)  ? 1.0 : -1.0);

   if (($first eq "") || ($second eq ""))
   {
      print "WARNING, there is no initial position listed in the bulletin.\n";
      return $FAILURE_GETPOSITION;
   }


 print "%%%%%%%%%%%lat_0, latPos, lon_0, and lonPos is $lat_0 $latPos $lon_0 and $lonPos\n";


   ## get the 12/24 forecast position

   $posMsg = `grep -i "UTC NEAR" $filename`;
   print "the position msg is ***\n$posMsg\n***\n";

   @splitMsgTmp = split(/\n/, $posMsg);

   # also check if the match line is less than 2, if so, check the FORECAST POSITION, 02/23/06 
   if (($posMsg eq "") || ($#splitMsgTmp + 1 < 2) )
   {
      $posMsg = `grep -i "FORECAST POSITION AT" $filename`;
      if ($posMsg eq "")
      {
         print "WARNING, there is no forecast position listed in the bulletin.\n";
         return $FAILURE_GETPOSITION;
      }
   }
   @splitMsg1 = split(/\n/, $posMsg);

   $i = 0;

   foreach $posLine (@splitMsg1)
   {

      # Add this on 01/24/2005, for the format:
      # "FORECAST POSITION AT 250000 UTC NEAR 20.3 S 150.5 E AND AT 251200 UTC
      # NEAR 21.2
      # S 152.1 E."
      if ($posLine =~ /FORECAST POSITION AT .+ UTC NEAR .+ UTC/i)
      {
         print "posLine is $posLine\n";

         # to get the forecast position line
         open(POS4, $filename) || die "Can not open $filename: $!";
         while (defined($line = <POS4>))
         {
            if ($line =~ /^FORECAST POSITION AT/i)
            {
               $posLine2 = $line;

               #while ($nameLine !~ /\d+UTC/)
               while ($posLine2 !~ /REMARKS/i)
               {
                  chomp $posLine2;
                  if (defined($line = <POS4>))
                  {
                     print "the position line in while is:\n$posLine2\n";
                     $posLine2 .= $line;
                  }
                  else
                  {
                     last;
                  }
               }
               print "the pos line outside while is:\n$posLine2\n";
               last;
            }
         }

         close(POS4) || die "Can not close $filename: $!";

         print "The connected posLine is $posLine2\n";

         ## i is a counter to determine if it is 12 or 24 hour forecast
         # $i++;
         # print "This is $i position.\n";

         @splitMsg = split(/UTC/i, $posLine2);
         @tempMsg = split(/\s+/, $splitMsg[0]);
         # @tempMsg = split(/\s+/, $splitMsg[0]);
         # $foreTime = $tempMsg[-1];


         ## if it is initial time , go to next
         # if ($iniTime =~ /$foreTime/)
         # {
         #    print "This is the initial time, go to next.\n";
         #   $i--;
         #   next;
         # }


         $_ = $posLine2;
         ($first, $second, $third, $forth) = /(\d+\.\d+\s?\w+)\D*(\d+\.\d+\s?\w+).*\D+(\d+\.\d+\s?\w+)\D*(\d+\.\d+\s?\w+)/;
print "%%%%first, second, third, forth is $first, $second, $third, $forth\n\n";


         if (($first eq "") || ($second eq "") || ($third eq "") || ($forth eq ""))
         {
            print "WARNING, forecast position can not be archived successfully.\n";
            return $FAILURE_GETPOSITION;
         }

         $lat_12 = $first;
         # if there is space "16.5 S/SOUTH"
         if ($lat_12 =~ /\d\s+\w/)
         {
            @tempMsg = split(/\s+/, $lat_12);
            $lat_12 = $tempMsg[0];
            $latPos = $tempMsg[1];
         }
         else
         {
            $latPos = chop($lat_12);
         }

         $lon_12 = $second;
         # there is space "146.5 E/EAST"
         if ($lon_12 =~ /\d\s+\w/)
         {
            @tempMsg = split(/\s+/, $lon_12);
            $lon_12 = $tempMsg[0];
            $lonPos = $tempMsg[1];
         }
         else
         {
            $lonPos = chop($lon_12);
         }



         # there are two formats for the position, For example,
         # "AT 0400 UTC 27 FEBRUARY 17.8S 119.5E 990HPA"  - 8 splits
         # "AT 1600UTC 10 MARCH  32.2 SOUTH  109.6 EAST  1002HPA" - 9 splits
         # "AT 1600UTC 09 DECEMBER 12.1S  099.5E  965HPA" - 7 splits     added on 12/09/2003
         # maybe this exist:
         # "AT 1600 UTC 09 DECEMBER 12.1 SOUTH  099.5 EAST  965HPA" - 10 splits

         if ($latPos =~ /^[Ss]/)
         {
            $lat_12 *= -1;
         }
         # the conversion follows Mcidas definition (East-lon is negative)
         if ($lonPos =~ /^[Ee]/)
         {
            $lon_12 *= -1;
         }


         $lat_24 = $third;
         # if there is space "16.5 S/SOUTH"
         if ($lat_24 =~ /\d\s+\w/)
         {
            @tempMsg = split(/\s+/, $lat_24);
            $lat_24 = $tempMsg[0];
            $latPos = $tempMsg[1];
         }
         else
         {
            $latPos = chop($lat_24);
         }

         $lon_24 = $forth;
         # there is space "146.5 E/EAST"
         if ($lon_24 =~ /\d\s+\w/)
         {
            @tempMsg = split(/\s+/, $lon_24);
            $lon_24 = $tempMsg[0];
            $lonPos = $tempMsg[1];
         }
         else
         {
            $lonPos = chop($lon_24);
         }



         # there are two formats for the position, For example,
         # "AT 0400 UTC 27 FEBRUARY 17.8S 119.5E 990HPA"  - 8 splits
         # "AT 1600UTC 10 MARCH  32.2 SOUTH  109.6 EAST  1002HPA" - 9 splits
         # "AT 1600UTC 09 DECEMBER 12.1S  099.5E  965HPA" - 7 splits     added on 12/09/2003
         # maybe this exist:
         # "AT 1600 UTC 09 DECEMBER 12.1 SOUTH  099.5 EAST  965HPA" - 10 splits

         if ($latPos =~ /^[Ss]/)
         {
            $lat_24 *= -1;
         }
         # the conversion follows Mcidas definition (East-lon is negative)
         if ($lonPos =~ /^[Ee]/)
         {
            $lon_24 *= -1;
         }



        # $forecastHh = substr($foreTime, 2, 2);
        # $iniHh =  substr($iniTime, 2, 2);
        # $diffHh = abs ($forecastHh - $iniHh);


        # # there is 12 hour forecast
        # #if (($i == 1) && ($diffHh == 12))
        # if (($i == 1) && (($diffHh >= 11) && ($diffHh <= 13)) )
        # {
        #    $lat_12 = $lat;
        #    $lon_12 = $lon;
        # }
        # #elsif (($i == 1) && ($diffHh == 0))
        # elsif (($i == 1) && (($diffHh >= -1) && ($diffHh <= 1)) )
        # {
        #    $lat_24 = $lat;
        #    $lon_24 = $lon;
        #    get12_point();
        #    last;
        # }
       ##  elsif (($i == 2) && ($diffHh == 0))
       #  elsif (($i == 2) && (($diffHh >= -1) && ($diffHh <= 1)) )
       #  {
       #     $lat_24 = $lat;
       #     $lon_24 = $lon;
       #     last;
       #  }

     #print "lat is $lat, pos is $latPos, lon is $lon, pos is $lonPos\n";

      }  # end of "if" for checking the 12/24 forecast position for "\d+UTC NEAR..AND..UTC"


      # get the 12/24 hour forecast position, expecting forecast for every 12 hour
      elsif ($posLine =~ /^FORECAST POSITION .*UTC NEAR/i)
      {
         print "posLine is $posLine\n";
         # i is a counter to determine if it is 12 or 24 hour forecast
         $i++;
         print "this is $i position.\n";

         @splitMsg = split(/UTC/i, $posLine);
         @tempMsg = split(/\s+/, $splitMsg[0]);
         $foreTime = $tempMsg[-1];


         # if it is initial time , go to next
         if ($iniTime =~ /$foreTime/)
         {
            print "This is the initial time, go to next.\n";
            $i--;
            next;
         }


         $_ = $posLine;
       
         # some has DEGREES infront of EAST, such as "FORECAST POSITION 110600UTC NEAR 15.0 DEGREES SOUTH 142.8 EAST"
       #  ($first, $second) = /(\d+\.\d+\s?\w+)\D*(\d+\.\d+\s?\w+)/;
         
         # ($first, $second) = /(\d+\.\d+\s?\w*D?E?G?R?E?E?S?\s?\w*)\D*(\d+\.\d+\s?\w*D?E?G?R?E?E?S?\s?\w*)/i;
         ($first, $second) = /(\d+\.\d+\s?\w*D?E?G?R?E?E?S?\s?[A-Z]*)\D*(\d+\.\d+\s?\w*D?E?G?R?E?E?S?\s?[A-Z]*)/i;

print "first is **$first**, second is **$second**\n";

         if (($first eq "") || ($second eq ""))
         {
            print "WARNING, forecast position can not be archived successfully.\n";
            return $FAILURE_GETPOSITION;
         }

         $lat = $first;
         # if there is space "16.5 S/SOUTH"
         if ($lat =~ /\d\s+\w/)
         {
            @tempMsg = split(/\s+/, $lat);
            $lat = $tempMsg[0];
            $latPos = $tempMsg[1];
            if ($latPos =~ /DEGREES/i)
            {
               $latPos = $tempMsg[2];
            }
         }
         else
         {
            $latPos = chop($lat);

            # for the one without DEGREE, most likely will have white space after the 'S' - "15.5S "
            if ($latPos eq " ")
            {
               $latPos = chop($lat);
            }
         }

         $lon = $second;
         # there is space "146.5 E/EAST"
         if ($lon =~ /\d\s+\w/)
         {
            @tempMsg = split(/\s+/, $lon);
            $lon = $tempMsg[0];
            $lonPos = $tempMsg[1];
            if ($lonPos =~ /DEGREES/i)
            {
               $lonPos = $tempMsg[2];
            }

         }
         else
         {
            $lonPos = chop($lon);
            if ($lonPos eq " ")
            {
               $lonPos = chop($lon);
            }

         }


         # there are two formats for the position, For example,
         # "AT 0400 UTC 27 FEBRUARY 17.8S 119.5E 990HPA"  - 8 splits
         # "AT 1600UTC 10 MARCH  32.2 SOUTH  109.6 EAST  1002HPA" - 9 splits
         # "AT 1600UTC 09 DECEMBER 12.1S  099.5E  965HPA" - 7 splits     added on 12/09/2003
         # maybe this exist:
         # "AT 1600 UTC 09 DECEMBER 12.1 SOUTH  099.5 EAST  965HPA" - 10 splits

         if ($latPos =~ /^[Ss]/)
         {
            $lat *= -1;
         }
         # the conversion follows Mcidas definition (East-lon is negative)
         if ($lonPos =~ /^[Ee]/)
         {
            $lon *= -1;
         }


         $forecastHh = substr($foreTime, 2, 2);
         $iniHh =  substr($iniTime, 2, 2);
         $diffHh = abs ($forecastHh - $iniHh);


         # there is 12 hour forecast
         #if (($i == 1) && ($diffHh == 12))
         if (($i == 1) && (($diffHh >= 11) && ($diffHh <= 13)) )
         {
            $lat_12 = $lat;
            $lon_12 = $lon;
         }
         #elsif (($i == 1) && ($diffHh == 0))
         elsif (($i == 1) && (($diffHh >= -1) && ($diffHh <= 1)) )
         {
            $lat_24 = $lat;
            $lon_24 = $lon;
            get12_point();
            last;
         }
       #  elsif (($i == 2) && ($diffHh == 0))
         elsif (($i == 2) && (($diffHh >= -1) && ($diffHh <= 1)) )
         {
            $lat_24 = $lat;
            $lon_24 = $lon;
            last;
         }

     print "lat is $lat, pos is $latPos, lon is $lon, pos is $lonPos\n";

      }  # end of "elsif" for checking the 12/24 forecast position for "FORECAST POSITION .*UTC NEAR"

      #elsif ($posLine =~ /^\s*A?T?\s*\d+UTC NEAR/)
      elsif ($posLine =~ /UTC NEAR/i)
      {
         print "posLine is $posLine\n";
         # i is a counter to determine if it is 12 or 24 hour forecast
         $i++;
         print "This is $i position.\n";

         @splitMsg = split(/UTC/i, $posLine);
         @tempMsg = split(/\s+/, $splitMsg[0]);
         $foreTime = $tempMsg[-1];


         # if it is initial time , go to next
         if ($iniTime =~ /$foreTime/)
         {
            print "This is the initial time, go to next.\n";
            $i--;
            next;
         }


         $_ = $posLine;
         ($first, $second) = /(\d+\.\d+\s?\w+)\D*(\d+\.\d+\s?\w+)/;



         if (($first eq "") || ($second eq ""))
         {
            print "WARNING, forecast position can not be archived successfully.\n";
            return $FAILURE_GETPOSITION;
         }

         $lat = $first;
         # if there is space "16.5 S/SOUTH"
         if ($lat =~ /\d\s+\w/)
         {
            @tempMsg = split(/\s+/, $lat);
            $lat = $tempMsg[0];
            $latPos = $tempMsg[1];
         }
         else
         {
            $latPos = chop($lat);
         }

         $lon = $second;
         # there is space "146.5 E/EAST"
         if ($lon =~ /\d\s+\w/)
         {
            @tempMsg = split(/\s+/, $lon);
            $lon = $tempMsg[0];
            $lonPos = $tempMsg[1];
         }
         else
         {
            $lonPos = chop($lon);
         }


         # there are two formats for the position, For example,
         # "AT 0400 UTC 27 FEBRUARY 17.8S 119.5E 990HPA"  - 8 splits
         # "AT 1600UTC 10 MARCH  32.2 SOUTH  109.6 EAST  1002HPA" - 9 splits
         # "AT 1600UTC 09 DECEMBER 12.1S  099.5E  965HPA" - 7 splits     added on 12/09/2003
         # maybe this exist:
         # "AT 1600 UTC 09 DECEMBER 12.1 SOUTH  099.5 EAST  965HPA" - 10 splits


         if ($latPos =~ /^[Ss]/)
         {
            $lat *= -1;
         }
         # the conversion follows Mcidas definition (East-lon is negative)
         if ($lonPos =~ /^[Ee]/)
         {
            $lon *= -1;
         }


         $forecastHh = substr($foreTime, 2, 2);
         $iniHh =  substr($iniTime, 2, 2);
         $diffHh = abs ($forecastHh - $iniHh);

         # there is 12 hour forecast
        # if (($i == 1) && ($diffHh == 12))
         if (($i == 1) && (($diffHh >= 11) && ($diffHh <= 13)) )
         {
            $lat_12 = $lat;
            $lon_12 = $lon;
         }
        # elsif (($i == 1) && ($diffHh == 0))
         elsif (($i == 1) && (($diffHh >= -1) && ($diffHh <= 1)) )
         {
            $lat_24 = $lat;
            $lon_24 = $lon;
            get12_point();
            last;
         }
        # elsif (($i == 2) && ($diffHh == 0))
         elsif (($i == 2) && (($diffHh >= -1) && ($diffHh <= 1)) )
         {
            $lat_24 = $lat;
            $lon_24 = $lon;
            last;
         }

     print "i is $i, lat is $lat, pos is $latPos, lon is $lon, pos is $lonPos\n";
      }  # end of "elsif" for checking the 12/24 forecast position for "\d+UTC NEAR"


      # Add this on 04/16/04, for the format:
      # "FORECAST POSITION AT 160600 12.4 147.6 1001 HPA"

      elsif ($posLine =~ /FORECAST POSITION AT \d+/i)
      {
         print "posLine is $posLine\n";
         # i is a counter to determine if it is 12 or 24 hour forecast
         $i++;
         print "this is $i position.\n";

         @splitMsg = split(/AT/i, $posLine);
         @tempMsg = split(/\s+/, $splitMsg[1]);
         $foreTime = $tempMsg[1];
         # in case there is UTC following the digits
         $foreTime = substr($foreTime, 0, 6);

         $_ = $posLine;
         ($first, $second) = /(\d+\.\d+\s?\w+)\D*(\d+\.\d+\s?\w+)/;

         if (($first eq "") || ($second eq ""))
         {
            print "WARNING, forecast position can not be archived successfully.\n";
            return $FAILURE_GETPOSITION;
         }

         $lat = $first;
         # if there is space "16.5 S/SOUTH"
         if ($lat =~ /\d\s+\w/)
         {
            @tempMsg = split(/\s+/, $lat);
            $lat = $tempMsg[0];
            $latPos = $tempMsg[1];
         }
         else
         {
            $latPos = chop($lat);
         }

         $lon = $second;
         # there is space "146.5 E/EAST"
         if ($lon =~ /\d\s+\w/)
         {
            @tempMsg = split(/\s+/, $lon);
            $lon = $tempMsg[0];
            $lonPos = $tempMsg[1];
         }
         else
         {
            $lonPos = chop($lon);
         }


         # there are two formats for the position, For example,
         # "AT 0400 UTC 27 FEBRUARY 17.8S 119.5E 990HPA"  - 8 splits
         # "AT 1600UTC 10 MARCH  32.2 SOUTH  109.6 EAST  1002HPA" - 9 splits
         # "AT 1600UTC 09 DECEMBER 12.1S  099.5E  965HPA" - 7 splits     added on 12/09/2003
         # maybe this exist:
         # "AT 1600 UTC 09 DECEMBER 12.1 SOUTH  099.5 EAST  965HPA" - 10 splits

         if ($latPos =~ /^[Ss]/)
         {
            $lat *= -1;
         }
         # the conversion follows Mcidas definition (East-lon is negative)
         if ($lonPos =~ /^[Ee]/)
         {
            $lon *= -1;
         }


         $forecastHh = substr($foreTime, 2, 2);
         $iniHh =  substr($iniTime, 2, 2);
         $diffHh = abs ($forecastHh - $iniHh);


         # there is 12 hour forecast
         #if (($i == 1) && ($diffHh == 12))
         if (($i == 1) && (($diffHh >= 11) && ($diffHh <= 13)) )
         {
            $lat_12 = $lat;
            $lon_12 = $lon;
         }
         #elsif (($i == 1) && ($diffHh == 0))
         elsif (($i == 1) && (($diffHh >= -1) && ($diffHh <= 1)) )
         {
            $lat_24 = $lat;
            $lon_24 = $lon;
            get12_point();
            last;
         }
       #  elsif (($i == 2) && ($diffHh == 0))
         elsif (($i == 2) && (($diffHh >= -1) && ($diffHh <= 1)) )
         {
            $lat_24 = $lat;
            $lon_24 = $lon;
            last;
         }

     print "lat is $lat, pos is $latPos, lon is $lon, pos is $lonPos\n";

      }   # end of "elsif" for checking the 12/24 forecast position for "FORECAST POSITION AT \d+"
 
   }     # end of "foreach" for forecast position

  if ($lat_24 !~ /\d$/)
  {
     print "WARNING, 24 hour forecast is not the right one or not available!!\n";
     return -1;
  }

  get6_18_points();

   return $SUCCESS;

}	# End of subroutine getBrisbaneBul



######################







############  Subroutine "getBulInfo5" ###############
############       Tokyo bulletin  ###############
## WTJP21 - 25
##################################################

sub getBulInfo5
{
   my($i) = 0;
   print "This is in function getBulInfo5.\n";

   if ($wxtFile ne "")
   {
      $filename = $wxtFile;
   }
   else
   {
      system("wxtlist.k WMO=$wmohd DTIME=$hourRange DEV=T $filename");
   }   

#   $filename = "wxt/wtjp21-1203";
# $wmohd=WTJP21;

   # If no bulletin match for this WMO header get back to the main program
   if (($tempMsg = `grep "No matches found" $filename`) =~ /^No matches found$/)
   {
      print "No matches found for bulletin $wmohd\n";      ## will delete the print
      return $NOMATCH;
   }

   ##------------ Start to get the storm information ------------##

   #-------- get the storm name --------#

   # open and read the bulletin file for the storm name
   open(POS5, $filename) || die "Can not open $filename: $!";
  
   while (defined($line = <POS5>))
   {
      #if (($line =~ /^WARNING \d{6}\.$/) && (($tempLine  = <POS5>) =~ /^WARNING VALID/))
       if (($line =~ /^WARNING \d{6}\.$/)) 
      {
         @tempMsg = split(/\s+/, $line);
         $iniTime = $tempMsg[1];
         chop($iniTime);		# remove the '.' at the end
         $iniTime =  $iniTime . 'Z';
         $forecastHour = substr ($iniTime, 2, 2);
         $line = <POS5>;		# skip the next two lines
         $line = <POS5>;

         # get the storm name and storm number

         if (($line =~ /^\w+ WARNING\.$/) && (($nameLine = <POS5>) =~ /\(\d+\)/))
         {
            @tempMsg = split(/\(/, $nameLine);
            @nameMsg = split(/\s+/, $tempMsg[0]);
            $stormName = $nameMsg[-1];

            # Try to catch the EX or SUBTROPICAL
            # There is no example collected so far. guess they won't issue one if so.  07/06/2007
            $prefix = "";
            if ($nameLine =~ /SUBTROPICAL/i)
            {
               $prefix = "SUB";
            }
            elsif ($nameLine =~ /EX[\-,\_,\s+]?TROPICAL/i)
            {
               $prefix = "EX";
            }
            elsif ($nameLine =~ /EXTRA[\-,\_]?TROPICAL/i)
            {
               $prefix = "EXTRA";
            }

            if ($prefix ne "")
            {
               $stormName = $prefix . '-' . $stormName;
            }


            $nameNNB = $stormName;
            $stNumber = $nameMsg[-2];
            # get the last two digits for the storm number
            $stNumber = substr($stNumber, 2,2);
         }
         else
         {
            return $FAILURE_BUL_FORMAT;
         }

         last;
      }	# End of the if to get the initial storm time and storm name 
   }	# End of the while loop

   close(POS5) || die "Can not close $filename: $!";

   #-------- get the time of the bulletine --------#
   $timeMsg = `grep $wmohd $filename`;
   @splitMsg = split(/\s+/, $timeMsg);
   $stationId = $splitMsg[1];
   $bulTime = $splitMsg[2];
   $bulYJd  = $splitMsg[-2];


   ##------------ Get the position track ------------##

   $posMsg = `grep -i "AT " $filename`;
   print "the positon msg is ***\n$posMsg\n***\n";
   if ($posMsg eq "")
   {
      print "WARNING, there is no position listed in the bulletin.\n";
      return -1;
   }
   @splitMsg = split(/\n/, $posMsg);
   print "split 1 is $splitMsg[0]\n";
   print "split 2 is $splitMsg[1]\n";
   print "split 3 is $splitMsg[2]\n";

   $i = 0;
   foreach $posLine (@splitMsg)
   {

      # get the initial position
      # if ($posLine =~ /^AT \d+.*MOVING/)
      if (($posLine =~ /^AT \d+.*MOVING/) || ($posLine =~ /^AT \d+.*STATIONARY/))
      # if (($posLine =~ /^POSITION/) && ($posLine =~ /[:\/]/))
      {
         print "this is the initial position.\n";
         @tempMsg = split(/\s+/, $posLine);
         print "temp1 is $tempMsg[1], temp2 is $tempMsg[2]\n";
         $lat_0 = $tempMsg[1];     
         $latPos = chop($lat_0);
         $lat_0 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         $lon_0 = $tempMsg[2];
         $lonPos = chop($lon_0);

         if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
         {
            print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
            return $FAILURE_GETPOSITION;
         }

         # the conversion follows Mcidas definition (East-lon is negative)
         $lon_0 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         print "lat is $lat_0, pos is $latPos, lon is $lon_0, pos is $lonPos\n";
      }

      # get the forecast position, expecting forecast for every 24 hour
      # only need to get the first 24 hour forecast
      elsif ($posLine =~ /^FORECAST POSITION FOR/)
      {
         $i++;
          print "posLine is $posLine\n";
         @tempMsg = split(/\s+/, $posLine);

         # there are 11 elements expected
         if (@tempMsg < 10)
         {
            next;
         }

         print "temp3 is $tempMsg[3], temp5 is $tempMsg[5], temp6 is $tempMsg[6]\n";
         $forecastHh = substr($tempMsg[3], 2, 2);
         $bulHh =  substr($bulTime, 2, 2);
         $diffHh = abs ($forecastHh - $bulHh);

         $lat = $tempMsg[5];
         $latPos = chop($lat);
         $lat *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         $lon = $tempMsg[6];
         $lonPos = chop($lon);

         if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
         {
            print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
            return $FAILURE_GETPOSITION;
         }

         # the conversion follows Mcidas definition (East-lon is negative)
         $lon *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);

         # there is 12 hour forecast
         if (($i == 1) && ($diffHh == 12))
         {
            $lat_12 = $lat;
            $lon_12 = $lon;
         }
         elsif (($i == 1) && ($diffHh == 0))
         {
            $lat_24 = $lat;
            $lon_24 = $lon;
            get12_point();
            last;
         }
         elsif (($i == 2) && ($diffHh == 0))
         {
            $lat_24 = $lat;
            $lon_24 = $lon;
            last;
         }


         print "lat is $lat, pos is $latPos, lon is $lon, pos is $lonPos\n";

       #  last;  # only get the first one, exit the foreach loop
      }  # end of "elseif" for checking the 12/24 forecast position

  }      # end of "foreach"


  if ($lat_24 !~ /\d$/) 
  {
     print "WARNING, 24 hour forecast is not the right one or not available!!\n";
     return -1;
  }

#  get12_point();
  get6_18_points();

  return 0;

}  	## End of subroutine getBulInfo5 

############  Subroutine "getBulInfo6" ###############
############     Fiji bulletin     ###############
##
## Please Note that: don't have the storm number in the bulletin!!! 04/16/03
## Thiw subroutine will work for lower case bulletin as well, change on 01/13/2006
##################################################

sub getBulInfo6
{
   my ($i) = 0;
   print "This is in function getBulInfo6.\n";

   if ($wxtFile ne "")
   {
      $filename = $wxtFile;
   }
   else
   {
      system("wxtlist.k WMO=$wmohd WSTN=NFFN DTIME=$hourRange DEV=T $filename");
   }


   # If no bulletin match for this WMO header get back to the main program
   if (($tempMsg = `grep "No matches found" $filename`) =~ /^No matches found$/)
   {
      print "No matches found for bulletin $wmohd\n";      ## will delete the print
      return $NOMATCH;
   }

   ##------------ Start to get the storm information ------------##

   #-------- get the storm name --------#

   $tempMsg = `grep -i "WAS LOCATED NEAR" $filename`;
   # if (($tempMsg  eq "") || ($tempMsg !~ /\[\d+ ?[Hh][Pp][Aa]\]/))
   if (($tempMsg  eq "") || ($tempMsg !~ /\[\d+ ?HPA\]/i))
   {
      return $FAILURE_BUL_FORMAT;
   }

   @splitMsg = split(/\[/, $tempMsg);
   @nameMsg = split(/\s+/, $splitMsg[0]);
   $stormName = $nameMsg[-1];

   # add the following "if" code on 01/06/2005,
   # for pattern "TROPICAL CYCLONE KERRY CENTRE [985 HPA] WAS LOCATED NEAR 17..."

   if ( ($stormName eq "CENTRE") || ($stormName eq "centre") )
   {
       $stormName = $nameMsg[-2];
   }

   # Try to catch the EX or SUBTROPICAL
   # There is no example collected so far. guess they won't issue one if so.  07/06/2007
   $prefix = "";
   if ($splitMsg[0] =~ /SUBTROPICAL/i)
   {
      $prefix = "SUB";
   }
   elsif ($splitMsg[0] =~ /EX[\-,\_,\s+]?TROPICAL/i)
   {
      $prefix = "EX";
   }
   elsif ($splitMsg[0] =~ /EXTRA[\-,\_]?TROPICAL/i)
   {
      $prefix = "EXTRA";
   }

   if ($prefix ne "")
   {
      $stormName = $prefix . '-' . $stormName;
   }


   $nameNNB = $stormName;
  
   #-------- get the storm number --------#
   # these bulletins don't included storm number, use the WMO header number to replace
   $stNumber = substr($wmohd, 4, 2);

   #-------- get the time of the bulletine --------#

   $timeMsg = `grep $wmohd $filename`;
   @splitMsg = split(/\s+/, $timeMsg);

   $stationId = $splitMsg[1];
   $bulTime = $splitMsg[2];
   $bulYJd = $splitMsg[-2];

   ##------------ Get the position track ------------##
   #--- the initial time of the storm will be got here ---#


   $posMsg = `grep -i " UTC" $filename`;
   print "the positon msg is ***\n$posMsg\n***\n";
   if ($posMsg eq "")
   {
      print "WARNING, there is no position listed in the bulletin.\n";
      return -1;
   }
   @splitMsg = split(/\n/, $posMsg);
   print "split 1 is $splitMsg[0]\n";
   print "split 2 is $splitMsg[1]\n";
   print "split 3 is $splitMsg[2]\n";

   foreach $posLine (@splitMsg)
   {
      # get the initial position
      # if ( ($posLine =~ /^REPEAT POSITION .* UTC\.?$/) || ($posLine =~ /^[Rr]epeat position .* UTC\.?$/) )
      if ($posLine =~ /^REPEAT POSITION .* UTC\.?$/i) 
      {
         print "this is the initial position.\n";
         @tempMsg = split(/\s+/, $posLine);

         #-------- get the initial time of the storm --------#

         $iniTime = $tempMsg[-2];
         $iniTime .= 'Z';
         $forecastHour = substr ($iniTime, 2, 2);

         $lat_0 = $tempMsg[2];     

         # Add on 04/06/07, for "REPEAT POSITION  26.2 SOUTH 171.5 WEST";
	 # It was "REPEAT POSITION 23.3S 175.6W "
         if ($lat_0 !~ /[A-Z]/i)
         {
           $latPos = $tempMsg[3];
           $lon_0 = $tempMsg[4];
           $lonPos = $tempMsg[5];
         }
         else
         {
           $latPos = chop($lat_0);
           # $lat_0 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
           $lon_0 = $tempMsg[3];
           $lonPos = chop($lon_0); 
         }

         if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
         {
            print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
            return $FAILURE_GETPOSITION;
         }

         $lat_0 *= (($latPos =~ /^[Nn]/)  ? 1.0 : -1.0);
         # the conversion follows Mcidas definition (East-lon is negative)
         # $lon_0 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         $lon_0 *= (($lonPos =~ /^[Ww]/) ? 1.0 : -1.0);
         print "lat is $lat_0, pos is $latPos, lon is $lon_0, pos is $lonPos\n";
      }

      # get the 12 hour forecast position, expecting forecast for every 12 hour
      # elsif ($posLine =~ /^FORECAST POSITION NEAR .* UTC$/)
      # elsif ( ($posLine =~ /^FORECAST POSITION NEAR .* UTC/) || ($posLine =~ /^[Ff]orecast position near .* UTC/) )
      elsif ($posLine =~ /^FORECAST POSITION NEAR .* UTC/i)
      {
        #  print "posLine is $posLine\n";
         @tempMsg = split(/\s+/, $posLine);
         $lat_12 = $tempMsg[3];
         $latPos =  chop($lat_12);
         $lon_12 = $tempMsg[4];
         $lonPos = chop($lon_12);

         if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
         {
            print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
            return $FAILURE_GETPOSITION;
         }

         $lat_12 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         $lon_12 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         print "lat is $lat_12, pos is $latPos, lon is $lon_12, pos is $lonPos\n";
      }  # end of "elseif" for checking the 12 forecast position

      # get the 24 hour forecast position
      # elsif ($posLine =~ /^\s+AND NEAR .* UTC\.?$/)
      # elsif ( ($posLine =~ /^\s+A?N?D? NEAR .* UTC\.?$/) || ($posLine =~ /^\s+a?n?d? near .* UTC\.?$/) )
      elsif ($posLine =~ /^\s+A?N?D? NEAR .* UTC\.?$/i)
      {
        #  print "posLine is $posLine\n";
         @tempMsg = split(/\s+/, $posLine);
         $lat_24 = $tempMsg[-5];
         $latPos =  chop($lat_24);
         $lon_24 = $tempMsg[-4];
         $lonPos = chop($lon_24);

         if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
         {
            print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
            return $FAILURE_GETPOSITION;
         }

         $lat_24 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         $lon_24 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         print "lat is $lat_24, pos is $latPos, lon is $lon_24, pos is $lonPos\n";
      }  # end of "elseif" for checking the 24 forecast position

   }     # end of "foreach"

  if ($lat_24 !~ /\d$/)
  {
     print "WARNING, 24 hour forecast is not the right one or not available!!\n";
     return -1;
  }

   get6_18_points();

   return 0;

}  	## End of subroutine getBulInfo6


############  Subroutine "getBulInfo7" ###############
############    Australia bulletin     ###############
############ RSMC Perth, RSMC Darwin   ###############
## FKAU01-06            -- what else?
## FKPS01 - 02 (without space after the ":" )
## Please Note that: don't have the storm number in the bulletin!!! 04/14/03
##################################################

sub getBulInfo7
{
   my($i) = 0;
   my($status);

   print "This is in function getBulInfo7.\n";

   if ($wxtFile ne "")
   {
      $filename = $wxtFile;
   }
   else
   {
      system("wxtlist.k WMO=$wmohd DTIME=$hourRange DEV=T $filename");
   }


   # If no bulletin match for this WMO header get back to the main program
   if (($tempMsg = `grep "No matches found" $filename`) =~ /^No matches found$/)
   {
      print "No matches found for bulletin $wmohd\n";      ## will delete the print
      return $NOMATCH;
   }


   ##------------ Start to get the storm information ------------##

   #-------- get the storm name --------#
  
   $nameLine = `grep "TC:" $filename`;
   
   # For FKIN20, "TC  :  SIDR",  11/13/07
   if ($nameLine eq "")
   {
     $nameLine = `grep "TC " $filename|grep "\:"`;
   }

   # will consider FKPS format (without space after ':')
   # split the message with space OR ":"

   # @splitMsg = split(/\s+|:\s*/, $nameLine);    # encounter "split loop" problem here
    @splitMsg = split(/[:\s]+/, $nameLine);
#print "the nameLine is $nameLine.\n";
#print "splitMsg 0 is $splitMsg[0], splitMsg 1 is $splitMsg[1], splitMsg 2 is $splitMsg[2], splitMsg 3 is $splitMsg[3]\n";

   $stormName = $splitMsg[-1];
   if ($splitMsg[-1] eq "LOW")
   {
      $centerLine = `grep "TCAC:" $filename`;
      @splitMsg2 = split(/[:\s]+/, $centerLine);

      $stormName = $splitMsg2[-1] . '_' . $splitMsg[-1];
   }
   # This is for Japan bulletin FKPQ, since inside the (), it is the storm number:$yy$stNum
   elsif ($splitMsg[-1] =~ /\(.*\)/)
   {
      $stormName = $splitMsg[-2];
   }

   # Try to catch the EX or SUBTROPICAL
   # There is no example collected so far. guess they won't issue one if so.  07/06/2007
   $prefix = "";
   if ($nameLine =~ /SUBTROPICAL/i)
   {
      $prefix = "SUB";
   }
   elsif ($nameLine =~ /EX[\-,\_,\s+]?TROPICAL/i)
   {
      $prefix = "EX";
   }
   elsif ($nameLine =~ /EXTRA[\-,\_]?TROPICAL/i)
   {
      $prefix = "EXTRA";
   }

   if ($prefix ne "")
   {
      $stormName = $prefix . '-' . $stormName;
   }

   if ($stormName =~ /^NIL$/i)
   {
      print "Have NIL show up as TD.\n";
      $stormName = "TD";
   }

   if ($stormName =~ /^NN$/i)
   {
      print "Have NN show up as TD.\n";
      $stormName = "TD";
   }

   $nameNNB = $stormName;

   $timeLine = `grep "DTG:" $filename`;

   # For FKIN20, "DTG :  20071113/0000Z",  11/13/07
   if ($timeLine eq "")
   {
     $timeLine = `grep "DTG " $filename|grep "\:"`;
   }

print "#####timeLine first is $timeLine\n";
   $timeLine =~ s/\///g;    # delete  the backslash between day and hour if there is any
print "#####timeLine is $timeLine\n";
   chomp $timeLine;
   $iniTime = substr($timeLine, -7, 2) . substr($timeLine, -5);
   
print "stormName is $stormName, iniTime is **$iniTime**.\n";

   # check if the bulletin format is valid by the archived storm name and initial time
   if (($stormName eq "") || ($iniTime !~ /^\d+/))
   {
      return $FAILURE_BUL_FORMAT;
   }

   ##-- get the initial storm position --##

   $positionLine = `grep "PSN:" $filename`;

   # For FKIN20, "PSN :  N1120 E08930",  11/13/07
   if ($positionLine eq "")
   {
     $positionLine = `grep "PSN " $filename|grep "\:"`;
   }
   @splitMsg = split(/[:\s]+/, $positionLine);
   $lat = $splitMsg[-2];
   $lon = $splitMsg[-1];
   $beforeLat = $splitMsg[-3];
   $beforeLat2 = $splitMsg[-4];
   if (($lat =~ /^[NnSn]\d+/) && ($lon =~ /^[EeWw]\d+/))
   {
      $latPos = substr($lat, 0, 1);
      $lonPos = substr($lon, 0, 1);
      $lat = substr($lat, 1);
      $lon = substr($lon, 1);
    }

   elsif (($lon =~ /^\d{2,}/) && ($lat !~ /\d{2,}/) && ($lat =~ /^[EeWw]\w*$/))
   {
      $lonPos = $lat; 
      if ($beforeLat =~ /^[NnSn]\d+/)
      { 
         $latPos = substr($beforeLat, 0, 1);
         $lat = substr($beforeLat, 1);
      }
      elsif (($beforeLat =~ /^\d+/) && ($beforeLat2 =~ /^[NnSn]\w*$/))
      {
         $latPos = $beforeLat2;
         $lat = $beforeLat;
      }
      else
      {
          print "ERROR: Unexpected format on the initial position line:$positionLine\n";
      }
   }
   elsif (($lon =~ /^[EeWw]\d+/) && ($lat =~ /^\d{2,}/) && ($beforeLat =~ /^[NnSn]\w*$/)) 
   {
      $lonPos = substr($lon, 0, 1);
      $lon = substr($lon, 1);  
      $latPos = $beforeLat;
   }
      
       
   # to handle "PSN:N0830 E 08430"

   $lat_0 = degreeToDecimal($lat);
   $lon_0 = degreeToDecimal($lon);

   if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
   {
      print "ERROR! latPos or lonPos has invalid value1:$latPos, $lonPos\n";
      return $FAILURE_GETPOSITION;
   }

   $lat_0 *= (($latPos  =~ /^[Nn]/) ? 1.0 : -1.0);
   $lon_0 *= (($lonPos =~ /^[Ww]/)  ? 1.0 : -1.0);


print "### the lat 0 is $lat_0 , latPos is $latPos, lon_0 is $lon_0, lonPos is $lonPos \n";


   ##-------- get the time of the bulletine --------##

   $timeMsg = `grep $wmohd $filename`;
   @splitMsg = split(/\s+/, $timeMsg);
   $stationId = $splitMsg[1];
   $bulTime = $splitMsg[2];
   $lastCh = substr ($bulTime, -1);
   #print "last Char of bulTime: $bulTime is $lastCh\n";
   if ($lastCh =~ /[A-Z]/)
   {
      print "Chop the last char: $lastCh.\n";
      chop($bulTime);
   }  

   $bulYJd  = $splitMsg[-2];
   $forecastHour = substr ($iniTime, 2, 2);

print "%%%%%%%% the storm name is $stormName, iniTime is $iniTime, lat_0 is $lat_0, lon_0 is $lon_0, bulYJd is $bulYJd,
                bulTime is $bulTime.\n";

   ##------------ Get the position track ------------##

   ## get the 6/12/18/24 forecast position
   $posMsg = `grep  "FCST PSN" $filename`;
   if ($posMsg eq "")
   {
      print "WARNING, there is no forecast position listed in the bulletin.\n";
      return $FAILURE_GETPOSITION;
   }
   @splitMsg1 = split(/\n/, $posMsg);

   # initial the 6/12/18 hour forecast positions
   $lat_6 = "";
   $lon_6 = "";
   $lat_12 = "";
   $lon_12 = "";
   $lat_18 = "";
   $lon_18 = "";

   foreach $posLine (@splitMsg1)
   {
      # get the hour forecast position, most of them have 12/18/24 hour forecasts,
      # there is 6 hour forecast available for FKAU (Australia)
      # 18 hour forecast is not available(NIL/NN) for FKPQ (Japan) 

      # handle the "N/A" for FKAU, modified on 03/31/08
      # FCST PSN +6HR:       N/A

      if (($posLine =~ /NIL$/) || ($posLine =~ /NN$/) || ($posLine =~ /\:\s*N\/A/)) 
      {
         next;
      }
 
      @splitMsg = split(/[:Z\s]+/, $posLine);
      print "splitmsg  $splitMsg[-2] $splitMsg[-1]\n";
      $lat = $splitMsg[-2];
      $lon = $splitMsg[-1];
      $beforeLat = $splitMsg[-3];
      $beforeLat2 = $splitMsg[-4];
      $j = -1;
      # to handle "FCST PSN +12HR:      06/0000Z N2235 E10625 TROPICAL DEPRESSION", 07/05/2007
      if ($lon !~ /\d{2,}/)
      {
         print "lon is not digit as expected, it is $lon, lat is $lat. Correct it!\n";
         #$j = -1;


         while (($splitMsg[$j] !~ /\d{2,}/) && ( abs($j) <= $#splitMsg))
         {
            $j--;
         }

         if ( abs($j) > $#splitMsg + 1)
         {
            print "WARNING, Forecast position is not in the right format or not available!!\n";
            return $FAILURE_GETPOSITION;
         }
         print "splitmsg $j $splitMsg[$j -1] $splitMsg[$j]\n";
         $lat = $splitMsg[$j - 1];
         $lon = $splitMsg[$j];
	 print "After split lat $lat lon $lon\n";
         $beforeLat = $splitMsg[$j - 2];
         $beforeLat2 = $splitMsg[$j - 3];

         # To handle the FKIN20 line as following, 05/24/09
         # " FCST PSN+24HRS: 251200 N 2230 E 08830(OVER LAND) "
         # delete the non digit
         # Don't want to delete the 'E' in front of the digit
         #if ( $lon =~ /[a-zA-Z\(\)]/ )
         if (( $lon =~ /\w{2,}/) || ( $lon =~ /[\(\)]/ ))
         {
            $tempLonPos = "";
            if ($lon =~ /^[EeWw]/)
            {
               $tempLonPos = substr($lon, 0, 1);
            }
            $lon =~ s/[a-zA-Z\(\)]//g;
            $lon = $tempLonPos .  $lon; 
            #print "lon is $lon, lon_test is $lon_test!!!!\n"; 
         }
         print "CORRECT lon is $lon, lat is $lat\n";
      }
      print "Before conv lat $lat lon $lon\n";
      if ($lat =~ /^[NnSn]\d+/)
      { 
         $latPos = substr($lat, 0, 1);
         $lat = degreeToDecimal(substr($lat, 1));
      }
      # this handle "FCST PSN+24HRS : 290000 N 1400 E 0850" in FKIN20, 04/28/08
      elsif ($lat =~ /^[EeWw]$/)
      {
      #   print " handle \"FCST PSN+24HRS : 290000 N 1400 E 08500\" in FKIN20, 04/28/08\n";
         $latPos = $beforeLat2;
         $lat = $beforeLat;
         $lat = degreeToDecimal($lat);
      }
      # the handle "FCST PSN+12HRS: 131200 N 1300 E08830" in FKIN20, 11/13/07
      elsif ($lat !~ /^[NnSn]/)
      {
       #  print "handle \"FCST PSN+12HRS: 131200 N 1300 E08830\" in FKIN20, 11/13/07\n";
         $latPos =  $beforeLat;
         $lat = degreeToDecimal($lat);
      }

      
      if ($lon =~ /^[EeWw]\d+/)
      {
         $lonPos = substr($lon, 0, 1);
         $lon = degreeToDecimal(substr($lon, 1));
      }
      # handle "FCST PSN+ 06 HRS: 160000 N 1630 E08600" in FKIN20, 04/15/09
      # handle "FCST PSN +12HRS:  160000 N 1630 E 08600" in FKIN20,
      elsif ($lon =~ /^\d+/)
      {
         $lon = degreeToDecimal($lon); 
         $lonPos = $splitMsg[$j - 1];
      }
      print "LatPos $latPos LonPos $lonPos\n";
      if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
      {
         print "ERROR! latPos or lonPos has invalid value2:$latPos, $lonPos\n";
         return $FAILURE_GETPOSITION;
      }

      $lat *= (($latPos  =~ /^[Nn]/) ? 1.0 : -1.0);
      $lon *= (($lonPos =~ /^[Ww]/)  ? 1.0 : -1.0);
      
print "### FORECAST position: i is $i, the lat 0 is $lat , latPos is $latPos, lon is $lon, lonPos is $lonPos \n";
      print "POSline $posLine\n";
      if (($posLine =~ /\PSN.\s?6\s?HR/) || ($posLine =~ /\+\s?6\s?HR/))
      {
         $lat_6 = $lat;
         $lon_6 = $lon;
      }
      elsif (($posLine =~ /\PSN.\s?12\s?HR/) || ($posLine =~ /\+\s?12\s?HR/))
      {
	  print "Went into 12?\n";
         $lat_12 = $lat;
         $lon_12 = $lon;
      }
      elsif (($posLine =~ /\PSN.\s?18\s?HR/) || ($posLine =~ /\+\s?18\s?HR/))
      {
	  print "Went into 18?\n";
         $lat_18 = $lat;
         $lon_18 = $lon;
      }
      elsif (($posLine =~ /\PSN.\s?24\s?HR/) || ($posLine =~ /\+\s?24\s?HR/))
      {
	  print "Went into 24?\n";
         $lat_24 = $lat;
         $lon_24 = $lon;
      }

   }   # end of "foreach" for forecast position
   print "lat_24 $lat_24 lat_18 $lat_18\n";
   if ($lat_24 !~ /\d$/)
   {
      print "WARNING, there is no 24 hour forecasti or it is not a right one!!\n";
      return $FAILURE_GETPOSITION;
   }

   # not seen any bulletin without 12 hour position yet, noted at 04/05/2004
   if ($lat_12 eq "") 
   {
      get12_point();
   }
 
   if (($lat_6 eq "") && ($lat_18 eq ""))
   {
      get6_18_points();
   }
   elsif (($lat_6 eq "") && ($lat_18 ne ""))
   {
      # only get 6 hour forecast
      $lat = $lat_18;
      $lon = $lon_18;
      get6_18_points();
      $lat_18 = $lat;
      $lon_18 = $lon;
   }

   #-------- get the storm number --------#
   # these bulletins don't included storm number, use 99
   # $stNumber = substr($wmohd, 4, 2);
   $stNumber = 99;


   return $SUCCESS;;

}       ## End of subroutine getBulInfo7


############  Subroutine "getBulInfo8" ###############
##################################################

sub getBulInfo8
{
   my($i,$j);
   print "This is in function getBulInfo8.\n";

   if ($wxtFile ne "")
   {
      $filename = $wxtFile;
   }
   else
   {
      system("wxtlist.k WMO=$wmohd MATCH='TROPICAL CYCLONE WARNING' DTIME=$hourRange DEV=T $filename");
   }

   # If no bulletin match for this WMO header get back to the main program
   if (($tempMsg = `grep "No matches found" $filename`) =~ /^No matches found$/)
   {
      print "No matches found for bulletin $wmohd\n";      ## will delete the print
      return $NOMATCH;
   }

   ##------------ Start to get the storm information ------------##

   #-------- get the storm name --------#

   $tempMsg = `grep -i "WARNING NR" $filename`;
   # $tempMsg = `grep "TROPICAL CYCLONE -" $filename`;

print "tempMsg is $tempMsg\n";

   if ($tempMsg  eq "")
   {
      return $FAILURE_BUL_FORMAT;
   }


   @splitMsg = split(/\-/, $tempMsg);

   @nameMsg = split(/\s+/, $splitMsg[1]);

   $stormName = $nameMsg[-1];

   # Try to catch the EX or SUBTROPICAL
   # There is no example collected so far. guess they won't issue one if so.  07/06/2007
   $prefix = "";
   if ($tempMsg =~ /SUBTROPICAL/i)
   {
      $prefix = "SUB";
   }
   elsif ($tempMsg =~ /EX[\-,\_,\s+]?TROPICAL/i)
   {
      $prefix = "EX";
   }
   elsif ($tempMsg =~ /EXTRA[\-,\_]?TROPICAL/i)
   {
      $prefix = "EXTRA";
   }

   if ($prefix ne "")
   {
      $stormName = $prefix . '-' . $stormName;
   }


   $nameNNB = $stormName;


   #-------- get the storm number  --------#

   #open(FILE, $filename) || die "Can not open $filename: $!";
   #my(@lines) = <FILE>;
   #@numberMsg = split(/\s+/, $lines[3]);

   #$stNumber = $numberMsg[-1];
   #close(FILE) || die "Can not close $filename: $!";
   
   @numberMsg = split(/\s+/, $tempMsg);
   $stNumber = $numberMsg[-1];


   #-------- get the time of the bulletine --------#
   $timeMsg = `grep $wmohd $filename`;
   @splitMsg = split(/\s+/, $timeMsg);
   $stationId = $splitMsg[1];
   $bulTime = $splitMsg[2];
   $bulYJd  = $splitMsg[-2];


   ##------------ Get the position track ------------##
   #--- the initial time of the storm will be got here ---#
   #---            and the forecast hour               ---#

   open(POS8, $filename) || die "Can not open $filename: $!";
#   my(@lines) = <POS8>;

  # if ($posMsg eq "")
  # {
  #    print "WARNING, there is no position listed in the bulletin.\n";
  #    return $FAILURE_GETPOSITION;
  # }

   # $i for counting the number of forecast position lines read
   $i = 0;

   while (defined($line = <POS8>))
   {
      #---------------------------------------------------#

      #-------- get the initial time and position --------#
      if ($line =~ /INITIAL POSITION/i)
      {
         # read the next line
         $line = <POS8>;

         if ($line =~ /NEAR/i)
         {
            print "initial posLine is $line\n";
            # @tempMsg = split(/\s+/, $line);
            @tempMsg = split(/NEAR/i, $line);

            #-------- get the initial time of the storm --------#

            $iniTime = $tempMsg[0];
            $iniTime =~ s/\s//g;          # delete the white space in the front if there is any 
            $forecastHour = substr ($iniTime, 2, 2);

            #-------- get the storm initial position --------#
 
            # use index -1 is better than 3, since whether there is space
            # in front of the line will result in different element "3"
 
            @splitMsg = split(/\s+/, $tempMsg[1]);
            $lat_0 = $splitMsg[1];
            
            if ($lat_0 =~ /[NnSs]$/)
            {
               $latPos = chop($lat_0);
               $j = 2;
            }
            else
            {
               $latPos = $splitMsg[2];
               $j = 3;
            }

            $lon_0 = $splitMsg[$j];
            if ($lon_0 =~ /[EeWw]$/)
            {
               $lonPos = chop($lon_0);
            }
            else
            {
               $lonPos = $splitMsg[$j + 1];
            }

            if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
            {
               print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
               return $FAILURE_GETPOSITION;
            }

            $lat_0 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
            $lon_0 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
            print "lat is $lat_0, pos is $latPos, lon is $lon_0, pos is $lonPos\n";
         }
         else
         {
            print "WARNING, initial position is not in the right format or not available!!\n";
            return $FAILURE_GETPOSITION;;
         }

      }	 # end of "if" for initial position

      #-------- get the storm forecast positions  --------#

      #usually is 06, 12, 18 and 24

      # elsif ($line =~ /^\s*FORECAST\s*$/i)
      elsif ($line =~ /HRS VALID AT:/i)
      {
         $i++;

         @tempMsg = split(/HRS VALID AT:/i, $line);

         $fHh = $tempMsg[0];
         
         # remove the white space after the hour and the zero in front of the hour
         $fHh += 0;
         @splitMsg = split(/\s+/, $tempMsg[1]);

         $j = 0;

         # in case there is no space after the ':'
         if ($splitMsg[$j] ne "")
         {
            $j = 1;
         } 
         else
         {
            $j = 2;
         }
     
         $lat = $splitMsg[$j];

         if ($lat =~ /[NnSs]$/)
         {
            $latPos = chop($lat);
            $j++;
         }
         else
         {
            $latPos = $splitMsg[++$j];
            $j++;
         }

         $lon = $splitMsg[$j];
         if ($lon =~ /[EeWw]$/)
         {
            $lonPos = chop($lon);
         }
         else
         {
            $lonPos = $splitMsg[$j + 1];
         }

         if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
         {
            print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
            return $FAILURE_GETPOSITION;
         }

         $lat *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         $lon *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
 
         $latName = "lat_" . $fHh;
         ${$latName} = $lat;
          
         $lonName = "lon_" . $fHh;
         ${$lonName} = $lon;
#print "latName is $latName, lat is ${$latName}\n";

      }  # end of "elseif" for checking the 12/24 forecast position

   }     # end of "While"

  close(POS8) || die "Can not close $filename: $!";

  print "i is $i\n";
  if (($i < 4) || ($lat_24 !~ /\d$/))
  {
     print "WARNING, 12/24 hour forecast is not the right one or not available!!\n";
     return $FAILURE_GETPOSITION;
  }

  return $SUCCESS;

}       ## End of subroutine getBulInfo8



############  Subroutine "getBulInfo9" ###############
##################################################

# Not checking the Extratropical and Subtropical, as not enough example collected
# 07/06/2007

sub getBulInfo9
{
   my($i,$j);
   print "This is in function getBulInfo9.\n";

   if ($wxtFile ne "")
   {
      $filename = $wxtFile;
   }
   else
   {
      system("wxtlist.k WMO=$wmohd DTIME=$hourRange DEV=T $filename");
   }

   # If no bulletin match for this WMO header get back to the main program
   if (($tempMsg = `grep "No matches found" $filename`) =~ /^No matches found$/)
   {
      print "No matches found for bulletin $wmohd\n";      ## will delete the print
      return $NOMATCH;
   }

   ##------------ Start to get the storm information ------------##

   #-------- get the storm name --------#

   $tempMsg = `grep -i "TROPICAL CYCLONE" $filename`;
   
   if ($tempMsg  eq "")
   {
      # will check if there is "HURRICANE replace of 'TROPICAL CYCLONE' in front of the storm name
      $testLine = `grep -i "HURRICANE" $filename |grep -i HPA`;
      if ($testLine ne "")
      {
         print "Pay Attention!!!, there is another pattern: $testLine\n";
      }
      return $FAILURE_BUL_FORMAT;
   }


   @splitMsg = split(/TROPICAL CYCLONE/i, $tempMsg);

   @nameMsg = split(/\s+/, $splitMsg[1]);

   $stormName = $nameMsg[1];
   $nameNNB = $nameMsg[1];
   
   # temporarily assign 99 for the storm number
   $stNumber = 99;


   #-------- get the time of the bulletine --------#
   $timeMsg = `grep $wmohd $filename`;
   @splitMsg = split(/\s+/, $timeMsg);
   $stationId = $splitMsg[1];
   $bulTime = $splitMsg[2];
   $bulYJd  = $splitMsg[-2];


   ##------------ Get the position track ------------##
   #--- the initial time of the storm will be got here ---#
   #---            and the forecast hour               ---#

   open(POS9, $filename) || die "Can not open $filename: $!";

   # $i for counting the number of forecast position lines read
   $i = 0;

   while (defined($line = <POS9>))
   {
      #---------------------------------------------------#

      #-------- get the initial time and position --------#
      if ($line =~ /REPEAT POSITION/i)
      {
         # print "initial posLine is $line\n";
         # @tempMsg = split(/\s+/, $line);
         @tempMsg = split(/REPEAT POSITION/i, $line);
         @splitMsg = split(/\s+/, $tempMsg[1]);

         #-------- get the initial time of the storm --------#

         $j = -1;
         while (($splitMsg[$j] !~ /UTC/) && ($splitMsg[$j] ne ""))
         {
            $j--;
         }

         if ($splitMsg[$j] =~ /^UTC/)
         {
            $iniTime = $splitMsg[$j - 1];
         }
         elsif ($splitMsg[$j] =~ /^\d+UTC/)
         {
            $iniTime = substr($splitMsg[$j], 0, 6);
         }
         else
         {
            print "WARNING, unsuccess to get initial time from the bulletin.\n";
            return $FAILURE_BUL_FORMAT;
         } 

         $forecastHour = substr ($iniTime, 2, 2);

         #-------- get the storm initial position --------#

         $j = 0;
         while (($splitMsg[$j] !~ /\d+\.\d+/) && ($j <= $#splitMsg))
         {
            $j++;
         }
         if ($j > $#splitMsg + 1)
         {
            print "WARNING, initial position is not in the right format or not available!!\n";
            return $FAILURE_GETPOSITION;
         }

         $lat_0 = $splitMsg[$j];
            
         if ($lat_0 =~ /[NnSs]$/)
         {
            $latPos = chop($lat_0);
            $j++;
         }
         else
         {
            $latPos = $splitMsg[++$j];
            $j++;
         }

         $lon_0 = $splitMsg[$j];
         if ($lon_0 =~ /[EeWw]$/)
         {
            $lonPos = chop($lon_0);
         }
         else
         {
            $lonPos = $splitMsg[$j + 1];
         }

         if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
         {
            print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
            return $FAILURE_GETPOSITION;
         }

         $lat_0 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         $lon_0 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         #print "lat_0 is $lat_0, pos is $latPos, lon_0 is $lon_0, pos is $lonPos\n";

      }	 # end of "if" for initial position

      #-------- get the storm forecast positions  --------#

      #usually is 12 and 24

      elsif (($line =~ /FORECAST POSITION/i) || ($line =~ /NEAR .+ \d+\s*UTC/i))
      {
         @splitMsg = split(/\s+/, $line);

         $j = 0;
         while (($splitMsg[$j] !~ /\d+\.\d+/) && ($j <= $#splitMsg))
         {
            $j++;
         }

         if ($j > $#splitMsg + 1)
         {
            print "WARNING, forecast position is not in the right format or not available!!\n";
            return $FAILURE_GETPOSITION;
         }
         $i++;

         $lat = $splitMsg[$j];

         if ($lat =~ /[NnSs]$/)
         {
            $latPos = chop($lat);
            $j++;
         }
         else
         {
            $latPos = $splitMsg[++$j];
            $j++;
         }

         $lon = $splitMsg[$j];
         if ($lon =~ /[EeWw]$/)
         {
            $lonPos = chop($lon);
         }
         else
         {
            $lonPos = $splitMsg[$j + 1];
         }

         if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
         {
            print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
            return $FAILURE_GETPOSITION;
         }

         $lat *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         $lon *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
 
         $fHh = $i * 12;
         $latName = "lat_" . $fHh;
         ${$latName} = $lat;
          
         $lonName = "lon_" . $fHh;
         ${$lonName} = $lon;

      }  # end of "elseif" for checking the 12/24 forecast position

   }     # end of "While"

  close(POS9) || die "Can not close $filename: $!";

  print "i is $i\n";
  if (($i < 2) || ($lat_24 !~ /\d$/))
  {
     print "WARNING, 12/24 hour forecast is not the right one or not available!!\n";
     return $FAILURE_GETPOSITION;
  }

  get6_18_points();

  return $SUCCESS;

}       ## End of subroutine getBulInfo9


############  Subroutine "getBulInfo10" ###############
##################################################

sub getBulInfo10
{
   my($i,$j);
   print "This is in function getBulInfo10.\n";
   
   if ($wxtFile ne "")
   {
      $filename = $wxtFile;
   }
   else
   {
      system("wxtlist.k WMO=$wmohd DTIME=$hourRange DEV=T $filename");
      system("echo wxtlist.k WMO=$wmohd DTIME=$hourRange DEV=T $filename");
   }
# $filename = "wxt/other/tpc/wtpz21-1020";

   # If no bulletin match for this WMO header get back to the main program
   if (($tempMsg = `grep "No matches found" $filename`) =~ /^No matches found$/)
   {
      print "No matches found for bulletin $wmohd\n";      ## will delete the print
      return $NOMATCH;
   }

   ##------------ Start to get the storm information ------------##

   #-------- get the storm name --------#

   $tempMsg = `grep -i "Tropical Cyclone:" $filename`;
   if ( $tempMsg eq "" )
   {
      $tempMsg = `grep -i "Name:" $filename`;
 
      if ($tempMsg  eq "")
      {
         print "Pay Attention!!!, there is another pattern for the name\n";
         return $FAILURE_BUL_FORMAT;
      }

      $splitStr = "Name:";
   }
   else
   {
      $splitStr = "Tropical Cyclone:";
   }

   @splitMsg = split(/$splitStr/i, $tempMsg);
   @nameMsg = split(/\s+/, $splitMsg[1]);
   $stormName = $nameMsg[-1];
   $nameNNB = $nameMsg[-1];
   
   # temporarily assign 99 for the storm number
   $stNumber = 99;

   ###########################################
   ## There is no example collected so far, so comment out the following lines  
   ## Try to catch the EX or SUBTROPICAL
   # $prefix = "";
   # # m ay need to change the "nameLine" to "tempMsg" 
   #if ($nameLine =~ /SUBTROPICAL/i)
   #{
   #   $prefix = "SUB";
   #}
   #elsif ($nameLine =~ /EX[\-,\_,\s+]?TROPICAL/i)
   #{
   #   $prefix = "EX";
   #}
   #elsif ($nameLine =~ /EXTRA[\-,\_]?TROPICAL/i)
   #{
   #   $prefix = "EXTRA";
   #}

   #if ($prefix ne "")
   #{
   #   $stormName = $prefix . '-' . $stormName;
   #}

   # $nameNNB = $stormName;
   ###########################################


   #-------- get the time of the bulletin --------#

   $timeMsg = `grep $wmohd $filename`;
   system("echo timeMsg = $timeMsg");
   @splitMsg = split(/\s+/, $timeMsg);
   $stationId = $splitMsg[1];
   $bulTime = $splitMsg[2];
   $bulYJd  = $splitMsg[-2];

   # assign the AREA to the storm name for Tropical Low
   if ($stormName =~ /^LOW$/i )
   {
      if ($stationId eq "APRF")
      {
         $stormName = "PERTH_LOW";
         $nameNNB = $stormName; 
      }
      elsif ($stationId eq "ABRF")
      {
         $stormName = "BRISBANE_LOW";
         $nameNNB = $stormName;
      }
      elsif ($stationId eq "ADRM")
      {
         $stormName = "DARWIN_LOW";
         $nameNNB = $stormName;
      }
   }

   ##------------ Get the position track ------------##
   #--- the initial time of the storm will be got here ---#
   #---            and the forecast hour               ---#

   #---------------------------------------------------#

   #-------- get the initial time of the storm --------#

   $tempMsg = `grep -i "Data At:" $filename`;
   @splitMsg = split(/\s+/, $tempMsg);
   $iniTime = $splitMsg[2];
            
   if ($tempMsg eq "")
   {         
      print "WARNING, unsuccess to get initial time from the bulletin.\n";
      return $FAILURE_BUL_FORMAT;
   } 

    #     $forecastHour = substr ($iniTime, 2, 2);
   $forecastHour = substr ($iniTime, 0, 2);

   #-------- get the storm initial position --------#

   $tempMsg = `grep -i "Latitude:" $filename`;
   @splitMsg = split(/\s+/, $tempMsg);
   $lat_0 = $splitMsg[1];

   $tempMsg = `grep -i "Longitude:" $filename`;
   @splitMsg = split(/\s+/, $tempMsg);
   $lon_0 = $splitMsg[1];

   if ($tempMsg eq "")
   {         
      print "WARNING, failure to get Latitude from the bulletin.\n";
      return $FAILURE_BUL_FORMAT;
   } 
            
         if ($lat_0 =~ /[NnSs]$/)
         {
            $latPos = chop($lat_0);
            $j++;
         }
         else
         {
            $latPos = $splitMsg[++$j];
            $j++;
         }

         $lon_0 = $splitMsg[$j];
         if ($lon_0 =~ /[EeWw]$/)
         {
            $lonPos = chop($lon_0);
         }
         else
         {
            $lonPos = $splitMsg[$j + 1];
         }
print "latPos is $latPos, lonPos is $lonPos.\n";
         if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
         {
            print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
            return $FAILURE_GETPOSITION;
         }

         $lat_0 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         $lon_0 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         #print "lat_0 is $lat_0, pos is $latPos, lon_0 is $lon_0, pos is $lonPos\n";


      #-------- get the storm forecast positions  --------#

      #usually is 12 and 24

         $tempMsg = `grep -i "+12:" $filename`;
         @splitMsg = split(/\s+/, $tempMsg);

         if (($tempMsg eq "") || ($tempMsg =~ /\:\s*N\/A/))
          {         
             print "WARNING, failure to get 12 hour forecast from the bulletin.\n";
             $lat_12= "";
             #return $FAILURE_GETPOSITION;
           } 
          else
           {
             $lat_12= $splitMsg[2];
             $lon_12 = $splitMsg[3];
             chop($lon_12);  # get rid of : at the end 

             $latPos = chop($lat_12);
             $lonPos = chop($lon_12);

             if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
             {
               print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
               return $FAILURE_GETPOSITION;
             }

             $lat_12 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
             $lon_12 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
           }


         $tempMsg = `grep -i "+24:" $filename`;
         @splitMsg = split(/\s+/, $tempMsg);

         if (($tempMsg eq "") || ($tempMsg =~ /\:\s*N\/A/))
          {         
            print "WARNING, failure to get 24 hour forecast from the bulletin.\n";
            return $FAILURE_GETPOSITION;
          } 
          else
           {
             $lat_24= $splitMsg[2];
             $lon_24 = $splitMsg[3];
             chop($lon_24);  # get rid of : at the end 

             $latPos = chop($lat_24);
             $lonPos = chop($lon_24);

             if (($latPos !~ /^[NnSs]/) || ($lonPos !~ /^[WwEe]/))
             {
               print "ERROR! latPos or lonPos has invalid value:$latPos, $lonPos\n";
               return $FAILURE_GETPOSITION;
             }

             $lat_24 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
             $lon_24 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
           }

  if ($lat_12 eq "")
  {
     get12_point();
  }

  get6_18_points();

  return $SUCCESS;

}       ## End of subroutine getBulInfo10



####*********** End of all those "get position" subroutines! ***********#####


############  Subroutine "degreeToDecimal" ###############
##################################################
sub degreeToDecimal
{
   my ($degreePos) = @_;
   my ($intPart, $decPart);
   $intPart = substr($degreePos, 0, (length($degreePos) -2 ));
   $decPart = substr($degreePos, -2);
#print "intPart is $intPart, decPart is $decPart\n";
   $decPart /= 60;
   return ($intPart + $decPart);
}


############  Subroutine "get12_point" ###############
##################################################
sub get12_point
{
   $lat_12 = ($lat_0 + $lat_24)/2;
   $lon_12 = ($lon_0 + $lon_24)/2;

 print "before adjust :
         lat-0 is $lat_0, lon-0 is $lon_0,
         lat-12 is $lat_12, lon-12 is $lon_12,
         lat-24 is $lat_24, lon-24 is $lon_24\n";

   # Check if it is cross dateline 180E/W
   if (($lon_0 / $lon_24) < 0 && (abs($lon_0) > 90))
   {
      if ($lon_12 <= 0)
      {
         $lon_12 += 180;
      }
      else
      {
         $lon_12 -= 180;
      }
   }

   print "after adjust:
         lat-0 is $lat_0, lon-0 is $lon_0,
         lat-12 is $lat_12, lon-12 is $lon_12,
         lat-24 is $lat_24, lon-24 is $lon_24\n";

}	## End of the subroutine "get12_point"


############  Subroutine "get6_18_points" ###############
##################################################
sub get6_18_points
{
   $lat_6 = ($lat_0 + $lat_12)/2;
   $lon_6 = ($lon_0 + $lon_12)/2;
   $lat_18 = ($lat_12 + $lat_24)/2;
   $lon_18 = ($lon_12 + $lon_24)/2;
  
 print "before adjust :
         lat-0 is $lat_0, lon-0 is $lon_0,
         lat-6 is $lat_6, lon-6 is $lon_6,
         lat-12 is $lat_12, lon-12 is $lon_12,
         lat-18 is $lat_18, lon-18 is $lon_18
         lat-24 is $lat_24, lon-24 is $lon_24\n";


 
   # Check if it is cross dateline 180E/W
   if (($lon_0 / $lon_12) < 0 && (abs($lon_0) > 90))
   {
      if ($lon_6 <= 0)
      {
         $lon_6 += 180;
      }
      else
      {
         $lon_6 -= 180;
      }   
   }

   if (($lon_12 / $lon_24) < 0 && (abs($lon_12) > 90))
   {
      if ($lon_18 <= 0)
      {
         $lon_18 += 180;
      }
      else
      {
         $lon_18 -= 180;
      }
   }

   print "after adjust:
         lat-0 is $lat_0, lon-0 is $lon_0,
         lat-6 is $lat_6, lon-6 is $lon_6,
         lat-12 is $lat_12, lon-12 is $lon_12,
         lat-18 is $lat_18, lon-18 is $lon_18
         lat-24 is $lat_24, lon-24 is $lon_24\n";

} 	## End of the subroutine "get6_18_points"


############  Subroutine "writeTrack" ###############
# This subrountine was replaced by another one "writeInfo"
##################################################

sub writeTrack
{  
   my($i);
   open(POSTRACK, "> $positionFile") || die "Can not open $positionFile for writing: $!";
   for($i = 0; $i <= 24; $i += 6)
   {
      $latName = "lat_" . $i;
      $lonName = "lon_" . $i;
      printf POSTRACK "%2d%8.2f%8.2f\n", $i,${$latName}, ${$lonName};

      # all these names can work: ${$latName}, $$latName, and ${"lat_" . $i}
      #printf POSTRACK "%2d%8.2f%8.2f%8.1f\n", $i,${$latName}, $$lonName, ${"lat_" . $i} ;
   }  
   close(POSTRACK) || die "Can not close $positionFile: $!";

   printInfo();

   ##--------- Write the storm information into the Storm structure ----------##

#   $storm1 = Storm->new();    should be declare in the main program
   $storm1->name($stormName);
   $storm1->nameNNB($stormName);
   $storm1->wmoHd($wmohd);
   $storm1->bulTime($bulTime);
   $storm1->bulYJd($bulYJd);
   $storm1->stationId($stationId);
   $storm1->iniTime($iniTime);
   $storm1->forecastHour($forecastHour);

   ##------ assign the forecast track to the Storm struct ------##
   for($i = 0; $i <= 24; $i += 6)
   {
      $latName = "lat_" . $i;
      $lonName = "lon_" . $i;
      $storm1->$latName($$latName);
      $storm1->$lonName($$lonName);

      printf "the i, lat and lon in the storm structure are %2d%8.2f%8.2f\n",
              $i, $storm1->$latName, $storm1->$lonName;  

      # all these names can work: ${$latName}, $$latName, and ${"lat_" . $i}
      #printf POSTRACK "%2d%8.2f%8.2f%8.1f\n", $i,${$latName}, $$lonName, ${"lat_" . $i} ;
   }


}	## End of subroutine "writeTrack"


############  Subroutine "writeInfo" ###############
# write the information got from bulletin to the storm structure
# and the file for pctrap to achive.  Will improve by "export"
# module in the future
##################################################

sub writeInfo
{
   my($i);


   # 03/03/04, check and delete the non-word charater within the storm name
   print "Check if there is non-word charater in the storm name.\n"; 
   # $stormName =~ s/\W//g;
 #  $stormName =~ s/[^a-zA-Z0-9_-]//g;
   $stormName =~ tr/a-z/A-Z/;
   $nameNNB = $stormName;

   ##--- get the forecast julian date from the bulletin issued date ---##
 
   # This bulletin year-jjj date is the date for issuing bulletin,
   # may not be the same as the forecast date/time, so has to adjust it
   print "In writeInfo BULYJD 1 $bulYJd XX\n";

   if (($bulYJd = adjustBulYJd($bulTime, $bulYJd)) == $FAILURE_BUL_JDATE )
   {
       print "There is something wrong with the bulletin issued date and forecast date, the time different is more than 1 day\n";
       return $FAILURE_BUL_JDATE;
   }

   ##-------- conver the bulYJd to bulMmdd --------##
   print "In writeInfo BULYJD 2 $bulYJd XX\n";
   my($dmy) = `$util_dir/TKcydtodmyExe -cyd $bulYJd`;
   @splitMsg = split(/\s+/, $dmy);
   $day = $splitMsg[0];
   $month = $splitMsg[1];   
   if ($day < 10)
   {
      $day = '0' . $day;
   }
   if ($month < 10)
   {
      $month = '0' . $month;
   }
 
   $bulMmdd = $month . $day;

   printInfo();

   ##--------- Write the storm information into the Storm structure ----------##

#   $storm1 = Storm->new();   xxxx should be declare in the main program
   $storm1->name($stormName);
   $storm1->nameNNB($stormName);
   $storm1->stNumber($stNumber);
   $storm1->wmoHd($wmohd);
   $storm1->bulTime($bulTime);
   $storm1->bulYJd($bulYJd);
   $storm1->bulMmdd($bulMmdd);
   $storm1->stationId($stationId);
   $storm1->iniTime($iniTime);
   $storm1->forecastHour($forecastHour);

   ## write the information to file
   print "BULYJD BULMMDD $bulYJd $bulMmdd\n";
   open(BULINFO, "> $infoFile") || die "Can not open $infoFile for writing: $!";
   print "wirte the storm structure to the Bulinfo file\n";

   printf BULINFO "%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n",$storm1->name, $storm1->nameNNB,
                  $storm1->stNumber, $storm1->wmoHd, $storm1->stationId,
                  $storm1->bulTime,$storm1->bulYJd, $storm1->bulMmdd, $storm1->iniTime, 
                  $storm1->forecastHour, $basin;

   ##------ assign the forecast track to the Storm struct ------##
   for($i = 0; $i <= 24; $i += 6)
   {
      $latName = "lat_" . $i;
      $lonName = "lon_" . $i;
      $storm1->$latName($$latName);
      $storm1->$lonName($$lonName);

      printf "the i, lat and lon in the storm structure are %2d%8.2f%8.2f\n", 
              $i, $storm1->$latName, $storm1->$lonName;

      printf BULINFO "%.2f\n%.2f\n", $storm1->$latName, $storm1->$lonName; 
   }

   close(BULINFO) || die "Can not close $infoFile: $!";

   return $SUCCESS;
}       ## End of subroutine "writeInfo"


############  Subroutine "checkPositions" ###############
##################################################

sub checkPositions
{

   #will modify the following which copied from writeInfo
   
   ##------ assign the forecast track to the Storm struct ------##
   for($i = 0; $i <= 24; $i += 6)
   {
      $latName = "lat_" . $i;
      $lonName = "lon_" . $i;

      # check for the 0 latitude, which is impossible position for tropical system
      if ($$latName == 0)
      {
          # pobably write the following to a string, and send email for some of the major error found.
          print "ERROR! The $latName is $$latName, and it has incorrect ZERO value!\n";
          return $FAILURE_GETPOSITION;
      }
 
      # Do a rough checking on the difference of lat (and lon) positions
      $i2 = $i + 6;

      if ( $i2 <= 24)
      {
         $lat2Name = "lat_" . $i2;
         $lon2Name = "lon_" . $i2;
         $latDiff = abs($$lat2Name - $$latName);
         $lonDiff = abs($$lon2Name - $$lonName);
   
         if (($latDiff > 10) || (($lonDiff > 10) && ($lonDiff < 350)) )
         {
            print "ERROR! The difference of the $latName and $lat2Name is $latDiff, and the difference of the $lonName and $lon2Name is $lonDiff.\n";
            return $FAILURE_GETPOSITION;
         }
      }

      if ($$lonName == 0)
      {
         print "ATTENTION! The $lonName is $$lonName, and it has ZERO value!\n";
      }

   }

   return $SUCCESS;

}  ## End of subroutine "checkPositions"


############  Subroutine "printInfo" ###############
##################################################

sub printInfo
{
   print "**** Storm Information *****\n";
   print "Storm name is $stormName, nameNNB is $nameNNB, wmoHd is $wmohd
         \rstationId is $stationId, bulTime is $bulTime, bulYJd is $bulYJd, bulMmdd is $bulMmdd,
         \riniTime is $iniTime, stNumber is $stNumber, forecast hour is $forecastHour.\n";
}
